#ifndef LAMBDADEPENDCORRECTION
#define LAMBDADEPENDCORRECTION

#include "ManipulatorBase.hh"
#include "ElementContainerMatrix.hh"
#include "UtsusemiHeader.hh"
#include "StringTools.hh"

////////////////////////////////////////
// Correct ECM with lambda dependency
////////////////////////////////////////

//! Function to correct ElementContainerMatrix using give lambda dependency
/*! 
 *   # Usage
 *   # DAT : ElementContainer, depend_data.txt : lambda dependency text file
 *   # Example of lambda dependency text is below. ( # charactor at head of a line is means ignored line. )
 *   #  # Lambda, Factor
 *   #  0.7, 1.0
 *   #  0.8, 1.2
 *   #  ...
 *   LD = LambdaDependCorrection(DAT)
 *   if LD.SetCurveTextDep("ALL","depend_data.txt"):
 *       if LD.Execute():
 *           .. Succeeded ..
 *       else:
 *           .. Failed ..
 *    
 *
 */
class LambdaDependCorrection:
public ManipulatorBase<ElementContainerMatrix>{
private:

protected:
    const string _MessageTag;
    vector<string> _bankNames;
    string _depType;
    Map< pair< string,vector<double> > > _depParams;
    StringTools* _st;
    static const string DEP_GIVENCURVE;
    static const string DEP_SINGLELINE;

    void _addToParams( string bank, string type, vector<double> params );
    double _calFact( double x0, double y0, double x1, double y1, double x );
    bool _debugmode;

public:
    LambdaDependCorrection(ElementContainerMatrix* ecm=NULL);
        //!< Constructor
        /*!<
         *  @param ecm ElementContainerMatrix for the target
         */
    ~LambdaDependCorrection();
        //!< Destructor
        /*!<
         */
    bool SetCurveTextDep( string bank, string filepath );
        //!< Set parameters for the dependency of given curve
        /*!< function of lambda
         *
         *  @param bank (string) bank name
         *  @param filepath (string) text file path
         *  @retval true  succeeded.
         *  @retval false failed.
         */
    bool SetSingleLineDep( string bank, string params );
        //!< Set parameters for single line dependency
        /*!< line defined : A+B*lambda
         *   params = "<A>,<B>"
         *
         *  @param bank (string) bank name
         *  @param params (string) parameters
         *  @retval true  succeeded.
         *  @retval false failed.
         */
    void ClearParams( string bank="ALL" );
    vector<string> PutPresetBankName(){ return _bankNames; }
    bool Execute(ElementContainerMatrix* _ecm=NULL);
        //!< Execute correction
        /*!<
         *  @retval true  succeeded.
         *  @retval false failed.
         */
    bool CheckBankName(string);
};
#endif
