#include "ElementContainerUtility.hh"
#include "SearchInHeader.hh"
const double ElementContainerUtility::MINIMUM_VALUE=1e-20;
const double ElementContainerUtility::MINIMUN_ERROR=1e-20;

ElementContainer* ElementContainerUtility::SetScaling(ElementContainer* ec, double scaling)
{
	vector<Double> v = ec->PutY();
	
	unsigned int ii;
	
	for (ii = 0; ii < v.size(); ii++)
	{
		v[ii] = scaling;
	}
	
	if (ec->CheckKey(SCALING()) == 0) {
		ec->Add(SCALING(), v);
	} else {
		ec->Remove(SCALING());
		ec->Add(SCALING(), v);
	}
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::SetScaling(ElementContainerMatrix* ecm, double scaling)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				SetScaling(eca->PutPointer(pixel), scaling);
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::SolidAngleCorrection(ElementContainer* ec, double base_dS)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	// Get Solid Angle
	double sa = ec->PutHeaderPointer()->PutDouble("PixelSolidAngle");
	
	// solid angle correction
	double coef = base_dS / sa;
	
	unsigned int ii;
	
	if (ec->CheckKey(SCALING()) == 0) {
		vector<Double> err = ec->PutE();
		
		for (ii = 0; ii < intensity.size(); ii++)
		{
			intensity[ii] *= coef;
			err[ii] *= coef;
		}
	} else {
		vector<Double> scaling = ec->Put(SCALING());
		
		for (ii = 0; ii < intensity.size(); ii++)
		{
			intensity[ii] *= coef;
			err[ii] *= coef;
			scaling[ii] /= coef;
		}
		ec->Remove(SCALING());
		ec->Add(SCALING(), scaling);
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::SolidAngleCorrection(ElementContainerMatrix* ecm, double base_dS)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				SolidAngleCorrection(eca->PutPointer(pixel), base_dS);
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::KiKfCorrection(ElementContainer* ec, double Ei)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> energy = ec->PutX();
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	unsigned int ii;
	double coef;
	
	if (ec->CheckKey(SCALING()) == 0) {
		for (ii = 0; ii < intensity.size(); ii++)
		{
			coef = sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
			intensity[ii] *= coef;
			err[ii] *= coef;
		}
	} else {
		vector<Double> scaling = ec->Put(SCALING());
		
		for (ii = 0; ii < intensity.size(); ii++)
		{
			coef = sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
			intensity[ii] *= coef;
			err[ii] *= coef;
			scaling[ii] /= coef;
		}
		ec->Remove(SCALING());
		ec->Add(SCALING(), scaling);
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::KiKfCorrection(ElementContainerMatrix* ecm)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	// Get Incident Energy
	double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				KiKfCorrection(eca->PutPointer(pixel), Ei);
			}
		}
	}
	
	return ecm;
}


ElementContainer* ElementContainerUtility::ZeroToMinumumValue(ElementContainer* ec)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	unsigned int ii;
	int flag = 0;

	for (ii = 0; ii < intensity.size(); ii++)
	{
		if (intensity[ii] == 0) {
			intensity[ii] = MINIMUM_VALUE;
			err[ii] = MINIMUN_ERROR;
			flag = 1;
		}
	}

	if (flag == 1) {
		ec->Remove(yname);
		ec->Remove(ename);
		ec->Add(yname,intensity);
		ec->Add(ename,err);
		ec->SetKeys(xname,yname,ename);
	}
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::ZeroToMinumumValue(ElementContainerMatrix* ecm)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				ZeroToMinumumValue(eca->PutPointer(pixel));
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::SetTimeIndependentBkg(ElementContainer* ec, double bkg, int flagKikfCorrection, double Ei)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> energy = ec->PutX();
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	vector<double> posi_vec = ec->PutHeaderPointer()->PutDoubleVector("PixelPosition");
	double L2 = sqrt( posi_vec[0]*posi_vec[0] + posi_vec[1]*posi_vec[1] + posi_vec[2]*posi_vec[2] );
	
	// sqrt(Mn/2*J2meV / 2) = 2.286287e-3
	double coefBkg = bkg * 2.286287e-3 * (L2 * 1e-3) * 1e6;
	
	unsigned int ii;
	
	if (ec->CheckKey(SCALING()) == 0) {
		if (flagKikfCorrection == 0) {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				intensity[ii] = coefBkg *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii]));
				err[ii] = 0;
			}
		} else {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				intensity[ii] = coefBkg *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
				err[ii] = 0;
			}
		}
	} else {
		vector<Double> scaling = ec->Put(SCALING());
		
		if (flagKikfCorrection == 0) {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				intensity[ii] = coefBkg *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii]));
				err[ii] = 0;
				scaling[ii] = 1;
			}
		} else {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				intensity[ii] = coefBkg *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
				err[ii] = 0;
				scaling[ii] = 1;
			}
		}
		ec->Remove(SCALING());
		ec->Add(SCALING(), scaling);
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::SetTimeIndependentBkg(ElementContainerMatrix* ecm, double bkg, int flagKikfCorrection)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	// Get Incident Energy
	double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				SetTimeIndependentBkg(eca->PutPointer(pixel), bkg, flagKikfCorrection, Ei);
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::SetTimeIndependentBkg(ElementContainer* ec, ElementContainer* ecBkg, double tofLeft, double tofRight, int flagKikfCorrection, double Ei)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> energy = ec->PutX();
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	vector<double> posi_vec = ec->PutHeaderPointer()->PutDoubleVector("PixelPosition");
	double L2 = sqrt( posi_vec[0]*posi_vec[0] + posi_vec[1]*posi_vec[1] + posi_vec[2]*posi_vec[2] );
	
	// Summation
	double avrInt, avrErr;
	
	IntegrateIntensity(avrInt, avrErr, ecBkg, tofLeft, tofRight);
	
	avrInt /= (tofRight - tofLeft);
	avrErr /= (tofRight - tofLeft);
	
	// sqrt(Mn/2*J2meV / 2) = 2.286287e-3
	double coefWidth = 2.286287e-3 * (L2 * 1e-3) * 1e6;
	
	unsigned int ii;
	double coef;
	
	if (ec->CheckKey(SCALING()) == 0) {
		if (flagKikfCorrection == 0) {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii]));
				
				intensity[ii] = avrInt * coef;
				err[ii] = avrErr * coef;
			}
		} else {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
				
				intensity[ii] = avrInt * coef;
				err[ii] = avrErr * coef;
			}
		}
	} else {
		vector<Double> scaling = ec->Put(SCALING());
		
		double baseScaling = avrInt / (avrErr * avrErr);
		
		if (flagKikfCorrection == 0) {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii]));
				
				intensity[ii] = avrInt * coef;
				err[ii] = avrErr * coef;
				scaling[ii] = baseScaling / coef;
			}
		} else {
			for (ii = 0; ii < intensity.size(); ii++)
			{
				coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2));
				
				intensity[ii] = avrInt * coef;
				err[ii] = avrErr * coef;
				scaling[ii] = baseScaling / coef;
			}
		}
		ec->Remove(SCALING());
		ec->Add(SCALING(), scaling);
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}


ElementContainerMatrix* ElementContainerUtility::SetTimeIndependentBkg(ElementContainerMatrix* ecm, ElementContainerMatrix* ecmBkg, double tofLeft, double tofRight, int flagKikfCorrection)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	ElementContainerArray* ecaBkg;
	
	// Get Incident Energy
	double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	if (ecm->PutTableSize() != ecmBkg->PutTableSize()) {
		//cout << "the containers must have the same size of array of ElementContainerArray" << endl;
		return ecm;
	}
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		ecaBkg = ecmBkg->PutPointer(psd);
		
		if (eca->PutTableSize() != ecaBkg->PutTableSize()) {
			//cout << "the containers must have the same size of array of ElementContainer" << endl;
			return ecm;
		}
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				SetTimeIndependentBkg(eca->PutPointer(pixel), ecaBkg->PutPointer(pixel), tofLeft, tofRight, flagKikfCorrection, Ei);
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::SubtractTimeIndependentBkg(ElementContainer* ec, ElementContainer* ecBkg, double tofLeft, double tofRight, int flagKikfCorrection, double factor, double Ei)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> energy = ec->PutX();
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	vector<double> posi_vec = ec->PutHeaderPointer()->PutDoubleVector("PixelPosition");
	double L2 = sqrt( posi_vec[0]*posi_vec[0] + posi_vec[1]*posi_vec[1] + posi_vec[2]*posi_vec[2] );
	
	// Summation
	double avrInt, avrErr;
	
	IntegrateIntensity(avrInt, avrErr, ecBkg, tofLeft, tofRight);
	
	avrInt /= (tofRight - tofLeft);
	avrErr /= (tofRight - tofLeft);
	
	// sqrt(Mn/2*J2meV / 2) = 2.286287e-3
	double coefWidth = 2.286287e-3 * (L2 * 1e-3) * 1e6;
	
	unsigned int ii;
	double coef;
	
	// scaling must not be changed.
	
	if (flagKikfCorrection == 0) {
		for (ii = 0; ii < intensity.size(); ii++)
		{
			coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * factor;
			
			intensity[ii] -= avrInt * coef;
			err[ii] = sqrt(err[ii] * err[ii] + avrErr * avrErr * coef * coef);
		}
	} else {
		for (ii = 0; ii < intensity.size(); ii++)
		{
			coef = coefWidth *  (1 / sqrt(Ei - energy[ii + 1]) - 1 / sqrt(Ei - energy[ii])) * sqrt(Ei / (Ei - (energy[ii] + energy[ii + 1]) / 2)) * factor;
			
			intensity[ii] -= avrInt * coef;
			err[ii] = sqrt(err[ii] * err[ii] + avrErr * avrErr * coef * coef);
		}
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}


ElementContainerMatrix* ElementContainerUtility::SubtractTimeIndependentBkg(ElementContainerMatrix* ecm, ElementContainerMatrix* ecmBkg, double tofLeft, double tofRight, int flagKikfCorrection, double factor)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	ElementContainerArray* ecaBkg;
	
	// Get Incident Energy
	double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	if (ecm->PutTableSize() != ecmBkg->PutTableSize()) {
		//cout << "the containers must have the same size of array of ElementContainerArray" << endl;
		return ecm;
	}
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		ecaBkg = ecmBkg->PutPointer(psd);
		
		if (eca->PutTableSize() != ecaBkg->PutTableSize()) {
			//cout << "the containers must have the same size of array of ElementContainer" << endl;
			return ecm;
		}
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				SubtractTimeIndependentBkg(eca->PutPointer(pixel), ecaBkg->PutPointer(pixel), tofLeft, tofRight, flagKikfCorrection, factor, Ei);
			}
		}
	}
	
	return ecm;
}

ElementContainerMatrix* ElementContainerUtility::PsdIntCorrection(ElementContainerMatrix* ecm, ElementContainerMatrix* ecmRef, double tofLeft, double tofRight)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	ElementContainerArray* ecaRef;
	ElementContainer* ec;
	HeaderBase *h_ec;
	
	vector<Double> vRefInt;
	double sumPsd = 0.0;
	int countPsd = 0;
	
	// Get Incident Energy
	//double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	if (ecm->PutTableSize() != ecmRef->PutTableSize()) {
		//cout << "the containers must have the same size of array of ElementContainerArray" << endl;
		return ecm;
	}
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		ecaRef = ecmRef->PutPointer(psd);
		
		if (eca->PutTableSize() != ecaRef->PutTableSize()) {
			//cout << "the containers must have the same size of array of ElementContainer" << endl;
			return ecm;
		}
		
		double sumPixel = 0.0;
		int countPixel = 0;
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				ec = eca->PutPointer(pixel);
				h_ec = ec->PutHeaderPointer();
				
				if ((h_ec->CheckKey("MASKED") == 1) && (h_ec->PutInt4("MASKED") == 1)){
					continue;
				}
				
				double avrInt, avrErr;
				
				IntegrateIntensity(avrInt, avrErr, ecaRef->PutPointer(pixel), tofLeft, tofRight);
				
				sumPixel += avrInt;
				countPixel++;
			}
		}
		
		if (countPixel > 0 && sumPixel > 0.0) {
			sumPsd += sumPixel / countPixel;
			countPsd += 1;
			
			vRefInt.push_back(sumPixel / countPixel);
		} else {
			vRefInt.push_back(-1.0);
		}
	}
	
	if (countPsd == 0) {
		//cout << "no data of reference ECM" << endl;
		return ecm;
	}
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		double refInt = vRefInt[psd];
		double factor = (sumPsd / countPsd) / refInt;
		
		if (refInt < 0.0) {
			continue;
		}
		
		//cout << "PSD-ID " << psd << ": factor=" << factor << endl;
		
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				Multiply(eca->PutPointer(pixel), factor);
			}
		}
		
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::AddConstValue(ElementContainer* ec, double value)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> intensity = ec->PutY();
	
	unsigned int ii;

	for (ii = 0; ii < intensity.size(); ii++)
	{
		intensity[ii] += value;
	}

	ec->Remove(yname);
	ec->Add(yname,intensity);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainer* ElementContainerUtility::SetConstValue(ElementContainer* ec, double value)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> intensity = ec->PutY();
	
	unsigned int ii;

	for (ii = 0; ii < intensity.size(); ii++)
	{
		intensity[ii] = value;
	}

	ec->Remove(yname);
	ec->Add(yname,intensity);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainer* ElementContainerUtility::CalcContainers(ElementContainer* ec1, ElementContainer* ec2, double coef1, double coef2)
{
	std::string xname = ec1->PutXKey();
	std::string yname = ec1->PutYKey();
	std::string ename = ec1->PutEKey();
	
	vector<Double> intensity1 = ec1->PutY();
	vector<Double> err1 = ec1->PutE();
	
	vector<Double> intensity2 = ec2->PutY();
	vector<Double> err2 = ec2->PutE();
	
	if (intensity1.size() != intensity2.size()) {
		//cout << "the containers must have the same size of y-vector" << endl;
		return ec1;
	}
	
	unsigned int ii;
	
	for (ii = 0; ii < intensity1.size(); ii++) {
		intensity1[ii] = intensity1[ii] * coef1 + intensity2[ii] * coef2;
		err1[ii] = sqrt(err1[ii] * err1[ii] * coef1 * coef1 + err2[ii] * err2[ii] * coef2 * coef2);
	}
	
	if (coef1 == 0) {
		if (ec2->CheckKey(SCALING()) != 0) {
			vector<Double> scaling2 = ec2->Put(SCALING());
			
			if (coef2 != 0) {
				for (ii = 0; ii < scaling2.size(); ii++) {
					scaling2[ii] /= coef2;
				}
			} else {
				for (ii = 0; ii < scaling2.size(); ii++) {
					scaling2[ii] = 0;
				}
			}
			
			ec1->Remove(SCALING());
			ec1->Add(SCALING(), scaling2);
		}
	} else {
		if (coef1 != 1 && ec1->CheckKey(SCALING()) != 0) {
			vector<Double> scaling1 = ec1->Put(SCALING());
			
			if (coef1 != 0) {
				for (ii = 0; ii < scaling1.size(); ii++) {
					scaling1[ii] /= coef1;
				}
			} else {
				for (ii = 0; ii < scaling1.size(); ii++) {
					scaling1[ii] = 0;
				}
			}
			ec1->Remove(SCALING());
			ec1->Add(SCALING(), scaling1);
		}
	}

	ec1->Remove(yname);
	ec1->Remove(ename);
	ec1->Add(yname,intensity1);
	ec1->Add(ename,err1);
	ec1->SetKeys(xname,yname,ename);
	
	return ec1;
}


ElementContainerMatrix* ElementContainerUtility::CalcContainers(ElementContainerMatrix* ecm1, ElementContainerMatrix* ecm2, double coef1, double coef2)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca1;
	ElementContainerArray* eca2;
	
	if (ecm1->PutTableSize() != ecm2->PutTableSize()) {
		//cout << "the containers must have the same size of array of ElementContainerArray" << endl;
		return ecm1;
	}
	
	for (psd = 0; psd < ecm1->PutTableSize(); psd++) {
		eca1 = ecm1->PutPointer(psd);
		eca2 = ecm2->PutPointer(psd);
		
		if (eca1->PutTableSize() != eca2->PutTableSize()) {
			//cout << "the containers must have the same size of array of ElementContainer" << endl;
			return ecm1;
		}
		
		for (pixel = 0; pixel < eca1->PutTableSize(); pixel++) {
			CalcContainers(eca1->PutPointer(pixel), eca2->PutPointer(pixel), coef1, coef2);
		}
	}
	
	return ecm1;
}

ElementContainer* ElementContainerUtility::Multiply(ElementContainer* ec, double value)
{
	std::string xname = ec->PutXKey();
	std::string yname = ec->PutYKey();
	std::string ename = ec->PutEKey();
	
	vector<Double> intensity = ec->PutY();
	vector<Double> err = ec->PutE();
	
	unsigned int ii;
	
	if (ec->CheckKey(SCALING()) == 0) {
		for (ii = 0; ii < intensity.size(); ii++) {
			intensity[ii] *= value;
			err[ii] *= value;
		}
	} else {
		vector<Double> scaling = ec->Put(SCALING());
		
		if (value != 0) {
			for (ii = 0; ii < intensity.size(); ii++) {
				intensity[ii] *= value;
				err[ii] *= value;
				scaling[ii] /= value;
			}
		} else {
			for (ii = 0; ii < intensity.size(); ii++) {
				intensity[ii] = 0;
				err[ii] = 0;
				scaling[ii] = 0;
			}
		}
		ec->Remove(SCALING());
		ec->Add(SCALING(), scaling);
	}
	
	ec->Remove(yname);
	ec->Remove(ename);
	ec->Add(yname,intensity);
	ec->Add(ename,err);
	ec->SetKeys(xname,yname,ename);
	
	return ec;
}

ElementContainerMatrix* ElementContainerUtility::Multiply(ElementContainerMatrix* ecm, double value)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca;
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++)
	{
		eca = ecm->PutPointer(psd);
		
		if (eca->PutHeaderPointer()->PutString("TYPE") == "PSD")
		{
			for (pixel = 0; pixel < eca->PutTableSize(); pixel++)
			{
				Multiply(eca->PutPointer(pixel), value);
			}
		}
	}
	
	return ecm;
}

ElementContainer* ElementContainerUtility::AverageContainers(ElementContainer* ec1, ElementContainer* ec2)
{
	std::string xname = ec1->PutXKey();
	std::string yname = ec1->PutYKey();
	std::string ename = ec1->PutEKey();
	
	if (ec1->CheckKey(SCALING()) == 0 || ec2->CheckKey(SCALING()) == 0) {
		return ec1;
	}
	
	vector<Double> intensity1 = ec1->PutY();
	vector<Double> err1 = ec1->PutE();
	vector<Double> scaling1 = ec1->Put(SCALING());
	
	vector<Double> intensity2 = ec2->PutY();
	vector<Double> err2 = ec2->PutE();
	vector<Double> scaling2 = ec2->Put(SCALING());
	
	if (intensity1.size() != intensity2.size()) {
		//cout << "the containers must have the same size of y-vector" << endl;
		return ec1;
	}
	
	unsigned int ii;
	double c1, c2, s1, s2;
	
	unsigned int num = intensity1.size();
	
	for (ii = 0; ii < num; ii++)
	{
		s1 = scaling1[ii];
		s2 = scaling2[ii];
		
		c1 = s1 * intensity1[ii];
		c2 = s2 * intensity2[ii];
		
		intensity1[ii] = (c1 + c2) / (s1 + s2);
		err1[ii] = sqrt(c1 + c2) / (s1 + s2);
		scaling1[ii] = s1 + s2;
	}
	
	ec1->Remove(yname);
	ec1->Remove(ename);
	ec1->Add(yname,intensity1);
	ec1->Add(ename,err1);
	ec1->SetKeys(xname,yname,ename);
	
	ec1->Remove(SCALING());
	ec1->Add(SCALING(), scaling1);
	
	return ec1;
}

ElementContainerMatrix* ElementContainerUtility::AverageContainers(ElementContainerMatrix* ecm1, ElementContainerMatrix* ecm2)
{
	unsigned int psd, pixel;
	ElementContainerArray* eca1;
	ElementContainerArray* eca2;
	
	if (ecm1->PutTableSize() != ecm2->PutTableSize()) {
		//cout << "the containers must have the same size of array of ElementContainerArray" << endl;
		return ecm1;
	}
	
	for (psd = 0; psd < ecm1->PutTableSize(); psd++)
	{
		eca1 = ecm1->PutPointer(psd);
		eca2 = ecm2->PutPointer(psd);
		
		if (eca1->PutTableSize() != eca2->PutTableSize()) {
			//cout << "the containers must have the same size of array of ElementContainer" << endl;
			return ecm1;
		}
		
		for (pixel = 0; pixel < eca1->PutTableSize(); pixel++)
		{
			AverageContainers(eca1->PutPointer(pixel), eca2->PutPointer(pixel));
		}
	}
	
	return ecm1;
}

ElementContainer ElementContainerUtility::GetAverage(ElementContainerMatrix* ecm, vector<Int4> &psd_v, vector<Int4> &pixel_v)
{
	ElementContainer* ec;
	
	vector<Double> x_vec, y_vec, e_vec, s_vec;
	x_vec.clear();
	y_vec.clear();
	e_vec.clear();
	s_vec.clear();
	
	ElementContainer* ec0 = ecm->PutPointer(psd_v[0])->PutPointer(pixel_v[0]);
	HeaderBase hh = ec0->PutHeader();
	string xkey = ec0->PutXKey();
	string ykey = ec0->PutYKey();
	string ekey = ec0->PutEKey();
	
	x_vec = ec0->PutX();
	
	UInt4 v_size = x_vec.size() - 1;
	UInt4 num_of_pixel = psd_v.size();
	
	for (UInt4 i = 0; i < v_size; i++){
		y_vec.push_back(0.0);
		e_vec.push_back(0.0);
		s_vec.push_back(0.0);
	}
	
	HeaderBase *h_ec;
	UInt4 num_of_counts = 0;
	
	double scaling, intensity, error;
	
	for (UInt4 i = 0; i < (num_of_pixel); i++){
		ec = ecm->PutPointer(psd_v[i])->PutPointer(pixel_v[i]);
		h_ec = ec->PutHeaderPointer();
		if ((h_ec->CheckKey("MASKED") == 1) && (h_ec->PutInt4("MASKED") == 1)){
			continue;
		} else {
			vector<Double>* intensity_vec = (*ec)(ykey);
			vector<Double>* error_vec = (*ec)(ekey);
			//cout << "v_size=" << v_size << ", Ysize=" << intensity_vec.size() << endl;
			
			if (ec->CheckKey(SCALING()) == 0) {
				for (UInt4 j = 0; j < (v_size); j++){
					intensity = (*intensity_vec)[j];
					error = (*error_vec)[j];
					
					if (error <= 0) {
						continue;
					}
					
					//scaling = intensity / (error * error);
					y_vec[j] += intensity;
					e_vec[j] += error * error;
					s_vec[j] += 1;
				}
			} else {
				vector<Double>* scaling_vec = (*ec)(SCALING());
				
				for (UInt4 j = 0; j < (v_size); j++){
					intensity = (*intensity_vec)[j];
					error = (*error_vec)[j];
					scaling = (*scaling_vec)[j];
					
					y_vec[j] += intensity * scaling;
					e_vec[j] += error * error * scaling * scaling;
					s_vec[j] += scaling;
				}
			}
			
			num_of_counts += 1;
		}
	}
	//cout << "num_of_counts = " << num_of_counts << endl;
	
	Int4 masked_flag = 0;
	
	if (num_of_counts != 0){
		for (UInt4 i = 0; i < (v_size); i++){
			y_vec[i] = y_vec[i] / s_vec[i];
			e_vec[i] = sqrt(e_vec[i]) / s_vec[i];
		}
	}else{
		masked_flag = 1;
	}
	
	if (hh.CheckKey("MASKED") == 0){
		hh.Add("MASKED", masked_flag);
	}else{
		hh.OverWrite("MASKED", masked_flag);
	}
	
	ElementContainer ret_ec;
	
	ret_ec.InputHeader(hh);
	ret_ec.Add(xkey, x_vec);
	ret_ec.Add(ykey, y_vec);
	ret_ec.Add(ekey, e_vec);
	ret_ec.SetKeys(xkey, ykey, ekey);
	
	ret_ec.Add(SCALING(), s_vec);
	
	return ret_ec;
}

ElementContainer* ElementContainerUtility::SumOfPixelNormalizedTof(ElementContainerMatrix* ecm, int PSD_min, int PSD_max, int Pixel_min, int Pixel_max, double t0shift, double baseL1, double baseL2)
{
	unsigned int ii, psd, pixel;
	double L2;
	
	ElementContainerArray* eca;
	ElementContainer* ec;
	ElementContainer* ec_res = 0;
	//HeaderBase* pHeader;
	
	SearchInHeader* SIH = new SearchInHeader(ecm);
	
	SIH->SearchArray("PSDID", PSD_min, PSD_max);
	//vector<int> psd_vec = SIH->PutResultsPsd();
	vector<UInt4> psd_vec = SIH->PutResultIndex(0);
	
	delete SIH;
	
	//if (psd_vec[0] == -1) {
	if (psd_vec.empty()){
		cout << "can not find such PSDs as " << PSD_min << " to " << PSD_max << endl;
	}
	else
	{
		for (psd = 0; psd < psd_vec.size(); psd++) {
			eca = ecm->PutPointer(psd_vec[psd]);
			
			for (pixel = Pixel_min; pixel <= Pixel_max; pixel++) {
				ec = new ElementContainer(*(eca->PutPointer(pixel)));
				vector<double> posi_vec = ec->PutHeaderPointer()->PutDoubleVector("PixelPosition");
				L2 = sqrt( posi_vec[0]*posi_vec[0] + posi_vec[1]*posi_vec[1] + posi_vec[2]*posi_vec[2] );
				
				string xkey = ec->PutXKey();
				string ykey = ec->PutYKey();
				string ekey = ec->PutEKey();
				vector<double> tof = ec->PutX();
				
				vector<double> binTof = ec->PutX();
				
				for (ii = 0; ii < tof.size(); ii++) {
					tof[ii] = (tof[ii] - t0shift) / (baseL1 + L2) * (baseL1 + baseL2) + t0shift;
				}
				
				ec->Replace(xkey,tof);
				ec->SetKeys(xkey,ykey,ekey);
				
				ElementContainer new_ec = ec->Binning(binTof);
				
				ec->Replace(xkey, new_ec.PutX());
				ec->Replace(ykey, new_ec.PutY());
				ec->Replace(ekey, new_ec.PutE());
				ec->SetKeys(xkey, ykey, ekey);
				
				if (ec_res == 0){
					ec_res = ec;
				}
				else
				{
					*ec_res = *ec_res + *ec;
					
					delete ec;
				}
			}
		}
		
		ec_res->ReSetUnit("Intensity","neutrons");
		ec_res->ReSetUnit("Error","neutrons");
	}
	
	return ec_res;
}

void ElementContainerUtility::IntegrateIntensity(double& intensity, double& error, ElementContainer* ec, double left, double right)
{
	vector<Double> vX = ec->PutX();
	vector<Double> vIntensity = ec->PutY();
	vector<Double> vErr = ec->PutE();
	
	unsigned int ii, kk;
	unsigned int size = vIntensity.size();
	
	if (vX.size() != size + 1) {
		intensity = 0;
		error = -1;
		return;
	}
	
	intensity = 0;
	error = 0;
	
	if (vX[0] > left) {
		ii = 0;
		kk = 1;
	} else {
		for (ii = 0, kk = 1; ii < size; ii += 1, kk += 1) {
			if (vX[kk] > left) {
				if (vX[kk] < right) {
					intensity = vIntensity[ii] * (vX[kk] - left) / (vX[kk] - vX[ii]);
					error = vErr[ii] * vErr[ii] * (vX[kk] - left) / (vX[kk] - vX[ii]);
					break;
				} else {
					intensity = vIntensity[ii] * (right - left) / (vX[kk] - vX[ii]);
					error = vErr[ii] * sqrt((right - left) / (vX[kk] - vX[ii]));
					return;
				}
			}
		}
	}
	
	for ( ; ii < size; ii += 1, kk += 1) {
		if (vX[kk] > right) {
			intensity += vIntensity[ii] * (right - vX[ii]) / (vX[kk] - vX[ii]);
			error += vErr[ii] * vErr[ii] * (right - vX[ii]) / (vX[kk] - vX[ii]);
			break;
		}
		
		intensity += vIntensity[ii];
		error += vErr[ii] * vErr[ii];
	}
	
	error = sqrt(error);
	
	return;
}

void ElementContainerUtility::DoDetailedMaskQ(ElementContainerMatrix* ecm, double leftQ, double rightQ, double leftE, double rightE){
	
	// Get Incident Energy
	double Ei = ecm->PutHeaderPointer()->PutDouble("Ei");
	
	double leftTheta, rightTheta;
	
	unsigned int psd, pixel, ii;
	double alpha;
	
	ElementContainerArray* eca;
	ElementContainer* ec;
	HeaderBase* pHeader;
	
	alpha = 1.0 - leftQ * leftQ * 2.0720953 / 2.0 / Ei;
	if (alpha < -1.0) {
		leftTheta = 180.0;
	} else if (alpha > 1.0) {
		leftTheta = 0.0;
	} else {
		leftTheta = acos(alpha) / M_PI * 180.0;
	}
	
	alpha = 1.0 - rightQ * rightQ * 2.0720953 / 2.0 / Ei;
	if (alpha < -1.0) {
		rightTheta = 180.0;
	} else if (alpha > 1.0) {
		rightTheta = 0.0;
	} else {
		rightTheta = acos(alpha) / M_PI * 180.0;
	}
	
	cout << "2theta: " << leftTheta << " - " << rightTheta << endl;
	
	for (psd = 0; psd < ecm->PutTableSize(); psd++) {
		eca = (*ecm)(psd);
		
		for (pixel = 0; pixel < eca->PutTableSize(); pixel++) {
			ec = (*eca)(pixel);
			pHeader = ec->PutHeaderPointer();
			
			double pa = (pHeader->PutDoubleVector("PixelPolarAngle"))[0];
			
			if (pa < leftTheta || rightTheta <= pa) {
				continue;
			}
			
			vector<Double>* x_vec = (*ec)(ec->PutXKey());
			unsigned int size = x_vec->size() - 1;
			
			if (pHeader->CheckKey("DetailedMask")) {
				vector<int> detailedMask = pHeader->PutInt4Vector("DetailedMask");
				
				for (ii = 1; ii <= size; ++ii) {
					if ((*x_vec)[ii] > leftE) {
						break;
					}
				}
				for (--ii; ii < size; ++ii) {
					if ((*x_vec)[ii] > rightE) {
						break;
					}
					
					detailedMask[ii] = 1;
				}
				
				pHeader->Erase("DetailedMask");
				pHeader->Add("DetailedMask", detailedMask);
				
			} else {
				vector<int> detailedMask(size);
				
				for (ii = 1; ii <= size; ++ii) {
					if ((*x_vec)[ii] > leftE) {
						break;
					}
				}
				for (--ii; ii < size; ++ii) {
					if ((*x_vec)[ii] > rightE) {
						break;
					}
					
					detailedMask[ii] = 1;
				}
				
				pHeader->Add("DetailedMask", detailedMask);
			}
		}
	}
}

