#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

import time
import os
import Manyo as mm
import TkLocal
import math
import codecs
import numpy

import ana.Reduction.BaseCommands as BaseCom

if os.environ["MLF_SYS_NAME"]=="AMR":
    Def_NumOfPixel=120
    Def_L1=30.0
    Def_L2=4.0
    Def_L2max=4.5
    Def_dS=25.4*25.0
elif os.environ["MLF_SYS_NAME"]=="SIK":
    Def_NumOfPixel=100
    Def_L1=18.03
    Def_L2=2.50
    Def_L2max=2.834
    Def_dS=25.4*19.0
elif os.environ["MLF_SYS_NAME"]=="HRC":
    Def_NumOfPixel=100
    Def_L1=18.03
    Def_L2=2.50
    Def_L2max=2.8
    Def_dS=25.4*19.0
else:
    Def_NumOfPixel=100
    Def_L1=18.03
    Def_L2=2.50
    Def_L2max=2.834
    Def_dS=25.4*19.0

#########################################
def SetScaling(dat, scaling=1.0):
    """
    Function to set scaling value to histogram
    
    @param dat        (ElementContainerMatrix)
    """
    PROCESS = "SetScaling"
    #ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("SetScaling >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.SetScaling(dat, scaling)
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat)"
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)

    print("SetScaling >>> Cost of time : ",time.time()-start)

#########################################
def ZeroToMinimumValue(dat):
    """
    Function to set minimum values to data of zero count
    
    @param dat        (ElementContainerMatrix)
    """
    PROCESS = "ZeroToMinimumValue"
    #ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("ZeroToMinimumValue >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.ZeroToMinumumValue(dat)
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat)"
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)

    print("ZeroToMinimumValue >>> Cost of time : ",time.time()-start)

#########################################
def SetTimeIndependentBkgConst(dat,bkg):
    """
    Function to set time-independent background
    
    @param dat        (ElementContainerMatrix)
    @param bkg        (double) intensity of time-independet background [s-1 psd-1 pixel-1]
    @param L2         (dounle) L2
    """
    PROCESS = "SetTimeIndependentBkgConst"
    #ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("SetTimeIndependentBkgConst >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.SetTimeIndependentBkg(dat,bkg)
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat, bkg=%f)" % (bkg)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)

    print("SetTimeIndependentBkgConst >>> Cost of time : ",time.time()-start)

#########################################
def SetTimeIndependentBkg(dat,datBkg,tofLeft,tofRight):
    """
    Function to set time-independent background
    
    @param dat      (ElementContainerMatrix)
    @param datBkg   (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of time-independet background [us]
    @param tofRight (double) right limit of tof range of time-independet background [us]
    """
    PROCESS = "SetTimeIndependentBkg"
    #ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("SetTimeIndependentBkg >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.SetTimeIndependentBkg(dat,datBkg,tofLeft,tofRight,1)
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat, datBkg, tofLeft=%f, tofRight=%f)" % (tofLeft,tofRight)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)
    
    print("SetTimeIndependentBkg >>> Cost of time : ",time.time()-start)

#########################################
def SubtractTimeIndependentBkg(dat,datBkg,tofLeft,tofRight,factor):
    """
    Function to set time-independent background
    
    @param dat      (ElementContainerMatrix)
    @param datBkg   (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of time-independet background [us]
    @param tofRight (double) right limit of tof range of time-independet background [us]
    @param factor   (double) normalization factor
    """
    PROCESS = "SubtractTimeIndependentBkg"
    #ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("SubtractTimeIndependentBkg >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.SubtractTimeIndependentBkg(dat,datBkg,tofLeft,tofRight,1,factor)
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat, datBkg, tofLeft=%f, tofRight=%f, factor=%f)" % (tofLeft,tofRight,factor)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)
    
    print("SubtractTimeIndependentBkg >>> Cost of time : ",time.time()-start)

#########################################
def PsdIntCorrection(dat,datRef,tofLeft, tofRight):
    """
    Correction of Intensity on each PSD for using R/C etc.
    
    @param dat      (ElementContainerMatrix)
    @param datRef   (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of reference data [us]
    @param tofRight (double) right limit of tof range of reference data [us]
    """
    PROCESS = "PsdIntCorrection"
    ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("PsdIntCorrection >>> executing...")
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.PsdIntCorrection(dat,datRef,tofLeft,tofRight)
    
    del ecu
    
    ## Add Header Info
    SS = PROCESS+"(dat, datRef, tofLeft=%f, tofRight=%f)" % (tofLeft,tofRight)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)
    
    print("PsdIntCorrection >>> Cost of time : ",time.time()-start)

##########################################
def ToPowder(dat,startQ=0.0,endQ=0.0,deltaQ=0.0):
    """
    Convert data from crystal to powder

    @param dat    (ElementContaienrMatrix)
    @param startQ (double) Q value of range
    @param endQ   (double) Q value of range
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    PROCESS = "ToPowder"
    
    cp = TkLocal.CalcPowder(dat)
    ecm = cp.ToPowder(startQ,endQ,deltaQ)
    del cp
    
    ## Add Header Info
    SS = PROCESS+"(dat,startQ=%f,endQ=%f,deltaQ=%f)" % (startQ,endQ,deltaQ)
    hh = ecm.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    hh.OverWrite("DATAPROCESSED",process_vect)
    
    return ecm

##########################################
def ToPowderAutoBin(dat,deltaQ=0.0):
    """
    Convert data from crystal to powder

    @param dat    (ElementContaienrMatrix)
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    PROCESS = "ToPowder"
    
    cp = TkLocal.CalcPowder(dat)
    ecm = cp.ToPowderAutoBin(deltaQ)
    del cp
    
    ## Add Header Info
    SS = PROCESS+"(dat,deltaQ=%f)" % (deltaQ)
    hh = ecm.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    hh.OverWrite("DATAPROCESSED",process_vect)
    
    return ecm

##########################################
def ToPowderRect(dat,startQ=0.0,endQ=0.0,deltaQ=0.0):
    """
    Convert data from crystal to powder

    @param dat    (ElementContaienrMatrix)
    @param startQ (double) Q value of range
    @param endQ   (double) Q value of range
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    PROCESS = "ToPowder"
    
    cp = TkLocal.CalcPowder(dat)
    ecm = cp.ToPowderRect(startQ,endQ,deltaQ)
    del cp
    
    ## Add Header Info
    SS = PROCESS+"(dat,startQ=%f,endQ=%f,deltaQ=%f)" % (startQ,endQ,deltaQ)
    hh = ecm.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    hh.OverWrite("DATAPROCESSED",process_vect)
    
    return ecm

#########################################
def SumOfPixelNormalizedTof(Target, PSD_min=1, PSD_max=1, Pixel_min=1, Pixel_max=1, t0shift=0.0, L1=-1, L2=-1):
    """
    Put summation of intensity along Pixel
    @param Target    (ElementContainerMatrix)
    @param PSD_min   (int) lower limit of PSD-ID range
    @param PSD_max   (int) upper limit of PSD-ID range
    @param Pixel_min (int) lower limit of Pixel range
    @param Pixel_max (int) upper limit of Pixel range
    @param t0shift   (double) T0 offset [micro-sec]
    @param L1        (double)  Distance between moderator to sample [m]
    @param L2        (double)  between sample to PSD avarage [m]
    @retval EC
    """
    
    if PSD_min>PSD_max:
        temp = PSD_max
        PSD_max = PSD_min
        PSD_min = temp
    if Pixel_min>Pixel_max:
        temp = Pixel_max
        Pixel_max = Pixel_min
        Pixel_min = temp
    
    if L1 < 0 :
        baseL1 = Def_L1 * 1000
    else :
        baseL1 = L1 * 1000
    
    if L2 < 0 :
        baseL2 = Def_L2 * 1000
    else :
        baseL2 = L2 * 1000
    
    ecu = TkLocal.ElementContainerUtility()
    ec_res = ecu.SumOfPixelNormalizedTof(Target,PSD_min,PSD_max,Pixel_min,Pixel_max,t0shift,baseL1,baseL2)
    del ecu

    ## Add Header Info
    SS = "SumOfPixelNormalizedTof(Target, PSD_min=%d, PSD_max=%d, Pixel_min=%d, Pixel_max=%d, L1=%f, L2=%f)" % (PSD_min,PSD_max,Pixel_min,Pixel_max,L1,L2)
    hh = ec_res.PutHeaderPointer()
    if hh.CheckKey("DATAPROCESSED")==0:
        process_vect = mm.StringVector()
        process_vect.append(SS)
        hh.Add("DATAPROCESSED",process_vect)
    else:
        process_vect = hh.PutStringVector("DATAPROCESSED")
        process_vect.append(SS)
        hh.OverWrite("DATAPROCESSED",process_vect)

    return ec_res

#########################################
def SumOfPixel(Target, PSD_min=1, PSD_max=1, Pixel_min=1, Pixel_max=1):
    """
    Put summation of intensity along Pixel
    @param Target    (ElementContainerMatrix)
    @param PSD_min   (int) lower limit of PSD-ID range
    @param PSD_max   (int) upper limit of PSD-ID range
    @param Pixel_min (int) lower limit of Pixel range
    @param Pixel_max (int) upper limit of Pixel range
    @retval EC
    """
    
    if PSD_min>PSD_max:
        temp = PSD_max
        PSD_max = PSD_min
        PSD_min = temp
    if Pixel_min>Pixel_max:
        temp = Pixel_max
        Pixel_max = Pixel_min
        Pixel_min = temp
    
    SIH = mm.SearchInHeaders()
    SIH.SetTarget(Target)
    SIH.SearchArray("PSDID",PSD_min,PSD_max)
    psds = SIH.PutResultsPsd()
    del SIH
    
    ec_res = None
    
    if psds[0]==-1:
        print("can not find such PSDs as ",PSD_min," to ",PSD_max)
    else:
        for psd in range(psds.size()):
            eca = Target.Put(psds[psd])
            
            for pixel in range(Pixel_min,(Pixel_max+1)):
                ec = eca.Put(pixel)
                
                if ec_res == None :
                    ec_res = ec
                else :
                    ec_res = ec_res + ec
        
        ec_res.ReSetUnit("Intensity","neutrons")
        ec_res.ReSetUnit("Error","neutrons")

    ## Add Header Info
    SS = "SumOfPixel(Target, PSD_min=%d, PSD_max=%d, Pixel_min=%d, Pixel_max=%d)" % (PSD_min,PSD_max,Pixel_min,Pixel_max)
    hh = ec_res.PutHeaderPointer()
    if hh.CheckKey("DATAPROCESSED")==0:
        process_vect = mm.StringVector()
        process_vect.append(SS)
        hh.Add("DATAPROCESSED",process_vect)
    else:
        process_vect = hh.PutStringVector("DATAPROCESSED")
        process_vect.append(SS)
        hh.OverWrite("DATAPROCESSED",process_vect)

    return ec_res

#########################################
def GetBeamCurrent(dat,StartTime=0.0,EndTime=0.0):
    """
    get Beam current from web.
    @param dat        (ElementContainer,-Array,-Matrix)
    @param start_time (string) the start time of measurement start [s]
    @param end_time   (string) the end time of measurement end [s]
    @retval beamCurrent        the number of protons [TelaProtons]
    """
    import datetime

    runNo = int(dat.PutHeader().PutString("RUNNUMBER"))
    mpv = None
    if dat.PutHeader().CheckKey("MEASPERIOD")==1:
        mpv = dat.PutHeader().PutDoubleVector("MEASPERIOD")
        if mpv.size()!=14:
            mpv = None
    
    if (mpv==None):
        TT = mm.T0TreatToolsPsd()
        st_RUNNO="%3s%06d" % (os.environ["MLF_SYS_NAME"],runNo)
        
        list_DATA = os.listdir( os.environ["MLF_DATA_DIR"] )
        target_dir = ""
        for data_dir in list_DATA:
            if st_RUNNO in data_dir:
                target_dir = data_dir
                break
        if target_dir=="":
            err_message = "There is no data;"+st_RUNNO
            raise UserWarning(err_message)
        edb_list = os.listdir( os.path.join( os.environ["MLF_DATA_DIR"], target_dir ) )
        edb_list.sort()
        target_edb=""
        for a_edb in edb_list:
            if ".edb" in a_edb:
                target_edb = a_edb
                break
        if target_edb=="":
            err_message = "There is no edb data;"+st_RUNNO
            raise UserWarning(err_message)
        TT.readOrgEvent( os.path.join( os.environ["MLF_DATA_DIR"], target_dir, target_edb ) )
        mpv = TT.putMeasPeriodFromT0()
        del TT
        
    if mpv.size()!=14:
        err_message = "False to get measuring period from Event Data"
        raise UserWarning(err_message)
    
    firstDatetime = datetime.datetime(int(mpv[0]),int(mpv[1]),int(mpv[2]),int(mpv[3]),int(mpv[4]),int(mpv[5]))
    lastDatetime = datetime.datetime(int(mpv[7]),int(mpv[8]),int(mpv[9]),int(mpv[10]),int(mpv[11]),int(mpv[12]))
    
    if StartTime > 0 :
        startDatetime = firstDatetime + datetime.timedelta(seconds=StartTime)
    else :
        startDatetime = firstDatetime
    
    if EndTime > 0 :
        endDatetime = firstDatetime + datetime.timedelta(seconds=EndTime)
        
        if (lastDatetime < endDatetime) :
            endDatetime = lastDatetime
        
    else :
        endDatetime = lastDatetime
    
    start_date = "%04d/%02d/%02d" % (startDatetime.year,startDatetime.month,startDatetime.day)
    start_time = "%02d:%02d:%02d" % (startDatetime.hour,startDatetime.minute,startDatetime.second)
    end_date   = "%04d/%02d/%02d" % (endDatetime.year,endDatetime.month,endDatetime.day)
    end_time   = "%02d:%02d:%02d" % (endDatetime.hour,endDatetime.minute,endDatetime.second)
    
    print("GetBeamCurrent(runNo=%d)" % runNo)
    print("     begin date and time = ",start_date," ",start_time)
    print("       end date and time = ",end_date," ",end_time)
    
    bc = BaseCom.getBeamCurrent(start_date, start_time, end_date, end_time)
    if bc<0:
        print("BeamCurrent is invalid.")
        return 0
    
    return bc
    

#########################################
def GetMeasuringTime(dat,StartTime=0.0,EndTime=0.0):
    """
    Get measuring time [min] from event data.
    @param dat        (ElementContainer,-Array,-Matrix)
    @param start_time (string) the start time of measurement start [s]
    @param end_time   (string) the end time of measurement end [s]
    @retval minutes
    """
    import datetime
    
    runNo = int(dat.PutHeader().PutString("RUNNUMBER"))
    mpv = None
    if dat.PutHeader().CheckKey("MEASPERIOD")==1:
        mpv = dat.PutHeader().PutDoubleVector("MEASPERIOD")
        if mpv.size()!=14:
            mpv = None
    
    if (mpv==None):
        TT = mm.T0TreatToolsPsd()
        st_RUNNO="%3s%06d" % (os.environ["MLF_SYS_NAME"],runNo)
        
        list_DATA = os.listdir( os.environ["MLF_DATA_DIR"] )
        target_dir = ""
        for data_dir in list_DATA:
            if st_RUNNO in data_dir:
                target_dir = data_dir
                break
        if target_dir=="":
            err_message = "There is no data;"+st_RUNNO
            raise UserWarning(err_message)
        edb_list = os.listdir( os.path.join( os.environ["MLF_DATA_DIR"], target_dir ) )
        edb_list.sort()
        target_edb=""
        for a_edb in edb_list:
            if ".edb" in a_edb:
                target_edb = a_edb
                break
        if target_edb=="":
            err_message = "There is no edb data;"+st_RUNNO
            raise UserWarning(err_message)
        TT.readOrgEvent( os.path.join( os.environ["MLF_DATA_DIR"], target_dir, target_edb ) )
        mpv = TT.putMeasPeriodFromT0()
        del TT
        
    if mpv.size()!=14:
        err_message = "False to get measuring period from Event Data"
        raise UserWarning(err_message)
    
    firstDatetime = datetime.datetime(int(mpv[0]),int(mpv[1]),int(mpv[2]),int(mpv[3]),int(mpv[4]),int(mpv[5]))
    lastDatetime = datetime.datetime(int(mpv[7]),int(mpv[8]),int(mpv[9]),int(mpv[10]),int(mpv[11]),int(mpv[12]))
    
    if StartTime > 0 :
        startDatetime = firstDatetime + datetime.timedelta(seconds=StartTime)
    else :
        startDatetime = firstDatetime
    
    if EndTime > 0 :
        endDatetime = measDatetime + datetime.timedelta(seconds=EndTime)
        
        if (lastDatetime < endDatetime) :
            endDatetime = lastDatetime
        
    else :
        endDatetime = lastDatetime
    
    mins = float((endDatetime - startDatetime).seconds) / 60.0 + float((endDatetime - startDatetime).days) * 24.0 * 60.0
    
    return mins

##########################################
def CalcContainers(dat1,dat2,coef1=1.0,coef2=0.0):
    """
    Execute caluculation ret = coef1*dat1 + coef2*dat2
    @param  dat1   ElementContainerMatrix or Array  first data
    @param  dat2   ElementContainerMatrix or Array  second data
    @param  coef1  Double    coefficient of dat1
    @param  coef2  Double    coefficient of dat2
    @param  ope    string    operator
    @retval ret    Return Value
    """ 

    if (coef1==0.0) and (coef2==0.0):
        raise UserWarning("Both Coefficents are 0.")

    if type(dat1)!=type(dat2):
        print("Arguments should be same type.")
        raise UserWarning("Arguments should be same type.")
        
    if type(dat1)==type(mm.ElementContainerMatrix()):
        ret = mm.ElementContainerMatrix(dat1)
        
        ecu = TkLocal.ElementContainerUtility()
        ecu.CalcContainers(ret, dat2, coef1, coef2)
        
        del ecu
        
        return ret
    else:
        raise UserWarning("Types of the containers must be ElementContainerMatrix.")

#########################################
def SolidAngleCorrection(dat,dS=-1.0):
    """
    Correction for solid angle of pixel
    Each intensity is devided by solid angle * default value
    @param dat (ElementContainerMatrix)
    @param dS (Double) the area of 1 pixel on PSD. -1.0 means default value.
    """
    PROCESS = "SolidAngleCorrection"
    ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    print("SolidAngleCorrection >>> executing...")
    L2 = Def_L2*1000.0     # [mm]
    if dS==-1.0:
        dS = Def_dS        # [mm]
    
    dOmega = dS/(L2*L2)
    
    start = time.time()
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.SolidAngleCorrection(dat, dOmega)
    
    del ecu

    ## Add Header Info
    SS = PROCESS+"(dat, dS=%f)" % (dS)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)

    print("SolidAngleCorrection >>> Cost of time : ",time.time()-start)

#########################################
def KiKfCorrection(dat):
    """
    Calculate polar-angle and azim-angle of each Pixel from Pixel Position
    @param dat (ElementContainerMatrix)
    """
    PROCESS = "Ki/kf"
    ret = BaseCom.CheckDataProcess(dat,PROCESS)
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.KiKfCorrection(dat)
    
    del ecu

#########################################
def DoDetailedMask(dat,path="./",filename="detaildMask.txt"):
    """
    MASKED flag controled by file.
    @param dat (ElementContainerMatrix)
    @param filename (string) filename of mask information 
    """
    PROCESS = "DoDetailedMask"
    
    print("DoDetailedMask >>> executing...")
    
    filepath = os.path.join(path,filename)
    
    if os.path.exists(filepath):
        
        mkfile = open(filepath, "r")
        lines = mkfile.readlines()
        
        ecu = TkLocal.ElementContainerUtility()
        
        for line in lines :
            ll = line.split(",")
            if len(ll) == 0:
                pass
            elif ll[0][0] is "#" :
                pass
            elif len(ll) == 5:
                if ll[0] is "Q" :
                    print("DoDetailedMaskQ(dat,%g,%g,%g,%g)" % (float(ll[1]),float(ll[2]),float(ll[3]),float(ll[4])))
                    ecu.DoDetailedMaskQ(dat,float(ll[1]),float(ll[2]),float(ll[3]),float(ll[4]))
                else:
                    print("DoDetailedMask >>> ERROR: [",line[:-1],"] can not be interpretated.")
            else:
                print("DoDetailedMask >>> ERROR: [",line[:-1],"] can not be interpretated.")
        
    else:
        print("DoDetailedMask >>> ERROR:",filepath," is not found.")

    ## Add Header Info
    SS = PROCESS+"(dat, path=%s, filename=%s)" % (path,filename)
    hh = dat.PutHeaderPointer()
    process_vect = hh.PutStringVector("DATAPROCESSED")
    process_vect.append(SS)
    if hh.CheckKey("DATAPROCESSED")==0:
        hh.Add("DATAPROCESSED",process_vect)
    else:
        hh.OverWrite("DATAPROCESSED",process_vect)

#########################################
def LoadDataFromDump(path="./",filename="Sample.dmp"):
    """
    Load ElementContainerMatrix from dump file made by ECMSerialize class
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval DAT        ElementContainer/Array/Matrix
    """
    import ana.Reduction.ECMSerializeTk as ECMS
    ecms = ECMS.ECMSerialize()
    filepath = os.path.join(path,filename)
    
    if os.path.exists(filepath):
        DAT = ecms.Decode(filepath)
        del ecms
        return DAT
    else:
        err_message="LoadDataFromDump >>> Could not find such file."+filepath
        del ecms
        raise UserWarning(err_message)
        return None


#########################################
def SaveDataToDump(dat, path="./",filename="Sample.dmp"):
    """
    Serialize ElementContainerMatrix to dump file using ECMSerialize class.
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval None
    """
    import ana.Reduction.ECMSerializeTk as ECMS
    ecms = ECMS.ECMSerialize()
    filepath = os.path.join(path,filename)
    if os.path.exists(filepath):
        print("SaveDataFromDump >>> This file is already existed.",filepath)
        os.system("mv "+filepath+" "+filepath+".pre")
        print("SaveDataFromDump >>> Original data was moved to.",(filepath+".pre"))
    
    ecms.Encode(dat,filepath)
    del ecms
    return

#########################################
def GetProfilesSum(dat, startDetID, endDetId, exDetIds="", bins=1):
    """
    Get Summation of Pulse Height Profiles of PSD
    Each PSD pulse heignt profile is stored in header of ElementContaienrArray.
    @param  dat   (ElementContainerMatrix) Loaded ElementContainerMatrix.
    @param  startDetID (int) Start Detector ID of summation
    @param  endDetID (int) End Detector ID of summation
    @param  exDetIds (String) Detector ID (-1 : put all)
    @param  bins (double) binning of pulse height
    @retval EC    ElementContainer or list of ElementContainers of pulse height profile
    """

    def MakeProfileEC(ecm,index):
        hh = ecm(index).PutHeader()
        vect = hh.PutInt4Vector("PHProfile")
        hh.Erase("MASKEDElementContainer")
        hh.Erase("PHProfile")
        ec = mm.ElementContainer(hh)
        x=[]
        y=[]
        e=[]
        num=vect.size()
        for i in range(num):
            x.append(i)
            y.append(float(vect[i]))
            e.append(math.sqrt(float(vect[i])))
        x.append(num)
        
        ec.Add("PulseHeight",x)
        ec.Add("Counts",y)
        ec.Add("Error",e)
        ec.SetKeys("PulseHeight","Counts","Error")

        return ec
    
    exDetIdList=exDetIds.split(";")
    
    flag=False
    
    for ii in range(dat.PutTableSize()):
        psdId=dat(ii).PutHeaderPointer().PutInt4("PSDID")
        
        if (startDetID == -1 or startDetID <= psdId) and (endDetId == -1 or psdId <= endDetId) :
            if not str(psdId) in exDetIdList :
                EC_tmp = MakeProfileEC(dat,ii)
                
                if flag :
                    EC += EC_tmp
                    del EC_tmp
                else :
                    flag=True
                    EC = EC_tmp
    
    if not flag :
        EC = None
        raise UserWarning("Can not find specified detIDs")
    
    EC.SetUnit(EC.PutYKey(), "None")
    EC.SetUnit(EC.PutEKey(), "None")
    
    if bins > 1 :
        num=EC.PutSize(EC.PutXKey()) - 1
        newBin=[float(ii) for ii in range(0, num + bins, bins)]
        CTP=mm.CppToPython()
        EC_bin=EC.Binning(CTP.ListToDoubleVector(newBin))
        del EC
        return EC_bin
    
    return EC

#########################################
def FlexCalcTk(dat,Expression=""):
    """
    Flexible calculation of S(Q,hw) on ElementContainerMatrix.
    @param dat (ElementContainerMatrix)
    @param Factor (string) the expression of multiplying factor
    @retval (ElementContainerMatrix)
    """
    print("FlexCalc >>> executing...")
    import ana.Reduction.FlexCalcTk as FCTk
    ffunc=FCTk.FlexCalcTk(dat,Expression)
    ret = ffunc.DoFunc()
    ffunc.finish()
    return ret

#########################################
def SaveQSliceFromPowder(dat,path="./",filename="Sample",startQ=1.0,endQ=1.0,frcQ=1,bin=0.0,fOutScaling=0):
    """
    Save text files of Q-sliced powder data
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ    (double) minimum limit of Q-range
    @param  endQ   (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  bin      (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    @retval None
    """
    if not mm.CheckDataProcessed(dat,"ToPowder"):
        print("Data must be powder data.")
        raise UserWarning("Data must be powder data.")
    
    if type(dat)!=type(mm.ElementContainerMatrix()):
        raise UserWarning("Not ElementContainerMatrix")
    
    if endQ < startQ or frcQ < 1 :
        raise UserWarning("Parameter Error")
    
    if frcQ > 100 :
        raise UserWarning("please input frcQ <= 100")
    
    from vis.M2PlotTk import D2Matrix
    dd = D2Matrix()
    #(dat,err,X,Y,type)=dd.ReadMatrix( dat )
    
    print("Slice and output text...")
    from vis.D2VisTk import MapObj
    #dmap = MapObj( (dat,err,X,Y,type),1 )
    dmap = MapObj( dd.ReadMatrix( dat ),1 )
    
    datalst = []
    
    runno, ei = dd.GetHeaderInfo()
    main = "Run no.: %s" % runno
    sub = "Ei (meV): %.4f" % ei
    
    hh = dat.PutHeader()
    xlabel = "Q(1/Angstrom)"
    ylabel = "Energy(meV)"
    zlabel = "Neutrons"
    if hh.CheckKey("XLABEL"):
        xlabel = hh.PutString("XLABEL")
    if hh.CheckKey("YLABEL"):
        ylabel = hh.PutString("YLABEL")
    if hh.CheckKey("ZLABEL"):
        zlabel = hh.PutString("ZLABEL")
    
    ql = startQ
    qd = (endQ - startQ) / frcQ
    
    for ii in range(frcQ) :
        qc = ql + qd / 2.0
        qr = ql + qd
        
        obj = dmap.GetHistY([qc,ql,qr,bin])
        
        header = {}
        
        strQ = "%.3f (%.3f - %.3f)" % (obj.mp,obj.fp,obj.ep) 
        header["Label"] = strQ
        
        # Xデータ(bin の境界値)数 - 1 の array を作る
        xx = numpy.arange(0.0, len(obj.bins)-1, 1.0)
        # 各bin の中央値を作る
        for i in range(len(obj.bins)-1):
            xx[i]= (obj.bins[i]+obj.bins[i+1])/2.0
        
        # data[0:x, 1:y, 2:error, 3:scaling, 4:title1, 5:title2, 6:header, 
        # 7:XLABLE, 8:XUNIT, 9:YLABLE, 10:YUNIT]
        if (fOutScaling == 0) :
            datalst.append([xx,obj.counts,obj.errs,None,main,sub,header,ylabel,"",zlabel,""])
        else :
            datalst.append([xx,obj.counts,obj.errs,obj.scalings,main,sub,header,ylabel,"",zlabel,""])
        
        ql = qr
    
    WriteToTextFile( os.path.join(path,filename),datalst )

#########################################
def SaveQSliceFromHist(dat,path="./",filename="Sample",startQ=1.0,endQ=1.0,frcQ=1,bin=0.0,fOutScaling=0):
    """
    Save text files of Q-sliced powder data
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ    (double) minimum limit of Q-range
    @param  endQ   (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  bin      (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    @retval None
    """
    if mm.CheckDataProcessed(dat,"ToPowder"):
        print("Data must be data before processing ToPowder.")
        raise UserWarning("Data must be data before processing ToPowder.")
    
    if not mm.CheckDataProcessed(dat,"TOF TO ENERGY TRANSFER"):
        print("This data is invarid.")
        raise UserWarning("This data is invarid.")
    
    if type(dat)!=type(mm.ElementContainerMatrix()):
        raise UserWarning("Not ElementContainerMatrix")
    
    if endQ < startQ or frcQ < 1 :
        raise UserWarning("Parameter Error")
    
    if frcQ > 100 :
        raise UserWarning("please input frcQ <= 100")
    
    deltaQ = (endQ - startQ) / frcQ
    
    ecm = ToPowderRect(dat,startQ,endQ,deltaQ)
    
    SavePowderToText(ecm,path,filename,fOutScaling)

#########################################
def SavePowderToText(dat,path="./",filename="Sample",fOutScaling=0):
    """
    Save text files of Q-sliced powder data
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  fOutScaling  (int) flag of whether to output scaling
    @retval None
    """
    
    if not mm.CheckDataProcessed(dat,"ToPowder"):
        print("Data must be powder data.")
        raise UserWarning("Data must be powder data.")
    
    if type(dat)!=type(mm.ElementContainerMatrix()):
        raise UserWarning("Not ElementContainerMatrix")
    
    datalst = []
    
    hh = dat.PutHeader()
    
    if hh.CheckKey("RUNNUMBER"):
        main = "Run no.: %s" % hh.PutString("RUNNUMBER")
    else:
        main = ""
    
    if hh.CheckKey("Ei"):
        sub = "Ei (meV): %.4f" % hh.PutDouble("Ei")
    else:
        sub = ""
    
    if hh.CheckKey("SAMPLETYPE"):
        sampletype = hh.PutString("SAMPLETYPE")
    else:
        sampletype = "Unknown"
    
    xlabel = "Q(1/Angstrom)"
    ylabel = "Energy(meV)"
    zlabel = "Neutrons"
    if hh.CheckKey("XLABEL"):
        xlabel = hh.PutString("XLABEL")
    if hh.CheckKey("YLABEL"):
        ylabel = hh.PutString("YLABEL")
    if hh.CheckKey("ZLABEL"):
        zlabel = hh.PutString("ZLABEL")
    
    header = {}
    
    eca = dat(0)
    
    bins = eca(0).PutX()
    # Xデータ(bin の境界値)数 - 1 の array を作る
    xx = numpy.arange(0.0, len(bins)-1, 1.0)
    # 各bin の中央値を作る
    for i in range(len(bins)-1):
        xx[i]= (bins[i]+bins[i+1])/2.0
    
    for pixel in range(eca.PutTableSize()):
        ec = eca(pixel)
        ech = ec.PutHeaderPointer()
        
        header = {}
        
        if sampletype == "Powder":
            pol = ech.PutDoubleVector("PixelPolarAngle")
            polr = pol[0]*math.pi/180.0
            
            strQ = "%.3f (%.3f - %.3f) (deg)" % (pol[0],pol[0]-pol[1],pol[0]+pol[1])
            header["Label"] = strQ
        else:
            if ech.CheckKey("XRANGE"):
                xr = ech.PutDoubleVector("XRANGE")
                strQ = "%.3f (%.3f - %.3f) (A-1)" % ((xr[0]+xr[1])/2,xr[0],xr[1])
                header["Label"] = strQ
            else:
                header["Label"] = pixel
        
        ecp = TkLocal.HistogramBinToPointTk(ec)
        yy = ecp.PutYList()
        ee = ecp.PutEList()
        sc = ecp.PutSList()
        
        for st in range(len(sc)):
            if sc[st] > 0 :
                break
        
        for ed in range(len(sc)-1,st,-1):
            if sc[ed] > 0 :
                break
        
        # data[0:x, 1:y, 2:error, 3:scaling, 4:title1, 5:title2, 6:header, 
        # 7:XLABLE, 8:XUNIT, 9:YLABLE, 10:YUNIT]
        if (fOutScaling == 0) :
            datalst.append([xx[st:ed+1],yy[st:ed+1],ee[st:ed+1],None,main,sub,header,ylabel,"",zlabel,""])
        else :
            datalst.append([xx[st:ed+1],yy[st:ed+1],ee[st:ed+1],sc[st:ed+1],main,sub,header,ylabel,"",zlabel,""])
    
    WriteToTextFile( os.path.join(path,filename),datalst )


######################################################

FILE_HEADER = "**  MPLOT FILE **"
XUNIT = "X unit"
YUNIT = "Y unit"
XLABEL = "X label"
YLABEL = "Y label"

######################################################
def WriteToTextFile(path, datalst):
    """
    テキストファイルにデータを出力(トレースの数だけ)
    @param  path ファイルパス
    @param  datalst データのリスト
    @retval 無し
    """ 
    
    # 拡張子を分離
    fbody, ext = os.path.splitext(path)
    
    num = 1
    for data in datalst:
        fname = "%s-%d.txt" % (fbody, num)
        #print "open file :",fname
        num += 1
        # 指定されたファイルを開く
        try:
            fp = codecs.open(fname, 'w', "utf-8")
        except:
            raise IOError()
        else:
            WriteTextFile(fp, data)

######################################################
def WriteTextFile(fp, data):
    """
    テキストファイルにデータを出力(1トレース)
    @param  fp ファイルポインタ
    @param  data データ
    @retval 無し
    """  
    # data[0:x, 1:y, 2:error, 3:scaling, 4:title1, 5:title2, 6:header, 
    # 7:XLABLE, 8:XUNIT, 9:YLABLE, 10:YUNIT]
    
    # ヘッダー書き込み
    fp.write(FILE_HEADER+"\n")
    fp.write("-------------------\n")
    # タイトル書き込み
    main = data[4]
    sub = data[5]
    fp.write(main+"\n")
    fp.write(sub+"\n")
    fp.write("-------------------\n")
    # ヘッダー書き込み
    header = data[6]
    for item in header:
        item = "%s: %s\n" % (item, header[item])
        fp.write(item) 
    fp.write("-------------------\n")
    xlabel = data[7]
    ylabel = data[9]
    labels = "%s:%s\n" % (XLABEL, xlabel)
    labels += "%s:%s\n" % (XUNIT, data[8])
    labels += "%s:%s\n" % (YLABEL, ylabel)
    labels += "%s:%s\n" % (YUNIT, data[10])
    fp.write(labels)
    
    # データヘッダ書き込み
    dhead = "\n"
    if len(xlabel)<=15:
        dhead += "%s " % xlabel.center(15)
    else:
        dhead += "%s " % xlabel
    if len(ylabel)<=15:
        dhead += "%s " % ylabel.center(15)
    else:
        dhead += "%s " % ylabel
    if (data[3] == None) :
        dhead += "%15s\n" % ("     Error     ")
    else :
        dhead += "%15s " % ("     Error     ")
        dhead += "%15s\n" % ("    Scaling    ")
    fp.write(dhead)
    fp.write("-----------------------------------------------\n")
    
    xx = data[0]
    
    # データ書き込み
    if (data[3] == None) :
        for i in range(len(data[1])):
            dline = "%15e %15e %15e\n" % (xx[i],data[1][i],data[2][i] )
            fp.write(dline)
    else :
        for i in range(len(data[1])):
            dline = "%15e %15e %15e %15e\n" % (xx[i],data[1][i],data[2][i],data[3][i])
            fp.write(dline)
    
    # ヒストグラムデータなら
    if len(xx) > len(data[1]):
        # 最後のX値を書き込む
        dline = "%15e \n" % xx[i+1]
        fp.write(dline)
    
    # データ終了
    fp.write("-----------------------------------------------\n")
    # ファイルをクローズ
    fp.close           
            

#########################################
def SaveSumDetectorMapToText(dat,startX=0.0,endX=0.0,path="./",filename="Sample"):
    """
    Save text files of Q-sliced powder data
    @param  dat      (ElementContainerMatrix)
    @param  startX (double) top x value to be summed
    @param  endX   (double) end x value to be summed
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @retval None
    """
    
    if mm.CheckDataProcessed(dat,"ToPowder"):
        print("Data must be data before processing ToPowder.")
        raise UserWarning("Data must be data before processing ToPowder.")
    
    if type(dat)!=type(mm.ElementContainerMatrix()):
        raise UserWarning("Not ElementContainerMatrix")
    
    try:
      fp = codecs.open(os.path.join(path,filename), 'w', "utf-8")
    except:
        raise IOError
    
    hh = dat.PutHeader()
    
    if hh.CheckKey("RUNNUMBER"):
        main = "Run no.: %s" % hh.PutString("RUNNUMBER")
    else:
        main = ""
    
    sub = "range: %.4f - %.4f" % (startX, endX)
    
    # ヘッダー書き込み
    fp.write("-------------------\n")
    # タイトル書き込み
    fp.write(main+"\n")
    fp.write(sub+"\n")
    fp.write("-------------------\n")
    fp.write("Intensity\n")
    
    str1 = ""
    str2 = ""
    
    for psd in range(dat.PutTableSize()):
      eca = dat(psd)
      
      s0 = eca(0).Sum(startX, endX)
      
      str1 += "%.15g" % s0[0]
      str2 += "%.15g" % s0[1]

      for pixel in range(1, eca.PutTableSize()):
        s0 = eca(pixel).Sum(startX, endX)
        
        str1 += ",%.15g" % s0[0]
        str2 += ",%.15g" % s0[1]

      str1 += "\n"
      str2 += "\n"
    
    fp.write(str1)
    
    fp.write("\n-------------------\n")
    fp.write("Error\n")
    
    fp.write(str2)
    
    fp.close
