#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
FlexCalc : S(Q,hw)に対して、Q,hwを使用した計算を行う
"""

from __future__ import print_function

import Manyo as mm
import Manyo.Utsusemi as mu
import os,sys,time
import numpy

import Manyo.TKtools as TkLocal

class InvalidArgument(Exception):
    pass

class FlexFuncImportError(Exception):
    pass


class FlexCalcTk(object):
    ##################################################
    def __init__(self,DAT,EX="1",PATH=""):
        """
        Constructor

        @param DAT   (ElementContainerMatrix)
        @param EX    (String) Intensityなどに掛ける計算式
        @param PATH  (String) 計算用関数ファイルの保存場所
        """
        self.CommentHead = "FlexCalcTk >>> "
        
        # DATはElementContainerMatrixか
        #if type(DAT)!=type(mm.ElementContainerMatrix()):
        if not isinstance( DAT, mm.ElementContainerMatrix ):
            print(self.CommentHead+"Error: DAT is not ElementContainerMatrix.")
            raise InvalidArgument()
        """
        PROCESS = DAT.PutHeader().PutStringVector("DATAPROCESSED")
        if not "TOF TO ENERGY CONVERSION" in PROCESS:
            print self.CommentHead+"Error: You must execute TofToEnergyTransfer function on this data"
            raise InvalidArgument()
        """
        if not mu.CheckDataProcessed( DAT, "TOF TO ENERGY"):
            print(self.CommentHead+"Error: You must execute TofToEnergyTransfer function on this data")
            raise InvalidArgument()
        
        self.DAT=DAT

        # DATのデータの情報取得
        if DAT.PutHeader().CheckKey("SAMPLETYPE")==1:
            self.SAMPLETYPE = DAT.PutHeader().PutString("SAMPLETYPE")
        else:
            self.SAMPLETYPE = None

        # 計算式は入っているか
        if (EX!="1") and (type(EX)!=type("")):
            raise ValueError("Error: EX is not String.")

        # 計算式や関数ファイルの名前の登録
        self.EX=EX

        if (PATH!="") and os.path.exists(PATH):
            self.PATH=PATH
        else:
            if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
                self.PATH=os.path.join( os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","tmp" )
            else:
                self.PATH=os.path.join( os.environ["HOME"],"ana","tmp" )
            if not os.path.exists(self.PATH):
                self.PATH=""
        
        self.filename = "TempFlexCalcFile"+str(int(time.time()))+".py"

        # 計算式が空でなければ関数ファイル作成
        if self.EX!="1":
            self._MakeTempFile()
        
        
    def _MakeTempFile(self):
        """
        関数ファイルの作成
        これを読み込んで実際の計算に用いる。
        @retval None
        """
        # データがPowderなら、引数は|Q|、PowderでなければQx,Qy,Qzで関数を作成
        if self.SAMPLETYPE=="Powder" or self.SAMPLETYPE=="Rect":
            print(self.CommentHead+"SAMPELTYPE is Powder.")
            SS = "from numpy import *\n"
            SS+= "def func(hw,I,Err,Sc,Q):\n"
            SS+= "    fc="+self.EX+"\n"
            SS+= "    I*=fc\n"
            SS+= "    Err*=abs(fc)\n"
            SS+= "    Sc=where(fc!=0,Sc/fc,0)\n"
            SS+= "    return (I,Err,Sc)\n"
        else:
            print(self.CommentHead+"SAMPELTYPE is not Powder.")
            SS = "from numpy import *\n"
            SS+= "def func(hw,I,Err,Sc,Qx,Qy,Qz):\n"
            SS+= "    fc="+self.EX+"\n"
            SS+= "    I*=fc\n"
            SS+= "    Err*=abs(fc)\n"
            SS+= "    Sc=where(fc!=0,Sc/fc,0)\n"
            SS+= "    return (I,Err,Sc)\n"

        # 関数ファイルのオープンと書き込み
        try:
            fi = open(os.path.join(self.PATH,self.filename),"w")
        except:
            print(self.CommentHead+"Cannot open file:",os.path.join(self.PATH,self.filename))
            raise IOError
            
        fi.write(SS)
        fi.close()
        

    def DoFuncSelf(self):
        """
        計算実行
        @retval None
        """
        self._dofunc(self.DAT)

    def DoFunc(self):
        """
        オリジナルデータをコピーして計算の実行
        @retval new ElementContainerMatrix
        """
        DAT = mm.ElementContainerMatrix(self.DAT)
        self._dofunc(DAT)
        return DAT
        
    def _dofunc(self,DAT):
        """
        計算実行
        関数ファイルを読み込み関数登録して、実行する
        @param DAT (ElementContainerMatrix)
        @retval None
        """

        # 関数ファイルのインポート
        sys.path.insert(0,self.PATH)
        try:
            TF = __import__(self.filename.split(".")[0])
        except:
            raise FlexFuncImportError()

        # 入射エネルギーの入手とki算出
        Ei = self.DAT.PutHeader().PutDouble("Ei")
        Ki = numpy.sqrt( Ei/2.07213 )
        
        ecu = TkLocal.ElementContainerUtility()

        # 関数の実行
        if self.SAMPLETYPE=="Powder":
            for i in range(DAT.PutTableSize()):
                eca = DAT(i)
                #print self.CommentHead+"PSD no=",i
                for j in range(eca.PutTableSize()):
                    pangle_vec = eca(j).PutHeader().PutDoubleVector("PixelPolarAngle")

                    xx_org = eca(j).PutXList()
                    xx = []
                    for k in range(len(xx_org)-1):
                        xx.append( (xx_org[k]+xx_org[k+1])/2.0 )
                    xx = numpy.array(xx)
                    yy = numpy.array(eca(j).PutYList())
                    ee = numpy.array(eca(j).PutEList())
                    qq = self._calcQarray(Ei,Ki,pangle_vec[0],xx)
                    xkey = eca(j).PutXKey()
                    ykey = eca(j).PutYKey()
                    ekey = eca(j).PutEKey()
                    
                    if ecu.ScalingExists(eca(j)):
                        ss = numpy.array(ecu.PutScalingList(eca(j)))
                    else:
                        #dummy
                        ss = yy

                    (res_yy,res_ee,res_ss)=TF.func(xx,yy,ee,ss,qq)
                    eca(j).Replace(ykey,res_yy.tolist())
                    eca(j).Replace(ekey,res_ee.tolist())
                    eca(j).SetKeys(xkey,ykey,ekey)
                    
                    if ecu.ScalingExists(eca(j)):
                        eca(j).Replace(ecu.SCALING(),res_ss.tolist())
            #print xx
            #print qq
        elif self.SAMPLETYPE=="Rect":
            for i in range(DAT.PutTableSize()):
                eca = DAT(i)
                #print self.CommentHead+"PSD no=",i
                for j in range(eca.PutTableSize()):
                    xrange_vec = eca(j).PutHeader().PutDoubleVector("XRANGE")
                    const_q = (xrange_vec[0] + xrange_vec[1]) / 2
                    xx_org = eca(j).PutXList()
                    xx = []
                    qq = []
                    for k in range(len(xx_org)-1):
                        xx.append( (xx_org[k]+xx_org[k+1])/2.0 )
                        qq.append(const_q)
                    xx = numpy.array(xx)
                    yy = numpy.array(eca(j).PutYList())
                    ee = numpy.array(eca(j).PutEList())
                    qq = numpy.array(qq)
                    xkey = eca(j).PutXKey()
                    ykey = eca(j).PutYKey()
                    ekey = eca(j).PutEKey()
                    
                    if ecu.ScalingExists(eca(j)):
                        ss = numpy.array(ecu.PutScalingList(eca(j)))
                    else:
                        #dummy
                        ss = yy

                    (res_yy,res_ee,res_ss)=TF.func(xx,yy,ee,ss,qq)
                    eca(j).Replace(ykey,res_yy.tolist())
                    eca(j).Replace(ekey,res_ee.tolist())
                    eca(j).SetKeys(xkey,ykey,ekey)
                    
                    if ecu.ScalingExists(eca(j)):
                        eca(j).Replace(ecu.SCALING(),res_ss.tolist())
            #print xx
            #print qq
        else:
            for i in range(DAT.PutTableSize()):
                eca = DAT(i)
                #print self.CommentHead+"PSD no=",i
                for j in range(eca.PutTableSize()):
                    pangle_vec = eca(j).PutHeader().PutDoubleVector("PixelPolarAngle")
                    aangle_vec = eca(j).PutHeader().PutDoubleVector("PixelAzimAngle")
                    xx_org = eca(j).PutXList()
                    xx = []
                    for k in range(len(xx_org)-1):
                        xx.append( (xx_org[k]+xx_org[k+1])/2.0 )
                    xx = numpy.array(xx)
                    yy = numpy.array(eca(j).PutYList())
                    ee = numpy.array(eca(j).PutEList())
                    (qx,qy,qz) = self._calcQxyzarray(Ei,Ki,pangle_vec[0],aangle_vec[0],xx)
                    xkey = eca(j).PutXKey()
                    ykey = eca(j).PutYKey()
                    ekey = eca(j).PutEKey()

                    if ecu.ScalingExists(eca(j)):
                        ss = numpy.array(ecu.PutScalingList(eca(j)))
                    else:
                        #dummy
                        ss = yy

                    (res_yy,res_ee,res_ss)=TF.func(xx,yy,ee,ss,qx,qy,qz)
                    eca(j).Replace(ykey,res_yy.tolist())
                    eca(j).Replace(ekey,res_ee.tolist())
                    eca(j).SetKeys(xkey,ykey,ekey)
                    
                    if ecu.ScalingExists(eca(j)):
                        eca(j).Replace(ecu.SCALING(),res_ss.tolist())

                    
    def _calcQarray(self,Ei,Ki,pangle,hw):
        """
        |Q|の計算
        @param Ei     (Double) 入射エネルギー
        @param Ki     (Double) 入射波数ベクトル
        @param pangle (Double) PixelのPolar Angle
        @param hw     (numpy.array) エネルギー遷移情報
        @retval numpy.array of MomentTransfer
        """
        Kf = numpy.sqrt( ( Ei- hw )/2.07213 )
        return numpy.sqrt( Ki*Ki + Kf*Kf -2*Ki*Kf*numpy.cos( pangle*numpy.pi/180.0 ) )

    def _calcQxyzarray(self,Ei,Ki,pangle,aangle,hw):
        """
        (Qx, Qy, Qz)の計算
        @param Ei     (Double) 入射エネルギー
        @param Ki     (Double) 入射波数ベクトル
        @param pangle (Double) PixelのPolar Angle
        @param aangle (Double) Azimuthal Angle
        @param hw     (numpy.array) エネルギー遷移情報
        @retval tupples of numpy.array of MomentTransfer
        """
        Kf = numpy.sqrt( ( Ei- hw )/2.07213 )
        qx = Kf*numpy.sin( pangle*numpy.pi/180.0 )*numpy.cos( aangle*numpy.pi/180.0 )
        qy = Kf*numpy.sin( pangle*numpy.pi/180.0 )*numpy.sin( aangle*numpy.pi/180.0 )
        qz = Kf*numpy.cos( pangle*numpy.pi/180.0 )
        return (qx,qy,qz)

    def finish(self):
        """
        終了処理
        """
        if sys.platform=='linux2':
            com = "rm -rf "+os.path.join( self.PATH,self.filename )
            os.system( com )
        
