#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

import time
import os
import Manyo as mm
import Manyo.TKtools as TkLocal
import math
import numpy

import utsusemi.TKtools.ana.Reduction.TkCommands as TkCom
import utsusemi.ana.Reduction.BaseCommands as BaseCom

class ContianerError(Exception):
    pass

# special reserved word for commands history
HISTORY = None
# reserved words for return value in this.
DAT = None
DAT_BKG = None
DAT_REF = None
ECM = None
AC = None
#from ana.Reduction.DefaultInstParams import *

#########################################
def GetDataOfMonochroEi2(runNo="0",Ei=0.0,deltaE=0.0,Emin=0.0,Emax=0.0,MaskFile="mask.txt",NormFactor=1000000.0,offset_sec=0.0,LLDiscri=560.0):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetHistogramOfEnergyTransfer(runNo,Ei,dHW,hw_min,hw_max,startTime,endTime)
    - SetScaling(DAT,1.0)
    - NormByBeamCurrent(DAT,NormFactor)
    - DoMask(DAT)
    - SolidAngleCorrection(DAT)
    - kikfCorrection(DAT)
    @param runNo   (string) List of Run Numbers (ex. "111,113-115")
    @param Ei       (double) Incident Energy [meV]
    @param deltaE   (double) Delta Energy Transfer [meV]
    @param Emin     (double) minimum of Energy Transfer [meV]
    @param Emax     (double) maximum of Energy Transfer [meV]
    @param MaskFile (string) file name for masking
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param LLDiscri (double) Lower limit of discrimation
    @retval DAT     ElementCotainerMatix as result
    """
    
    #dat = BaseCom.GetHistogramOfEnergyTransfer(runNo,Ei,deltaE,Emin,Emax)
    dat = TkCom.GetHistogramOfEnergyTransfer(runNo,Ei,deltaE,Emin,Emax,-1,-1,LLDiscri,4096.0)
    
    if len(dat.PutHeaderPointer().PutString("RUNNUMBER"))==0 :
        raise UserWarning("Failure in making histogram.")
    
    TkCom.SetScaling(dat, 1.0)
    
    ecu = TkLocal.ElementContainerUtility()
    
    if NormFactor > 0:
        #bc = TkCom.GetBeamCurrent(dat, -1, -1, offset_sec)
        bc = BaseCom.GetBeamCurrentFromECM(dat,offset_sec)
        
        ecu.Multiply(dat, float(NormFactor) / float(bc))
        dat.PutHeaderPointer().Add("NORM_VALUE_TK", float(NormFactor) / float(bc))
        
    elif NormFactor < 0:
        ecu.Multiply(dat, 1.0/float(abs(NormFactor)) )
        dat.PutHeaderPointer().Add("NORM_VALUE_TK", 1.0/float(abs(NormFactor)))
    
    BaseCom.DoMask(dat,MaskFile)
    TkCom.SolidAngleCorrection(dat)
    TkCom.KiKfCorrection(dat)
    
    return dat

#########################################
def GetDataOfMonochroEi3(runNo="0",Ei=0.0,deltaE=0.0,Erange="0.0 0.0",TimeSlice="-1.0 -1.0",MaskFile="mask.txt",NormFactor=1000000.0,offset_sec=0.0,Discri="560.0 4096.0"):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetHistogramOfEnergyTransfer(runNo,Ei,dHW,hw_min,hw_max,startTime,endTime)
    - SetScaling(DAT,1.0)
    - NormByBeamCurrent(DAT,NormFactor)
    - DoMask(DAT)
    - SolidAngleCorrection(DAT)
    - kikfCorrection(DAT)
    @param runNo   (string) List of Run Numbers (ex. "111,113-115")
    @param Ei       (double) Incident Energy [meV]
    @param deltaE   (double) Delta Energy Transfer [meV]
    @param Erange   (double) min and max of Energy Transfer [meV]
    @param TimeSlice(double) first second large second
    @param MaskFile (string) file name for masking
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param Discri (double) Lower and higher limit of discrimation
    @retval DAT     ElementCotainerMatix as result
    """
    
    ErangeList = TkCom.SplitStringParameters( Erange, 2, "float" )
    if ErangeList==-1 or ErangeList==-2:
        raise UserWarning("E range is invalid.")
    
    TimeList = TkCom.SplitStringParameters( TimeSlice, 2, "float" )
    if TimeList==-1 or TimeList==-2:
        raise UserWarning("Time Slice range is invalid.")
    
    DiscriList = TkCom.SplitStringParameters( Discri, 2, "float" )
    if DiscriList==-1 or DiscriList==-2:
        raise UserWarning("Discriminate values are invalid.")
    
    Emin = ErangeList[0]
    Emax = ErangeList[1]
    st_time = TimeList[0]
    ed_time = TimeList[1]
    LLDiscri = DiscriList[0]
    HLDiscri = DiscriList[1]
    
    dat = TkCom.GetHistogramOfEnergyTransfer(runNo,Ei,deltaE,Emin,Emax,st_time,ed_time,LLDiscri,HLDiscri)
    
    if len(dat.PutHeaderPointer().PutString("RUNNUMBER"))==0 :
        raise UserWarning("Failure in making histogram.")
    
    sumBc = BaseCom.GetBeamCurrentFromECM(dat,offset_sec)
    #sumBc = TkCom.GetBeamCurrent(dat, st_time, ed_time, offset_sec)
    
    TkCom.SetScaling(dat, 1.0)
    
    ecu = TkLocal.ElementContainerUtility()
    hh = dat.PutHeaderPointer()
    
    if NormFactor>0 and sumBc > 0:
        ecu.Multiply(dat, float(NormFactor)/float(sumBc))
        hh.Add("NORM_VALUE_TK", float(NormFactor) / float(sumBc))
    if NormFactor<0:
        ecu.Multiply(dat, 1.0/float(abs(NormFactor)) )
        hh.Add("NORM_VALUE_TK", 1.0/float(abs(NormFactor)))
    
    BaseCom.DoMask(dat,MaskFile)
    TkCom.SolidAngleCorrection(dat)
    TkCom.KiKfCorrection(dat)

    return dat

##########################################
def ToPowderAutoBin(dat=DAT,deltaQ=0.1):
    """
    Convert data from crystal to powder

    @param dat [Def:DAT]   (ElementContaienrMatrix)
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    
    return TkCom.ToPowderAutoBin(dat,deltaQ)

##########################################
def ToPowder(dat=DAT,startQ=-1.0,endQ=-1.0,deltaQ=0.1):
    """
    Convert data from crystal to powder

    @param dat [Def:DAT]   (ElementContaienrMatrix)
    @param startQ (double) Q value of range
    @param endQ   (double) Q value of range
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    
    return TkCom.ToPowder(dat,startQ,endQ,deltaQ)

##########################################
def ToPowderRect(dat=DAT,startQ=-1.0,endQ=-1.0,deltaQ=0.1):
    """
    Convert data from crystal to powder (rect matrix)

    @param dat [Def:DAT]   (ElementContaienrMatrix)
    @param startQ (double) Q value of range
    @param endQ   (double) Q value of range
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    
    return TkCom.ToPowderRect(dat,startQ,endQ,deltaQ)

#########################################
def DoDetailedMask(dat=DAT,path="./",filename="detaildMask.txt"):
    """
    MASKED flag controled by file.
    @param dat [Def:DAT] (ElementContainerMatrix)
    @param filename (string) filename of mask information 
    """
    TkCom.DoDetailedMask(dat, path, filename)

#########################################
def SumOfPixelAMR(Target=DAT, PSD_min=6, PSD_max=271, Pixel_min=61, Pixel_max=80):
    """
    Put summation of intensity along Pixel
    @param Target [Def:DAT]   (ElementContainerMatrix)
    @param PSD_min   (int) lower limit of PSD-ID range
    @param PSD_max   (int) upper limit of PSD-ID range
    @param Pixel_min (int) lower limit of Pixel range
    @param Pixel_max (int) upper limit of Pixel range
    @retval EC
    """
    return TkCom.SumOfPixel(Target, PSD_min, PSD_max, Pixel_min, Pixel_max)

#########################################
def GetProfilesSum(dat=DAT, firstPsdId=-1, lasPsdId=-1, exPsdIds="-1", bins=1):
    """
    Get Summation of Pulse Height Profiles of PSD
    Each PSD pulse heignt profile is stored in header of ElementContaienrArray.
    @param  dat [Def:DAT]  (ElementContainerMatrix) Loaded ElementContainerMatrix.
    @param  startDetID (int) Start Detector ID of summation
    @param  endDetID (int) End Detector ID of summation
    @param  exDetIds (String) excluding Detector IDs (-1 : put all)
    @param  bins (double) binning of pulse height
    @retval EC    ElementContainer or list of ElementContainers of pulse height profile
    """
    return TkCom.GetProfilesSum(dat, firstPsdId, lasPsdId, exPsdIds, bins)

########################################
def GetRefHistogram(runNo=-1,startTOF=0.0,endTOF=40000.0,binTOF=1000.0,NormFactor=0.0,LLDiscri=560.0):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetNeunetHist(runNo,"tof,startTOF,endTOF,binTOF","All","All","None","psd,discri,4096","None")
    - SetScaling(dat, 1.0)
    - SolidAngleCorrection()
    @param runNo      (int)    Run Number
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @param LLDiscri (double) Lower limit of discrimation
    @retval DAT     (ElementContainerMatrix)
    """
    
    dat = TkCom.GetHistogram(runNo,startTOF,endTOF,binTOF,-1.0,-1.0,LLDiscri,4096.0)

    if runNo!=0:
        TkCom.SetScaling(dat, 1.0)
        
        ecu = TkLocal.ElementContainerUtility()
        
        if NormFactor > 0:
            bc = TkCom.GetBeamCurrent(dat)
            ecu.Multiply(dat, float(NormFactor) / float(bc))
            dat.PutHeaderPointer().Add("NORM_VALUE_TK", float(NormFactor) / float(bc))
            
        elif NormFactor < 0:
            ecu.Multiply(dat, 1.0/float(abs(NormFactor)) )
            dat.PutHeaderPointer().Add("NORM_VALUE_TK", 1.0/float(abs(NormFactor)))
        
        TkCom.SolidAngleCorrection(dat)
        
    return dat

#########################################
def PsdIntCorrection(dat=DAT, datRef=DAT_REF, tofLeft=0.0, tofRight=40000.0):
    """
    Correction of Intensity on each PSD for using R/C etc.
    - PsdIntCorrection(dat,datRef,,tofLeft, tofRight)
    
    @param dat [Def:DAT]  (ElementContainerMatrix)
    @param datRef [Def:DAT_REF]   (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of reference data [us]
    @param tofRight (double) right limit of tof range of reference data [us]
    """
    
    #if type(dat)!=type(mm.ElementContainerMatrix()):
    if not isinstance(dat, mm.ElementContainerMatrix):
        raise UserWarning("Contianer Error")
    
    TkCom.PsdIntCorrection(dat, datRef, tofLeft, tofRight)

#########################################
def SubtractTIDBkg(dat=DAT, datBkg=DAT_BKG, tofLeft=2000.0, tofRight=40000.0):
    """
    Make time-independent background data
    - Copy container()
    - SetTimeIndependentBkg(dat)
    
    @param dat [Def:DAT]   (ElementContainerMatrix)
    @param datBkg [Def:DAT_BKG]  (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of time-independet background [us]
    @param tofRight (double) right limit of tof range of time-independet background [us]
    @retval DAT     ElementCotainerMatix as result
    """
    
    #if type(dat)==type(mm.ElementContainerMatrix()):
    if isinstance(dat, mm.ElementContainerMatrix):
        ret = mm.ElementContainerMatrix(dat)
    else:
        raise UserWarning("Contianer Error")
    
    hh = datBkg.PutHeaderPointer()
    if hh.CheckKey("KICKERCOUNT") == 1:
        kc_bkg = hh.PutInt4("KICKERCOUNT")
    else:
        raise UserWarning("No KICKERCOUNT")
    
    if hh.CheckKey("NORM_VALUE_TK") != 0:
        normValue_bkg = hh.PutDouble("NORM_VALUE_TK")
    else :
        normValue_bkg = 1
    
    hh = dat.PutHeaderPointer()
    if hh.CheckKey("KICKERCOUNT") == 1:
        kc = hh.PutInt4("KICKERCOUNT")
        
        print("kicker count: %f(data), %f(background)" % (kc, kc_bkg))
    else:
        raise UserWarning("No KICKERCOUNT")
    
    if hh.CheckKey("NORM_VALUE_TK") != 0:
        normValue = hh.PutDouble("NORM_VALUE_TK")
    else :
        normValue = 1
    
    factor = float(kc) / float(kc_bkg) * normValue / normValue_bkg
    
    TkCom.SubtractTimeIndependentBkg(ret, datBkg, tofLeft, tofRight, factor)
    
    return ret

#########################################
def MakeDataOfTIDBkg(datRef=DAT, datBkg=DAT_BKG, tofLeft=0.0, tofRight=0.0):
    """
    Make time-independent background data
    - Copy container)
    - SetTimeIndependentBkg(dat)
    
    @param datRef [Def:DAT]   (ElementContainerMatrix)
    @param datBkg [Def:DAT_BKG]  (ElementContainerMatrix)
    @param tofLeft  (double) left limit of tof range of time-independet background [us]
    @param tofRight (double) right limit of tof range of time-independet background [us]
    @retval DAT     ElementCotainerMatix as result
    """
    
    #if type(datRef)==type(mm.ElementContainerMatrix()):
    if isinstance(datRef, mm.ElementContainerMatrix):
        ret = mm.ElementContainerMatrix(datRef)
    else:
        raise UserWarning("Contianer Error")
    
    hh = datRef.PutHeaderPointer()
    if hh.CheckKey("KICKERCOUNT") == 1:
        kc = hh.PutInt4("KICKERCOUNT")
    else:
        raise UserWarning("No KICKERCOUNT")
    
    hh = datBkg.PutHeaderPointer()
    if hh.CheckKey("KICKERCOUNT") == 1:
        kc_bkg = hh.PutInt4("KICKERCOUNT")
        
        print("kicker count: %f(data), %f(background)" % (kc, kc_bkg))
    else:
        raise UserWarning("No KICKERCOUNT")
    
    hh = datRef.PutHeaderPointer()
    if hh.CheckKey("NORM_VALUE_TK")!=0:
        normValue = hh.PutDouble("NORM_VALUE_TK")
        factor = float(kc) / float(kc_bkg) * normValue
    else :
        raise UserWarning("No NORM_VALUE_TK")
    
    TkCom.SetTimeIndependentBkg(ret, datBkg, tofLeft, tofRight)
    
    ecu = TkLocal.ElementContainerUtility()
    ecu.Multiply(ret, factor)
    
    return ret

#########################################
def FlexCalcTk(dat=DAT,Expression="1"):
    """
    Flexible calculation of S(Q,hw) on ElementContainerMatrix.
    @param  dat [Def:DAT] (ElementContainerMatrix)
    @param  Expression (string) the expression of multiplying factor
    @retval DAT (ElementContainerMatrix)
    """

    return TkCom.FlexCalcTk(dat,Expression)

#########################################
def DetailedBalance(dat=DAT,temperature=300.0):
    """
    Correction of detailed balace of S(Q,hw) on ElementContainerMatrix.
    @param  dat [Def:DAT]   (ElementContainerMatrix)
    @param  temperature (double) temperature of sample
    @retval DAT         (ElementContainerMatrix)
    """

    Expression = "exp(-11.6045*hw/2/" + str(temperature) + ")"
    
    return TkCom.FlexCalcTk(dat,Expression)

##########################################
def CalcContainers(dat1=DAT,dat2=DAT,coef1=1.0,coef2=0.0):
    """
    Execute caluculation ret = coef1*dat1 + coef2*dat2
    @param  dat1 [Def:DAT]  ElementContainerMatrix or Array  first data
    @param  dat2 [Def:DAT]  ElementContainerMatrix or Array  second data
    @param  coef1  Double    coefficient of dat1
    @param  coef2  Double    coefficient of dat2
    @param  ope    string    operator
    @retval DAT    Return Value
    """ 

    return TkCom.CalcContainers(dat1,dat2,coef1,coef2)

#########################################
def AppendContainer(dat1=DAT,dat2=DAT):
    """
    Function to append another container
    
    @param dat1 [Def:DAT]    (ElementContainerMatrix)
    @param dat2 [Def:DAT]    (ElementContainerMatrix)
    """
    ecu = TkLocal.ElementContainerUtility()
    ecu.AverageContainers(dat1,dat2)

#########################################
def ZeroToMinimumValue(dat=DAT):
    """
    Function to set minimum values to data of zero count
    
    @param dat  [Def:DAT]    (ElementContainerMatrix)
    """
    TkCom.ZeroToMinimumValue(dat)

#########################################
def SaveQSliceFromPowder(ecm=ECM,path="./",filename="Sample.txt",startQ=1.0,endQ=10.0,frcQ=1,bin=0.0,fOutScaling=0,startNo=1):
    """
    Save files of Q-sliced data from powder data
    
    @param  ecm [Def:ECM]   (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ   (double) minimum limit of Q-range
    @param  endQ     (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  bin      (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    @param  startNo     (int) first number of saving file
    """
    TkCom.SaveQSliceFromPowder(ecm,path,filename,startQ,endQ,frcQ,bin,fOutScaling,startNo)

#########################################
def SaveQSliceFromHist(dat=DAT,path="./",filename="Sample.txt",startQ=1.0,endQ=10.0,frcQ=1,bin=0.0,fOutScaling=0):
    """
    Save files of Q-sliced data from histogram data
    
    @param  dat [Def:DAT]   (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ   (double) minimum limit of Q-range
    @param  endQ     (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  bin      (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    """
    TkCom.SaveQSliceFromHist(dat,path,filename,startQ,endQ,frcQ,bin,fOutScaling)

#########################################
def SaveESliceFromPowder(ecm=ECM,path="./",filename="Sample.txt",startE=1.0,endE=10.0,frcE=1,bin=0.0,fOutScaling=0):
    """
    Save files of E-sliced data from powder data
    
    @param  ecm [Def:ECM]   (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startE   (double) minimum limit of E-range
    @param  endE     (double) maximum limit of E-range
    @param  frcE     (int) number of division
    @param  bin      (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    """
    TkCom.SaveESliceFromPowder(ecm,path,filename,startE,endE,frcE,bin,fOutScaling)

#########################################
def SavePowderToText(ecm=ECM,path="./",filename="Sample.txt",fOutScaling=0):
    """
    Save files of Q-sliced data from histogram data
    
    @param  ecm [Def:ECM]   (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  fOutScaling  (int) flag of whether to output scaling
    """
    TkCom.SavePowderToText(ecm,path,filename,fOutScaling)

#########################################
def SaveSumDetectorMapToText(dat=DAT,startX=0.0,endX=0.0,path="./",filename="Sample.txt"):
    """
    Save files of Q-sliced data from histogram data
    
    @param  dat [Def:DAT]   (ElementContainerMatrix)
    @param  startX (double) top x value to be summed
    @param  endX   (double) end x value to be summed
    @param  path     (String) path to data file
    @param  filename (String) data file name
    """
    TkCom.SaveSumDetectorMapToText(dat,startX,endX,path,filename)

#########################################
def InitAbsorptionCorrection(dummy="No need to change"):
    """
    Initialize an object for absorption correction
    
    @param  dummy (String) Dummy parameter to avoid error
    @retval AC   (AbsorptionCorrection) an object for absorption correction
    """
    return TkLocal.AbsorptionCorrection()

#########################################
def SetSampleInfoCylinder(ac=AC,rLarge=7.0,rSmall=5.0,height=0.0,density=1.0):
    """
    Set form information of the sample
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  rLarge  (double) r-max of cylinder [mm]
    @param  rSmall  (double) r-min of cylinder [mm]
    @param  height  (double) height of cylinder [mm] (dummy)
    @param  density (double) density [kg/meter^3]
    """
    ac.SetSampleRLarge(rLarge)
    ac.SetSampleRSmalle(rSmall)
    #ac.SetSampleRHeight(height)
    ac.SetSampleDensity(density)

#########################################
def AddSampleElementWeight(ac=AC,atom1="-",wtPercent1=0.0,atom2="-",wtPercent2=0.0,atom3="-",wtPercent3=0.0):
    """
    Set elements information of the sample
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  atom1   (String) atom name
    @param  wtPercent1  (double) weight percent of atom1
    @param  atom2   (String) atom name
    @param  wtPercent2  (double) weight percent of atom2
    @param  atom3   (String) atom name
    @param  wtPercent3  (double) weight percent of atom3
    """
    if len(atom1) > 0 and atom1 != "-" :
        ac.AddSampleElementInfoWeight(atom1, wtPercent1)
    
    if len(atom2) > 0 and atom2 != "-" :
        ac.AddSampleElementInfoWeight(atom2, wtPercent2)
    
    if len(atom3) > 0 and atom3 != "-" :
        ac.AddSampleElementInfoWeight(atom3, wtPercent3)

#########################################
def AddSampleElementMol(ac=AC,atom1="-",molPercent1=0.0,atom2="-",molPercent2=0.0,atom3="-",molPercent3=0.0):
    """
    Set elements information of the sample
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  atom1   (String) atom name
    @param  molPercent1  (double) mol% of atom1
    @param  atom2   (String) atom name
    @param  molPercent2  (double) mol% of atom2
    @param  atom3   (String) atom name
    @param  molPercent3  (double) mol% of atom3
    """
    if len(atom1) > 0 and atom1 != "-":
        ac.AddSampleElementInfoMol(atom1, molPercent1)
    
    if len(atom2) > 0 and atom2 != "-" :
        ac.AddSampleElementInfoMol(atom2, molPercent2)
    
    if len(atom3) > 0 and atom3 != "-" :
        ac.AddSampleElementInfoMol(atom3, molPercent3)

#########################################
def SetCellInfoCylinder(ac=AC,rLarge=7.0,rSmall=5.0,height=0.0,density=1.0):
    """
    Set form information of the cell
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  rLarge  (double) r-max of cylinder [mm]
    @param  rSmall  (double) r-min of cylinder [mm]
    @param  height  (double) height of cylinder [mm] (dummy)
    @param  density (double) density [kg/meter^3]
    """
    ac.SetCellRLarge(rLarge)
    ac.SetCellRSmalle(rSmall)
    #ac.SetCellRHeight(height)
    ac.SetCellDensity(density)

#########################################
def AddCellElementWeight(ac=AC,atom1="-",wtPercent1=0.0,atom2="-",wtPercent2=0.0,atom3="-",wtPercent3=0.0):
    """
    Set elements information of the cell
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  atom1   (String) atom name
    @param  wtPercent1  (double) weight percent of atom1
    @param  atom2   (String) atom name
    @param  wtPercent2  (double) weight percent of atom2
    @param  atom3   (String) atom name
    @param  wtPercent3  (double) weight percent of atom3
    """
    if len(atom1) > 0 and atom1 != "-" :
        ac.AddCellElementInfoWeight(atom1, wtPercent1)
    
    if len(atom2) > 0 and atom2 != "-" :
        ac.AddCellElementInfoWeight(atom2, wtPercent2)
    
    if len(atom3) > 0 and atom3 != "-" :
        ac.AddCellElementInfoWeight(atom3, wtPercent3)

#########################################
def AddCellElementMol(ac=AC,atom1="-",molPercent1=0.0,atom2="-",molPercent2=0.0,atom3="-",molPercent3=0.0):
    """
    Set elements information of the cell
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  atom1   (String) atom name
    @param  molPercent1  (double) mol% of atom1
    @param  atom2   (String) atom name
    @param  molPercent2  (double) mol% of atom2
    @param  atom3   (String) atom name
    @param  molPercent3  (double) mol% of atom3
    """
    if len(atom1) > 0 and atom1 != "-" :
        ac.AddCellElementInfoMol(atom1, molPercent1)
    
    if len(atom2) > 0 and atom2 != "-" :
        ac.AddCellElementInfoMol(atom2, molPercent2)
    
    if len(atom3) > 0 and atom3 != "-" :
        ac.AddCellElementInfoMol(atom3, molPercent3)

#########################################
def AbsorptionCorrection(ac=AC,dat=DAT):
    """
    Correct intensity for absorption effect
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  dat     (ElementContainerMatrix) SQW of sample
    """
    ac.Correct(dat)

#########################################
def AbsorptionCorrectionSubCell(ac=AC,dat=DAT,datCell=DAT):
    """
    Correct intensity for absorption effect
    
    @param  ac [Def:AC]  (AbsorptionCorrection)
    @param  dat [Def:DAT]  (ElementContainerMatrix) SQW of sample & cell
    @param  datCell [Def:DAT] (ElementContainerMatrix) SQW of only cell
    """
    ac.Correct(dat, datCell)

#########################################
def EnergyToPath(dat=DAT,L2=-1.0):
    """
    Correct intensity for absorption effect
    
    @param  dat [Def:DAT] (ElementContainerMatrix)
    @param  L2      (double) L2
    """
    #TkCom.EnergyToDistance(dat,L2)
    
    if L2 <= 0 :
        #L2 = Def_L2
        L2 = dat.PutHeader().PutDouble(mu.UTSUSEMI_KEY_HEAD_TYPICALL2)
    
    ei = dat.PutHeaderPointer().PutDouble("Ei");
    
    for psd in range(dat.PutTableSize()):
        eca = dat(psd)
        
        for pixel in range(eca.PutTableSize()):
            ec = eca(pixel)
            
            xkey = ec.PutXKey()
            ykey = ec.PutYKey()
            ekey = ec.PutEKey()
            
            xx = numpy.array(ec.PutXList())
            
            res_xx = (numpy.sqrt(1.0 / (1.0 - xx / ei)) - 1.0) * L2 * 1000.0
            
            ec.Add("Path",res_xx.tolist(),"mm")
            #ec.Replace(xkey,res_xx.tolist())
            ec.SetKeys("Path",ykey,ekey)
            #ec.SetKeys(xkey,ykey,ekey)
    
#########################################
def OutputQSliceToGrpText(dat=ECM,path="./",filename="Sample.grp",startQ=1.0,endQ=1.0,frcQ=1,binE=0.0):
    """
    Save grouped text (DAVE format) files of Q-sliced powder data
    @param  dat [Def:ECM]    (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ   (double) minimum limit of Q-range
    @param  endQ     (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  binE     (double) rebinning width of energy
    @retval None
    """
    if not isinstance(dat, mm.ElementContainerMatrix):
        raise ContianerError()
    
    TkCom.OutputQSliceToGrpText(dat, path, filename, startQ, endQ, frcQ, binE)


#########################################
def OutputTimeSlicedElastic(runNo="0",Ei=0.0,ElaWidth=0.1,deltaQ=1.0,StTime=0.0,binTime=300.0,sliceNum=1,MaskFile="mask.txt",path="./",filename="Sample.txt",NormFactor=1000000.0,offset_sec=0.0,Discri="560.0 4096.0",fOutScaling=0):
#def OutputTimeSlicedElastic(runNo="11154",Ei=5.664,ElaWidth=0.1,deltaQ=0.2,StTime=0.0,binTime=300.0,sliceNum=2,MaskFile="mask_kikuchi_r6.txt",path="./",filename="Sample.txt",NormFactor=1000000.0,offset_sec=0.0,Discri="560.0 4096.0",fOutScaling=0):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetHistogramOfEnergyTransfer(runNo,Ei,dHW,hw_min,hw_max,startTime,endTime)
    - SetScaling(DAT,1.0)
    - NormByBeamCurrent(DAT,NormFactor)
    - DoMask(DAT)
    - SolidAngleCorrection(DAT)
    - kikfCorrection(DAT)
    @param runNo   (string) List of Run Numbers (ex. "111,113-115")
    @param Ei       (double) Incident Energy [meV]
    @param ElaWidth   (double) full width of Energy Transfer for elastic[meV]
    @param deltaQ (double) binning of Q
    @param StTime (double) first second of analysis [sec]
    @param binTime (double) binning of time slicing [sec]
    @param sliceNum (double) the number of time slicing
    @param MaskFile (string) file name for masking
    @param path     (String) path to data file
    @param filename (String) data file name
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param Discri (double) Lower and higher limit of discrimation
    @param fOutScaling  (int) flag of whether to output scaling
    @retval ECM
    """
    
    DiscriList = TkCom.SplitStringParameters( Discri, 2, "float" )
    if DiscriList==-1 or DiscriList==-2:
        raise UserWarning("Discriminate values are invalid.")
    
    Emin = -ElaWidth / 2.0
    Emax = ElaWidth / 2.0
    LLDiscri = DiscriList[0]
    HLDiscri = DiscriList[1]
    
    deltaE = Emax - Emin
    
    slices = []
    flag = False
    
    for ii in range(sliceNum):
        st_time = StTime + ii * binTime
        ed_time = st_time + binTime
        
        dat = TkCom.GetHistogramOfEnergyTransfer(runNo,Ei,deltaE,Emin,Emax,st_time,ed_time,LLDiscri,HLDiscri)
        
        if len(dat.PutHeaderPointer().PutString("RUNNUMBER"))==0 :
            raise UserWarning("Failure in making histogram.")
        
        sumBc = BaseCom.GetBeamCurrentFromECM(dat,offset_sec)
        #sumBc = TkCom.GetBeamCurrent(dat, st_time, ed_time, offset_sec)
        
        if sumBc <= 0:
            slices.append(None)
            continue
        
        TkCom.SetScaling(dat, 1.0)
        
        ecu = TkLocal.ElementContainerUtility()
        hh = dat.PutHeaderPointer()
        
        if NormFactor>0:
            ecu.Multiply(dat, float(NormFactor)/float(sumBc))
            hh.Add("NORM_VALUE_TK", float(NormFactor) / float(sumBc))
        if NormFactor<0:
            ecu.Multiply(dat, 1.0/float(abs(NormFactor)) )
            hh.Add("NORM_VALUE_TK", 1.0/float(abs(NormFactor)))
        
        BaseCom.DoMask(dat,MaskFile)
        TkCom.SolidAngleCorrection(dat)
        TkCom.KiKfCorrection(dat)
        
        ecm = ToPowder(dat=dat,startQ=-1.0,endQ=-1.0,deltaQ=deltaQ)
        
        datalst = TkCom.SaveESliceFromPowder(ecm,path=path,filename=filename,startE=Emin,endE=Emax,frcE=1,bin=0.0,fOutScaling=fOutScaling,stNo=(ii+1))
        
        # data[0:x, 1:y, 2:error, 3:scaling, 4:title1, 5:title2, 6:header, 
        slices.append(datalst[0])
        
        if not flag :
            xlist = datalst[0][0]
            flag = True
            
            ecm_res = mm.ElementContainerMatrix(dat.PutHeader());
    
    if not flag:
        return None
    
    ylist = [(StTime + binTime * ii) / 3600.0 for ii in range(sliceNum + 1)]
    
    eca_res = mm.ElementContainerArray()
    
    for ii in range(len(xlist)):
        
        ec_res = mm.ElementContainer()
        
        xrList = [float(xlist[ii] - deltaQ / 2.0), float(xlist[ii] + deltaQ / 2.0)]
        
        ec_res.PutHeaderPointer().AddDoubleList("XRANGE", xrList)
        
        zlist = []
        elist = []
        slist = []
        
        for data in slices:
            if data == None:
                zlist.append(0.0)
                elist.append(-1.0)
                slist.append(0.0)
                continue
            
            zlist.append(float(data[1][ii]))
            elist.append(float(data[2][ii]))
            if data[3] == None:
                slist.append(1.0)
            else:
                slist.append(float(data[3][ii]))
        
        ec_res.Add("Hours", ylist);
        ec_res.Add("Intensity", zlist);
        ec_res.Add("Error", elist);
        ec_res.SetKeys("Hours", "Intensity", "Error");
        
        ec_res.Add("Scaling", slist);
        
        #eca_res.AddPointer(ec_res);
        eca_res.Add(ec_res);
    
    eca_res.PutHeaderPointer().Add("SAMPLETYPE", "Rect")
    
    #ecm_res.AddPointer(eca_res)
    ecm_res.Add(eca_res)
    
    hh = ecm_res.PutHeaderPointer()
    if hh.CheckKey("SAMPLETYPE") :
        hh.OverWrite("SAMPLETYPE", "Rect");
    else:
        hh.Add("SAMPLETYPE", "Rect");
    
    return ecm_res


#########################################
def OutputForMDA(dat=ECM,path="./",filename="Sample.txt",startQ=0.1,endQ=1.0,frcQ=1,binE=0.0,fOutScaling=1):
    """
    Save grouped text (DAVE format) files of Q-sliced powder data
    @param  dat [Def:ECM]    (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  startQ   (double) minimum limit of Q-range
    @param  endQ     (double) maximum limit of Q-range
    @param  frcQ     (int) number of division
    @param  binE     (double) rebinning width of energy
    @param  fOutScaling  (int) flag of whether to output scaling
    @retval None
    """
    if not isinstance(dat, mm.ElementContainerMatrix):
        raise ContianerError()
    
    TkCom.OutputForMDA(dat, path, filename, startQ, endQ, frcQ, binE,fOutScaling)


#########################################
def test(dat=DAT):
    
    ecu = TkLocal.ElementContainerUtility()
    
    eca = dat.Put(0)
    ec = eca.Put(0)
    
    print(ecu.ScalingExists(ec))
    print(ecu.PutScaling(ec))

#########################################
# Dictionary for entry the name of functions
_functions = {
              "GetDataOfMonochroEi2":GetDataOfMonochroEi2,
              "GetDataOfMonochroEi3":GetDataOfMonochroEi3,
              "ToPowderAutoBin":ToPowderAutoBin,
              "ToPowder":ToPowder,
              "ToPowderRect":ToPowderRect,
              "DoDetailedMask":DoDetailedMask,
              "SumOfPixelAMR":SumOfPixelAMR,
              "GetProfilesSum":GetProfilesSum,
              "GetRefHistogram":GetRefHistogram,
              "PsdIntCorrection":PsdIntCorrection,
              "SubtractTIDBkg":SubtractTIDBkg,
              "MakeDataOfTIDBkg":MakeDataOfTIDBkg,
              "SaveQSliceFromPowder":SaveQSliceFromPowder,
              "SaveQSliceFromHist":SaveQSliceFromHist,
              "SaveESliceFromPowder":SaveESliceFromPowder,
              "FlexCalcTk":FlexCalcTk,
              "DetailedBalance":DetailedBalance,
              "CalcContainers":CalcContainers,
              "AppendContainer":AppendContainer,
              "SaveSumDetectorMapToText":SaveSumDetectorMapToText,
              "InitAbsorptionCorrection":InitAbsorptionCorrection,
              "SetSampleInfoCylinder":SetSampleInfoCylinder,
              "AddSampleElementWeight":AddSampleElementWeight,
              "AddSampleElementMol":AddSampleElementMol,
              "SetCellInfoCylinder":SetCellInfoCylinder,
              "AddCellElementWeight":AddCellElementWeight,
              "AddCellElementMol":AddCellElementMol,
              "AbsorptionCorrection":AbsorptionCorrection,
              "AbsorptionCorrectionSubCell":AbsorptionCorrectionSubCell,
              "OutputQSliceToGrpText":OutputQSliceToGrpText,
              "OutputTimeSlicedElastic":OutputTimeSlicedElastic,
              "OutputForMDA":OutputForMDA
          }
         
_functionsOrder = [
    "GetDataOfMonochroEi2",
    "GetDataOfMonochroEi3",
    "ToPowderAutoBin",
    "ToPowder",
    "ToPowderRect",
    "SumOfPixelAMR",
    "OutputForMDA",
    "GetRefHistogram",
    "PsdIntCorrection",
    "SubtractTIDBkg",
    "MakeDataOfTIDBkg",
    "DetailedBalance",
    "OutputQSliceToGrpText",
    "InitAbsorptionCorrection",
    "SetSampleInfoCylinder",
    "AddSampleElementWeight",
    "AddSampleElementMol",
    "SetCellInfoCylinder",
    "AddCellElementWeight",
    "AddCellElementMol",
    "AbsorptionCorrection",
    "AbsorptionCorrectionSubCell",
    "DoDetailedMask",
    "CalcContainers",
    "FlexCalcTk",
    "AppendContainer",
    "GetProfilesSum",
    "SaveQSliceFromPowder",
    "SaveQSliceFromHist",
    "SaveESliceFromPowder",
    "SaveSumDetectorMapToText",
    "OutputTimeSlicedElastic"
    ]
