#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
D2Vis.py
2次元プロッタ処理
"""
from __future__ import print_function

import wx
import os
import time
import tempfile
import matplotlib
# 先に指定されていなかったら、バックエンドを指定
if not matplotlib.get_backend() == 'WXAgg':
    matplotlib.use('WXAgg')
## [inamura 100726] for checking version of  matplotlib
vers = matplotlib.__version__.split('.')
matplotlib_verNo = int(vers[0])*1000 + int(vers[1])*10 + int(vers[0])
## <--[inamura 100726]
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigCanvas
from pylab import *
from matplotlib.colors import LogNorm
from matplotlib.figure import Figure
#import matplotlib.numerix as numpy
from numpy import *
import sys
import zlib

#import matplotlib.numerix.ma as ma
try:
    import numpy.ma as ma
except:
    import numpy.core.ma as ma
from matplotlib import cm
from vis.UtilPlot import *
from vis.FastPlot import *

##[inamura 101022]-->
print("This is smooth version") 
try:
    from scipy import signal
except:
    pass
##<--[inamura 101022]

##[T.Kikuchi 130507]-->
try:
    import Manyo.TKtools as TkLocal
except:
    pass
##<--[T.Kikuchi 130507]
"""
brief update history
2012.11.04 Update SaveAsText
2012.10.29 Debug around font control in SetXyRange2D and add listners for dialog
2012.07.11 Enable to change font size and font name on labels or titles
2012.07.10 Add Clickable area to show dialogs
2012.04.17 Add Quiet mode and CUI for Save figure
2012.02.11 Add Single plotter mode and Quiet mode
2011.01.25 Revise befavior to plot with smoothing (can replot with Apply button)
2011.01.25 Add new color map as default
2011.01.06 Add cursol on/off button, revise behavior of checkbox for smoothing
2010.12.14 Add function to output TEXT data
2010.10.22 Add smoothing function
2010.07.26 matplotlib 1.0.0 version
2010.06.22 Revise style to show Z-value (log scale
2009.09.24 Add color map dialog and x-y scale dialog

"""
#######################################
#  HistObj
#######################################  
class HistObj(object):
    
    """
    ヒストグラムデータクラス
    """
    #########################################
    def __init__(self, bins, counts, errs, scalings, fp=0.0, mp=0.0, ep=0.0):
        """
        コンストラクタ
        @param  bins  ヒストグラムのビン(array型) 
        @param  counts  強度値(array型) 
        @param  errs  エラー値(array型)
        @param  scaling  スケーリング値(array型)
        @param  fp    始値
        @param  mp    中央値
        @param  ep    終値
        @retval 無し
        """ 
        # ヒストグラムビン
        self.bins = bins
        # Y データ
        self.counts = counts
        # エラー値
        self.errs = errs
        # スケーリング値
        self.scalings = scalings
        # 始値
        self.fp = fp
        # 中央値
        self.mp = mp
        # 終値
        self.ep = ep 
    
        # スケーリング値を用いたマスク
        self.counts[self.scalings <= 0] = MASKVALUE
    
    ## AddData() was removed by T.Kikuchi in 130508.
    ## DevidData() was removed by T.Kikuchi in 130508.
    ## DevidInteg() was removed by T.Kikuchi in 130508.
    ## ClearInteg() was removed by T.Kikuchi in 130508.
    ## ReBin() was removed by T.Kikuchi in 130508.
    

#######################################
#  MapObje
#######################################  
class MapObj(object):
    
    """
    データハンドリングクラス
    """

    #########################################
    def __init__(self, data, order):
        """
        データを読み込み、カラーコンツァーマップを作成
        @param  data  データオブジェクト
        @param  order  イベントディスパッチャのID
        @retval 無し
        """ 
        # 初期化
        self.X = self.Y = None
        self.qArray = None
        self.arData = None
        self.erData = None
        ##[T.Kikuchi 130514]-->
        self.scData = None
        ##<--[T.Kikuchi 130514]
        self.zRange = (0.0, 1.0, 0.005)
        self.type = 0   #  0: 直交 1: 斜交
        
        # プロパティ管理クラスのインスタンスを取得
        self.ifi = IFEvtProp(order)
        
        # 公開プロパティを登録
        self.ifi.AddProperty('grid',       lambda : (self.X, self.Y))
        self.ifi.AddProperty('mapdata',    lambda : self.arData)
        self.ifi.AddProperty('erdata',    lambda : self.erData)
        ##[T.Kikuchi 130514]-->
        self.ifi.AddProperty('scdata',    lambda : self.scData)
        ##<--[T.Kikuchi 130514]
        self.ifi.AddProperty('zrange',     lambda : self.zRange)
        self.ifi.AddProperty('qlist',     lambda : self.qList)
        # データが指定されていれば
        if data != None:
            # データを読み込み
            self.ChangeData(data)


    ###################################################             
    def ChangeData(self, data):
        """
        データ変更イベント処理
        @param  data データオブジェクト
        @retval 無し
        """ 
        # データを格納
        self.arData = data[0]
        self.erData = data[1]
        self.X = data[2]
        self.Y = data[3]
        try:
            self.type = data[4]
        except:
            # タイプの指定が無ければ直交データ
            self.type = 0
        ##[T.Kikuchi 130507]-->
        try:
            self.scData = data[5]
        except:
            # スケーリングの指定無ければ作成
            self.scData = where(self.arData < MASKVALUE-1.0, 1.0, 0.0)
        
        self.m2s = TkLocal.M2Slicer()
        self.m2s.SetData(self.arData, self.erData, self.scData, self.X, self.Y, self.type)
        ##<--[T.Kikuchi 130507]
        
        # マスク値を除く最大値を検索
        self.maxCount = 0.0
        for i in range(len(self.arData)):
            for j in range(len(self.arData[0])):
                if self.arData[i][j] < MASKVALUE-1.0:
                    if self.arData[i][j] > self.maxCount :
                        self.maxCount = self.arData[i][j]
        self.numBins = len(self.X)       
        
        # データの属性を取得
        self.GetZRange()
        # Q代表値値のリストを作成
        qnum = len(self.X[0])-1
        self.qArray = zeros(qnum, float64)
        qg = (self.X.max()-self.X.min()) / qnum
        qvalue = self.X.min() + qg/2.0
        
        for i in range(qnum):
            self.qArray[i] = qvalue
            qvalue += qg

    #########################################    
    def GetZRange(self):
        """
        データの最小値と最大値、オフセット値を返す
        (ログモードでは、0ゃ負値を扱えないため)
        @param  無し
        @retval 最小値、最大値、オフセット値のタプル
        """ 
        # オフセットは、最大値の1/1000
        offset = self.maxCount / 1000.0
        # オフセットは、0.05 より大きくしない
        if offset > 0.05:
            offset = 0.05

        offset = 1.0E-25 #[inamura 100618]
        self.zRange =  (self.arData.min(), self.maxCount,offset)
        

    #########################################
    def GetHistX(self, eslice):
        """
        指定されたY値におけるヒストグラムを取得
        @param  eslice ( Y中央値、Y始値、Y終値) のタプル 
        @retval ヒストグラムオブジェクト
        """
        # 始値 <= 中央値 <= 終値 であること
        if eslice[1] > eslice[0] or eslice[0] > eslice[2]:
            raise PlotException('Common', 'C001', ("GetHistX", "Index"))
            return
        # ebin の数
        nY = len(self.Y)-1
        # 範囲チェック
        if eslice[1] > self.Y[nY,0] or eslice[2] < self.Y[0,0]:
            raise PlotException('Common', 'C001', ("GetHistX", "Index"))
            return
        
        ##[T.Kikuchi 130507]-->
        #start = time.time()
        
        # スライスは TkLocalパッケージにておこなう
        ##[T.Kikuchi 150511]-->
        if len(eslice) >= 6 and eslice[5] :
            # 積分モード
            slice = self.m2s.XSliceIntegrate(eslice[1], eslice[0], eslice[2], eslice[3])
        else :
            # 平均モード
            slice = self.m2s.XSliceAverage(eslice[1], eslice[0], eslice[2], eslice[3])
        ##<--[T.Kikuchi 150511]
        
        if slice is None :
            raise PlotException('Common', 'C001', ("GetHistX", "Index"))
            return
        
        obj = HistObj(array(slice.PutXList(), float64), array(slice.PutYList(), float64), array(slice.PutErList(), float64), array(slice.PutScList(), float64), slice.PutFp(), slice.PutMp(), slice.PutEp() )
        
        #print "GetHistX >>> Cost of time : ",time.time()-start
        return obj
        ##<--[T.Kikuchi 130507]

    ## InciceDat() was removed by T.Kikuchi in 130507.
    
    #########################################
    def GetHistY(self, qslice):
        """
        指定されたX値におけるヒストグラムを取得
        @param  qslice  X中央値、X始値、X終値のタプル 
        @retval ヒストグラムオブジェクト
        """ 
        
        # 2010.9.28 T.Kikuchi 普通に範囲内の領域で平均をとるように変更
        # 始値 <= 中央値 <= 終値 であること
        if qslice[1] > qslice[0] or qslice[0] > qslice[2]:
            raise PlotException('Common', 'C001', ("GetHistY", "Index"))
            return
        # qbin の数
        nX = len(self.qArray)
        # 範囲チェック
        if qslice[1] < self.X.min() or qslice[2] > self.X.max():
            raise PlotException('Common', 'C001', ("GetHistY", "Index"))
            return
        
        ##[T.Kikuchi 130507]-->
        #start = time.time()
        
        # スライスは TkLocalパッケージにておこなう
        ##[T.Kikuchi 150511]-->
        if len(qslice) >= 6 and qslice[5] :
            # 積分モード
            slice = self.m2s.YSliceIntegrate(qslice[1], qslice[0], qslice[2], qslice[3])
        else :
            # 平均モード
            slice = self.m2s.YSliceAverage(qslice[1], qslice[0], qslice[2], qslice[3])
        ##<--[T.Kikuchi 150511]
        
        if slice is None :
            raise PlotException('Common', 'C001', ("GetHistX", "Index"))
            return
        
        obj = HistObj(array(slice.PutXList(), float64), array(slice.PutYList(), float64), array(slice.PutErList(), float64), array(slice.PutScList(), float64), slice.PutFp(), slice.PutMp(), slice.PutEp() )
        
        #print "GetHistY >>> Cost of time : ",time.time()-start
        return obj
        ##<--[T.Kikuchi 130507]
    
    ## _hAdjast() was removed by T.Kikuchi in 130507.
    ## _GetQSlice() was removed by T.Kikuchi in 130507.
    ## _WeightingAverage() was removed by T.Kikuchi in 130507.
    
    ##[inamura 101214]-->
    #########################################
    def MakeMeshList(self,dQ=0.0):
        """
        Make Mesh data using GetHistX
        @param dQ (double) bin width of Q
        @return None
        """
        elastic = self.GetHistX( (0.0,0.0,0.0,dQ) )
        num_Xbin = len(elastic.bins)-1
        delta_Xbin = (elastic.bins[num_Xbin]-elastic.bins[0])/num_Xbin

        print(elastic.bins)
        print("deltaXbin=",delta_Xbin)
        
        Ybin = []
        num_Ybin = len(self.Y)-1
        for i in range(num_Ybin):
            Ybin.append( (self.Y[i,0]+self.Y[i+1,0])/2.0 )

        XX = []
        YY = []
        II = []
        EE = []
        for y in Ybin:
            sl = self.GetHistX( (y,y,y,0.0) )
            sl.ReBin( elastic.bins )
            for i in range( len(elastic.bins)-1 ):
                XX.append( (elastic.bins[i]+elastic.bins[i+1])/2.0 )
                YY.append( y )
                II.append( sl.counts[i] )
                EE.append( sl.errs[i] )

        return (XX,YY,II,EE)
        
    #########################################
    def SaveAsText(self,filename,dQ=0.0):
        """
        Make mesh data and save data as text file
        @param filename (string) File name
        @param dQ (double) bin width of Q
        @return None
        """
        try:
            fd = open( filename,'w' )
        except:
            raise

        ##[inamura 121104]->
        fd.write("## X,Y,Intensity,Error \n")
        if self.type==0:
            for i in range( len(self.arData) ):
                for j in range( len(self.arData[i]) ):
                    if self.arData[i][j] < MASKVALUE-1.0:
                        tmp = (self.X[i][j],self.Y[i][j],self.arData[i][j],self.erData[i][j] )
                    else:
                        tmp = (self.X[i][j],self.Y[i][j],0.0,0.0)
                    
                    fd.write( "%f, %f, %f, %f\n" % (tmp) )
        else:
            (XX,YY,II,EE) = self.MakeMeshList( dQ )
            
            for (xx,yy,ii,ee) in zip(XX,YY,II,EE):
                if ii<MASKVALUE:
                    fd.write( "%f, %f, %f, %f\n" % (xx,yy,ii,ee) )
        ##<--[inamura 121104]
        
        fd.close()
        
    ##<--[inamura 101214]
#######################################
#  Legacy2DFile
####################################### 
class Legacy2DFile(object):
    """
    以前のファイル形式クラス
    """
    ###################################################             
    def ReadData(self, fname1, fname2, ei):
        """
        データ変更イベント処理
        @param  fname1  SPE ファイル名
        @param fname2   PHX ファイル名
        @param ei       入射エネルギー
        @retval 無し
        """ 
        
        # 疑似データファイル読み込み
        try:
            self._ReadSPEFile(fname1)
            self._ReadPHXFile(fname2)
        except:
            raise
            return None
        else:
            # グリッドを作成
            X, Y = self._MakeXYGrid(ei)
            return (self.arData, self.erData, X, Y,1)

    #########################################
    def _ReadSPEFile(self, fname):
        """
        ダミーデータを読み込み、カラーコンツァーマップを作成
        @param  fname  ダミーデータファイル
        @retval 無し
        """ 
        try:
            datFile = open(fname, 'r')
        except:
            raise PlotException('Common', 'C006', (fname,))
            return
                
        # エネルギービン格納リストを準備 
        self.ebins = []
        # データ格納リストを準備 
        self.dataList = []
        self.errList = []
        
        self.numPsd = 0
        self.numBins = 0
        self.maxCount = 0.0

        # データファイルから、検出器数とビンの数-1を取得
        line = datFile.readline()
        self.numPsd, self.numBins = list(map(int, line.split()))
        if self.numPsd > 3000:
            datFile.close()
            raise PlotException('VisCont', 'V011', ("SPE",))
        # 
        self.numBins +=1
        
        # エネルギービンのデータまで読み飛ばし
        while True:
            line = datFile.readline()
            if "Energy" in line:
                break
        
        # エネルギービンのデータ読み込み
        while True:
            line = datFile.readline()
            if "S(Phi,w)" in line:
                break
            dat = self._ChangeNums(line)

            # 1行分のデータ(8個)をリストに追加
            self.ebins.extend(dat)
           
        # 検出器数分のデータ読み込み    
        for i in range(self.numPsd):
            # 1検出器のデータ読み込み
            hist = []
            errs = []
            while True:
                line = datFile.readline()
                if "###" in line:
                    break
                dat = self._ChangeNums(line)
                # 1行分のデータ(8個)をリストに追加
                hist.extend(dat)
                 
            # エラーデータ読み込み
            while True:
                line = datFile.readline()
                try:
                    line.split()
                except:
                    # ファイルの終了、クローズ
                    datFile.close()
                    break
                if len(line)==0:
                    # ファイルの終了、クローズ
                    datFile.close()
                    break
                    
                if "###" in line:
                    break
                dat = self._ChangeNums(line)
                # 1行分のエラー(8個)をリストに追加
                errs.extend(dat)

            # 1検出器分のヒストグラムとエラーをデータリストに追加
            self.dataList.append(hist)
            self.errList.append(errs)

    #########################################
    def _ReadPHXFile(self, fname):
        """
        角度ファイル読み込み
        @param  fname  ダミーデータファイル
        @retval 無し
        """ 
        # 角度ファイルを開く
        try:
            fra = open(fname, 'r')
        except:
            raise PlotException('Common', 'C006', (fname,))
            return
                
        # 散乱角データ格納用リストを準備
        self.faiPols = []
        # 検出器の個数読み込み
        line = fra.readline().split()
         
        # 検出器ギャップ用のリストを準備
        self.gap =[]
        bFai = 0.0
        # 検出器数分の角度データ読み込み  
        for i in range(self.numPsd):
            try:
                line = fra.readline().split()
            except:
                break
            fai = float(line[2])/180.0*pi
            dfai = float(line[4])/180.0*pi
            if float(line[3]) > 0.0:
                # 角度ファイルをクローズ
                fra.close()
                raise PlotException('VisCont', 'V011', ("PHX",))
                return
                
            self.faiPols.append((fai,dfai))

            if fai - bFai > 0.01:
                self.gap.append(i-1)
            bFai = fai
        # 角度ファイルをクローズ
        fra.close()

        self.gap.pop(0)
        # ギャップ数を格納
        self.numGap = len(self.gap)
        
        # 全点 0 のアレイを作成        
        self.arData =zeros((self.numBins-1,self.numPsd+self.numGap), float64)
        self.erData =zeros((self.numBins-1,self.numPsd+self.numGap), float64)

        # 縦横を逆にして取り込む
        for i in range(self.numBins-1):
            ng = 0
            for j in range(self.numPsd):
                self.arData[i][j+ng] = self.dataList[j][i]
                self.erData[i][j+ng] = self.errList[j][i]
                if ng < self.numGap:
                    if j == self.gap[ng]:
                        self.arData[i][j+ng+1] = MASKVALUE
                        self.erData[i][j+ng+1] = 0.0
                        ng +=1

    #########################################    
    def _ChangeNums(self,strLine ):
        """
        全検出単位のマップデータを返す
        @param  strLine 数値文字列
        @retval 実数に変換後のリスト
        """ 

        numList = []
        
        lnum = len(strLine)
        i = 0
        j = 10
        # 10 文字の数値を切り出す
        while j < lnum:
            num = float(strLine[i:j])
            numList.append(num)
            i += 10
            j += 10
        return numList

    #########################################    
    def _MakeXYGrid(self, ei):
        """
        XグリッドとYグリッドを計算
        @param  ei  入射エネルギー
        @retval XグリッドとYグリッド
        """ 
        nGap = len(self.gap)
        x = arange(0.0,self.numPsd+nGap+1 , 1.0)
        y = array(self.ebins)
        
        X,Y = meshgrid(x, y)

        ki = sqrt(ei/2.072)
        ki2 = ki*ki

        for i in range(self.numBins):

            hw = self.ebins[i]
            ng = 0
            for j in range(self.numPsd):
                # 角度読み込み
                faix, deltaFai = self.faiPols[j]
                fai = faix - deltaFai/2.0
                kf = sqrt((70.0-hw)/2.072)

                # Q を計算
                q =sqrt( ki2 + kf*kf - 2.0*ki*kf*cos(fai))
               
                X[i][j+ng]= q
                
                if ng < nGap:
                    if j == self.gap[ng]:
                        fai = faix + deltaFai/2.0
                        q =sqrt( ki2 + kf*kf - 2.0*ki*kf*cos(fai))
                        X[i][j+ng+1]= q
                        ng +=1
                        
            fai = faix + deltaFai/2.0
            q =sqrt( ki2 + kf*kf - 2.0*ki*kf*cos(fai))            
            X[i][j+ng+1]= q
            
        return (X,Y)    


##[inamura 090924]->
#########################################
#       SetColorMapDialog
#########################################
class SetColorMap2DDialog(wx.Dialog):
    """
    ColorMap 設定ダイアログクラス
    """
    ##########################################################
    def __init__(self, parent, order, postfix):
        """
        コンストラクタ
        @param  parent   親ウィンドウのID
        @param  order  プロッタの起動順序
        @param  postfix 画面タイトルに付ける親の識別子    
        @retval 無し
        """ 
        
        self.ifi = IFEvtProp(order)
        
        # リソース取得      
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        self.dialog = res.GetDialog(parent, 'Dg2DColorMap')
        # アイコンを設定
        self.dialog.SetIcon(Images().GetMainIcon())
        
        if order > 0:
            # タイトル文字列を取得
            strT = self.dialog.GetTitle()
            # 順番を付してタイトル文字列を作成
            self.dialog.SetTitle(strT + postfix)
            
        # コンボボックス
        self.cbCS = res.GetCtrl(self.dialog, 'ColorMap')
        # コンボボックスのイベントハンドラ登録
        self.cbCS.Bind(wx.EVT_COMBOBOX, self.OnComboCS, self.cbCS)
        # コンボボックスの初期値設定
        self.orgCS=self.ifi.GetProperty("colormap")
        self.cbCS.SetValue(self.orgCS)
        
        # ボタンのイベントハンドラ登録
        self.dialog.Bind(wx.EVT_BUTTON, self.OnCancel, id=res.GetId('btCancel'))
        self.dialog.Bind(wx.EVT_BUTTON, self.OnClose, id=res.GetId('btClose'))
        #[X]ボタン押下時のイベントハンドラ登録
        self.dialog.Bind(wx.EVT_CLOSE, self.OnClose)
        
        self.ifi.NotifyEvent(self, "setcolormap2ddg", False) ##[inamura 121029]
        
        # ダイアログ表示
        self.dialog.ShowModal()
        
    ##########################################################
    def OnComboCS(self, evt):
        """
        Color Map 変更イベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        # ColorMapの名前を取得
        value=self.cbCS.GetValue()
        print("Apply value=",value)

        # トレース属性変更要求をだす
        self.ifi.NotifyEvent(self, "colormap", value )

    ##########################################################
    def OnClose(self, evt):
        """
        Closeボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        self.ifi.NotifyEvent(self, "setcolormap2ddg", True) ##[inamura 121029]
        self.dialog.Destroy()

    ##########################################################
    def OnCancel(self, evt):
        """
        Cancelボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 

        self.ifi.NotifyEvent(self, "colormap", self.orgCS )
        self.ifi.NotifyEvent(self, "setcolormap2ddg", True) ##[inamura 121029]
        self.dialog.Destroy()



#########################################
#       Set XY range Dialog
#########################################        
class SetXYRange2DDialog(wx.Dialog):
    """
    X,Y Range パラメータ設定ダイアログクラス
    """
    
    ##########################################################
    def __init__(self, parent, order, postfix):
        """
        コンストラクタ
        @param  parent   親ウィンドウのID
        @param  order  プロッタの起動順序
        @param  postfix 画面タイトルに付ける親の識別子    
        @retval 無し
        """ 
        # インターフェイスクラスのインスタンスを取得
        self.ifi = IFEvtProp(order)
        
        # リソースを取得
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        self.dg = res.GetDialog(parent, 'Dg2DSetting')
        # アイコンを設定
        self.dg.SetIcon(Images().GetMainIcon())
        # アイコンを設定
        #self.dg.SetIcon(Images().GetMainIcon())
        # タイトル文字列を取得
        strT = self.dg.GetTitle()
        # 順番を付してタイトル文字列を作成
        self.dg.SetTitle(strT + postfix)
        # ボタンのイベントハンドラを登録
        self.dg.Bind(wx.EVT_BUTTON, self.OnClose, id=res.GetId('btClose'))
        self.dg.Bind(wx.EVT_BUTTON, self.OnApplyX, id=res.GetId('btXApply'))
        self.dg.Bind(wx.EVT_BUTTON, self.OnApplyY, id=res.GetId('btYApply'))
        
        # リソースよりコントロールを取得
        panel1 = res.GetCtrl(self.dg, 'xyrangepanel')
        
        # チェックボックス
        self.ckXAuto = res.GetCtrl(panel1, 'ckXAuto')
        self.ckYAuto = res.GetCtrl(panel1, 'ckYAuto')
 
        # テキストボックス
        self.tx0 = res.GetCtrl(panel1, 'x0')
        self.tx1 = res.GetCtrl(panel1, 'x1')
        self.ty0 = res.GetCtrl(panel1, 'y0')
        self.ty1 = res.GetCtrl(panel1, 'y1')
        # プッシュボタン
        self.applyX = res.GetCtrl(self.dg, 'btXApply')
        self.applyY = res.GetCtrl(self.dg, 'btYApply')
        
        ##[inamura 120711]-->
        ifdrawdic = self.ifi.GetProperty('drawdic')
        self.isOnly = self.ifi.GetProperty('only2d')
        self.init_drawdic = None
        if self.isOnly:
            self.init_drawdic = ifdrawdic["SingleMode"].copy()
        else:
            self.init_drawdic = ifdrawdic["MultiMode"].copy()

        self.spinFS = res.GetCtrl(self.dg,'spFS')
        self.spinFS.Bind(wx.EVT_SPINCTRL, self.OnSetFont, self.spinFS)
        self.chFontName = res.GetCtrl(self.dg, 'chFontName')
        self.chFontName.Bind(wx.EVT_CHOICE, self.OnSetFont, self.chFontName)
        ##<--[inamura 120711]
        
        #[X]ボタン押下時のイベントハンドラ登録
        self.dg.Bind(wx.EVT_CLOSE, self.OnClose)
        # チェックボックスのイベントハンドラ登録
        self.ckXAuto.Bind(wx.EVT_CHECKBOX, self.OnAutoX, self.ckXAuto)
        self.ckYAuto.Bind(wx.EVT_CHECKBOX, self.OnAutoY, self.ckYAuto)
        
        # 各部品に値を設定
        self._SetValues()
        
        # ダイアログオープンイベント発行
        self.ifi.NotifyEvent(self, "setxyrange2ddg", False) ##[inamura 121029]
        
        # ダイアログ表示
        self.dg.Show()
        
    ##########################################################
    def _SetValues(self):
        """
        現在のパラメータを取得して画面に設定
        @param  無し
        @retval 無し
        """ 
       
        # 現在のスケールX範囲をテキストボックスにセット
        self.ckXAuto.SetValue(self.ifi.GetProperty('autox2d'))
        self._SetEnableXRange()

        # 現在のスケールY範囲をテキストボックスにセット
        self.ckYAuto.SetValue(self.ifi.GetProperty('autoy2d'))
        self._SetEnableYRange()
        
        ##[inamura 120711]-->
        self.spinFS.SetValue( self.init_drawdic["TickLabelSize"] )
        fdic = self.ifi.GetProperty('drawdic')
        fnl = fdic["FontList"]
        for it_fnl in fnl:
            self.chFontName.Append(it_fnl)
        self.chFontName.SetStringSelection( self.init_drawdic["ScaleFont"] )
        ##<--[inamura 120711]
        
    ##########################################################
    def OnClose(self, evt):
        """
        ダイアログクローズイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        ##[inamura 120711]-->
        params = []
        #params.append( ['TickLabelSize', self.init_drawdic["TickLabelSize"] ] )
        #params.append( ['ScaleFont',self.init_drawdic["ScaleFont"] ] )
        #self.ifi.NotifyEvent(self, 'setdrawinfo', params )
        ##<--[inamura 120711]

        # パラメータ設定ダイアログクローズイベントを発行
        self.ifi.NotifyEvent(self, "setxyrange2ddg", True) ##[inamura 121029]
        
        self.dg.Destroy() 

    ##########################################################        
    def OnAutoX(self, evt=None):
        """
        AutoXチェックボックスイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        self._SetEnableXRange()
        # スケール変更通知をリスナーへ送信
        self.ifi.NotifyEvent(self, "xscale2d", (True, None, None))

    ##########################################################        
    def _SetEnableXRange(self):
        """
        X Range の選択可/不可を設定
        @param  evt イベント情報
        @retval 無し
        """ 
  
        # 自動モードであれば
        if self.ckXAuto.GetValue():
            # テキストボックスを選択不可とする
            self.tx0.Enable(False)    
            self.tx1.Enable(False)
            self.applyX.Enable(False)
            
        # マニュアルモードなら
        else:
            # 範囲プロパティを取得し文字列に変換
            strx0 = "%.5f" % (self.ifi.GetProperty('xrange2d'))[0]
            strx1 = "%.5f" % (self.ifi.GetProperty('xrange2d'))[1]
            # テキストボックスに値を設定    
            self.tx0.SetValue(strx0)
            self.tx1.SetValue(strx1)
            # スケール入力テキストボックスを選択可とする
            self.tx0.Enable(True)
            self.tx1.Enable(True)
            # Apply ボタンを選択可とする
            self.applyX.Enable(True)

            
    ##########################################################        
    def OnAutoY(self, evt=None):
        """
        AutoYチェックボックスイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        self._SetEnableYRange()
        # スケール変更通知をリスナーへ送信
        self.ifi.NotifyEvent(self, "yscale2d", (True, None, None))
        
    ##########################################################        
    def _SetEnableYRange(self):
        """
        X Range の選択可/不可を設定
        @param  evt イベント情報
        @retval 無し
        """     
        # 自動モードであれば
        if self.ckYAuto.GetValue():
            # テキストボックスを選択不可とする
            self.ty0.Enable(False)    
            self.ty1.Enable(False)
            self.applyY.Enable(False)
        # マニュアルモードなら
        else:
            # 範囲プロパティを取得し文字列に変換
            stry0 = "%.5f" % (self.ifi.GetProperty('yrange2d'))[0]
            stry1 = "%.5f" % (self.ifi.GetProperty('yrange2d'))[1]
            # テキストボックスに値を設定    
            self.ty0.SetValue(stry0)
            self.ty1.SetValue(stry1) 
            # スケール入力テキストボックスを選択可とする
            self.ty0.Enable(True)    
            self.ty1.Enable(True)
            # Apply ボタンを選択可とする
            self.applyY.Enable(True)

    ##########################################################    
    def OnApplyX(self, evt):
        """
        Applyボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        # テキストボックスより値を取得
        # 数値か
        try:
            x0 = float(self.tx0.GetValue())
        except:
            x0 = 0.0 
        try:
            x1 = float(self.tx1.GetValue())
        except:
            x1 = 1.0
        # 同値か
        if x0 == x1:
            x1 = x0+1.0
        # 逆転していたら
        if x0 > x1 :
            # 入れ替える
            tmp = x0
            x0 = x1
            x1 = tmp
        
        # 数値を再表示  
        strX0 = "%.5f" % x0   
        strX1 = "%.5f" % x1
        self.tx0.SetValue(strX0)
        self.tx1.SetValue(strX1)
         
        # スケール変更通知をリスナーへ送信
        self.ifi.NotifyEvent(self, "xscale2d", (False, x0, x1))
        
    ##########################################################    
    def OnApplyY(self, evt):
        """
        Applyボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        # テキストボックスより値を取得
        # 数値か
        try:
            y0 = float(self.ty0.GetValue())
        except:
            y0 = 0.0 
        try:
            y1 = float(self.ty1.GetValue())
        except:
            y1 = 1.0
        # 同値か
        if y0 == y1:
            y1 = y0+1.0
        # 逆転していたら
        if y0 > y1 :
            # 入れ替える
            tmp = y0
            y0 = y1
            y1 = tmp
        
        # 数値を再表示  
        strY0 = "%.5f" % y0   
        strY1 = "%.5f" % y1
        self.ty0.SetValue(strY0)
        self.ty1.SetValue(strY1)
        # スケール変更通知をリスナーへ送信
        self.ifi.NotifyEvent(self, "yscale2d", (False, y0, y1))
    ##[inamura 120711]-->
    ##########################################################
    def OnSetFont(self, evt):
        """
        """
        params = []
        params.append(['TickLabelSize', self.spinFS.GetValue()])
        params.append(['ScaleFont',self.chFontName.GetStringSelection()])
                       
        self.ifi.NotifyEvent(self, 'setdrawinfo', params )
    ##<--[inamura 120711]

##<-[inamura 090924]
##[inamura 110106]-->
#######################################
#  ChartToolBarD2Vis
#######################################         
class ChartToolBarD2Vis(ChartToolBar):
    """
    D2Vis用ツールバーの拡張クラス
    Cursor表示処理用のアイコン追加
    """
    def __init__(self, parent, order):
        """
        コンストラクタ
        @param order  イベントディスパッチャのID
        @param parent  親クラスのインスタンス
        @param order   イベントディスパッチャのID
        @retval 無
        """
        self.parent = parent
        # イベント管理クラスのインスタンスを取得
        self.ifi = IFEvtProp(order)
        #先にスーパークラスのコンストラクタを実行させる
        ChartToolBar.__init__(self, parent, order)

    ##################################    
    def _init_toolbar(self):
        """
        ツールバーを作成
        @param 無し
        @retval 無し
        """
        #先にスーパークラスのツールバーを作る
        ChartToolBar._init_toolbar(self)
        
        self._parent = self.parent.canvas.GetParent()
        # アイコンのIDを生成
        self._CURSOR = wx.NewId()
        self._SINGLM = wx.NewId() ##[inamura 120211]
        
        # アイコンのサイズ指定
        self.SetToolBitmapSize(wx.Size(24, 24))
        
        # セパレータを入れる
        self.AddSeparator()
        #アイコンをツールバーに追加
        self.AddCheckTool(self._CURSOR, self.GetCursorIconData(), shortHelp='Cursor On/Off')
        self.AddCheckTool(self._SINGLM, self.GetSingleModeIconData(), shortHelp='SingleMode On/Off')##[inamura 120211]
        #　各アイコン押下時のイベントハンドラ作成 
        if wx.VERSION_STRING >= '2.5':
            self.Bind(wx.EVT_TOOL, self.OnCursor, id=self._CURSOR)
            self.Bind(wx.EVT_TOOL, self.OnSingleMode, id=self._SINGLM) ##[inamura 120211]
        else:
            wx.EVT_TOOL(self, self._CURSOR, self.OnCursor)
            wx.EVT_TOOL(self, self._SINGLM, self.OnIngleMode) ##[inamura 120211]
            
        # turn off Cursor
        self.SetCursorBtn(False)
        
        # ツールバーの再表示
        self.Realize()

    ##################################    
    def OnCursor(self, *args):
        """
        Cursorモード変更
        @param  *args    イベント情報
        @retval 無し
        """
        # Cursorラベルモードの設定または解除 
        self.SetCursorBtn(self.GetToolState(self._CURSOR))

        # update D2Chart
        self.parent.d2.UpdateCursol(3)
 
    ##################################    
    def SetCursorBtn(self, flag): 
        """
        CURSOR ボタンオン/オフ処理
        @param  flag  True: On  False: Off
        @retval 無し
        """ 
        self.cursorMode = flag 
        # Turn on/off Cursor button
        self.ToggleTool(self._CURSOR, flag)

    ###################################################
    def GetCursorIconData(self):
        """
        Return Icom picture data as Bitmap
        Image text data is XPM format.
        @param None
        @retval wx.Bitmap
        """
        xpm_icon_data = [
            "24 24 4 1",
            " 	c None",
            ".	c #BACDF3",
            "+	c #000000",
            "@	c #8896B2",
            "                        ",
            " ...................... ",
            " ..........++.......... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " .++++++++++++++++++++. ",
            " .++++++++++++++++++++. ",
            " ..........++.@@....... ",
            " ..@@@@@@@@++@@@@@@@@@@ ",
            " ..@@@@@@@@++@@@@@@@@@@ ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " ..........++.@@....... ",
            " .............@@....... ",
            "                        "]
        
        return wx.BitmapFromXPMData(xpm_icon_data)
    
##<--[inamura 110106]
##[inamura 120211]-->
    ##################################    
    def OnSingleMode(self, *args):
        """
        Cursorモード変更
        @param  *args    イベント情報
        @retval 無し
        """
        # Cursorラベルモードの設定または解除 
        ret = self.SetSingleModeBtn(self.GetToolState(self._SINGLM))

        # update D2Chart
        self.parent.d2.OnNotifyShowOnly2d(None,None,ret) ##[inamura 120710]
        #self.parent.d2.isOnly = ret
        #self.parent.d2.PlotD2Chart()
        #self.parent.d2.UpdateCursol(3)
        #self.parent.d2.canvas.draw()
 
    ##################################    
    def SetSingleModeBtn(self, flag): 
        """
        CURSOR ボタンオン/オフ処理
        @param  flag  True: On  False: Off
        @retval 無し
        """ 
        self.SingleMode = flag 
        # Turn on/off Cursor button
        self.ToggleTool(self._SINGLM, flag)
        return flag

    ###################################################
    def GetSingleModeIconData(self):
        """
        Return Icom picture data as Bitmap
        Image text data is XPM format.
        @param None
        @retval wx.Bitmap
        """
        xpm_icon_data = [
            "22 21 69 2",
            "AA c #FFFFFFFFFFFF",
            "BA c #FFFFCCCCCCCC",
            "CA c #FFFFCCCC9999",
            "DA c #FFFF99999999",
            "EA c #CCCCFFFFFFFF",
            "FA c #CCCCFFFFCCCC",
            "GA c #CCCCCCCCFFFF",
            "HA c #CCCCCCCCCCCC",
            "IA c #CCCCCCCC9999",
            "JA c #CCCC99999999",
            "KA c #CCCC66666666",
            "LA c #CCCC33336666",
            "MA c #9999FFFFCCCC",
            "NA c #9999FFFF9999",
            "OA c #9999CCCCFFFF",
            "PA c #9999CCCCCCCC",
            "AB c #9999CCCC9999",
            "BB c #99999999FFFF",
            "CB c #99999999CCCC",
            "DB c #999999999999",
            "EB c #999999996666",
            "FB c #999966669999",
            "GB c #999966666666",
            "HB c #999966663333",
            "IB c #999933330000",
            "JB c #6666CCCCFFFF",
            "KB c #6666CCCCCCCC",
            "LB c #66669999FFFF",
            "MB c #66669999CCCC",
            "NB c #666699999999",
            "OB c #66666666CCCC",
            "PB c #666666669999",
            "AC c #666666666666",
            "BC c #666666663333",
            "CC c #666633333333",
            "DC c #666633330000",
            "EC c #33339999CCCC",
            "FC c #33336666CCCC",
            "GC c #333366669999",
            "HC c #33333333CCCC",
            "IC c #333333339999",
            "JC c #333333336666",
            "KC c #333300006666",
            "LC c #333300000000",
            "MC c #00006666CCCC",
            "NC c #000066669999",
            "OC c #00003333CCCC",
            "PC c #000033339999",
            "AD c #000033336666",
            "BD c #000033333333",
            "CD c #000000006666",
            "DD c #000000003333",
            "ED c #777700000000",
            "FD c #555500000000",
            "GD c #444400000000",
            "HD c #00000000BBBB",
            "ID c #000000008888",
            "JD c #000000007777",
            "KD c #000000005555",
            "LD c #000000004444",
            "MD c #EEEEEEEEEEEE",
            "ND c #DDDDDDDDDDDD",
            "OD c #BBBBBBBBBBBB",
            "PD c #AAAAAAAAAAAA",
            "AE c #888888888888",
            "BE c #777777777777",
            "CE c #555555555555",
            "DE c #222222222222",
            "EE c #000000000000",
            "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEE",
            "AAAAMDAAAAAAAAAAMDMDMDAAAAAAMDAAAAAAAAAAAACE",
            "AAMDAAAAMDAAAADBAEFBDBMDAAMDAAAAMDAAMDAAAACE",
            "AAAAAAAAAAAAODLDLDDDDDNBAAAAAAAAAAAAAAMDAACE",
            "AAMDAAAAAAAAICKDBDJCADJDCBAAAAAAAAAAAAHAJACE",
            "AAAAAAAAAAPDIDJCLDBDICMCPCGAAAAAAAAAAAJALACE",
            "AAAAAAAAMDPBADADBDKDADICCDPBMDAAAAAAAABAKACE",
            "AAMDAAAAHAIDOCJCADADPCMCJCLDODAAAAAAAAJADACE",
            "AAAAAAAAPDIDFCADCDADMCHCPCKDGCAAAAAAAABAIACE",
            "AAAAAAMDGCPCICPCBDKCPCMCICADIDGAAAAAAAFACACE",
            "AAMDAANDADOCMCADLDADGCOCMCICIDMBAAAAAANDNACE",
            "AAAAAAPDJDICOCPCDEADOCGCOCADPCPCMDAAAAFAABCE",
            "AAAAAABEPCMCFCECLDGCECOCFCADICMCCBAAAAPAMACE",
            "AAAAMDJCMCFCECECCDNCLBMCFCNCICOCMBAAAAEAKBCE",
            "AAAANDPCPCFCECFCICPCECECNCHCOCNCMCGAAAPAOBCE",
            "AAAAPDOCPCECECOCMCHDGCJBOCMCMCOCOCPAAABBMBCE",
            "AAAACBGCFCKBPAMCFCFCECPAECNBPBFCECCBAAODOBCE",
            "AAMDAECCACGBEBBEGBBEEBGBGBCEHBBEBEBEMDPDJCCE",
            "AAMDFDGDIBBEHBGBBEHBACHBBCEDLCFDHBDCJANDABCE",
            "AANDPDJAMAHAOAMAHAPAGAMANDODDBJAHAFAHAAAAACE",
            "EECECECECECECECECECECECECECECECECECECECECECE"]
        
        return wx.BitmapFromXPMData(xpm_icon_data)

##<--[inamura 120211]
#########################################
#       Title2DDialog
#########################################
class Title2DDialog(wx.Dialog):
    """
    タイトル設定ダイアログクラス
    """
    ##########################################################
    def __init__(self, parent, order, postfix):
        """
        コンストラクタ
        @param  parent   親ウィンドウのID
        @param  order  プロッタの起動順序
        @param  postfix 画面タイトルに付ける親の識別子    
        @retval 無し
        """ 

        self.ifi = IFEvtProp(order)
        
        # リソース取得      
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        self.dialog = res.GetDialog(parent, 'Dg2DTitle')
        
        if order > 0:
            # タイトル文字列を取得
            strT = self.dialog.GetTitle()
            # 順番を付してタイトル文字列を作成
            self.dialog.SetTitle(strT + postfix)
        
        # テキストボックスのコントローラ取得        
        self.text1 = res.GetCtrl(self.dialog, 'tx1')
        self.text2 = res.GetCtrl(self.dialog, 'tx2')
        
        self.text1.SetValue(self.ifi.GetProperty("title1"))
        self.text2.SetValue(self.ifi.GetProperty("title2"))
        # ボタンのイベントハンドラ登録
        self.dialog.Bind(wx.EVT_BUTTON, self.OnOk, id=res.GetId('btOk'))
        self.dialog.Bind(wx.EVT_BUTTON, self.OnCancel, id=res.GetId('btCancel'))
        #[X]ボタン押下時のイベントハンドラ登録
        self.dialog.Bind(wx.EVT_CLOSE, self.OnCancel)
        # ダイアログオープンイベント発行
        self.ifi.NotifyEvent(self, "title2ddg", False) ##[inamura 121029]
        
        ##[inamura 120711]-->
        # Font Info
        ifdrawdic = self.ifi.GetProperty('drawdic')
        self.isOnly = self.ifi.GetProperty('only2d')
        self.init_drawdic = None
        if self.isOnly:
            self.init_drawdic = ifdrawdic["SingleMode"].copy()
        else:
            self.init_drawdic = ifdrawdic["MultiMode"].copy()

        self.spinFS = res.GetCtrl(self.dialog,'spFS')
        self.spinFS.Bind(wx.EVT_SPINCTRL, self.OnSetFont, self.spinFS)
        self.spinFS.SetValue( self.init_drawdic["TitleSize"] )
        
        self.chFontName = res.GetCtrl(self.dialog, 'chFontName')
        self.chFontName.Bind(wx.EVT_CHOICE, self.OnSetFont, self.chFontName)
        fdic = self.ifi.GetProperty('drawdic')
        fnl = fdic["FontList"]
        for it_fnl in fnl:
            self.chFontName.Append(it_fnl)
        self.chFontName.SetStringSelection( self.init_drawdic["TitleFont"] )
        ##<--[inamura 120711]
                
        # ダイアログ表示
        self.dialog.ShowModal()

    ##########################################################
    def OnOk(self, evt):
        """
        OK ボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        # テキストボックスから値を取得
        main = self.text1.GetValue()
        sub = self.text2.GetValue()
        try:
            # 日本語文字列が含まれているかどうかをチェック
            try:
                str(main)
            except:
                raise PlotException('Common', 'C025', ("Main Title",))
            try:
                str(sub)
            except:
                raise PlotException('Common', 'C025', ("Sub Title",))
        except PlotException as ex:
            PlotMessage(self.dialog, ex)
        
        else:    
            # 前と変わらなかったら
            if self.ifi.GetProperty("title1") == main:
                # 変更無記号を送信(自動更新を解除しないため)
                # 注: 一度メインタイトル変更すると、自動更新モードを解除する
                # サブタイトルは、自動更新モードに影響しない
                main = '*'
            # タイトル変更要求送信    
            self.ifi.NotifyEvent(self, "title", (main, sub))
            
            self.ifi.NotifyEvent(self, "title2ddg", True) ##[inamura 121029]
            
            self.dialog.Destroy()
          
    ##########################################################
    def OnCancel(self, evt):
        """
        Cancel ボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        ##[inamura 120711]-->
        params = []
        params.append(['TitleSize', self.init_drawdic["TitleSize"]])
        params.append(['TitleFont', self.init_drawdic["TitleFont"]])
        self.ifi.NotifyEvent(self, 'setdrawinfo', params )
        ##<--[inamrua 120711]
        
        self.ifi.NotifyEvent(self, "title2ddg", True) ##[inamura 121029]
        
        self.dialog.Destroy()        
        
    ##[inamura 120711]-->
    ##########################################################
    def OnSetFont(self, evt):
        """
        """
        params = []
        params.append(['TitleSize', self.spinFS.GetValue()])
        params.append(['TitleFont',self.chFontName.GetStringSelection()])
                       
        self.ifi.NotifyEvent(self, 'setdrawinfo', params )
    ##<--[inamura 120711]



#########################################
#       ScaleLabel3Dialog
#########################################
class ScaleLabel3Dialog(wx.Dialog):
    """
    スケールラベル設定ダイアログクラス
    """
    ##########################################################
    def __init__(self,parent, order, postfix):
        """
        コンストラクタ
        @param  parent   親ウィンドウのID
        @param  order  プロッタの起動順序
        @param  postfix
        @retval 無し
        """ 

        self.ifi = IFEvtProp(order)
        
        # リソース取得      
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        self.dialog = res.GetDialog(parent, 'Dg2ScaleLabel')
        
        # タイトル文字列を取得
        strT = self.dialog.GetTitle()
        # 順番を付してタイトル文字列を作成
        self.dialog.SetTitle(strT + postfix)
 
        # テキストボックスのコントローラ取得        
        self.xlabel = res.GetCtrl(self.dialog, 'xlabel')
        self.ylabel = res.GetCtrl(self.dialog, 'ylabel')
        self.zlabel = res.GetCtrl(self.dialog, 'zlabel')
        # プロパティを取得
        xl= self.ifi.GetProperty("xlabel")
        yl= self.ifi.GetProperty("ylabel")
        zl= self.ifi.GetProperty("zlabel")
        
        # プロパティが取得できたら、テキストボックスに設定
        self.xlabel.SetValue(xl)
        self.ylabel.SetValue(yl)
        self.zlabel.SetValue(zl)
            
        # ボタンのイベントハンドラ登録
        self.dialog.Bind(wx.EVT_BUTTON, self.OnOk, id=res.GetId('btOk'))
        self.dialog.Bind(wx.EVT_BUTTON, self.OnCancel, id=res.GetId('btCancel'))
        #[X]ボタン押下時のイベントハンドラ登録
        self.dialog.Bind(wx.EVT_CLOSE, self.OnCancel)
        
        ##[inamura 120711]-->
        # Font Info
        ifdrawdic = self.ifi.GetProperty('drawdic')
        self.isOnly = self.ifi.GetProperty('only2d')
        self.init_drawdic = None
        if self.isOnly:
            self.init_drawdic = ifdrawdic["SingleMode"].copy()
        else:
            self.init_drawdic = ifdrawdic["MultiMode"].copy()

        self.spinFS = res.GetCtrl(self.dialog,'spFS')
        self.spinFS.Bind(wx.EVT_SPINCTRL, self.OnSetFont, self.spinFS)
        self.spinFS.SetValue( self.init_drawdic["LabelSize"] )
        
        self.chFontName = res.GetCtrl(self.dialog, 'chFontName')
        self.chFontName.Bind(wx.EVT_CHOICE, self.OnSetFont, self.chFontName)
        fdic = self.ifi.GetProperty('drawdic')
        fnl = fdic["FontList"]
        for it_fnl in fnl:
            self.chFontName.Append(it_fnl)
        self.chFontName.SetStringSelection( self.init_drawdic["LabelFont"] )
        ##<--[inamura 120711]

        # ダイアログオープンイベント発行
        self.ifi.NotifyEvent(self, "scalelabel3dg", False) ##[inamura 121029]
        
        # ダイアログ表示
        self.dialog.ShowModal()
        

    ##########################################################
    def OnOk(self,evt):
        """
        OK ボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        # テキストボックスから値を取得
        xlabel = self.xlabel.GetValue()
        ylabel = self.ylabel.GetValue()
        zlabel = self.zlabel.GetValue()
        try:        # エレメントコンテナ変換クラスのインスタンス取得?
#        self.ce = ConvertEC()
            # 日本語文字列が含まれているかどうかをチェック
            try:
                str(xlabel)
            except:
                raise PlotException('Common','C025',("X Label",))
            try:
                str(ylabel)
            except:
                raise PlotException('Common','C025',("Y Label",))
            try:
                str(zlabel)
            except:
                raise PlotException('Common','C025',("Z Label",))
                        
        except PlotException as ex:
            PlotMessage(self.dialog,ex)
        else:    
            self.ifi.NotifyEvent(self, "scalelabel", (xlabel,ylabel,zlabel))
            self.ifi.NotifyEvent(self, "scalelabel3dg", True) ##[inamura 121029]
            self.dialog.Destroy()
          
    ##########################################################
    def OnCancel(self,evt):
        """
        Cancel ボタンイベント処理
        @param  evt イベント情報
        @retval 無し
        """ 
        ##[inamura 120711]-->
        params = []
        params.append(['LabelSize', self.init_drawdic["LabelSize"]])
        params.append(['LabelFont', self.init_drawdic["LabelFont"]])
        self.ifi.NotifyEvent(self, 'setdrawinfo', params )
        ##<--[inamrua 120711]
        
        self.ifi.NotifyEvent(self, "scalelabel3dg", True) ##[inamura 121029]
        
        self.dialog.Destroy()        
        
    ##[inamura 120711]-->
    ##########################################################
    def OnSetFont(self, evt):
        """
        """
        params = []
        params.append(['LabelSize', self.spinFS.GetValue()])
        params.append(['LabelFont',self.chFontName.GetStringSelection()])
                       
        self.ifi.NotifyEvent(self, 'setdrawinfo', params )
    ##<--[inamura 120711]

#######################################
#  SlicingPanel
#######################################  
class SlicingPanel(object):
    
    """
    スライス条件パネル
    """

    #########################################
    def __init__(self, frame, panel, id, order):
        """
        スライス条件パネルの初期化、コントロールを取得、
        イベントハンドラ登録等を実行
        @param  frame  フレームID
        @param  panel  XパネルまたはYパネルのID
        @param  id     X=1, Y=2
        @param  order  イベントディスパッチャのID
        @retval 無し
        """ 
        # 入力チェッククラスのインスタンスを取得、親ウィンドウは、MapD2Frame
        self.val = Validator(frame)

        self.id = id
        # プロッタのインターフェイスを取得
        self.ifi = IFEvtProp(order)
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        # コントロールを取得
        self.txV = res.GetCtrl(panel, 'value')
        self.txdiff = res.GetCtrl(panel, 'dif')
        self.txr0 = res.GetCtrl(panel, 'tx0')
        self.txr1 = res.GetCtrl(panel, 'tx1')
        self.rdCenter = res.GetCtrl(panel, 'rdcenter')
        self.rdRange = res.GetCtrl(panel, 'rdrange')
        self.frac = res.GetCtrl(panel, 'spfrac')
        self.ckrbin = res.GetCtrl(panel, 'ckrb')
        self.txbin = res.GetCtrl(panel, 'bin')
        self.ckwith = res.GetCtrl(panel, 'ckwith')
        ##[T.Kikuchi 150511]-->
        self.ckinteg = res.GetCtrl(panel, 'ckinteg')
        ##<--[T.Kikuchi 150511]

        # ラジオボタンのイベントハンドラ登録
        self.rdCenter.Bind(wx.EVT_RADIOBUTTON, self.OnCenter, self.rdCenter)
        self.rdRange.Bind(wx.EVT_RADIOBUTTON, self.OnRange, self.rdRange) 
        
        # チェックボックスのイベントハンドラ登録
        self.ckrbin.Bind(wx.EVT_CHECKBOX, self.OnRebin, self.ckrbin)

        #ボタンのイベントハンドラ登録
        panel.Bind(wx.EVT_BUTTON, self.OnBtShow, id=res.GetId('btshow'))
        panel.Bind(wx.EVT_BUTTON, self.OnBtAppend, id=res.GetId('btappend'))
        panel.Bind(wx.EVT_BUTTON, self.OnBtClear, id=res.GetId('btclear'))
        
        # プロッタを初期化
        self.plot = None
        self.sliceParams = (0.0, 0.0, 1, 0.0, False)
        
        # スライス条件変更のリスナー登録
        self.ifi.AddListner('slicex',  self.OnNotifySlice)
        self.ifi.AddListner('slicey',  self.OnNotifySlice)
        self.ifi.AddListner('slicexrange',  self.OnNotifySliceRange)
        self.ifi.AddListner('sliceyrange',  self.OnNotifySliceRange)
        # スライス条件取得要求のリスナ登録
        self.ifi.AddListner('pickparams',  self.OnNotifyPickParams)
        # プロッタ送信モード変更得要求のリスナ登録
        self.ifi.AddListner('setwith',  self.OnNotifySetWith)

        # 公開プロパティを登録
        if id ==1:
            self.ifi.AddProperty('xparam',  lambda : self.sliceParams)
        else:
            self.ifi.AddProperty('yparam',  lambda : self.sliceParams)

    #########################################    
    def OnNotifyPickParams(self, wid, evt, value=None):
        """
        パネルからスライス条件を取得
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  value 
        @retval　無し
        """ 
        # 中心値指定か
        if self.rdCenter.GetValue():
            try:
                # 中心値を取得
                pos = float(self.txV.GetValue())
            except:
                pos = 0.0
                self.txV.SetValue("0.000")
            try:  
                #　加減算値を取得  
                diff= float(self.txdiff.GetValue())
            except:
                diff = 0.0
                self.txdiff.SetValue("0.000")
            if diff < 0.0:
                diff = 0.0
                self.txdiff.SetValue("0.000")        
            r0 = pos - diff
            r1 = pos + diff
            # 中心値指定の場合は、切片数は常に1
            frac = 1
        # レンジ指定
        else:
            try:
                # 始値を取得
                r0 = float(self.txr0.GetValue())
            except:
                r0 = 0.0
            try: 
                # 終値を取得   
                r1 = float(self.txr1.GetValue())
            except:
                r1 = r0 + 1.0        
            # 分割数を取得
            try:
                frac = int(self.frac.GetValue())
            except:
                frac = 1
            if r0 > r1:
                r1 = r0
                frac = 1
            strR0 = "%.3f" % r0
            self.txr0.SetValue(strR0)
            strR1 = "%.3f" % r1 
            self.txr1.SetValue(strR1)   
            self.frac.SetValue(frac)    
        # リビニングが指定されているか
        if self.ckrbin.GetValue():
            try:
                bin = float(self.txbin.GetValue())
                # 負値だったら
                if bin < 0.0:
                    self.txbin.SetValue("0.000")
                    bin = 0.0
            except:
                # 文字列等は空白とみなす
                self.txbin.SetValue("0.000")
                bin = 0.0 
        else:
            bin = 0.0
        
        self.sliceParams = (r0, r1, frac, bin, self.ckwith.GetValue(), self.ckinteg.GetValue())  # [T.Kikuchi 150511]

    ######################################################
    def OnNotifySlice(self, wid, evt, params):
        """
        スライス条件指定イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  params スライス条件のタプル
        @retval 無し
        """ 

        # X パネルか
        if evt=="slicex" and self.id == 1 :
            self.SetCenterValue(params[0], params[1], params[2])
        elif evt=="slicey" and self.id == 2:
            self.SetCenterValue(params[0], params[1], params[2])
        
    ######################################################
    def OnNotifySliceRange(self, wid, evt, params):
        """
        スライスレンジ条件指定イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  params スライス条件のタプル
        @retval 無し
        """ 
        # X パネルか
        if evt=="slicexrange" and self.id == 1 :
            self.SetRangeValues(params[0], params[1], params[2], params[3])
        elif evt=="sliceyrange" and self.id == 2:
            self.SetRangeValues(params[0], params[1], params[2], params[3])
    
    #########################################    
    def SetCenterValue(self, pos, diff = None, bin = None):
        """
        中心位置と範囲をテキストボックスに反映
        @param   pos   中心位置   
        @param   diff  範囲 
        @param   bin   ビン値  
        @retval　無し
        """ 
        # テキストボックスへの設定文字列作成
        strV = "%.3f" % pos
        # 中心値に設定
        self.txV.SetValue(strV)
        # 範囲が指定されているか
        if not diff == None:
            # 範囲をテキストボックスへ設定
            strDiff = "%.3f" % diff
            self.txdiff.SetValue(strDiff)
        # リビニング値が指定されているか
        if not bin == None:
            # ビン値をテキストボックスへ設定
            strBin = "%.3f" % bin
            self.txbin.SetValue(strBin)
            if bin > 0.0:
                binflag = True
            else:
                binflag = False
            # ビンモードを変更
            self.ckrbin.SetValue(binflag)
            self.OnRebin()
        # ラジオボタンを On にセット
        self.rdCenter.SetValue(True)
        self.OnCenter()

    #########################################    
    def SetRangeValues(self, pos0, pos1, fract=None, bin = None):
        """
        積算範囲をテキストボックスに反映
        @param   pos0  レンジ始値  
        @param   pos1  レンジ終値 
        @param   fract 分割数 
        @param   bin   ビン値   
        @retval　無し
        """ 
        # テキストボックスへの設定文字列作成
        strR0 = "%.3f" % pos0
        strR1 = "%.3f" % pos1
        
        # レンジテキストボックスに設定
        self.txr0.SetValue(strR0)
        self.txr1.SetValue(strR1)
        # ラジオボタンを On にセット
        self.rdRange.SetValue(True)
        
        # 分割数が指定されていたなら
        if not fract == None:
            # スピンボックスに値を設定
            self.frac.SetValue(fract)
        # リビニング値が指定されているか
        if not bin == None:
            # ビン値をテキストボックスへ設定
            strBin = "%.3f" % bin
            self.txbin.SetValue(strBin)
            if bin > 0.0:
                binflag = True
            else:
                binflag = False
            # ビンモードを変更
            self.ckrbin.SetValue(binflag)
            self.OnRebin()
            
        self.OnRange()

    ######################################################
    def OnNotifySetWith(self, wid, evt, params):
        """
        プロッタ送信モード変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  params プロッタ送信モード条件のタプル
        @retval 無し
        """ 
        # X パネルに対する要求か
        if params[1]==1 or params[1]==3 :
            if self.id == 1:
                self.ckwith.SetValue(params[0])
        # Y パネルに対する要求か        
        if params[1]==2 or params[1]==3 :
            if self.id == 2:
                self.ckwith.SetValue(params[0])
        
    #########################################    
    def OnCenter(self, event=None):
        """
        Centerラジオボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # チェックされたのか
        if self.rdCenter.GetValue():
            # Value と diff テキストボックスを選択可
            self.txV.Enable(True)   
            self.txdiff.Enable(True)
            # Range テキストボックスを選択不可
            self.txr0.Enable(False)   
            self.txr1.Enable(False)
            # Fracturing スピンボタンを選択不可
            self.frac.Enable(False)
            # Range ラジオボタンのチェックを外す
            self.rdRange.SetValue(False)

    #########################################    
    def OnRange(self, event=None):
        """
        Rangeラジオボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # チェックされたのか
        if self.rdRange.GetValue():
            # Value と diff テキストボックスを選択不可
            self.txV.Enable(False)   
            self.txdiff.Enable(False)
            # Range テキストボックスを選択可
            self.txr0.Enable(True)   
            self.txr1.Enable(True)
            # Fracturing スピンボタンを選択不可
            self.frac.Enable(True)
            # Range ラジオボタンのチェックを外す
            self.rdCenter.SetValue(False) 
    
    #########################################    
    def OnRebin(self, event=None):
        """
        Rebinning チェックボックスイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # チェックされたのか
        if self.ckrbin.GetValue():
            # bin テキストボックスを選択不可
            self.txbin.Enable(True) 
        else:  
            self.txbin.Enable(False)

    #########################################    
    def OnBtShow(self, event):
        """
        Showボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        self.ifi.NotifyEvent(self, "show", self.id)
        
    #########################################    
    def OnBtAppend(self, event):
        """
        Appendボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        self.ifi.NotifyEvent(self, "add", self.id)    
    
    #########################################    
    def OnBtClear(self, event):
        """
        Peplace(X)ボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # データクリア要求発行
        self.ifi.NotifyEvent(self, "clear", self.id)  

#######################################
#  ZScalePanel
#######################################  
class ZScalePanel(object):
    """
    Z　スケールパネルクラス
    """

    #########################################
    def __init__(self, panel, order):
        """
        Zスケールパネルのコントロールクラス
        @param  panel  ZパネルのID
        @param  order イベントディスパッチャのID
        @retval 無し
        """ 
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        # ボタンのイベントハンドラ登録
        panel.Bind(wx.EVT_BUTTON, self.OnBtzApply, id=res.GetId('btzapply'))
        # コントロールの取得
        self.ckauto = res.GetCtrl(panel, 'ckauto')
        self.cklog = res.GetCtrl(panel, 'cklog')
        self.txz0 = res.GetCtrl(panel, 'z0')
        self.txz1 = res.GetCtrl(panel, 'z1')
        self.btzApply = res.GetCtrl(panel, 'btzapply')
        ##[inamura 101022]-->
        self.cksmooth = res.GetCtrl(panel, 'cksmooth')
        self.txsmooth = res.GetCtrl(panel, 'window')
        ##<--[inamura 101022]
        # チェックボックスのイベントハンドラ登録
        self.ckauto.Bind(wx.EVT_CHECKBOX, self.OnZAuto, self.ckauto)
        self.cklog.Bind(wx.EVT_CHECKBOX, self.OnLog, self.cklog) 
        self.cksmooth.Bind(wx.EVT_CHECKBOX, self.OnSmooth, self.cksmooth) ##[inamura 101022]
        
        self.ifi = IFEvtProp(order)
        # Zスケール変更のリスナー登録
        self.ifi.AddListner('setzscale', self.OnNotifySetZScale)
        # ログモード設定のリスナー登録
        self.ifi.AddListner('setlog', self.OnNotifySetLog)

        # keeping smoothing window
        self.smoothwindow = 1      ##[inamura 110124]

    ##############################################
    def OnNotifySetZScale(self, wid, evtType, zscale):
        """
        Z軸設定イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  zscale 　スケール範囲タプル
        @retval 無し
        """ 
        # マニュアルモードであれば
        if not zscale[0]:
            # テキストボックスへ値を設定
            ##[inamura 100622]-->
            #strZ0 = "%.3f" % zscale[1]
            #strZ1 = "%.3f" % zscale[2]
            strZ0 = "%g" % zscale[1]
            strZ1 = "%g" % zscale[2]
            ##<--[inamura 100622]
            self.txz0.SetValue(strZ0)
            self.txz1.SetValue(strZ1)
        # 自動モードチェックボックスへ値を設定
        self.ckauto.SetValue(zscale[0])
        # Zスケール変更処理
        self.OnZAuto()
        
    ##############################################
    def OnNotifySetLog(self, wid, evtType, logFlag):
        """
        Zログモード設定イベント処理関数
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  logFlag　ログフラグ
        @retval 無し
        """ 
        # チェックボックスにログフラグを設定
        self.cklog.SetValue(logFlag)
        # Zスケール変更イベント発行
        self.ifi.NotifyEvent(self, "log", logFlag)     

    ##############################################
    def ChangeAutoRange(self, z0, z1):
        """
        データ変更時のZ軸設定処理(自動モード時)
        @param  z0 Z軸始値
        @param  z1 Z軸終値
        @retval 無し
        """ 
        # 自動モードであれば
        if self.ckauto.GetValue():
            # テキストボックスへ値を設定
            self.SetRange(z0, z1)
        
    ##############################################
    def SetRange(self, z0,z1):
        """
        Z軸設定処理
        @param  z0 Z軸始値
        @param  z1 Z軸終値
        @retval 無し
        """ 
        # テキストボックスへ値を設定
        ##[inamura 100622]-->
        #strZ0 = "%.3f" % z0
        #strZ1 = "%.3f" % z1
        strZ0 = "%g" % z0
        strZ1 = "%g" % z1
        ##<--[inamura 100622]
        self.txz0.SetValue(strZ0)
        self.txz1.SetValue(strZ1)
        
    ##############################################
    def OnLog(self, evt):
        """
        ログメニューイベント処理関数
        ログとリニアのモード切替
        @param  evt 　イベント
        @retval 無し
        """
        print("OnLog")
        # チェックボックスの値を取得し、ログフラグに設定
        logFlag = self.cklog.GetValue()
        # ログモードであれば
        if logFlag:
           z0 = float(self.txz0.GetValue())
           # 負値は不可   
           if z0 < 0.0:
               ##[inamura 100622]-->
               #self.txz0.SetValue("0.000")
               self.txz0.SetValue("1.0e-25")
               ##<--[inamura 100622]
        # ログモード変更イベント発行
        self.ifi.NotifyEvent(self, "log", logFlag) 

    ##############################################    
    def OnZAuto(self, evt=None):
        """
        Z軸範囲自動チェックボックスイベント処理
        @param  evt 　イベント
        @retval 無し
        """     
        # 自動モードであれば
        if self.ckauto.GetValue():
            # テキストボックスとボタンを選択不可
            self.txz0.Enable(False)    
            self.txz1.Enable(False)
            self.btzApply.Enable(False)
            # Zスケール変更イベント発行
            self.ifi.NotifyEvent(self, "zscale", (True,0.0,0.0))
        else:
            # テキストボックスとボタンを選択可に
            self.txz0.Enable(True)    
            self.txz1.Enable(True)
            self.btzApply.Enable(True)
            # 現在のテキストボックスの値を取得し送信
            self.OnBtzApply()
            
    #####################################################
    def OnBtzApply(self, ezt=None):
        """
        Zパネルアプライボタンイベント処理
        @param  evt 　イベントの種類
        @retval 無し
        """
        ##[inamura 110124]-->
        ## smoothing 
        if self.cksmooth.GetValue():
            window = float( self.txsmooth.GetValue() )
            if self.smoothwindow!=window:
                self.ifi.NotifyEvent(self, "smooth", (True,window) )
                self.smoothwindow = window
        ##<--[inamura 110124]
        # テキストボックスの値を取得
        try:
            z0 = float(self.txz0.GetValue())
        except:
            z0 = 0.0
        # ログモードであれば    
        if self.cklog.GetValue() :
            # 負値は不可   
            if z0 < 0.0:
                ##[inamura 100622]-->
                #z0 = 0.0
                z0 = 1.0E-25
                ##<--[inamura 100622]
        try:
            z1 = float(self.txz1.GetValue())
        except:
            z1 = z0+1.0
        # 1.0 以下は不可
        ##[inamura 091210]-->
        ##if z1 < z0+1.0:
        ##    z1 = z0+1.0
        ##<--[inamura 091210]
        # 再表示
        ##[inamura 100622]-->
        #strZ0 = "%.3f" % z0
        strZ0 = "%g" % z0
        self.txz0.SetValue(strZ0)
        #strZ1 = "%.3f" % z1
        strZ1 = "%g" % z1
        self.txz1.SetValue(strZ1)
        
        # Zスケール変更イベント発行
        self.ifi.NotifyEvent(self, "zscale", (False,z0,z1)) 
              
    ##[inamura 101022]-->
    #####################################################
    def OnSmooth(self, evt):
        """
        checkbox event for smoothing
        @param evt type of event
        @retval None
        """
        # チェックボックスの値を取得
        smtFlag = self.cksmooth.GetValue()
        window = float( self.txsmooth.GetValue() )
        self.smoothwindow = window
        print("OnSmooth::",smtFlag,window)
        self.ifi.NotifyEvent(self, "smooth", (smtFlag,window) )

    ##<--[inamura 101022]
    ##[inamura 110106]-->
    #####################################################
    def DisableSmooth(self):
        """
        Turn off smoothing mode
        @retval None
        """
        smtFlag = False
        self.cksmooth.SetValue(smtFlag)
        window = float( self.txsmooth.GetValue() )
        self.smoothwindow = window
        print("DisableSmooth::",smtFlag,window)
        self.ifi.NotifyEvent(self, "smooth", (smtFlag,window) )
    ##<--[inamura 110106]

#######################################
#  MapD2Frame
#######################################  
class MapD2Frame(object):
    """
    検出器マップ表示画面クラス
    """

    #########################################
    def __init__(self, parent=None, data=None, pno=1, fTitle = None, isQuiet=False):
        """
        2次元マップ表示処理のコンストラクタ
        @param  parent 親画面のインスタンス   
        @param  data  データオブジェクト 
        @param  pno   プロッタ番号
        @param  fTitle   画面タイトルの後に表示する文字列
                         (VisualCont から起動したときに起動元識別用に付与)
        @retval 無し
        """ 

        # 初期化処理
        self.plot = [None, None]
        self.dragFlag = False
        self.dowmP = (0, 0, 0, 0)
        self.pno = pno
        self.dmap = None
        
        # リソースからメイン画面を取得
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        self.frame = res.GetFrame(parent, "sqeMapframe")
        # リソースからフレームが取得できなかった
        if self.frame == None:
            return
        # フレームのタイトルに番号を付ける
        strT = self.frame.GetTitle()
        if fTitle:
            self.post = " (%s)" % fTitle
        else:
            self.post = " (%d)" % pno
            
        self.frame.SetTitle(strT + self.post)
        
        # アイコンを設定
        self.frame.SetIcon(Images().GetMainIcon())
        # メニューバーを取得
        self.menuBar= res.GetMenuBar('sqeMenuBar')
        self.frame.SetMenuBar(self.menuBar)

        # グラフ描画用のパネルを取得
        panel = res.GetCtrl(self.frame, 'mappanel')

        # グラフ描画用のフィギュアとキャンバスを作成
        self.fig   = Figure((6.0,6.4), 100)
        self.canvas= FigCanvas(panel, -1, self.fig)
        sizer = wx.BoxSizer(wx.VERTICAL)

        # ツールバー作成
        #self.toolbar = ChartToolBar(self,pno)
        self.toolbar = ChartToolBarD2Vis(self,pno) ## [inamura 110106]
        sizer.Add(self.toolbar, 0, wx.LEFT|wx.TOP|wx.GROW)
        self.toolbar.Show()
        sizer.Add(self.canvas, 1, wx.LEFT | wx.EXPAND)
        # サイザーに入れる
        panel.SetSizer(sizer)
        panel.Fit()
        # イベントハンドラーの設定
        self._SetEventHandler(res)
        
        # 文字列ならファイル名
        try:
            if type(data[0]) == type(""):
                self.lf = Legacy2DFile()
                # ファイルを読み込み、マップデータ(非直方形)を作る
                try:
                    mapXY = self.lf.ReadData(data[0],data[1])
                except:
                    raise
                if mapXY == None:
                    pass
                else:
                    # マップデータをセットし、データクラスのインスタンスを取得
                    self.dmap = MapObj(mapXY, pno)
            else:
                # データクラスのインスタンス取得
                self.dmap = MapObj(data, pno)  
        except:
            # データクラスのインスタンス取得
            self.dmap = MapObj(data, pno)

        # Z スケールレンジに初期値(データの最小値と最大値)を設定
        min, max, self.offset = self.ifi.GetProperty("zrange")
        # ログモードであれば
        if min > 0:
            min = 0.0
        # Zパネルに値を設定    
        self.zp.SetRange(min, max)

        # メイン画面を表示
        if not isQuiet:
            self.frame.Show() ##[inamura 120211]
        
        # 検出器マップの描画
        self.d2 = D2Chart(self, panel,pno )
        
        self.res = res ##[inamura 121029]

    #########################################
    def _SetEventHandler(self, res):
        """
        イベントハンドラーの登録
        @param  res リソースファイルのインスタンス   
        @retval 無し
        """ 
        # リソース取得
        res = ResourceFile(os.path.join("TKtools", "vis", "D2VisTk"))
        
        # プロパティ管理クラスのインスタンスを取得
        self.ifi = IFEvtProp(self.pno)

        #メニューのイベントハンドラ登録
        self.frame.Bind(wx.EVT_MENU, self.OnLegacy, id=res.GetId('menu_legacy'))
        self.frame.Bind(wx.EVT_MENU, self.OnPrint, id=res.GetId('menu_print'))
        self.frame.Bind(wx.EVT_MENU, self.OnClose, id=res.GetId('menu_exit'))
        self.frame.Bind(wx.EVT_MENU, self.OnLabel, id=res.GetId('menu_label'))
        self.frame.Bind(wx.EVT_MENU, self.OnTitle, id=res.GetId('menu_title'))
        ##[inamura 090924]->
        self.frame.Bind(wx.EVT_MENU, self.OnColorMap, id=res.GetId('menu_colormap'))
        self.frame.Bind(wx.EVT_MENU, self.OnSetXYRange, id=res.GetId('menu_xyrange'))
        ##<-[inamura 090924]
        ##[inamura 101214]-->
        self.frame.Bind(wx.EVT_MENU, self.OnSaveAsText, id=res.GetId('menu_saveas'))
        ##<--[inamura 101214]
        
        #クローズイベント登録
        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)
 
        #マウスイベントのリスナ登録
        self.canvas.mpl_connect('motion_notify_event', self.OnMouseMove)
        self.canvas.mpl_connect('button_press_event', self.OnMouseDown)
        self.canvas.mpl_connect('button_release_event', self.OnMouseUp)
        
        # Xパネルのコントロール取得
        xp = res.GetCtrl(self.frame, 'xpanel')
        self.xp = SlicingPanel(self.frame, xp, 1, self.pno)
         
        # Yパネルのコントロール取得
        yp = res.GetCtrl(self.frame, 'ypanel')
        self.yp = SlicingPanel(self.frame, yp, 2, self.pno)
        
        # Zパネルのコントロール取得
        zp = res.GetCtrl(self.frame, 'zpanel')
        self.zp = ZScalePanel(zp, self.pno)

        # 1次元プロッタへのデータ追加イベントのリスナー登録
        self.ifi.AddListner('add',  self.OnNotifyAppend)
        # 1次元プロッタのクリアイベントのリスナー登録
        self.ifi.AddListner('clear',  self.OnNotifyClear)
        # 印刷イベントのリスナー登録
        self.ifi.AddListner('print',  self.OnPrint)
        # 画像保存要求のリスナー登録
        self.ifi.AddListner('save',  self.OnNotifySave)
        # 2Dプロッタクローズ要求のリスナー登録
        self.ifi.AddListner('close',  self.OnClose)
        # アイコン化処理のリスナー登録
        self.ifi.AddListner('doiconize', self.OnNotifyDoIconize)
        # 2次元プロッタデータ変更のリスナー登録
        self.ifi.AddListner('changedata', self.OnNotifyChangeData)
        # Add Listner for open/close of dialogs [inamura 121029]-->
        self.ifi.AddListner('scalelabel3dg', self.OnNotifyChildDg)
        self.ifi.AddListner('title2ddg', self.OnNotifyChildDg)
        self.ifi.AddListner('setxyrange2ddg', self.OnNotifyChildDg)
        self.ifi.AddListner('setcolormap2ddg', self.OnNotifyChildDg)
        ##<--[inamura 121029]
        
        
        # データ情報表示用に文字列のコントロール取得
        panely = res.GetCtrl(self.frame, 'panel2')
        self.infotxt = res.GetCtrl(panely, 'infotxt')
        
        # Double Click state
        self.doubleClick_state1 = [ 0, 0.0] ##[inamura 120710]
        self.doubleClick_interval = 0.6     ##[inamura 120710]
            
    ######################################################
    def OnNotifySave(self, wid, evt, fname):
        """
        画像保存処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  fname ファイル名
        @retval 無し
        """ 
        try:
            try:
                self.canvas.print_figure(fname,dpi=300)
            except:
                raise PlotException('Common','C018',(fname,))
        except PlotException as ex:
                PlotMessage(self.frame,ex)

    #########################################    
    def OnNotifyAppend(self, wid, evt, pID):
        """
        Appendボタン押下イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  pID 　プロッタID
        @retval 無し
        """
        # ヘッダー作成
        header = {}
        if pID == 1 or pID == 3:
            # Qヒストグラム取得
            hist = self.ifi.GetProperty("qdata")
            strLabel = self.ifi.GetProperty("strenergy")
            header["Label"] = strLabel
            # データ追加
            self.AddToPlot((hist.bins, hist.counts, hist.errs, header), 1)
        if pID == 2 or pID == 3:
            # エネルギーヒストグラム取得
            hist = self.ifi.GetProperty("edata")
            strLabel = self.ifi.GetProperty("strq")
            header["Label"] = strLabel
            # データ追加
            self.AddToPlot((hist.bins, hist.counts, hist.errs, header), 2)

        
    #########################################    
    def OnNotifyClear(self, wid, evt, pID):
        """
        Peplace(X)ボタン押下イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  pID 　プロッタID
        @retval 無し
        """ 
        print(pID)
        if pID == 1 or pID == 3:
            # データクリア要求発行
            IFEvtProp(self.pno+1).NotifyEvent(self, "remove", 0) 
        if pID == 2 or pID == 3:
            # データクリア要求発行
            IFEvtProp(self.pno+2).NotifyEvent(self, "remove", 0)     

    ##############################################        
    def OnLegacy(self, *args):
        """
        レガシィファイル読み込み
        @param  *args 　イベント
        @retval 無し
        """

        # SPE ファイルオープンダイアログ
        dlg = wx.FileDialog(self.frame, 'SPE File Open ...', os.getcwd(), style=wx.OPEN, wildcard='SPE(*.SPE)|*.SPE')

        # キャンセルか
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        spefile = dlg.GetPath()
        fspe = dlg.GetFilename()
        dir = os.path.dirname(spefile)
        
        # PHX ファイルオープンダイアログ
        dlg = wx.FileDialog(self.frame, 'PHX File Open ...', dir, style=wx.OPEN, wildcard='PHX(*.PHX)|*.PHX')

        # キャンセルか
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        phxfile = dlg.GetPath()
        fphx = dlg.GetFilename()
        # タイトルをファイル名から作る
        strMain = fspe + ', ' + fphx
        
        # Ei 指定、正常に入力されるまで繰り返す
        while True:
            dlg = wx.TextEntryDialog(self.frame, "Main Ei (mev):",
                                  "Input Ei", style=wx.OK|wx.CANCEL)
            # キャンセルか
            if dlg.ShowModal() != wx.ID_OK:
                return
            try:
                try:
                    ei = float(dlg.GetValue()) 
                except:
                    raise PlotException('Common', 'C027', ("Ei",))
                else:
                    if ei > 0.0:
                        break
                    raise PlotException('Common', 'C027', ("Ei","0.0"))
            except PlotException as ex:
                PlotMessage(self.frame, ex)

        # レガシィファイルの読み込みクラスのインスタンス取得
        spe = Legacy2DFile()
        # SPE ファイルの読み込み
        try:
            # ファイルを読み込み、マップデータ(非直交)を作る
            mapXY = spe.ReadData(spefile, phxfile, ei)
        except PlotException as ex:
                PlotMessage(self.frame, ex)
        else:
            ifi = IFEvtProp(self.pno)
            # データ変更
            ifi.NotifyEvent(self, "changedata", mapXY)
            # サブタイトル作成
            strSub = "Ei (meV): %.1f\n" % ei  
            # タイトル設定コマンド送信
            ifi.NotifyEvent(None, "title", (strMain, strSub))
            # スケールラベル設定コマンド送信
            ifi.NotifyEvent(None, "scalelabel", ("Q(1/Angstrom)", "Energy(meV)","Neutrons"))   
        
    #########################################    
    def AddToPlot(self, data, pID):
        """
        データをプロッタへ追加
        @param  data   追加データ
        @param  pID 　プロッタID
        @retval 無し
        """ 
        ifi = IFEvtProp(pID+self.pno)
        
        pt = " - %d" % pID
        # プロッタが起動されていなければ
        if self.plot[pID-1] == None:
            # プロッタフレームクローズイベントのリスナー登録
            ifi.AddListner('plotclose',  self.OnNotifyPlotClose)
            # Q ヒストグラム用のプロッタウィンドウを開く
            self.plot[pID-1] = PlotFrame(self.frame, data, pID+self.pno, self.post+pt)
            # タイトル取得
            main = IFEvtProp(self.pno).GetProperty("title1")
            subs = IFEvtProp(self.pno).GetProperty("title2").split('\n')
            sub = ""
            for line in subs:
                sub = sub + line.strip() + ', '
            sub = sub[:-2]    

            # タイトル設定コマンド送信
            ifi.NotifyEvent(self, "title", (main, sub)) 
            # 軸ラベル取得
            xlb = IFEvtProp(self.pno).GetProperty("xlabel")
            ylb = IFEvtProp(self.pno).GetProperty("ylabel")
            zlb = IFEvtProp(self.pno).GetProperty("zlabel") 
            
#            zlb = "Newtrons"
            # 軸ラベル設定コマンド送信 
            if pID == 1:
                ifi.NotifyEvent(self, "scalelabel", (xlb, zlb)) 
            else:
                ifi.NotifyEvent(self, "scalelabel", (ylb, zlb))
             
        else:
            # データ追加要求イベント発行
            ifi.NotifyEvent(self, "add", [data])
            
  
            
    #####################################################
    def OnNotifyPlotClose(self, wid, evt, value=None):
        """
        プロッタクローズイベント対応処理
        @param  wid 　 イベント発生元のインスタンス
        @param  evt 　イベントの種類
        @param  value  無し   
        @retval 無し
        """ 
        # プロッタ(1)がクローズされたのか
        if self.plot[0] == wid:
            IFEvtProp(self.pno+1).InitIF()
            self.plot[0] = None
        # プロッタ(2)がクローズされたのか    
        elif self.plot[1] == wid:
            IFEvtProp(self.pno+2).InitIF()
            self.plot[1] = None 
    
    ###################################################
    def OnNotifyDoIconize(self, wid, evtType, flag):
        """
        ウィンドウアイコン化または復元要求受信処理
        @param wid
        @param evtType
        @param  flag    True: アイコン化　False: 元に戻す  
        @retval 無し
        """ 
        # アイコン化する、または元に戻す
        self.frame.Iconize(flag)        

    ###################################################             
    def OnNotifyChangeData(self, wid, evtType, data):
        """
        データ変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  data データオブジェクト
        　　　　　　　文字列の場合は、ダミーデータファイル名とみなす
        @retval 無し
        """
        self.dmap.ChangeData(data)
        # Zデータの範囲を取得
        min, max, offset = self.ifi.GetProperty("zrange")
        if min >= 0.0:
            min = 0.0
        else:
            if self.ifi.GetProperty("logflag"):
                min = 0.0
            
        self.zp.ChangeAutoRange(min, max)
        self.zp.DisableSmooth() ##[inamura 110106]
        self.d2.PlotD2Chart()
         

    #########################################
    def OnMouseMove(self, event):
        """
        マウス-ポインタ移動イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # データ無しなら
        if self.ifi.GetProperty("mapdata") == None:
            return
          
        # マウス位置がグラフ内かどうかをチェック
        if not event.inaxes:
            self.infotxt.SetLabel("")
            return
        
        # グラフ内であっても、None が来ることがあるのでプロテクト
        if event.ydata == None or event.xdata == None:
            self.infotxt.SetLabel("")
            return

        # Qヒストグラム内か
        if event.y > self.ifi.GetProperty("y1dpi"):
            strTx = "X=%.3f    Z= %.3f" % (event.xdata, event.ydata)
        
        # エネルギーヒストグラム内か
        elif event.x > self.ifi.GetProperty("x1dpi"):
            strTx = "Y=%.3f    Z= %.3f" % (event.ydata, event.xdata)
        
        # カラーマップ外か
        if event.y < self.ifi.GetProperty("y0dpi"):
            return
        else:
            # マウスのY位置におけるヒストグラムを取得
            try:
                hist = self.dmap.GetHistX((event.ydata,event.ydata,event.ydata, 0.0))
            except:
                self.infotxt.SetLabel("")
                return
            # Z データの無い領域か
            if hist.bins[0] > event.xdata or hist.bins[len(hist.bins)-1] < event.xdata:
                # マウス位置の情報を表示、Z データ無し
                strTx = "X=%.3f    Y=%.3f    Z= --" % (event.xdata, event.ydata)
            # カラーコンツァー表示領域
            else:
                for i in range(len(hist.bins)):
                    if hist.bins[i]> event.xdata:
                        break
                    # マスク値であれば
                    if hist.counts[i] >= MASKVALUE-1.0:
                        # Z データは無し 
                        strTx = "X=%.3f    Y=%.3f    Z= --" % (event.xdata, event.ydata)
                    else:
                        strTx = "X=%.3f    Y=%.3f    Z= %.3f" % (event.xdata, event.ydata, hist.counts[i])

           # ドラッグ中か
            if self.dragFlag:
                # ドラッグ範囲を、半透明の領域で表示
                self.d2.DrawRectCursol(event, self.downP)   
        # マウス位置情報表示
        self.infotxt.SetLabel(strTx)

    #########################################
    def OnMouseDown(self, event):
        """
        マウス-ボタンクリックイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 

        # パン処理中またはズーム処理中であれば、以下の処理を実行しない
        if self.toolbar._active == 'PAN' or self.toolbar._active == 'ZOOM':
            return

        # グラフ範囲外か
        if not event.inaxes:
            ##[inamura 120710]-->
            # if click in title area
            if event.y > self.ifi.GetProperty("y1dpi"):
                self.doubleClick_state1[0] = 3
            elif event.x < (self.ifi.GetProperty("x0dpi")*0.4) or event.y < (self.ifi.GetProperty("y0dpi")*0.7):
                self.doubleClick_state1[0] = 2
            elif event.x < self.ifi.GetProperty("x0dpi") or event.y < self.ifi.GetProperty("y0dpi"):
                self.doubleClick_state1[0] = 1
            else:
                self.doubleClick_state1[0] = 0
            ##<--[inamura 120710]

            return
        # カラーマップグラフ内か
        if event.y <= self.ifi.GetProperty("y1dpi") and event.x <= self.ifi.GetProperty("x1dpi"):
            # マウスクリック点のデータを保存
            self.downP = (event.x, event.y, event.xdata, event.ydata)
            # ドラッグ開始
            self.dragFlag = True 
        
            self.doubleClick_state1[0] = 0 ##[inamura 120710]
        

    #########################################
    def OnMouseUp(self, event):
        """
        マウス-ボタンアップイベント処理
        @param   event イベント情報   
        @retval 無し
        """
        ##[inamura 120710]-->
        if (self.doubleClick_state1[1]!=0.0) and (self.doubleClick_state1[1]+self.doubleClick_interval)>time.time():
            if self.doubleClick_state1[0]==1:
                self.OnSetXYRange(None)
            if self.doubleClick_state1[0]==2:
                self.OnLabel(None)
            if self.doubleClick_state1[0]==3:
                self.OnTitle(None)
            self.doubleClick_state1[1] = 0.0
        else:
            self.doubleClick_state1[1] = time.time()            
        ##<--[inamura 120710]

        # ドラッグ中(マップ内でボタン押下された)か
        if self.dragFlag:
            # ドラッグ終了
            self.dragFlag = False
        else:
            return
        # データ無しなら
        if self.ifi.GetProperty("mapdata") == None:
            return
        
        # パン処理中またはズーム処理中であれば、以下の処理を実行しない
        if self.toolbar._active == 'PAN' or self.toolbar._active == 'ZOOM':
            return

        # グラフ範囲外か
        if not event.inaxes:
            return
        # カラーマップグラフ外であれば何もしない
        if event.y > self.ifi.GetProperty("y1dpi") or event.x > self.ifi.GetProperty("x1dpi"):
            return

        # 開始点と終了点との差が、1 pixel 以下であればクリックとみなす
        if abs(self.downP[0] - event.x) < 2.0 and abs(self.downP[1] - event.y) < 2.0 :
            # 中心値のテキストボックスに値を設定
            self.ifi.NotifyEvent(self, "slicex", (event.ydata, None, None)) 
            self.ifi.NotifyEvent(self, "slicey", (event.xdata, None, None)) 

        # ドラッグならば
        else:
            # X方向のドラッグ開始点と終了点の大小比較
            if event.xdata > self.downP[2]:
                x0 = self.downP[2]
                x1 = event.xdata
            else:
                x0 = event.xdata
                x1 = self.downP[2]
            # Y Slice パネルの　Range へ　値を設定
            self.ifi.NotifyEvent(self, "sliceyrange", (x0,x1, 1, None)) 

            # Y方向のドラッグ開始点と終了点の大小比較
            if event.ydata > self.downP[3]:
                y0 = self.downP[3]
                y1 = event.ydata
            else:
                y0 = event.ydata
                y1 = self.downP[3]
            # X Slice パネルの　Range へ　値を設定
            self.ifi.NotifyEvent(self, "slicexrange", (y0,y1, 1, None)) 
        
        # カーソルとグラフを表示
        self.ifi.NotifyEvent(self, "show", 3)       

    ######################################################
    def OnTitle(self,event):
        """
        Set Graph Title メニューイベント処理
        スケール設定ダイアログを表示
        @param  event イベント情報
        @retval 無し
        """ 
        # タイトル設定ダイアログを開く 
        dlg = Title2DDialog(self.frame, self.pno, self.post)
 

    ######################################################
    def OnLabel(self,event):
        """
        Set Scale Label メニューイベント処理
        スケール設定ダイアログを表示
        @param  event イベント情報
        @retval 無し
        """ 
        dlg = ScaleLabel3Dialog(self.frame, self.pno, self.post)

    ##[inamura 090924]->
    ######################################################
    def OnColorMap(self,event):
        """
        Set ColorMap メニューイベント処理
        スケール設定ダイアログを表示
        @param  event イベント情報
        @retval 無し
        """ 
        
        dlg = SetColorMap2DDialog(self.frame, self.pno, self.post)

    ######################################################
    def OnSetXYRange(self,event):
        """
        Set Graph Parameter メニューイベント処理
        スケール設定ダイアログを表示
        @param  event イベント情報
        @retval 無し
        """ 
        
        dlg = SetXYRange2DDialog(self.frame, self.pno, self.post)
        
    ##<-[inamura 090924]

    ##############################################        
    def OnPrint(self, *args):
        """
        デテクタマップ印刷
        @param  *args 　イベント情報
        @retval 無し
        """ 
       
        # プラットフォームを判定     
        if os.name == "posix" or os.name == "mac" :
            # 領域を破線で表示し直す
            xc,yc = self.ifi.GetProperty("cursol")
            self.d2.DrawCursols(xc, yc, False)
            fname = "_d2plot.ps" 
            # テンポラリファイルの絶対パスを作成(指定されたテンポラリディレクトリの下)
            tmpf = os.path.join(tempfile.gettempdir(), fname)
            # Linux または Mac OS なら
            try:
                try:
                    # ポストスクリプトファイルを作成
                    self.canvas.print_figure(tmpf, dpi=600)
                except:
                    raise PlotException('Common','C018',(tmpf,))
                # スプーラにファイルを送信
                ret = os.system("lpr "+tmpf)
                # エラーがあったなら
                if ret > 0:
                    # エラーメッセージを表示
                    raise PlotException('Common','C023',())
                
            except PlotException as ex:
                PlotMessage(self.frame,ex)
            time.sleep(2.0)
            # カーソルを元に戻す
            self.d2.DrawCursols(xc, yc, True)  
        # Linux または Mac OS 以外(Windowsを想定)
        else:
            #用紙をA4に設定
            self.canvas.printerData.SetPaperId(wx.PAPER_A4)
            # 印刷画像の幅を設定 
            self.canvas.printer_width = 7.0
            # マージンを設定
            self.canvas.printer_margin = 1.0
            #プリントダイアログを表示        
            self.canvas.Printer_Print() 

    ##############################################        
    def OnClose(self, *args):
        """
        デテクタマップ画面クローズ
        @param  evt 　イベント
        @retval 無し
        """
        # インターフェイスのクリア
        IFEvtProp(self.pno+1).InitIF()
        IFEvtProp(self.pno+2).InitIF()

        # 2次元プロッタフレームクローズイベント発行
        self.ifi.NotifyEvent(self, "2dclosed")
        
        IFEvtProp(self.pno).InitIF()
        
        # 検出器マップ画面のクローズ       
        self.frame.Destroy() 

    ##[inamura 101214]-->
    ##############################################
    def OnSaveAsText(self, *args):
        """
        """
        print("OnSaveAsText")
        dlg =wx.FileDialog(self.frame, "Save data as text files", style=wx.SAVE, wildcard='Text(*.*)|*.*')
        if dlg.ShowModal() == wx.ID_OK:
            try:
                self.dmap.SaveAsText(dlg.GetPath())
            except:
                raise
            
    ##<--[inamura 101214]
    ##[inamura 121029]-->
    ###############################################
    def OnNotifyChildDg(self, wid, evt, flag):
        """
        """
        if evt == "scalelabel3dg":
            self.menuBar.Enable(self.res.GetId('menu_label'), flag)
        elif evt == "title2ddg":
            self.menuBar.Enable(self.res.GetId('menu_title'), flag)
        elif evt == "setxyrange2ddg":
            self.menuBar.Enable(self.res.GetId('menu_xyrange'), flag)
        elif evt == "setcolormap2ddg":
            self.menuBar.Enable(self.res.GetId('menu_colormap'), flag)
    ##<--[inamura 121029]
            
#######################################
#  D2Chart
#######################################  
class D2Chart(object):
    """
    グラフ描画 Class 
    """
    #########################################
    def __init__(self, parent, panel, order):
        """
        コンストラクタ
        @param  parent  起動クラスインスタンス
        @param  panel   描画用パネル
        @param  order　　イベントディスパッチャのID
        @retval 無し
        """
        self.fig = parent.fig
        self.canvas = parent.canvas
        self.dmap = parent.dmap
        self.frame = parent.frame
        self.xcyc = [None, None]
        self.toolbar = parent.toolbar ##[inamura 110106]
        self.isOnly = False ##[inamura 120111]
        ##[inamura 120711]-->
        # Draw managements
        sModeDic = {}
        mModeDic = {}
        sModeDic["MainPosi"] = [0.13, 0.105, 0.75, 0.75]
        mModeDic["MainPosi"] = [0.1, 0.095, 0.63, 0.62]
        sModeDic["ColorBarPosi"] = [0.90, 0.15, 0.05 ,0.62]
        mModeDic["ColorBarPosi"] = [0.02, 0.04, 0.28 ,0.02]
        sModeDic["ColorBarOri"] = 'vertical'
        mModeDic["ColorBarOri"] = 'horizontal'
        sModeDic["TickLabelSize"] = 10
        mModeDic["TickLabelSize"] = 8
        sModeDic["LabelSize"] = 12
        mModeDic["LabelSize"] = 8
        sModeDic["TitleSize"] = 12
        mModeDic["TitleSize"] = 12
        sModeDic["LabelFont"] = 'serif'
        mModeDic["LabelFont"] = 'serif'
        sModeDic["TitleFont"] = 'serif'
        mModeDic["TitleFont"] = 'serif'
        sModeDic["ScaleFont"] = 'serif'
        mModeDic["ScaleFont"] = 'serif'
        fDic = ["serif","sans-serif","monospace"]
        self.DrawDic = {"SingleMode":sModeDic,"MultiMode":mModeDic,"FontList":fDic}
        
        ##<--[inamura120711]
        
        # 初期化
        self.logFlag = False
 
        # プロパティ管理クラスのインスタンスを取得
        self.ifi = IFEvtProp(order) 
        # データ変更のリスナー登録
        self.ifi.AddListner('changedata', self.OnNotifyChangeData)
        # グラフ表示イベントのリスナー登録
        self.ifi.AddListner('show',  self.OnNotifyShow)
        # ログモード変更イベントのリスナー登録
        self.ifi.AddListner('log',  self.OnNotifyLog)
        # XYスケール範囲変更イベントのリスナー登録
        self.ifi.AddListner('changexy',  self.OnChangeXY)
        # Homeボタン押下イベントのリスナー登録
        self.ifi.AddListner('home',  self.OnHome)
        # タイトル変更のリスナー登録
        self.ifi.AddListner('title', self.OnNotifyTitle)
        # ラベル変更のリスナー登録
        self.ifi.AddListner('scalelabel', self.OnNotifyLabel)
        # Zスケール変更のリスナー登録
        self.ifi.AddListner('zscale', self.OnNotifyZScale)
        ##[inamura 090924]->
        # Color Scale変更のリスナー登録
        self.ifi.AddListner('colormap', self.OnNotifyColorMap)
        # X,Y Range変更のリスナー登録
        self.ifi.AddListner('xscale2d', self.OnNotifyXScale2D)
        self.ifi.AddListner('yscale2d', self.OnNotifyYScale2D)
        ##<-[inamura 090924]
        ##[inamura 101022]-->
        self.smoothFlag = False
        self.ifi.AddListner('smooth', self.OnNotifySmooth)
        ##<--[inamura 101022]
        self.ifi.AddListner('showonly2d', self.OnNotifyShowOnly2d) ##[inamura 120417]
        self.ifi.AddListner('setdrawinfo', self.OnNotifySetDrawInfo) ##[inamura 120711]
        # 2D画面描画準備
        self._DispInitialMap()
        # 描画
        self.PlotD2Chart()
        
        # Mapパネルのサイズ取得
        xsize,ysize = panel.GetSize()
        self.panel = panel ##[inamura 120710]
        
        #  グラフ境界のピクセル値を求める
#        pos = self.ax.get_position()   matplotlib 0.98.5 対応
        pos = VerHandlePlotLib().GetSubPlotPos(self.ax)
        
        self.x1dpi = (pos[0]+pos[2])*xsize
        self.y1dpi = (pos[1]+pos[3])*(ysize-20)
        self.y0dpi = pos[1]*(ysize-20)
        self.x0dpi = pos[0]*xsize   ##[inamura 120710]
        
        # 公開プロパティを登録
        self.ifi.AddProperty('logflag',    lambda : self.logFlag)
        self.ifi.AddProperty('title1',     lambda : self.title1)
        self.ifi.AddProperty('title2',     lambda : self.title2)
        self.ifi.AddProperty('xlabel',     lambda : self.xlabel)
        self.ifi.AddProperty('ylabel',     lambda : self.ylabel)
        self.ifi.AddProperty('zlabel',     lambda : self.zlabel)
        self.ifi.AddProperty('x0dpi',      lambda : self.x0dpi) ##[inamura 120710]
        self.ifi.AddProperty('x1dpi',      lambda : self.x1dpi)
        self.ifi.AddProperty('y0dpi',      lambda : self.y0dpi)
        self.ifi.AddProperty('y1dpi',      lambda : self.y1dpi)
        self.ifi.AddProperty('qdata',      lambda : self.qdata)
        self.ifi.AddProperty('edata',      lambda : self.edata)
        self.ifi.AddProperty('strenergy',  lambda : self.strEnergy)
        self.ifi.AddProperty('strq',       lambda : self.strQ)
        self.ifi.AddProperty('cursol',     lambda : self.xcyc)
        ##[inamura 090924]->
        self.ifi.AddProperty('colormap',   lambda : self.colormap)
        self.ifi.AddProperty('autox2d',    lambda : self.autox2DFlag)
        self.ifi.AddProperty('autoy2d',    lambda : self.autoy2DFlag)
        self.ifi.AddProperty('xrange2d',   lambda : self.xrange)
        self.ifi.AddProperty('yrange2d',   lambda : self.yrange)
        ##<-[inamura 090924]
        self.ifi.AddProperty('only2d',   lambda : self.isOnly) ##[inamura 120417]
        self.ifi.AddProperty('drawdic',  lambda : self.DrawDic)##[inamura 120711]
        
        # データ無しか
        if self.Z == None:
            return
                
        # カーソル位置の初期値をテキストボックスに設定
        cx = self.xrange[0]+ (self.xrange[1] - self.xrange[0] ) / 2.0
        # スライス条件指定イベント発行
        self.ifi.NotifyEvent(self, "slicey", (cx, 0.0, 0.0)) 
        cy = self.yrange[0]+ (self.yrange[1] - self.yrange[0] ) / 2.0
        # スライス条件指定イベント発行
        self.ifi.NotifyEvent(self, "slicex", (cy, 0.0, 0.0))
        # カーソルとグラフを表示
        self.UpdateCursol(3) 
    
    #########################################
    def _DispInitialMap(self):
        """
        データの準備
        @param  無し   
        @retval 無し
        """ 
        self.title1 = ""
        self.title2 = ""
        # 各軸のラベル文字列を作成
        self.xlabel = "X"
        self.ylabel = "Y"
        self.zlabel = "Z" 

        ##[inamura 090924]->
        # Color Scale 0: gray scale, 1: jet
        self.colormap = "default"
        self.autox2DFlag = True
        self.autoy2DFlag = True
        ##<-[inamura 090924]
        ##[inamura 110125]-->
        # Original ColorMap
        cdict = {'blue': ((0.0, 0.0, 0.0),
                          (0.16, 1, 1),
                          (0.377, 1, 1),
                          (0.67, 0, 0),
                          (1, 0,0)),
                 'green': ((0.0, 0, 0),
                           (0.174, 0, 0),
                           (0.410, 1, 1),
                           (0.66, 1, 1),
                           (0.915, 0, 0),
                           (1, 0, 0)),
                 'red': ((0.0, 0, 0),
                         (0.387, 0, 0),
                         (0.680, 1, 1),
                         (0.896, 1, 1),
                         (1, 0.5, 0.5))
                 }
        
        self.colormap_default = matplotlib.colors.LinearSegmentedColormap('default_utsusemi',cdict,256)
        ##<--[inamura 110125]
        # マップデータ取得
        self.Z = self.ifi.GetProperty("mapdata")
        ##[T.Kikuchi 130514]-->
        self.Zsc = self.ifi.GetProperty("scdata")
        ##<--[T.Kikuchi 130514]
        # データ無しなら
        if self.Z == None:
            return


        # Zデータの範囲を取得
        min, max, self.offset = self.ifi.GetProperty("zrange")
        if min >= 0.0:
            min = 0.0
        else:
#            if self.ifi.GetProperty("logflag"):  # 2008/04/28 Minakawa modify
           if self.logFlag:
                min = 0.0
            
        # 描画範囲を作る
        self.zMin = self.datMin = min
        self.zMax = self.datMax = max
        # グリッドデータ作成
        self.X,self.Y = self.ifi.GetProperty("grid")
        # マップデータからマスクデータ作成
        ##[T.Kikuchi 150511]-->
        #self.Zm = ma.masked_where(self.Z >= MASKVALUE-1.0, self.Z)
        self.Zm = ma.masked_where(self.Zsc <= 0.0, self.Z)
        ##<--[T.Kikuchi 150511]
        
        # 描画範囲の初期設定設定
        self.xrange = (self.X.min(), self.X.max())
        self.yrange = (self.Y.min(), self.Y.max())
        
       
    #########################################        
    def _ConvertCenterValue(self, bins):
        """
        bin値の中央値に変換する
        @param   bins  bin値をいれた配列
        @retval 中央値に変換後の配列
        """ 
        # データ(bin の境界値)数 - 1 の array を作る
        xx = zeros(len(bins)-1, float64)
        # 各bin の中央値を作る
        for i in range(len(bins)-1):
            xx[i]= (bins[i]+bins[i+1])/2.0
            
        return xx    

    #########################################        
    def GetMax(self, data, rng ):
        """
        指定範囲内で、マスク値を除く最大値を取得
        @param   data データ
        @param   rng 　探索範囲始値と終値のタプル
        @retval 最小値と最大値のタプル
        """ 
        minV = 10000000.0
        maxV = -10000000.0
        # データ配列を探索
        for i in range(len(data.counts)):
            # 指定された範囲内か
            if data.bins[i+1] >= rng[0] and data.bins[i+1] <= rng[1]:
                # 最小値サーチ
                if data.counts[i] < minV:
                    minV = data.counts[i]
                # マスク値でなければ
                if data.counts[i] < MASKVALUE-1.0:
                    # 最大値サーチ
                    if data.counts[i] > maxV:
                        maxV = data.counts[i]
        if minV > 0.0:
            minV = 0.0     
        return (minV, maxV*1.05)                         
  
    #########################################        
    def UpdateQbinGraph(self, eslice):
        """
        指定された E値におけるQ bin グラフの表示
        @param   eslice  E中央値、E始値、E終値のタプル 
        @retval 無し
        """ 
        # エネルギー値の等しい点におけるヒストグラムを取得
        try:
            self.qdata = self.dmap.GetHistX(eslice)
        except PlotException as ex:
            PlotMessage(self.frame, ex)
            
        # Xデータ(bin の境界値)の中央値に変換
        self.qxx = self._ConvertCenterValue(self.qdata.bins)
        
        # 指定範囲における、マスク値を除いた最大値を求める
        qzmin, qzmax = self.GetMax(self.qdata, self.ax.get_xlim())

        # マスクデータに変換        
        self.qzm = ma.masked_where(self.qdata.counts >= MASKVALUE, self.qdata.counts)
        
        # グラフ上に表示する値を保存

        if self.qdata.fp-self.qdata.ep == 0.0:
            self.strEnergy = "%.3f" % self.qdata.mp
        else:
            self.strEnergy = "%.3f (%.3f - %.3f)" % (self.qdata.mp,self.qdata.fp,self.qdata.ep) 

        # グラフの描画
        self.DrawQbinGraph(self.ax.get_xlim(), (qzmin, qzmax))

    #########################################        
    def DrawQbinGraph(self, xlim, ylim):
        """
        指定された E値におけるQ bin グラフの表示
        @param   xlim  X軸範囲のタプル
        @param   ylim  Y軸範囲のタプル
        @retval 無し
        """ 
        if self.isOnly:
            return ##[inamura 120211]
        # Q bin グラフをクリア
        self.axq.clear() 

        # 指定された E値におけるQ bin グラフ表示
        self.axq.plot(self.qxx,self.qzm, linewidth=0.6)

        # 座標ラベルの文字を小さくする
        xcl = getp(self.axq, 'xticklabels')
        setp(xcl, size=7)
        ycl = getp(self.axq, 'yticklabels')
        setp(ycl, size=7)     

        # 表示範囲を設定

        self.axq.set_xlim(xlim)
        self.axq.set_ylim(ylim)
        # グラフタイトルを表示
        strGtitle = "Y = %s" % self.strEnergy
        yy = (ylim[1]-ylim[0])*0.05+ylim[1]
        ##[inamura 120711]-->
        target = self.DrawDic["MultiMode"]
        #self.axq.text(xlim[0], yy, strGtitle, fontsize = 8)
        self.axq.text(xlim[0], yy, strGtitle, fontname=target["TitleFont"],fontsize = 8)
        # Y軸のラベルを表示
        #self.axq.set_ylabel(self.zlabel, fontsize = 9)
        self.axq.set_ylabel(self.zlabel, fontname=target["TitleFont"],fontsize = 9)
        ##<--[inamura 120711]
        
    #########################################        
    def UpdateEbinGraph(self, qslice):
        """
        指定された Q値におけるE bin グラフの表示
        @param   qslice  Q中央値、Q始値、Q終値のタプル 
        @retval 無し
        """ 
        # 指定されたQ値におけるヒストグラムを取得
        try:
            self.edata = self.dmap.GetHistY(qslice)
        except PlotException as ex:
            PlotMessage(self.frame, ex)
            
        # Xデータ(bin の境界値)の中央値に変換
        yy = self._ConvertCenterValue(self.edata.bins)
        # 指定範囲における、マスク値を除いた最大値を求める
        ezmin, ezmax = self.GetMax(self.edata, self.ax.get_ylim())

        # 縦グラフ用にマスクデータを変換
        for i in range(len(yy)):
            if self.edata.counts[i] >= MASKVALUE-1.0:
                # マスクデータセット
                yy[i] = MASKVALUE
        # 表示用のマスクデータを作成
        self.ym = ma.masked_where(yy >= MASKVALUE, yy)

        # Q値のテキストを保存
        if self.edata.fp-self.edata.ep == 0.0:
            self.strQ = "%.3f" % self.edata.mp
        else:
            self.strQ = "%.3f (%.3f - %.3f)" % (self.edata.mp,self.edata.fp,self.edata.ep) 
            
        # グラフの描画
        self.DrawEbinGraph((ezmin, ezmax), self.ax.get_ylim(),)

    #########################################        
    def DrawEbinGraph(self, xlim, ylim):
        """
        指定された E値におけるQ bin グラフの表示
        @param   xlim  X軸範囲のタプル
        @param   ylim  Y軸範囲のタプル
        @retval 無し
        """         
        if self.isOnly:
            return ##[inamura 120211]
        # E bin グラフをクリア
        self.axe.clear()
        # E bin グラフ表示
        self.axe.plot(self.edata.counts, self.ym, linewidth=0.6)
        # 座標ラベルの文字を小さくする

        xcl = getp(self.axe, 'xticklabels')
        setp(xcl, size=7)
        ycl = getp(self.axe, 'yticklabels')
        setp(ycl, size=7)   
        # Q値のタイトル文字列を作る
        strGtitle = "X = %s" % self.strQ
        # スケール範囲を設定
        self.axe.set_xlim(xlim)
        self.axe.set_ylim(ylim)

        yy = (ylim[1]-ylim[0])*0.015+ylim[1] 
        # タイトル文字列表示
        ##[inamura 120711]-->
        target = self.DrawDic["MultiMode"]
        #self.axe.text(xlim[0], yy, strGtitle, fontsize = 8)
        self.axe.text(xlim[0], yy, strGtitle, fontname=target["LabelFont"],fontsize = 8)
        # X軸のラベルを表示
        #self.axe.set_xlabel(self.zlabel, fontsize = 9)
        self.axe.set_xlabel(self.zlabel, fontname=target["LabelFont"],fontsize = 9)
        ##<--[inamura 120711]

    #########################################        
    def UpdateCursol(self, graphid):
        """
        十字カーソルとカーソル位置のグラフを表示
        @param   graphid 
                 1: Q bin graph のみ
                 2: Energy bin graph のみ
                 3: 両方のグラフ
        @retval 無し
        """ 
        # パラメータの取得要求
        self.ifi.NotifyEvent(self, "pickparams") 

        # パネルからX 軸スライス条件を取得
        x0, x1, frcX, xbin, xflag, xinteg = self.ifi.GetProperty("xparam")  # [T.Kikuchi 150511]
        if graphid == 2:
            xc = self.xcyc[0]
            frcX = 0

        # パネルからY 軸スライス条件を取得
        y0, y1, frcY, ybin, yflag, yinteg = self.ifi.GetProperty("yparam")  # [T.Kikuchi 150511]
        if graphid == 1:
            frcY = 0
            yc = self.xcyc[1]
        
        # Q bin と E bin 両方のグラフの描画が終了するまで繰り返す
        while True:
            # Q bin グラフの描画は必要か?
            if frcX > 0:
                xr = (x1-x0)/frcX
                xx = x0+xr/2.0
                xc = [xx, x0, x0+xr, xbin, xflag, xinteg]  # [T.Kikuchi 150511]
            # E bin グラフの描画は必要か?    
            if frcY > 0:
                yr = (y1-y0)/frcY
                yy = y0+yr/2.0
                yc = [yy, y0, y0+yr, ybin, yflag, yinteg]  # [T.Kikuchi 150511]

                
            # Q bin　と E bin の両方のグラフ描画か     
            if graphid == 3:
                # Q bin グラフの描画は終了しているか

                if frcX <= 0:
                   # E bin グラフのみの描画指定   
                   graphid = 2
                # E bin グラフの描画は終了しているか   
                elif frcY <= 0:
                   # Q bin グラフのみの描画指定  
                   graphid = 1
                    
            # カーソルとグラフ表示
            self.UpdateGraphs(xc,yc,graphid)

            if frcX:
                x0 = x0+xr
                frcX = frcX - 1
            if frcY:
                y0 = y0+yr
                frcY = frcY - 1
            # 描画終了か
            if frcX <= 0 and frcY <= 0:
                break
            else:
                # 次のグラフを描画する場合、すぐに消えないようにする
                time.sleep(1) 

    #########################################        
    def DrawCursols(self, xc, yc, spanflag):
        """
        十字カーソルとカーソル位置のグラフを表示
        @param   xc　　Q bin グラフ表示条件
        @param   yc　　E bin グラフ表示条件  
        @param   spanflag    
                 True: 領域を半透明の帯で表示
                 False: 領域を破線で表示
        @retval 無し
        """ 
        if xc == None:
            return
        # カーソルをクリア
        self.cursol.clear()

        ##[inamura 110106]-->
        # If Cursor Off mode
        if not self.toolbar.cursorMode:
            self.cursol.set_xlim(self.ax.get_xlim()) 
            self.cursol.set_ylim(self.ax.get_ylim())
            self.cursol.set_xticks([])
            self.cursol.set_yticks([])
            return
        ##<--[inamura 110106]

        # 半透明の帯で表示か
        if spanflag:
            # X軸のスライス範囲を半透明な領域で表示
            self.cursol.axhspan(xc[1], xc[2],facecolor='w', alpha=0.4) 
            # Y軸のスライス範囲を半透明な領域で表示
            self.cursol.axvspan(yc[1], yc[2],facecolor='w', alpha=0.4) 
        else:
            # X軸のスライス範囲を点線で表示
            self.cursol.axhline(y=xc[1], linestyle='--', color='k', linewidth=0.6)
            self.cursol.axhline(y=xc[2], linestyle='--', color='k', linewidth=0.6)
            # Y軸のスライス範囲を点線で表示
            self.cursol.axvline(x=yc[1], linestyle='--', color='k', linewidth=0.6)
            self.cursol.axvline(x=yc[2], linestyle='--', color='k', linewidth=0.6)
            
        # 中心値に十字カーソルを表示
        self.cursol.axhline(y=xc[0], color='k', linewidth=0.6)
        self.cursol.axvline(x=yc[0], color='k', linewidth=0.6)
        
        # カーソルのスケールをカラーマップのスケールに合わせる
        self.cursol.set_xlim(self.ax.get_xlim()) 
        self.cursol.set_ylim(self.ax.get_ylim())
        
        # カーソルのスケール文字を消す
        self.cursol.set_xticks([])
        self.cursol.set_yticks([])

    #########################################        
    def UpdateGraphs(self, xc, yc, graphid, pflag = True):
        """
        十字カーソルとカーソル位置のグラフを表示
        @param   xc　　Q bin グラフ表示条件
        @param   yc　　E bin グラフ表示条件  
        @param   grahid    
                 1: Q bin graph のみ
                 2: Energy bin graph のみ
                 3: 両方のグラフ
        @param   pflag プロッタ送信フラグ
        　　　　　True: プロッタ送信が指定されていれば送信
        　　　　　False: プロッタ送信が指定されていても送信しない
                        (タイトル、ラベル変更時等に使用)          
        @retval 無し
        """ 
        # データ無しか
        if xc == None:
            self.canvas.draw()
            return
        # Q bin graph　表示か
        if graphid == 1 or graphid == 3:
            # Q bin グラフを描画
            self.UpdateQbinGraph(xc)
            xc[0] = self.qdata.mp
            xc[1] = self.qdata.fp
            xc[2] = self.qdata.ep
        # E bin グラフ　表示か
        if graphid == 2 or graphid == 3:
            #  E bin グラフを描画
            self.UpdateEbinGraph(yc)
            yc[0] = self.edata.mp
            yc[1] = self.edata.fp
            yc[2] = self.edata.ep
        # カーソルを再表示
        self.DrawCursols(xc, yc, True)      
        # グラフを再表示              
        self.canvas.draw()
        self.frame.Update()
        
        # Q bin graph　表示か
        if graphid == 1 or graphid == 3:
            # with Plotter Sending が指定されていたなら
            if xc[4] and pflag:
                # プロッタ(1) にデータ追加指定
                self.ifi.NotifyEvent(self, "add", 1) 
        # E bin グラフ　表示か
        if graphid == 2 or graphid == 3:
            # with Plotter Sending が指定されていたなら
            if yc[4] and pflag:
                # プロッタ(2) にデータ追加指定
                self.ifi.NotifyEvent(self, "add", 2) 
                
        # 印刷用に、現在の領域を保存
        self.xcyc = (xc,yc)

    #########################################
    def PlotD2Chart(self):
        """
        2Dチャートを描画する
        @param   無し   
        @retval 無し
        """ 
        ##[inamura 120711]-->
        if self.isOnly:
            target = self.DrawDic["SingleMode"]
        else:
            target = self.DrawDic["MultiMode"]
        ##<--[inamura 120711]

        # 全グラフクリア
        self.fig.clf()
        # グラフのメインタイトルを表示
        #self.fig.text(0.1, 0.97, self.title1, fontsize = 10)
        self.fig.text(0.1, 0.97, self.title1, fontname=target["TitleFont"], fontsize=target["TitleSize"]) ##[inamura 120711]
        # グラフのサブタイトルを表示
        ##[inamura 120211]-->
        if self.isOnly:
            colms = self.title2.split(';')
            colm_offset = float( int(800/len(colms)) )/1000.0
            xxt = 0.15
            for colm in colms:
                yyt = 0.945
                lines = colm.split('\n')
                for line in lines:
                    #self.fig.text( xxt, yyt, line, fontsize=9 )
                    self.fig.text( xxt, yyt, line, fontname=target["TitleFont"], fontsize=target["TitleSize"]) ##[inamura 120711]
                    yyt = yyt-0.025
                xxt = xxt + colm_offset
        else:
            title2 = self.title2.replace(';','')
            lines = title2.split('\n')
            yyt = 0.945
            for line in lines:
                #self.fig.text(0.76, yyt, line, fontsize = 9)
                self.fig.text(0.76, yyt, line, fontname=target["TitleFont"], fontsize=target["TitleSize"]) ##[inamura 120711]
                yyt = yyt - 0.025
        ##<--[inamura 120211]
        
        # SQEマップ用のサブプロットを準備
        self.ax = self.fig.add_subplot(111)
        #self.ax.set_position([0.1, 0.095, 0.63 ,0.62])
        self.ax.set_position(target["MainPosi"]) ##[inamura 120211]
        
        if not self.isOnly:
            # Q bin ヒストグラム用の座標を用意
            self.axq = self.fig.add_axes([0.1, 0.74, 0.63 ,0.19])
            
            # Energy bin ヒストグラム用の座標を用意
            self.axe = self.fig.add_axes([0.790, 0.095, 0.18 ,0.62])

        # データがあれば
        if self.Z != None:
            # カラーバー用の座標軸を準備
            #axc = self.fig.add_axes([0.02, 0.04, 0.28 ,0.02])
            axc = self.fig.add_axes(target["ColorBarPosi"]) ##[inamura 120211]
            # カーソル表示用のサブプロットを準備(透過モード)
            self.cursol = self.fig.add_subplot(111,frameon=False)
            #self.cursol.set_position([0.1, 0.095, 0.63 ,0.62])
            self.cursol.set_position(target["MainPosi"])    ##[inamura 120211]

            # マスクデータの色作成(白色)
            ##[inamura 090924]->
            #print "PlotD2Chart >> self.colormap=",self.colormap
            ##[inamura 110125]-->
            if self.colormap=="default":
                cmap=self.colormap_default
            else:
                cmap=get_cmap(self.colormap)
            ##<--[inamura 110125]
            #cmap = cm.jet
            ##<-[inamura 090924]
            cmap.set_bad('w', 1.0)

            # ログモードか
            if self.logFlag:
                logZm = self.Zm + self.offset
                zmin = self.zMin
                # 表示データの最小値以下か
                #[inamura 100618]-->
                #if zmin < 0.05:
                #    zmin = 0.05
                if zmin < 1.0E-25:
                    zmin = 1.0E-25
                pc = self.ax.pcolormesh(self.X, self.Y, logZm,  cmap=cmap, \
                                    norm=LogNorm(vmin=zmin, vmax=self.zMax))
            # リニアモード
            else:
                pc = self.ax.pcolormesh(self.X, self.Y, self.Zm, cmap=cmap, \
                                        norm=Normalize(vmin=self.zMin, vmax=self.zMax))
                              

            # カラーマップと関連付けて、カラーバーを表示
            #self.fig.colorbar(pc, cax=axc, orientation='horizontal')
            self.fig.colorbar(pc, cax=axc, orientation=target["ColorBarOri"]) ##[inamura 120211]
            # 座標範囲を設定
            self.ax.set_xlim(self.xrange[0], self.xrange[1])
            self.ax.set_ylim(self.yrange[0], self.yrange[1])
            # カラーバーの座標ラベルの文字を小さくする
            xlabels = getp(axc, 'xticklabels')
            setp(xlabels, size=6)
            ylabels = getp(axc, 'yticklabels') ##[inamura 120211]
            setp(ylabels, size=6)
        
        else:
            if not self.isOnly:
                setp( self.axq.get_xticklabels(), visible=False)
                setp( self.axq.get_yticklabels(), visible=False)
                setp( self.axe.get_xticklabels(), visible=False)
                setp( self.axe.get_yticklabels(), visible=False)
            #setp( self.axq.get_xticklabels(), visible=False)
            #setp( self.axq.get_yticklabels(), visible=False)
            #setp( self.axe.get_xticklabels(), visible=False)
            #setp( self.axe.get_yticklabels(), visible=False)
        
        # カラーマップの座標ラベルの文字を小さくする
        #setp(self.ax.get_xticklabels(), size=8)
        #setp(self.ax.get_yticklabels(), size=8)     
        setp(self.ax.get_xticklabels(), size=target["TickLabelSize"])
        setp(self.ax.get_yticklabels(), size=target["TickLabelSize"]) ##[inamura 120211]
        setp(self.ax.get_xticklabels(), fontname=target["ScaleFont"]) ##[inamura 120711]
        setp(self.ax.get_yticklabels(), fontname=target["ScaleFont"]) ##[inamura 120711]
        
        # X軸のラベルを表示
        #self.ax.set_xlabel(self.xlabel, fontsize = 9)
        self.ax.set_xlabel(self.xlabel, fontname=target["LabelFont"], fontsize=target["LabelSize"]) ##[inamura 120711]
        # Y軸のラベルを表示
        self.ax.yaxis.tick_left()
        #self.ax.set_ylabel(self.ylabel, fontsize = 9)
        self.ax.set_ylabel(self.ylabel, fontname=target["LabelFont"], fontsize=target["LabelSize"]) ##[inamura 120711]

    #########################################
    def DrawRectCursol(self, event, downP):
        """
        ドラッグ範囲を半透明のカーソルにて表示
        @param   event マウス移動時のイベント情報 
        @param   downP マウスボタン押下時の位置情報  
        @retval 無し
        """ 
        # カーソルをクリア
        self.cursol.clear()
        ##[inamura 100726]-->
        ## if version of matplotlib > 0.98
        if matplotlib_verNo > 980:
            #self.cursol.autoscale(False)
            self.cursol.set_autoscale_on(False) #[inamura 110129]
        ##<--[inamura 100726]
        # ドラッグ開始点と現在のX位置の大小比較
        if event.xdata > downP[2]:
            x0 = downP[2]
            x1 = event.xdata
        else:
            x0 = event.xdata
            x1 = downP[2]
        # X軸のマウス移動範囲を半透明な領域で表示
        self.cursol.axvspan(x0, x1,facecolor='w', alpha=0.4) 
        
        # ドラッグ開始点と現在のY位置の大小比較
        if event.ydata > downP[3]:
            y0 = downP[3]
            y1 = event.ydata
        else:
            y0 = event.ydata
            y1 = downP[3]
        # Y軸のマウス移動範囲を半透明な領域で表示
        self.cursol.axhspan(y0, y1,facecolor='w', alpha=0.4)
        
        # カーソルのスケール文字を消す
        self.cursol.set_xticks([])
        self.cursol.set_yticks([])
        
        # グラフを再表示              
        self.canvas.draw()      

    ###################################################        
    def OnHome(self, wid, evt, value):
        """
        Home ボタン押下イベント処理関数
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @retval 無し
        """ 
        # カラーマップのX軸に合わせて、Q bin グラフを描画
        qzmin, qzmax = self.GetMax(self.qdata, self.ax.get_xlim())

        self.DrawQbinGraph(self.ax.get_xlim(), (qzmin, qzmax))
        # カラーマップのY軸に合わせて、E bin グラフを描画
        ezmin, ezmax = self.GetMax(self.edata, self.ax.get_ylim())

        self.DrawEbinGraph((ezmin,ezmax), self.ax.get_ylim())
        # グラフを再表示              
        self.canvas.draw()

    ###################################################        
    def OnNotifyChangeData(self, wid, evtType, graphid):
        """
        データ変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  data データオブジェクト
        @param graphid グラフID
         @retval 無し
        """ 
        # データ表示
        self._DispInitialMap()
        self.PlotD2Chart()
       
        # カーソル位置の初期値をテキストボックスに設定
        cx = self.xrange[0]+ (self.xrange[1] - self.xrange[0] ) / 2.0
        # スライス条件指定イベント発行
        self.ifi.NotifyEvent(self, "slicey", (cx, 0.0, 0.0)) 
        cy = self.yrange[0]+ (self.yrange[1] - self.yrange[0] ) / 2.0
        # スライス条件指定イベント発行
        self.ifi.NotifyEvent(self, "slicex", (cy, 0.0, 0.0))
        # カーソルとグラフを表示
        self.UpdateCursol(3)  
   

    ###################################################        
    def OnNotifyShow(self, wid, evtType, graphid):
        """
        グラフ表示要求イベント処理
        @param  wid       イベント発生元のインスタンス
        @param  evtType       イベントタイプ
        @param  graphid 　1: Energy bin graph
　                         2: Q bin graph
        @retval 無し
        """  
    
        self.UpdateCursol(graphid)       

    ###################################################        
    def OnNotifyZScale(self, wid, evtType, zscale):
        """
        Z軸変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  zscale 　スケール範囲タプル
        @retval 無し
        """ 
        # 自動モードであれば
        if zscale[0]:
            # データの最小値と最大値を設定
            self.zMax = self.datMax
            self.zMin = self.datMin

        # マニュアルモードであれば
        else:
            # データの最小値より小さい値は不可
            if zscale[1] < self.datMin - self.offset:
               self.datMin = self.offset
            # 設定値にオフセットを付加
            self.zMin = zscale[1] + self.offset
            self.zMax = zscale[2] + self.offset

        # マップの再描画
        self.PlotD2Chart()    
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)         

    ###################################################             
    def OnNotifyTitle(self, wid, evtType, titles):
        """
        グラフタイトル変更イベント処理
        Set Plotting Param ボタンを選択可とする
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  titles 1行目と2行目の文字列タプル
        @retval 無し
        """ 
        # グラフタイトルを取得
        title1, title2 = titles

        # メインタイトルが変更されたか
        if not title1 == '*':
            self.title1 = title1
        # サブタイトルが変更されたか    
        if not title2 == '*':
            self.title2 = title2
        # グラフを再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False) 

    ###################################################        
    def OnNotifyLabel(self, wid, evtType,labels):
        """
        スケールラベル変更イベント処理
        Set Plotting Param ボタンを選択可とする
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  labels X軸ラベル、Y軸ラベル、Z軸ラベル文字列タプル
        @retval 無し
        """ 
        # 縦軸、横軸のラベルを取得
        xlabel, ylabel, zlabel = labels
        # Xラベルが変更されたか
        if xlabel != '*':
            self.xlabel = xlabel
        # Yラベルが変更されたか   
        if ylabel != '*':
            self.ylabel = ylabel    
        # Zラベルが変更されたか   
        if zlabel != '*':
            self.zlabel = zlabel    
        # グラフを再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)  
             

    ###################################################                        
    def OnNotifyLog(self, wid, evtType, value):
        """
        Log モード変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  value 　Log モードフラグ
        @retval 無し
        """ 
        # ログモード変更
        self.logFlag = value
        # ログモードでは負値は不可
        if value and self.zMin < self.offset:
            self.zMin = self.offset
            
        # マップの再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)            

    ###################################################        
    def OnChangeXY(self, wid, evt, mouseEvent):
        """
        ズーム・パン　イベント処理関数
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  mouseEvent  マウスボタンダウン時とアップ時のイベントのタプル
        @retval 無し
        """ 
        # ズームまたはパン時のドラッグ開始点のイベント取得
        event = mouseEvent[0]
        # Q bin グラフ内か
        if event.y > self.y1dpi:
            # Q bin グラフを描画
            #self.DrawQbinGraph(self.axq.get_xlim(), self.axq.get_ylim())
            if not self.isOnly:
                self.DrawQbinGraph(self.axq.get_xlim(), self.axq.get_ylim()) ##[inamura 120211]
        # E bin グラフ内か
        elif event.x > self.x1dpi:
            # E bin グラフを描画
            #self.DrawEbinGraph(self.axe.get_xlim(), self.axe.get_ylim())
            if not self.isOnly:
                self.DrawEbinGraph(self.axe.get_xlim(), self.axe.get_ylim()) ##[inamura 120211]
        
        # カラーマップ内か
        else:
            # カラーマップのX軸に合わせて、Q bin グラフを描画
            qzmin, qzmax = self.GetMax(self.qdata, self.ax.get_xlim())
            self.DrawQbinGraph(self.ax.get_xlim(), (qzmin, qzmax))
            # カラーマップのY軸に合わせて、E bin グラフを描画
            ezmin, ezmax = self.GetMax(self.edata, self.ax.get_ylim())
            
            self.DrawEbinGraph((ezmin,ezmax), self.ax.get_ylim())
            
        # グラフを再表示              
        self.canvas.draw()

    ##[inamura 090924]->
    ###################################################                        
    def OnNotifyColorMap(self, wid, evtType, value):
        """
        ColorScale変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  value 　Name of Colormap
        @retval 無し
        """
        print("OnNotifyColorMap value=",value)
        # ColorScale変更
        self.colormap = value
            
        # マップの再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)

    ###################################################        
    def OnNotifyXScale2D(self, wid, evtType, xscale):
        """
        プロットデータ変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  scale 　スケール範囲タプル
        @retval 無し
        """ 
        
        # 自動モードか
        if xscale[0]:
            self.autoXFlag = True
        else:
            # 範囲が指定されていたらマニュアル指定
            self.autoXFlag = False
            self.xrange = ( xscale[1], xscale[2] )
            
        # マップの再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)
       
    ###################################################        
    def OnNotifyYScale2D(self, wid, evtType, yscale):
        """
        プロットデータ変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  scale 　スケール範囲タプル
        @retval 無し
        """ 
        
        # 自動モードか
        if yscale[0]:
            self.autoYFlag = True
        else:
            # 範囲が指定されていたらマニュアル指定
            self.autoYFlag = False
            self.yrange = ( yscale[1], yscale[2] )

        # マップの再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)

    ##<-[inamura 090924]
    ##[inamura 101022]-->
    ###################################################                        
    def OnNotifySmooth(self, wid, evtType, value):
        """
        Smoothing モード変更イベント処理
        @param  wid イベント発生元のインスタンス
        @param  evtType イベントタイプ
        @param  value 　smoothing モードフラグ
        @retval 無し
        """
        print("OnNotifySmooth")
        # smoothモード変更
        self.smoothFlag = value[0]
        if self.smoothFlag and self.Z!=None:
            ##[T.Kikuchi 130514]-->
            sw_org = int(abs( value[1] ) )
            Zm=self.Z*self.Zsc
            
            g = self.gauss_kern( sw_org, None)
            tmpZm = signal.convolve( Zm, g, mode='same')
            tmpZsc = signal.convolve( self.Zsc, g, mode='same')
            
            #print "Zm.size/Zm[0].size=",Zm.size/Zm[0].size
            #print "Zm[0].size = ",Zm[0].size
            #print "tmpZm.size/tmpZm[0].size=",tmpZm.size/tmpZm[0].size
            #print "tmpZm[0].size = ",tmpZm[0].size
            #print "tmpZsc.size/tmpZsc[0].size=",tmpZsc.size/tmpZsc[0].size
            #print "tmpZsc[0].size = ",tmpZsc[0].size
            
            ##[T.Kikuchi 130802]-->
            self.Zm = ma.masked_where(tmpZsc <= 0.0, tmpZm/tmpZsc)
            #self.Zm = ma.masked_where(self.Z >= MASKVALUE-1.0, tmpZm/tmpZsc)
            ##<--[T.Kikuchi 130802]
            ##<--[T.Kikuchi 130514]
        else:
            ##[T.Kikuchi 150508]-->
            #self.Zm = ma.masked_where(self.Z >= MASKVALUE-1.0, self.Z)
            self.Zm = ma.masked_where(self.Zsc <= 0.0, self.Z)
            ##<--[T.Kikuchi 150508]
            
        # マップの再描画
        self.PlotD2Chart()
        # カーソルを表示
        self.UpdateGraphs(self.xcyc[0], self.xcyc[1], 3, False)

    ###################################################                        
    def gauss_kern(self, size, sizey=None):
        size = int(size)
        if not sizey:
            sizey = size
        else:
            sizey = int(sizey)
        x, y = mgrid[-size:size+1, -sizey:sizey+1]
        g = exp(-(x**2/float(size) + y**2/float(sizey) ) )
        return g / g.sum()
    
    ##<--[inamura 101022]
    ##[inamura 120417]-->
    ###################################################
    def OnNotifyShowOnly2d(self, wid, evtType, value):
        """
        """
        if value in [True,False]:
            self.isOnly=value
            self.PlotD2Chart()
            self.UpdateCursol(3)
            self.canvas.draw()
            ##[inamura 120710]-->
            xsize,ysize = self.panel.GetSize()
            pos = VerHandlePlotLib().GetSubPlotPos(self.ax)
            self.x1dpi = (pos[0]+pos[2])*xsize
            self.y1dpi = (pos[1]+pos[3])*(ysize-20)
            self.y0dpi = pos[1]*(ysize-20)
            self.x0dpi = pos[0]*xsize
            ##<--[inamura 120710]
    ##<--[inamura 120417]
    ##[inamura 120711]-->
    ###################################################
    def OnNotifySetDrawInfo(self, wid, evtType, params):
        """
        """
        for param in params:
            if self.isOnly:
                self.DrawDic["SingleMode"][param[0]]=param[1]
            else:
                self.DrawDic["MultiMode"][param[0]]=param[1]
        
        self.PlotD2Chart()
        self.UpdateCursol(3)
        self.canvas.draw()
        
#######################################
#  CuiIF
####################################### 
class D2CuiIF(object):
    """
    CUI からのコマンドを解析し、プロッタに対し
    イベントを発行する
    """
    ###################################################    
    def StartPlot(self, plotNum, func,isQuiet=False): ##[inamura 120417]
        """
        コンストラクタ
        @param  plotNum  プロットナンバー
        @param  func   起動した画面のクローズイベント発生時に実行する関数
        @retval 無し
        """  
        self.func = func
        # テンポラリファイルクラスのインスタンス取得
        self.tp = TempFile()
        
        # 2次元プロッタ画面の起動
        self.plot = MapD2Frame(None, None, plotNum,"",isQuiet) ##[inamura 120417]
 
        # プロッタが公開しているインターフェイスクラスのインスタンスを取得
        self.ifi = IFEvtProp(plotNum)
        
        # プロッタフレームクローズイベントのリスナー登録
        self.ifi.AddListner('plotclose', self.OnNotifyClose)
        
        # コマンドとその実行関数登録
        self.functions = {"changedata"  :self.ChangeData, 
                          "add"         : self.ExecInt, 
                          "clear"       : self.ExecInt, 
                          "setlog"      : self.ChangeMode, 
                          "setwith"     : self.SetWithPlot, 
                          "setzscale"   : self.SetScale,  
                          "save"        : self.SaveFile,
                          "slicex"      : self.Slice,
                          "slicey"      : self.Slice, 
                          "slicexrange" : self.SliceRange,
                          "sliceyrange" : self.SliceRange, 
                          "xlabel"      : self.SetXLabel, 
                          "ylabel"      : self.SetYLabel,
                          "zlabel"      : self.SetZLabel, 
                          "maintitle"   : self.SetMain, 
                          "subtitle"    : self.SetSub,
                          "doiconize"   : self.ChangeMode,  
                          "print"       : self.Exec, 
                          "close"       : self.Exec,
                          "showonly2d"  : self.ShowOnly2d }  ##[inamura 120417]
        
    #####################################################
    def OnNotifyClose(self, *arg):
        """
        プロッタクローズイベント対応処理
        @param  wid 　 イベント発生元のインスタンス
        @param  evt 　イベントの種類
        @param  value  無し   
        @retval 無し
        """
        # メッセージの解放 
        MessageFile().CloseDoc()
        # 通信クラスの終了処理コールバック関数を実行
        self.func(*())     
        
    ###################################################    
    def TakeCommand(self, strCmd):
        """
        コマンドを解析し、プロッタ画面にイベントを発行
        @param  strCmd コマンド文字列
        @retval コマンドに対する応答
        """  
        # 文字列を空白区切りで分解
        args = strCmd.split()
        # コマンド実行関数を取得
        try:
            function = self.functions[args[0]]
        except:
            print("Command error!!", args)
            return
        # 関数の実行
        return function(args)
        
    ###################################################    
    def ChangeData(self, args):
        """
        データ変更イベント発行
        @param  args コマンドと引数のタプル
        @retval 無し
        """
        # テンポラリファイルを読んでデシリアライズ
        data = self.tp.ReadTempFile(args[1])
        if data == None:
            # 否定応答を返信
            return "NACK\n"
        # データ変更イベント発行
        self.ifi.NotifyEvent(self, args[0], data[0])
        # タイトル設定イベント発行
        self.ifi.NotifyEvent(self, "title", (data[1], data[2]))
        # ラベル指定ありか
        if len(data) == 6:
            # ラベル設定イベント発行
            self.ifi.NotifyEvent(self, "scalelabel", (data[3], data[4],data[5]))   

                
    ###################################################    
    def ExecInt(self, args):
        """
        引数が整数1個のコマンドイベント発行
        @param  args コマンドと引数のタプル
        @retval 無し
        """
        # プロッタ追加またはプロッタクリアイベント発行
        self.ifi.NotifyEvent(self, args[0], int(args[1]))
        
                
    ###################################################    
    def SetScale(self, args):
        """
        モード変更イベント発行
        @param  args コマンドと引数のタプル
        @retval 無し
        """          
        # 引数無しであれば自動
        if len(args) == 1:
            # スケール変更(自動)イベント発行
            self.ifi.NotifyEvent(self, args[0], (True,))
        else:
            # マニュアルスケール Z軸範囲指定
            self.ifi.NotifyEvent(self, args[0], (False, float(args[1]), float(args[2])))         

    #########################################         
    def SetWithPlot(self, args):
        """
        プロッタ送信モード設定
        @param  args コマンドと引数のタプル
        @retval 無し
        """ 
        # 変更フラグはFalse か    
        if args[1]== "0":
            # モードイベント発行
            self.ifi.NotifyEvent(self, args[0], (False, int(args[2])))
        else:
            self.ifi.NotifyEvent(self, args[0], (True, int(args[2])))
 
    ###################################################    
    def ChangeMode(self, args):
        """
        モード変更イベント発行(Boolean 型引数)
        ログモード変更、オーバレイモード変更、アイコン化
        @param  args コマンドと引数のタプル
        @retval 無し
        """ 
        # 変更フラグはFalse か    
        if args[1]== "0":
            # モードイベント発行
            self.ifi.NotifyEvent(self, args[0], False)
        else:
            self.ifi.NotifyEvent(self, args[0], True)

    ###################################################    
    def Slice(self, args):
        """
        スライス条件指定イベント発行
        @param  args コマンドと引数のタプル
        @retval 無し
        """ 
        # スライス条件指定イベント発行(パネルの書き換え)
        self.ifi.NotifyEvent(self, args[0], (float(args[1]), float(args[2]), float(args[3]))) 
        # グラフ描画要求
        if args[0] == "slicex":
            self.ifi.NotifyEvent(self, "show", 1)
        else:
            self.ifi.NotifyEvent(self, "show", 2)
            
    ###################################################    
    def SliceRange(self, args):
        """
        スライス範囲条件指定イベント発行
        @param  args コマンドと引数のタプル
        @retval 無し
        """ 
        # スライス条件指定イベント発行(パネルの書き換え)
        self.ifi.NotifyEvent(self, args[0], (float(args[1]), float(args[2]),int(args[3]), float(args[4])))
        # グラフ描画要求
        if args[0] == "slicexrange":
            self.ifi.NotifyEvent(self, "show", 1)
        else:
            self.ifi.NotifyEvent(self, "show", 2)
                
    ###################################################    
    def SaveFile(self, args):
        """
        画像保存コマンドを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """ 
        strArg = self._Rebuild(args)         
        # 画像保存イベント発行
        self.ifi.NotifyEvent(self, args[0], strArg) 
        
    ###################################################    
    def _Rebuild(self, args):
        """
        文字列引数を復元(文字列に空白が含まれている場合)
        @param  args コマンドと引数のタプル
        @retval 復元後の文字列
        """ 
        strArg = ""
        # 引数文字列を連結
        for i in range(len(args)-1):
            strArg = strArg + " " + args[i+1]
        # 頭の余分な空白を消す    
        ret = strArg[1:]      
        # 復元後の文字列を返す
        return ret     
        
    ###################################################    
    def SetXLabel(self, args):
        """
        X軸ラベルコマンドを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """  
        strArg = self._Rebuild(args) 
        # スケールラベル設定コマンド送信
        self.ifi.NotifyEvent(self, "scalelabel", (strArg, "*", "*"))   
        
    ###################################################    
    def SetYLabel(self, args):
        """
        Y軸ラベルコマンドを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """  
        strArg = self._Rebuild(args) 
        # スケールラベル設定コマンド送信
        self.ifi.NotifyEvent(self, "scalelabel", ("*", strArg,"*"))    
        
    ###################################################    
    def SetZLabel(self, args):
        """
        Z軸ラベルコマンドを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """  
        strArg = self._Rebuild(args) 
        # スケールラベル設定コマンド送信
        self.ifi.NotifyEvent(self, "scalelabel", ("*", "*", strArg))             
        
    ###################################################    
    def SetMain(self, args):
        """
        メインタイトルを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """  
        strArg = self._Rebuild(args) 
                
        # タイトル設定コマンド送信
        self.ifi.NotifyEvent(self, "title", (strArg, "*"))   
        
    ###################################################    
    def SetSub(self, args):
        """
        サブタイトルを送信
        @param  args コマンドと引数のタプル
        @retval 無し
        """  
        strArg = self._Rebuild(args) 
                
        # タイトル設定コマンド送信
        self.ifi.NotifyEvent(self, "title", ("*", strArg))              
          
    ###################################################    
    def Exec(self, args):
        """
        コマンド実行イベント発行(引数無)
        Close 
        @param  args コマンドと引数のタプル
        @retval 無し
        """          
        # コマンド実行要求イベント発行
        self.ifi.NotifyEvent(self, args[0])   
       
    ###################################################    
    def GetAttr(self, args):
        """
        属性の取得
        @param  args コマンドと引数のタプル
        @retval 無し
        """          
        # 属性取得
        attr = self.ifi.GetProperty(args[1])
     
        # 属性が文字列か
        if args[2]=="0":
            return attr+'\n' 
        # 整数か
        elif args[2]=="1":
            return "%d\n" % attr  
        # 実数か
        elif args[2]=="2":
            return "%.5f\n" % attr
        # ブーリアンか
        elif args[2]=="3":
            if attr== True:
                return "True\n"
            else:
                return "False\n"
        # オブジェクトか
        elif args[2]=="4":
            # 現在の絶対時間(0.01秒単位)と、プロッタ番号よりテンポラリファイル名を作成
            tsec = int(time.time()*100.0)
            tmpf = "attrs%d" % tsec
            # テンポラリファイルの絶対パスを作成(指定されたテンポラリディレクトリの下)
            tmppath = os.path.join(tempfile.gettempdir(), tmpf)
            try:
                # テンポラリファイルを開く
                fw = open(tmppath, "wb")
                # オブジェクトをバイナリモードでシリアライズ
                pickle.dump(attr, fw, True)
            except:
                print("\nError!! Cant create a temp file.")
            else:
                # テンポラリファイルをクローズ
                fw.close()
                # テンポラリファイル名を返信
                return tmppath + '\n'  

    ##[inamura 120417]-->
    ###################################################
    def ShowOnly2d(self,args):
        """
        """
        if args[1]=="True":
            isOnly=True
        else:
            isOnly=False
        print("ShowOnly2d=",isOnly)
        self.ifi.NotifyEvent(self,'showonly2d', isOnly)
        
    ##<--[inamura 120417]

    
if __name__ == '__main__':
    """
    CUI-GUI コミュニケータから、プロセスとして起動されるときに
    実行する処理
    """
        # socket 経由で他のプロセスから起動
    if len(sys.argv) > 1:
        import vis.WxCom as WxCom
        # CUI-GUI コミュニケータのサーバアプリ(wxPython) 起動
        app = WxCom.WxApp() 
        # 2次元プロッタ(D2Plot)の CUI IF のインスタンス取得 
        cuif = D2CuiIF()
        # 起動時引数より、通信ポート番号を取得
        portno = int(sys.argv[1])
        # 起動時引数より、プロッタ番号を取得
        pno = int(sys.argv[3])
        # プロッタ終了フラグが指定されているか
        if len(sys.argv) >= 5: 
            if sys.argv[4] == "True":
                closeFlag = True
            else:
                closeFlag = False
        else:
            # 指定無しの場合は、True とする。
            closeFlag = True
        
        ##[inamrua 120417]-->
        # If Quiet Flag is set.
        isQuiet=False
        if len(sys.argv) >=6:
            if sys.argv[5]=="True":
                isQuiet=True
        # プロッタを起動
        cuif.StartPlot(pno, app.OnGUIClose,isQuiet) 
        ##<--[inamura 120417]
        # CUI との通信スタート(ポート番号と、コマンド処理関数を引数として渡す)
        app.OnStart(portno, closeFlag, cuif.TakeCommand)
        # wxPython のメインループ開始
        app.MainLoop()
    # 以下はデバッグ用スクリプト
    else:
        # 以下は、デバッグ用スクリプト
        app = wx.PySimpleApp()

        h = arange(1.0,10.0 , 1.0)
        v = arange(-1.0,0.1 , 0.1)
        
        numh = len(h)-1
        numv = len(v)-1
        # 斜方グリッド作成
        H, V = meshgrid(h, v)

        aa = 1.0
        for i in range(11):
            for j in range(9):
                H[i,j]= H[i,j]*aa
            aa = aa + 0.1
#            aa = aa - 0.1
        data =ones((numv,numh), float64)

        data[0,3] = 2.0
        data[0,4] = 5.0
        data[0,5] = 3.0
        data[1,1] = 8.0
        data[1,3] = 4.0
        data[1,4] = 10.0
        data[1,5] = 6.0
        data[1,7] = 7.0
        data[2,4] = 1.5
        data[3,3] = 4.0
        data[3,4] = 2.5
        data[3,5] = 3.5
        data[4,5] = 0.0
        data[6,0] = 0.5
        data[7,6] = 9.0
        data[0,2] = MASKVALUE
        data[1,2] = MASKVALUE
        data[2,2] = MASKVALUE
        data[3,2] = MASKVALUE
        data[4,2] = MASKVALUE
        data[5,2] = MASKVALUE
        data[6,2] = MASKVALUE
        data[7,2] = MASKVALUE
        data[8,3] = MASKVALUE
        data[9,4] = MASKVALUE
        """
        h = arange(5.69,5.85 , 0.04)
        v = arange(-10.0,-9.0 , 0.2)
        
        numh = len(h)-1
        numv = len(v)-1
        # 斜方グリッド作成
        H, V = meshgrid(h, v)
        data =zeros((numv,numh), float64)
        
        data[0,0]=0.761
        data[0,1]=0.897
        data[1,0]=1.136
        data[1,1]=1.121
        data[2,0]=0.938
        data[2,1]=1.129
        
        aa = 0.0
        for i in range(5):
            for j in range(4):
                H[i,j]= H[i,j]-aa
                
            aa += 0.005
        """
        err = sqrt(data)
        mapXY = (data, err, H,V, 1)
        MapD2Frame(None, mapXY,1)

        IFEvtProp(1).NotifyEvent(None, "title", ("dummy.spe", "Ei(meV)=20"))
        # スケールラベル設定コマンド送信
        IFEvtProp(1).NotifyEvent(None, "scalelabel", ("Q(1/Angstrom)", "Energy(meV)","Neutrons"))  
        app.MainLoop()
