#include "ShamotoDymPDF.hh"

ShamotoMagFormFact::ShamotoMagFormFact(){
    // Mn2+ magnetic form factor
    A1=0.4220;
    a2=17.684;
    B1=0.5948;
    b2=6.005;
    C1=0.0043;
    c2=-0.609;
    D=-0.0219;
}
ShamotoMagFormFact::~ShamotoMagFormFact(){

}
Double ShamotoMagFormFact::Calc(Double Q){
    Double s=Q/4.0/M_PI;
    Double mgf=A1*exp(-a2*s*s)+B1*exp(-b2*s*s)+C1*exp(-c2*s*s)+D;
    return mgf;
}


ShamotoDymPDF::ShamotoDymPDF(bool useMF){
    _ecm = NULL;
    _sArray = NULL;
    _st = new StringTools();
    _dHW = 0.0;
    _minHW = 0.0;
    _maxHW = 0.0;
    _Cf = 74.84;
    _dR = 0.1;
    _maxR = 40.0;
    _useMF = useMF;
    _mfg = new ShamotoMagFormFact();
}
ShamotoDymPDF::~ShamotoDymPDF(){
    delete _st;
    delete _mfg;
}
void ShamotoDymPDF::SetData( ElementContainerArray* eca ){
    if (_ecm!=NULL) delete _ecm;
    _ecm = new ElementContainerMatrix( eca->PutHeader() );
    _ecm->Add( *eca );
}
void ShamotoDymPDF::SetSlicedData( ElementContainerArray* eca ){
    if (_sArray != NULL) delete _sArray;
    _sArray = eca;
}
void ShamotoDymPDF::SetSliceWidth( Double delta_hw, Double min_hw, Double max_hw ){
    _dHW = fabs( delta_hw );
    _minHW = min_hw;
    _maxHW = max_hw;
    if (_minHW > _maxHW){
        Double tmp = _minHW;
        _minHW = _maxHW;
        _maxHW = tmp;
    }
}
void ShamotoDymPDF::SetMagFormFact(Double _A1, Double _a2, Double _B1, Double _b2, Double _C1, Double _c2, Double _D){
    _mfg->A1 = _A1;
    _mfg->a2 = _a2;
    _mfg->B1 = _B1;
    _mfg->b2 = _b2;
    _mfg->C1 = _C1;
    _mfg->c2 = _c2;
    _mfg->D = _D;
}
bool ShamotoDymPDF::SetRRange( Double dR, Double maxR ){
    if ( (dR<0.0)&&(maxR<0.0) ) return false;
    _dR = dR;
    _maxR = maxR;
    return true;
}
bool ShamotoDymPDF::Slice( ElementContainerMatrix* ecm, ElementContainerArray* eca, Double dHW, Double min_hw, Double max_hw){
    if (min_hw==max_hw){
        vector<Double> hw = ecm->PutPointer(0)->PutPointer(0)->PutX();
        min_hw = hw[0];
        max_hw = hw.back();
    }
    if (min_hw>max_hw){
        Double tmp = min_hw;
        min_hw = max_hw;
        max_hw = tmp;
    }
    vector<Double> r_hw = CalcRangeAsBinCenterZero( min_hw, max_hw , dHW, true );
    min_hw = r_hw[0];
    max_hw = r_hw[1];
    UInt4 num_hw = UInt4( r_hw[2] );

    MlfArraySlicer* AS = new MlfArraySlicer( ecm );
    Double cur_hw = min_hw;
    for (UInt4 i=0; i<num_hw; i++){
        ElementContainer* _ec = new ElementContainer();
        if ( AS->CutAlongX( _ec, cur_hw, cur_hw+dHW, true ) ){
            _ec->AddToHeader(UTSUSEMI_KEY_HW, cur_hw + dHW/2.0 );
            vector<Double> hws;
            hws.push_back( cur_hw );
            hws.push_back( cur_hw+dHW );
            _ec->AddToHeader(UTSUSEMI_KEY_HW+"s", hws );
            eca->Add( *_ec );
        }
        delete _ec;
        cur_hw += dHW;
    }
    delete AS;
    return true;
}
bool ShamotoDymPDF::Execute( bool doSlice ){

    UInt4 j, j0;
    Double Pi, dr, Qmax, SSsum, Smax, Smaxerr;
    Double b, b1, b2;
    Pi = M_PI;

    if (doSlice){
        if (_sArray!=NULL) delete _sArray;
        _sArray = new ElementContainerArray();
        if (!(Slice( _ecm, _sArray, _dHW, _minHW, _maxHW))){
            UtsusemiError(" Execute>> Slice Error");
            return false;
        }
    }
    if (_sArray==NULL){
        UtsusemiError("");
        return false;
    }else{
        for (UInt4 i1=0; i1<(_sArray->PutSize()); i1++){
            ElementContainer* ec = _sArray->PutPointer(i1);
            vector<Double> Qo = ec->PutX();
            vector<Double> I = ec->PutY();
            vector<Double> E = ec->PutE();
            vector<Double> Q;
            UInt4 Size = (UInt4)I.size();
            if ((UInt4)Qo.size()==Size){
                Q.resize(Size);
                copy(Qo.begin(),Qo.end(),Q.begin());
            }else{
                for (UInt4 i2=0; i2<Size; i2++)
                    Q.push_back( (Qo[i2]+Qo[i2+1])/2.0 );
            }

            j=3;
            //dr=0.1;
            dr = _dR;
            j0=0;
            SSsum=0;
            Smax=0;
            Qmax = ( Q[ Size-3 ] + Q[ Size-2 ] + Q[ Size-1 ] )/3.0;
            Smax = ( I[ Size-3 ] + I[ Size-2 ] + I[ Size-1 ] )/3.0;
            Smaxerr = sqrt( Q[ Size-3 ]*Q[ Size-3 ] + Q[ Size-2 ]*Q[ Size-2 ] + Q[ Size-1 ]*Q[ Size-1 ] )/3.0;
            UtsusemiMessage("ShamotoDymPDF::Execute >> N_data="+_st->UInt4ToString( Size ));
            UtsusemiMessage("ShamotoDymPDF::Execute >> Q_max, Smax="+_st->DoubleToString( Qmax )+","+_st->DoubleToString( Smax ) );

            // Least squre fit of S(Q,E) by b*(Q-Qmax)(Q+Qmax)
            b1=0;
            b2=0;
            vector<Double> X(Size,0.0);
            vector<Double> Y(Size,0.0);
            for (UInt4 M=0; M<Size; M++){
                X[M] = (Q[M]-Qmax)*(Q[M]-Qmax);
                Y[M] = I[M]-Smax;
                b1 = b1+X[M]*Y[M];
                b2 = b2+X[M]*X[M];
                //cout<< X[M] << ","<<Y[M]<<endl;
            }
            b = b1/b2;
            //cout << b << endl;

            vector<Double> Q_av(Q.size(),0.0);
            vector<Double> I_av(Size,0.0);
            vector<Double> E_av(E.size(),0.0);
            for (UInt4 M=0; M<Size; M++){
                if (Q[M]==0.0) Q[M] = 0.001;
                Double WF = Qmax*sin(Pi*Q[M]/Qmax)/Pi/Q[M];
                Q_av[M] = Q[M];
                if (_useMF){
                    I_av[M] = _Cf*Q[M]*(I[M]-Smax-b*(Q[M]-Qmax)*(Q[M]-Qmax))*WF/_mfg->Calc(Q[M])/_mfg->Calc(Q[M]);
                    E_av[M] = _Cf*Q[M]*E[M]*WF/_mfg->Calc(Q[M])/_mfg->Calc(Q[M]);
                }else{
                    I_av[M] = _Cf*Q[M]*(I[M]-Smax-b*(Q[M]-Qmax)*(Q[M]-Qmax))*WF;
                    E_av[M] = _Cf*Q[M]*E[M]*WF;
                }
                SSsum = SSsum+I_av[M];
                //if (M==(Size-1)){
                //    cout << Q[M] << "," << I[M] << "," << E[M] << endl;
                //    cout << _Cf << endl;
                //    cout << WF << endl;
                //    cout << _mfg->Calc(Q[M]) << endl;
                //    cout << Q_av[M] << "," << I_av[M] << "," << E_av[M] << endl;
                //}
            }
            UtsusemiMessage("ShamotoDymPDF::Execute >> SUM of S(Q,E)="+_st->DoubleToString( SSsum ));

            // DyPDF G(r,E)/n(E) calculation ***************
            //UInt4 SizeOfG = 400;
            UInt4 SizeOfG = (UInt4)(_maxR/_dR);
            vector<Double> Q_rho(SizeOfG,0.0);
            vector<Double> I_rho(SizeOfG,0.0);
            vector<Double> E_rho(SizeOfG,0.0);
            for (UInt4 N=0; N<SizeOfG; N++){
                Q_rho[N] = (Double)N*dr;
                for (UInt4 M=0;M<(Size-1);M++){
                    I_rho[N]=I_rho[N]+I_av[M]*sin( Q_av[M]*Q_rho[N] )*Q_av[M]*(Q_av[M+1]-Q_av[M]);
                    if (I_av[M] < 0.001){
                        E_rho[N] = E_rho[N]+((E_av[M]/0.1)*(E_av[M]/0.1))*Q_av[M]*Q_av[M]*(Q_av[M+1]-Q_av[M]);
                    }else{
                        E_rho[N] = E_rho[N]+((E_av[M]/I_av[M])*(E_av[M]/I_av[M]))*Q_av[M]*Q_av[M]*(Q_av[M+1]-Q_av[M]);
                    }
                }
                I_rho[N] = 2.0*I_rho[N]/Pi;
                E_rho[N] = sqrt( E_rho[N])*sqrt(2.0)/Pi;
            }
            ec->Add(UTSUSEMI_KEY_MOMENTTRANSFER+"_RHO",Q_rho,UTSUSEMI_KEY_MOMENTTRANSFER_UNIT);
            ec->Add(UTSUSEMI_KEY_INTENSITY+"_RHO",I_rho,UTSUSEMI_KEY_ARB_UNIT);
            ec->Add(UTSUSEMI_KEY_ERROR+"_RHO",E_rho,UTSUSEMI_KEY_ARB_UNIT);
            ec->SetKeys(UTSUSEMI_KEY_MOMENTTRANSFER+"_RHO",UTSUSEMI_KEY_INTENSITY+"_RHO",UTSUSEMI_KEY_ERROR+"_RHO");
         }
    }
    return true;
}
ElementContainerArray ShamotoDymPDF::PutResult( bool isXaxisR ){
    if (!isXaxisR) return *_sArray;

    ElementContainerArray ret;
    vector<Double> vr = _sArray->PutPointer(0)->PutX();
    UInt4 num_hw = _sArray->PutSize();
    ret.Allocate(vr.size());
    for (UInt4 i = 0; i<vr.size(); i++){
        ElementContainer* ec = new ElementContainer();
        ec->AddToHeader( "R", vr[i] );
        ec->AddToHeader( "_UNIT_R", UTSUSEMI_KEY_LAMBDA_UNIT );
        vector<Double> hw(num_hw);
        vector<Double> II(num_hw);
        vector<Double> EE(num_hw);
        for (UInt4 j=0; j<num_hw; j++){
            hw[j] = _sArray->PutPointer(j)->PutHeaderPointer()->PutDouble(UTSUSEMI_KEY_HW);
            ElementContainer* _ec = _sArray->PutPointer(j);
            vector<Double>* ii = _ec->PutP( _ec->PutYKey() );
            vector<Double>* ee = _ec->PutP( _ec->PutEKey() );
            II[j] = ii->at(i);
            EE[j] = ee->at(i);
        }

        ec->Add( UTSUSEMI_KEY_ENERGYTRANSFER, hw, UTSUSEMI_KEY_ENERGY_UNIT); // hw, "EnergyTransfer", "meV"
        ec->Add( UTSUSEMI_KEY_INTENSITY, II, UTSUSEMI_KEY_ARB_UNIT );       // II, "Intensity", "arb. unit"
        ec->Add( UTSUSEMI_KEY_ERROR, EE, UTSUSEMI_KEY_ARB_UNIT );           // II, "Error", "arb. unit"
        ec->SetKeys(UTSUSEMI_KEY_ENERGYTRANSFER, UTSUSEMI_KEY_INTENSITY, UTSUSEMI_KEY_ERROR );
        ret.Set( i, ec );
    }
    return ret;
}
bool ShamotoDymPDF::Test( string _inputfile, string _outputfile, ElementContainerArray* _eca ){
    vector<Double> X;
    vector<Double> I;
    vector<Double> E;
    X.clear();
    I.clear();
    E.clear();

    FILE* fi;
    if ((fi=fopen(_inputfile.c_str(),"r"))==NULL){
        UtsusemiError("ShamotoDymPDF::Test file open error "+_inputfile);
        return false;
    }
    char s[2000];
    if (fgets( s, 2000, fi )==NULL){ // Read Title line to be ignored
        UtsusemiError("ShamotoDymPDF::Test file read error "+_inputfile);
        return false;
    }
    while( fgets( s, 2000, fi )!=NULL ){
        string a_line(s);
        vector<string> v = _st->SplitString( a_line);
        vector<Double> s;
        for (UInt4 i=0; i<v.size(); i++){
            if (v[i]!=""){
                s.push_back( _st->StringToDouble( v[i] ) );
            }
        }
        if (s.size()!=3){
            UtsusemiError("ShamotoDymPDF::Test >>> file format is invalid : line = "+a_line+", size= "+_st->UInt4ToString(s.size()));
            return false;
        }
        X.push_back(s[0]);
        I.push_back(s[1]);
        E.push_back(s[2]);
    }
    fclose(fi);

    if (_sArray!=NULL) delete _sArray;
    _sArray = new ElementContainerArray();
    ElementContainer EC;

    EC.Add(UTSUSEMI_KEY_MOMENTTRANSFER,X,UTSUSEMI_KEY_MOMENTTRANSFER_UNIT);
    EC.Add(UTSUSEMI_KEY_INTENSITY,I,UTSUSEMI_KEY_ARB_UNIT);
    EC.Add(UTSUSEMI_KEY_ERROR,E,UTSUSEMI_KEY_ARB_UNIT);
    EC.SetKeys(UTSUSEMI_KEY_MOMENTTRANSFER,UTSUSEMI_KEY_INTENSITY,UTSUSEMI_KEY_ERROR);
    _sArray->Add(EC);

    Execute(false);
    ElementContainer ec_result = _sArray->Put(0);

    X.clear();
    I.clear();
    E.clear();

    FILE* fo;
    if ((fo=fopen(_outputfile.c_str(),"r"))==NULL){
        UtsusemiError("ShamotoDymPDF::Test file error "+_outputfile);
        return false;
    }
    while( fgets( s, 2000, fo )!=NULL ){
        string a_line(s);
        vector<string> v = _st->SplitString( a_line," ",true );
        vector<Double> v2;
        for (UInt4 i=0; i<v.size(); i++){
            if (v[i]!=""){
                v2.push_back( _st->StringToDouble( v[i] ) );
            }
        }
        if (v2.size()!=3){
            UtsusemiError("ShamotoDymPDF::Test >>> file format is invalid : line = "+a_line+", size= "+_st->UInt4ToString(v2.size()));
            return false;
        }
        X.push_back(v2[0]);
        I.push_back(v2[1]);
        E.push_back(v2[2]);
    }
    fclose(fo);

    ElementContainer ec_comp;
    ec_comp.Add(UTSUSEMI_KEY_MOMENTTRANSFER,X,UTSUSEMI_KEY_MOMENTTRANSFER_UNIT);
    ec_comp.Add(UTSUSEMI_KEY_INTENSITY,I,UTSUSEMI_KEY_ARB_UNIT);
    ec_comp.Add(UTSUSEMI_KEY_ERROR,E,UTSUSEMI_KEY_ARB_UNIT);
    ec_comp.SetKeys(UTSUSEMI_KEY_MOMENTTRANSFER,UTSUSEMI_KEY_INTENSITY,UTSUSEMI_KEY_ERROR);


    _eca->Add(EC);
    _eca->Add(ec_result);
    _eca->Add(ec_comp);

    return true;
}
