#ifndef SHAMOTODYMPDF
#define SHAMOTODYMPDF

#include "StringTools.hh"
#include "MlfArraySlicer.hh"
#include "UtsusemiHeader.hh"

//////////////////////
//  ShamotoDymPDF
/////////////////////

//!
/*!
 *  Magnetic Form factor
 *  s = Q/(4*PI)
 *  A1*exp(-a2*s*s) + B1*exp(-b2*s*s) + C1*exp(-c2*s*s) + D
 *  SetMagFormFact(A1, a2, B1, b2, C1, c2, D)
 *
 *
 *  DPDF = mu.ShamotoDymPDF()
 *  DPDF.SetData(DAT)
 *  DPDF.SetSliceWidth(0.2, 1.0, 12.5)
 *  DPDF.SetCf(74.84)
 *  DPDF.SetMagFormFact(0.4220, 17.684, 0.5948, 6.005, 0.0043, -0.609, -0.0219)
 *  DPDF.SetRRange(0.1, 40.0)
 *  DPDF.Execute()
 *  ECA = DPDF.PutResult(True)
 */

class ShamotoMagFormFact
{
public:
    ShamotoMagFormFact();
    ~ShamotoMagFormFact();
    Double A1, a2, B1, b2, C1, c2, D;
    Double Calc(Double Q);
};

class ShamotoDymPDF
{
private:
    ElementContainerMatrix* _ecm;   /* Given Powder Data */
    ElementContainerArray* _sArray; /* Sliced Data Array */
    StringTools* _st;
    Double _dHW;
    Double _minHW, _maxHW;
    Double _Cf;
    Double _dR, _maxR;
    ShamotoMagFormFact* _mfg;
    bool _useMF;

public:
    ShamotoDymPDF(bool useMF=true);
        //!< Constructor
        /*!<
         */
    ~ShamotoDymPDF();
        //!< Destructor
        /*!<
         */

    void SetData( ElementContainerMatrix* ecm ){ _ecm=ecm; }
        //!< Set data to be calculated.
        /*!<
         *   @param ecm (ElementContainerMatrix)
         *   @retval None
         */
    void SetData( ElementContainerArray* eca );
        //!< Set data to be calculated.
        /*!<
         *   @param eca (ElementContainerArray)
         *   @retval None
         */
    void SetSlicedData( ElementContainerArray* eca );
        //!< Set the empty data container to be stored the sliced data by the Execute method
        /*!<
         *   @param eca (ElementContainerArray)
         *   @retval None
         */

    void SetSliceWidth( Double delta_hw, Double min_hw=0.0, Double max_hw=0.0 );
        //!< Set parameters used for slicing along to energy axis
        /*!< min_hw == max_hw means the use of all energy reange in data.
         *
         *   @param delta_hw (Double) Energy bin width
         *   @param min_hw (Double) Minimun value of enargy range
         *   @param max_hw (Double) Maximun value of enargy range
         *   @retval None
         */
    void SetCf( Double cf ){ _Cf = cf; }
        //!< Set correction factor parameters used for the magnetic form factor calculation
        /*!<
         *   @param cf (Double)
         *   @retval None
         */
    void SetMagFormFact(Double _A1, Double _a2, Double _B1, Double _b2, Double _C1, Double _c2, Double _D);
        //!< Set parameters used for the magnetic form factor calculation
        /*!<  A1*exp(-a2*s*s) + B1*exp(-b2*s*s) + C1*exp(-c2*s*s) + D
         *    where s = Q/(4*PI)
         *
         *   @param _A1 (Double)
         *   @param _a2 (Double)
         *   @param _B1 (Double)
         *   @param _b2 (Double)
         *   @param _C1 (Double)
         *   @param _c2 (Double)
         *   @param _D (Double)
         *   @retval None
         */
    bool Slice( ElementContainerMatrix* ecm, ElementContainerArray* eca, Double delta_hw, Double min_hw=0.0, Double max_hw=0.0);
        //!< Do slice of the original data using the energy range and width given by SetSliceWidth.
        /*!< min_hw == max_hw means the use of all energy reange in data.
         *   G(r, E) is calcualted from this sliced data
         *
         *   @param ecm (ElementContainerMatrix) original data (S(Q,E))
         *   @param eca (ElementContainerArray) sliced data
         *   @param delta_hw (Double) Energy bin width
         *   @param min_hw (Double) Minimum value of enargy range
         *   @param max_hw (Double) Maximum value of enargy range
         *   @retval true succeeded.
         *   @retval false failed.
         */
    bool SetRRange( Double dR, Double maxR );
        //!< Set r range of calculated data G(r, E)
        /*!<
         *   @param dR (Double) bin width of r
         *   @param maxR (Double) Maximum value of r range
         *   @retval true succeeded.
         *   @retval false failed.
         */
    bool Execute( bool doSlice=true);
        //!< Execute the calculation of G(r, E) from the sliced data.
        /*!<
         *   @param doSlice (bool) plot S(Q, E) or not befor G(r, E) calculation.
         *   @retval true succeeded.
         *   @retval false failed.
         */
    void TurnOffMF(bool off=true){ _useMF= !(off); }
        //!< Change the flag to use the magnetic form factor when G(r, E) calculation.
        /*!<
         *   @param off (bool) true means "Not use the magnetic form factor"
         *   @retval None
         */
    ElementContainerArray PutResult( bool isXaxisR = true);
        //!< Put the result G(r, E) or S(Q, E)
        /*!<
         *   @param isXaxisR (bool) true means G(r, E) returns, false means S(Q, E)
         *   @retval None
         */
    ElementContainerArray PutSlicedMap(){ cout<<"############## PUT sArray ################";return *_sArray; }
        //!< Put the sliced data S(Q, E)
        /*!<
         *   @param None
         *   @retval None
         */

    bool Test( string _inputfile, string _outputfile, ElementContainerArray* _eca);
        //!< Test code
        /*!<
         *   @param _inputfile (string) test file path of sample 1d (hw, Intensity, Error) data at const hw
         *   @param _outputfile (string) output file path which is calcutated as G(r, hw)
         *   @retval true succeeded.
         *   @retval false failed.
         */
};

#endif
