#ifndef UTSUSEMINEUNETEVENTDECODERBASE
#define UTSUSEMINEUNETEVENTDECODERBASE

#include "Header.hh"
#include "UtsusemiHeader.hh"
#include "UtsusemiNeutronEventDecoderBase.hh"
#include "WiringInfoEditorNeunet.hh"
#include "DetectorInfoEditorNeunet.hh"
#include "WiringInfoConversionDictionary.hh"
#include "StringTools.hh"

#ifdef MULTH
    #include <omp.h>
#endif

//////////////////////////////////
// UtsusemiNeunetEventDecoderBase
/////////////////////////////////

//! Decode Event Data of NEUNET for Utsusemi 
/*! 
 *
 */


class UtsusemiNeunetEventDecoderBase:
    public UtsusemiNeutronEventDecoderBase
{
private:
    string _MessageTag;  /**<  Message Tag */
    UInt4 _NumOfMulTh;
    StringTools* _st;

protected:

    UChar* _prevT0Event;  /**< for use to keep previous T0 Event */
    void Initialize();
        //!< Initialize some parameters
        /*!<
         *   @return None
         */
    void Clear();
        //!< Clear some parameters
        /*!<
         *   @return None
         */
public:
    UtsusemiNeunetEventDecoderBase();
        //!< Constructor
        /*!<
         */
    ~UtsusemiNeunetEventDecoderBase();
        //!< Destructor
        /*!<
         */

    
    //////////////////////////////////////////////////////////////////////////
    // public instances
    //////////////////////////////////////////////////////////////////////////
    //UtsusemiDetectorInfoReader3HePsd *_detInfo;
    DetectorInfoEditorNeunet *_detInfo;
    WiringInfoEditorNeunet *_wirInfo;
    //UtsusemiTofBinPatternFor3HePsd *_tofBinConv;
    WiringInfoConversionDictionary *_wirDict;
    //////////////////////////////////////////////////////////////////////////
    // public variables
    //////////////////////////////////////////////////////////////////////////
    vector< vector<Double>* > _pixelPositionVect;   /**< pixel positions ( index : outer pixel id ) */
    vector< Double > _pixelSolidAngleVect;          /**< pixel solid angle ( index : outer pixel id ) */
    bool _isSetFrameBoundary;
    bool _isUsedModuleNoInEvent;
    
    //////////////////////////////////////////////////////////////////////////
    // virtual functions from UtsusemiNeutronEventDecorderBase
    //////////////////////////////////////////////////////////////////////////
    virtual Int4 SetParametersFromFiles( std::string wiring_file, std::string detector_file );
        //!< set parameters from files
        /*!<
         *   @param wiring_file (string) Wiring Info xml file
         *   @param detector_file (string) DetectorInfo xml file
         *   @retval 0   no error
         *   @retval -1  Troubles happened
         */
    
    //////////////////////////////////////////////////////////////////////////
    // virtual functions required by UtsusemiEventDataConverterTemplate
    //////////////////////////////////////////////////////////////////////////
    virtual void ClearPrevT0Event( UInt4 ThNum );
        //!< Clear Previous T0 Event
        /*!<
         *   @param ThNum (UInt4) Thread number
         *   @retval None
         */
    virtual UInt4 DecodeEventData( UInt4 daqId, UInt4 moduleNo, const UChar *data, UInt4 *pixel_id, double *tof, vector<Double>* Clock, UInt4 ThNum );
        //!< decode Event data
        /*!< This decodes any one event data and return type of event.
         *   If a given event is neutron, this sets pixel_id and tof and return 1 (_isNeutronEvent)
         *   If T0 event, this event is stored and return 2 (_isT0Event)
         *   If clock event, return 3
         *   If any other event, return 0
         *   @param daqId (UInt4) daq ID
         *   @param moduleNo (UInt4) module number
         *   @param data  (UChar) one event data
         *   @param pixel_id (UInt4) Pixel ID calculated by given event
         *   @param tof (double) TOF calculated by given event
         *   @param Clock (vector<Double>)
         *   @param ThNum (UInt4) Thread number
         *   @retval _isNeutronEvent if neutron event
         *   @retval _isT0Event if T0 event
         *   @retval _isClockEvent if clock event
         *   @retval 0 if any other event
         */
    virtual UInt4 ConvertPixelIdToInnerId( UInt4 pixel_id );
        //!< convert outer PixelID to inner( memory allocated ) ID
        /*!< This is used for conversion from outer PixelID to inner id for allocated vectors
         *   
         *   @param pixel_id (UInt4) pixel ID
         *   @return inner ID (UInt4)
         */
    virtual UInt4 ConvertPixelIdToDetId( UInt4 pixel_id );
        //!< convert outer PixelID to outer DetID
        /*!< This is used to get det Id from outer PixelID to 
         *   
         *   @param pixel_id (UInt4) pixel ID
         *   @return detID (UInt4)
         */
    virtual vector<UInt4> PutPixelIdList( bool isInner );
        //!< return vector of existed id from conversion table
        /*!< Conversion table( PixelID <-> inner ID ) is made by WiringFile.
         *   If isInner is true, this puts list of existed inner ID
         *   if false, list of existed pixel ID
         *   @param isInner (bool) parameter to toggle type of return values.
         *   @return list of Id (vector<UInt4>)
         */
    virtual vector<Double> PutHistBin( UInt4 outerPixelId );    
        //!< put binning calculated by Efix
        /*!< For direct convertion from tof to phisical values, 
         *   calculated binning(tof) is required to construct histograms.
         *   
         *   @param  outerPixelId (UInt4) pixel id 
         *   @return binning vector
         */
    UInt4 PutNumOfInnerPixels();
        //!< put the number of inner pixels
        /*!<
         *   @return the number of pixels (UInt4)
         */
    vector<Double> PutPixelPosition( UInt4 outerPixelId );
        //!< put pixel position from given outerPixelId
        /*!< 
         *   @param  outerPixelId (UInt4) pixel id 
         *   @return vector of pixel position
         */
    virtual pair<string,string> PutXLabel( Int4 convtype );
    virtual pair<string,string> PutYLabel( Int4 convtype );
    virtual pair<string,string> PutELabel( Int4 convtype );
    virtual void SetMaskOnError( UInt4 _outerPixelId, vector<Double> &_bin, vector<Double> &_err );
    //////////////////////////////////////////////////////////////////////////    
    // public functions
    //////////////////////////////////////////////////////////////////////////
    UInt4 PutMaxOfPixelId();
        //!< put maximum number of pixel id
        /*!<
         *   @return maximum number (UInt4)
         */
    virtual void CalcPixelPosition();
        //!< calculate the position of each pixel
        /*!< This calculates the position of pixel from stored information.
         *   Potisions are stored to _pixelPositionVect.
         *   @return None
         */
    Int4 PutTofBinType( UInt4 outerPixelId );
        //!< put Tof Bin Type of given outerPixelId
        /*!< Tof Bin Type comes from Wiring File
         *
         *   @param  outerPixelId (UInt4) pixel id 
         *   @return tof bin type
         */
    virtual vector<Double> PutXaxis();
        //!< put x axis values
        /*!< When TofBinType is for direct conversion from event data to energy,
         *   this returns calculated x-axis values.
         *
         *   @param  None
         *   @return vector for values on calculated x-axis
         */
    bool isReverseAxis( Int4 convtype );
    bool isValidConvType( Int4 convtype );
    //void SetEfix( vector< Double > efix );
    //void SetEfix( Double efix );
        //!< set fixed Energy value for direct or indirect geometry
        /*!< fixed Energy values are required to calculate tof array 
         *   for direct unit convertion from tof
         *   If single Double efix is given, this value is used to calculate unit conversion
         *   for direct geometry analysis. The vector<Double> efix is used for inverted one.
         *   
         *   @param  efix (vector<Double> or Double) each fixed Energy ( index of vector is PixelId )
         *   @return None
         */
    //void SetTimeFocParams();
        //!< imports Time focusing parameters to TofBinPattern from DetInfo
        /*!< 
         *   @param None
         *   @return None
         */
    bool SetTofOriginShift( UInt4 cType );
    bool SetTofOriginShift( UInt4 cType, vector<Double> params );
        //!< Sets type and calculation parameter for Tof origin shift 
        /*!< This shift is defined by the lamda-depend function given in codes
         *
         *   @param cType  (UInt4) pattern Id for function
         *   @param params (vector<Double>) parameters of cType function
         *   @retval true  succeeded
         *   @retval false failed
         */
    vector<Double> PutLambda( UInt4 pixel_id );
        //!< Puts lambda axis from data container of pixel_id
        /*!< 
         *   @param pixel_id  (UInt4) pixel id
         *   @return vector<Double> lambda axis
         */
    vector<double>* PutTimeDependBackGroundInfo( UInt4 _outerPixelId ); //[inamura 160809]
        //!< Puts tof range of time depend background for each pixel_id
        /*!< 
         *   @param pixel_id  (UInt4) pixel id
         *   @return vector<Double>* Tof range of Time Depend Back Ground
         */
    bool PutDetIdPixNoFromEvent(UInt4 daqId, UInt4 moduleNo, const UChar *data, UInt4 *detId, UInt4 *pixNo, UInt4 ThNum);
    UInt4 PutNumOfPixelForDet(UInt4 _detId=0);
    Double PutL1();
    Double PutTypicalL2();
    Double PutTypicalDS();
    vector<Double> PutTypicalPixelSize(UInt4 _detId=0);

    void SetSimpleMode( bool _isSimple = false );
};

#endif
