#ifndef UTSUSEMISQECALC
#define UTSUSEMISQECALC

#include "UtsusemiHeader.hh"
#include "CppToPython.hh"
#include "StringTools.hh"
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include "HistogramBinToPoint.hh"
#include "UtsusemiUnitConverter.hh"
#include <time.h>
#include "UtsusemiSqeCalcXtalParams.hh"

//////////////////////
//  UtsusemiSqeCalc
/////////////////////

//! Class for supporting functions to treat Q value
/*! 
 *  This includes some methods to replace functions in VisualCont2 (VisualContCommon.py)
 *   
 * 
 */


class UtsusemiSqeCalc :
  public ManipulatorBase< ElementContainerMatrix >
{
private:
    void _Initialize();
        //!< Initializes some parameter 
        /*!< 
         *   @param None
         *   @return None
         */
    vector<gsl_vector*> _MakeEmpty3DMatrix();
    void _Delete3DMatrix( vector<gsl_vector*> *A );
    bool _Calc3DMatrixProduct( vector<gsl_vector*> *A, vector<gsl_vector*> *B, vector<gsl_vector*> *C );
    bool _Calc3DReversedMatrix( vector<gsl_vector*> *A, vector<gsl_vector*> *C );
    bool _CalcRotateUVWMatrix( UInt4 rot_axis, Double angle, vector<gsl_vector*> *A );
    void _CalcReciprocalLatticeMatrix( vector<Double> latticeConst, vector<gsl_vector*> *M );
    void _CalcUVWMatrix( vector<Double> Uvec, vector<Double> Vvec, vector<gsl_vector*> *M, vector<gsl_vector*> *N );
    void _CalcReciproLattice2( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec, vector<Double> rotSteps, vector< gsl_vector* > *ret );
    
    void _CalcCross3d( gsl_vector* v1, gsl_vector* v2, gsl_vector* ret );
        //!< Calculates the cross between two vectors of gsl_vector.
        /*!< Before calling this, returned gsl_vector must be created.
         *   @param v1    first vector 
         *   @param v2    second vector
         *   @param ret   result
         *   @return None
         */
    void _CalcReciproLattice( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec, Double phi, vector< gsl_vector* > *ret );
        //!< Calculates reciprocal lattice from parameters
        /*!< 
         *   @param latticeConst  lattice constants [a,b,c,alpha,beta,gamma]
         *   @param Uvec          U vector (parallel to incident beam)
         *   @param Vvec          V vector
         *   @param phi           angle around W axis 
         *   @param ret           result
         *   @return None
         */

    void _Make4DVect( gsl_vector* vec, vector< gsl_vector*> latticeVec, gsl_vector* ret );
        //!< Makes 4 dimension vector on Q-E space from 3-D vector on Q-space
        /*!< 
         *   @param vec           target vector on 3-D vector on Q-space
         *   @param latticeVec    Reciprocal vector infor (returned from _CalcReciproLattice)
         *   @param ret           result 
         *   @return None
         */
    StringTools *stool;
    vector< vector<Double> > datArray, errArray; /**< Vectors of sliced data*/
    vector<Double> XArray, YArray;               /**< Vectors of XY axes for sliced data*/
    Double MASKVALUE;                            /**< value for masking*/
    bool isDEBUG;                                /**< flag for DEBUG mode*/
    string MessageTag;
    UtsusemiUnitConverter* UCC;
    bool isDirectGeometry;
    UInt4 _NumOfMulTh;
    UtsusemiSqeCalcXtalParams* _XP;
    bool isSetXtalParams;
    bool _ConvPyListToDoubleVect( PyObject *ax, vector<double> *ret, UInt4 s );
    
public:
    UtsusemiSqeCalc();
        //!< Constructor
        /*!<
         */
    UtsusemiSqeCalc( bool isDebug );
        //!< Constructor
        /*!<
         *   @param isDebug   when debug mode, set true
         */
    UtsusemiSqeCalc( string instType );
        //!< Constructor
        /*!<
         *   @param instType   Direct:direct geometry, Inverted:inverted geometry
         */
    ~UtsusemiSqeCalc();
        //!< Destructor
        /*!<
         */
    
    void SetInstGeometry( string instType="Direct" );
        //!< Sets the instrument geometry
        /*!<
         *   @param instType   Direct:direct geometry, Inverted:inverted geometry
         */
    
    vector<Double> CalcReciproLattice( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec, Double phi);
        //!< Calculates reciprocal lattice from parameters
        /*!< 
         *   @param latticeConst   Lattice Constants a,b,c,alpha,beta,gamma
         *   @param Uvec           U vector
         *   @param Vvec           V vector
         *   @param phi            angle around W axis 
         *   @return Reciprocal Lattice vector
         */
    
    vector<Double> Make4DVect( vector<Double> vaxis, vector<Double> lc );
        //!< Makes 4 dimension vector on Q-E space from 3-D vector on Q-space
        /*!< 
         *   @param vaxis         view axes
         *   @param lc            Reciprocal vector infor (returned from CalcReciproLattice)
         *   @return None
         */

    vector<Double> Projection( vector<Double> vaxis);
        //!< Calculates projction of ElementContainerMatrix
        /*!< 
         *   @param vaxis          view axes
         *   @return Reciprocal Lattice vector
         */
    vector<Double> Projection_QxQyQz( vector<Double> vaxis);
        //!< Calculates projction of ElementContainerMatrix
        /*!< 
         *   @param vaxis          view axes
         *   @return Reciprocal Lattice vector
         */
    Int4 Slice( vector<Double> Ax1range,vector<Double> Ax2range,vector<Double> Ax3range,vector<Double> Ax4range, 
                vector<string> AxesType, vector<Double> Folding, bool isAverageMode=true );
        //!< Slice
        /*!< 
         *   @param Ax1range    first view axis vector
         *   @param Ax2range    second view axis vector
         *   @param Ax3range    third view axis vector
         *   @param Ax4range    forth view axis vector
         *   @param AxesType    set how to treat each axis. can use only "X","Y" or "T".
         *   @param Folding     [0]to[3] are for along-axis-folding 0..off, 1..on, [4]to[6] are parameters for diagonal folding, [4] is type 0..no, 1..1st-3rd quadrant, 2..2nd-4th quadrant, [5]and[6] are axes-id to be used.
         *   @param isAverageMode   
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    vector<Double> PutXArray(){return XArray;}
        //!< Puts XArray (X-axis values) calculated at Slice method.
        /*!< 
         *   @return  < XArray >
         */
    vector<Double> PutYArray(){return YArray;}
        //!< Puts YArray (Y-axis values) calculated at Slice method.
        /*!< 
         *   @return  < YArray >
         */
    vector<Double> PutDArray();
        //!< Puts DArray (Intensity values) calculated at Slice method.
        /*!< 
         *   @return  < DArray >
         */
    vector<Double> PutEArray();
        //!< Puts EArray (Error values) calculated at Slice method.
        /*!< 
         *   @return  < EArray >
         */
    vector<Double> GetAxLimit();
        //!< returns Q1,Q2,Q3,hw ranges
        /*!< 
         *   @return  < Q1 min, Q1 max, Q2 min, Q2 max, Q3 min, Q3 max, Q4 min, Q4 max >
         */
    PyObject* PutQRange(){ return __gCppToPython.VectorDoubleToList( GetAxLimit() );}
        //!< Puts axes ranges of D4Mat
        /*!< 
         *   @param None
         *   @return python list object [ax1_min, ax1_max, ax2_min, ... , ax4_max]
         */
    Int4 ConvertToD4Mat(string filepath, vector< Double> ax1,vector< Double> ax2,
                        vector< Double> ax3,vector< Double> ax4, vector<Double> foldings, bool isAverageMode=true, bool useBinCorr=true);
        //!< Convert and save data using format which D4MatSlicer can load.
        /*!< 
         *   @param filepath    Path/to/file
         *   @param ax1         first view axis information < min, max, step >
         *   @param ax2         second view axis information < min, max, step >
         *   @param ax3         third view axis information < min, max, step >
         *   @param ax4         forth view axis information < min, max, step >
         *   @param foldings    folding 0..off, 1..on
         *   @param isAverageMode  true: average calculation of intensity, false: summation
         *   @param useBinCorr true: do bin width correction
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    Int4 ConvertToD4Mat(string filepath, bool isAverageMode=true, bool useBinCorr=true);
        //!< Convert and save data using format which D4MatSlicer can load.
        /*!< 
         *   @param filepath    Path/to/file
         *   @param isAverageMode  true: average calculation of intensity, false: summation
         *   @param useBinCorr true: do bin width correction
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    void Test();
    vector<Double> _LastConv3DMatrixElements;
    vector<Double> QConvFromReciprocalToRealSpace( vector<Double> Qvec, vector<Double> LC, vector<Double> Uv, vector<Double> Vv, vector<Double> RS, vector<Double> VA );
        //!< Convert list of Qx,Qy,Qz in Qvec to real space using sample information
        /*!< 
         *   @param Qvec  list of Qx,Qy,Qz : Qx0,Qy0,Qz0,Qx1,Qy1,Qz1,Qx2,...,Qxn,Qyn,Qzn
         *   @param LC    Latice constants
         *   @param Uv    U vector
         *   @param Vv    V vector
         *   @param RS    Rotation info
         *   @param VA    View axes info
         *   @return list of x,y,z in real space, its size is same as Qvec
         */
    vector<Double> MakeProjectionMatrix( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec, vector<Double> rotateSteps, vector<Double> viewAxes );
        //!< Makes the 4d Matrix for projection from (Qa,Qb,Qc,hw) -> (Vx,Vy,Vz,Vx)
        /*!<    Vx = Qz*A[0]  + Qx*A[1]  + Qy*A[2]  + hw*A[3];
         *      Vy = Qz*A[4]  + Qx*A[5]  + Qy*A[6]  + hw*A[7];
         *      Vz = Qz*A[8]  + Qx*A[9]  + Qy*A[10] + hw*A[11];
         *      Vw = Qz*A[12] + Qx*A[13] + Qy*A[14] + hw*A[15];

         *   @param laticeConst (vector<double>) a,b,c,alpha,beta,gamma
         *   @param Uvec (vector<double>) U vector
         *   @param Vvec (vector<double>) V vector
         *   @param rotateSteps (vector<double>) axis1, rot1[, axis2, rot2[, axis3, rot3] ] 
         *                                       axisX is the number of rotation axis ("X":0,"Y":1,"Z":2) and the rotation value
         *   @param viewAxes (vector<Double>) View Axes
         *   @return vector<Double> Matrix A with size of 16
         */
    bool SetSlicedElementContainerArray( ElementContainerArray* _eca, string _bin_key="X", string _xval_key="XVAL", string _xrange_key=UTSUSEMI_KEY_HEAD_XRANGE );

    bool LoadXtalParam( string file );
    bool SaveXtalParam( string file );
    bool SetLatticeConstants( PyObject* LC );
    bool SetUVvector( PyObject* UV, PyObject* VV );
    bool SetRotationSteps( string steps );
    bool Projection( PyObject* arg_ax1, PyObject* arg_ax2, PyObject* arg_ax3, PyObject* arg_ax4 );
    bool Projection();
    bool Slice( PyObject* Ax1, PyObject* Ax2, PyObject* Ax3, PyObject* Ax4, PyObject* DiagFolding, bool isAverageMode=true );
    bool Slice(bool isAverageMode=true);
    static const Int4 ROTATE_AXIS_X;
    static const Int4 ROTATE_AXIS_Y;
    static const Int4 ROTATE_AXIS_Z;
    void SetDebugMode( bool _isDebugMode=true ){ isDEBUG=_isDebugMode; }
};
#endif
