#ifndef UTSUSEMISQECALC2
#define UTSUSEMISQECALC2

#include "UtsusemiHeader.hh"
#include "CppToPython.hh"
#include "StringTools.hh"
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_linalg.h>
#include "HistogramBinToPoint.hh"
#include "UtsusemiUnitConverter.hh"
#include <time.h>
#include "UtsusemiSqeCalcXtalParams.hh"
#include <omp.h>

//////////////////////
//  UtsusemiSqeCalc2
/////////////////////

//! Class for supporting functions to treat Q value
/*! 
 *  This includes some methods to replace functions in VisualCont2 (VisualContCommon.py)
 *   
 * 
 */


class UtsusemiSqeCalc2 :
  public ManipulatorBase< ElementContainerMatrix >
{
private:
    void _Initialize();
        //!< Initializes some parameter 
        /*!< 
         *   @param None
         *   @return None
         */
    gsl_matrix *L;  // Matrix of a* b* c*
    gsl_matrix *U;  // Matrix of U and V vectors
    gsl_matrix *UBI;  // Inverse of UBmatrix
    gsl_matrix *R;  // Matrix of rotation
    gsl_matrix *V;  // Matrix of view axes
    gsl_matrix *A;  // A = [V] [L] [U] [R]

    void _CalcReciprocalLatticeMatrix( vector<Double> latticeConst, gsl_matrix* M );
    void _CalcUVWMatrix( vector<Double> Uvec, vector<Double> Vvec, gsl_matrix* M, gsl_matrix* N );
    bool _CalcRotateMatrix( UInt4 rot_axis, Double angle, gsl_matrix* M);
    bool _MakeUBIMatrix( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec );
    bool _MakeUBIMatrix( vector<Double> UBmat );
    bool _MakeProjectionMatrix( vector<Double> rotateSteps, vector<Double> viewAxes );

    void _CalcCrossProduct( gsl_vector* v1, gsl_vector* v2, gsl_vector* ret );
        //!< Calculates the cross between two vectors of gsl_vector.
        /*!< Before calling this, returned gsl_vector must be created.
         *   @param v1    first vector 
         *   @param v2    second vector
         *   @param ret   result
         *   @return None
         */
    void _SetBasisVectToMatrix( gsl_vector* v1, gsl_vector* v2, gsl_vector* v3, gsl_matrix* M );
        //!< Creates Matrix to change Basis Vectors
        /*!< |v1[0] v2[0] v3[0]|
         *   |v1[1] v2[1] v3[1]|
         *   |v1[2] v2[2] v3[2]|
         *   @param v1 (gsl_vector*)
         *   @param v2 (gsl_vector*)
         *   @param v3 (gsl_vector*)
         *   @param M  (gsl_matrix*)
         *   @retval None
         */
    void _SetBasisVectToMatrix( vector<Double> va, gsl_matrix* M );
        //!< Creates Matrix to change Basis Vectors
        /*!< |va[0] va[3] va[6]|
         *   |va[1] va[4] va[7]|
         *   |va[2] va[5] va[8]|
         *   @param va (vector<Double>)
         *   @param M  (gsl_matrix*)
         *   @retval None
         */
    bool _CalcInverseMatrix( gsl_matrix *Min, gsl_matrix *Mout );
    vector<Double> _ConvMatrixToVect( gsl_matrix *Min );

    StringTools *stool;
    vector< vector<Double> > datArray, errArray; /**< Vectors of sliced data*/
    vector<Double> XArray, YArray;               /**< Vectors of XY axes for sliced data*/
    Double MASKVALUE;                            /**< value for masking*/
    bool isDEBUG;                                /**< flag for DEBUG mode*/
    string MessageTag;
    UtsusemiUnitConverter* UCC;
    bool isDirectGeometry;
    UInt4 _NumOfMulTh;
    UtsusemiSqeCalcXtalParams* _XP;
    bool isSetXtalParams;
    bool _ConvPyListToDoubleVect( PyObject *ax, vector<double> *ret, UInt4 s );
    
public:
    UtsusemiSqeCalc2();
        //!< Constructor
        /*!<
         */
    UtsusemiSqeCalc2( bool isDebug );
        //!< Constructor
        /*!<
         *   @param isDebug   when debug mode, set true
         */
    UtsusemiSqeCalc2( string instType );
        //!< Constructor
        /*!<
         *   @param instType   Direct:direct geometry, Inverted:inverted geometry
         */
    ~UtsusemiSqeCalc2();
        //!< Destructor
        /*!<
         */

    vector<Double> MakeProjectionMatrix( vector<Double> latticeConst, vector<Double> Uvec, vector<Double> Vvec, vector<Double> rotateSteps, vector<Double> viewAxes );
    vector<Double> MakeProjectionMatrix( vector<Double> UBmat, vector<Double> rotateSteps, vector<Double> viewAxes );
        //!< Makes the 4d Matrix for projection from (Qa,Qb,Qc,hw) -> (Vx,Vy,Vz,Vx)
        /*!<    Vx = Qx*A[0]  + Qy*A[1]  + Qz*A[2]  + hw*A[3];
         *      Vy = Qx*A[4]  + Qy*A[5]  + Qz*A[6]  + hw*A[7];
         *      Vz = Qx*A[8]  + Qy*A[9]  + Qz*A[10] + hw*A[11];
         *      Vw = Qx*A[12] + Qy*A[13] + Qz*A[14] + hw*A[15];

         *   @param laticeConst (vector<double>) a,b,c,alpha,beta,gamma
         *   @param Uvec (vector<double>) U vector
         *   @param Vvec (vector<double>) V vector
         *   @param rotateSteps (vector<double>) axis1, rot1[, axis2, rot2[, axis3, rot3] ] 
         *                                       axisX is the number of rotation axis ("X":0,"Y":1,"Z":2) and the rotation value
         *   @param viewAxes (vector<Double>) View Axes
         *   @return vector<Double> Matrix A with size of 16
         */
    void SetInstGeometry( string instType="Direct" );
        //!< Sets the instrument geometry
        /*!<
         *   @param instType   Direct:direct geometry, Inverted:inverted geometry
         */
    vector<Double> Projection( vector<Double> vaxis);
        //!< Calculates projction of ElementContainerMatrix
        /*!< 
         *   @param vaxis          view axes
         *   @return Reciprocal Lattice vector
         */
    vector<Double> Projection_QxQyQz( vector<Double> vaxis);
        //!< Calculates projction of ElementContainerMatrix
        /*!< 
         *   @param vaxis          view axes
         *   @return Reciprocal Lattice vector
         */
    Int4 Slice( vector<Double> Ax1range,vector<Double> Ax2range,vector<Double> Ax3range,vector<Double> Ax4range, 
                vector<string> AxesType, vector<Double> Folding, bool isAverageMode=true );
        //!< Slice
        /*!< 
         *   @param Ax1range    first view axis vector
         *   @param Ax2range    second view axis vector
         *   @param Ax3range    third view axis vector
         *   @param Ax4range    forth view axis vector
         *   @param AxesType    set how to treat each axis. can use only "X","Y" or "T".
         *   @param Folding     [0]to[3] are for along-axis-folding 0..off, 1..on, [4]to[6] are parameters for diagonal folding, [4] is type 0..no, 1..1st-3rd quadrant, 2..2nd-4th quadrant, [5]and[6] are axes-id to be used.
         *   @param isAverageMode   
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    vector<Double> PutXArray(){return XArray;}
        //!< Puts XArray (X-axis values) calculated at Slice method.
        /*!< 
         *   @return  < XArray >
         */
    vector<Double> PutYArray(){return YArray;}
        //!< Puts YArray (Y-axis values) calculated at Slice method.
        /*!< 
         *   @return  < YArray >
         */
    vector<Double> PutDArray();
        //!< Puts DArray (Intensity values) calculated at Slice method.
        /*!< 
         *   @return  < DArray >
         */
    vector<Double> PutEArray();
        //!< Puts EArray (Error values) calculated at Slice method.
        /*!< 
         *   @return  < EArray >
         */
    vector<Double> GetAxLimit();
        //!< returns Q1,Q2,Q3,hw ranges
        /*!< 
         *   @return  < Q1 min, Q1 max, Q2 min, Q2 max, Q3 min, Q3 max, Q4 min, Q4 max >
         */
    PyObject* PutQRange(){ return __gCppToPython.VectorDoubleToList( GetAxLimit() );}
        //!< Puts axes ranges of D4Mat
        /*!< 
         *   @param None
         *   @return python list object [ax1_min, ax1_max, ax2_min, ... , ax4_max]
         */
    Int4 ConvertToD4Mat(string filepath, vector< Double> ax1,vector< Double> ax2,
                        vector< Double> ax3,vector< Double> ax4, vector<Double> foldings, bool isAverageMode=true );
        //!< Convert and save data using format which D4MatSlicer can load.
        /*!< 
         *   @param filepath    Path/to/file
         *   @param ax1         first view axis information < min, max, step >
         *   @param ax2         second view axis information < min, max, step >
         *   @param ax3         third view axis information < min, max, step >
         *   @param ax4         forth view axis information < min, max, step >
         *   @param foldings    folding 0..off, 1..on
         *   @param isAverageMode  true: average calculation of intensity, false: summation
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    Int4 ConvertToD4Mat(string filepath, bool isAverageMode=true);
        //!< Convert and save data using format which D4MatSlicer can load.
        /*!< 
         *   @param filepath    Path/to/file
         *   @param isAverageMode  true: average calculation of intensity, false: summation
         *   @retval -1  Something wrong
         *   @retval  0  No trouble
         */
    void Test();
    vector<Double> _LastConv3DMatrixElements;
    vector<Double> QConvFromReciprocalToRealSpace( vector<Double> Qvec, vector<Double> LC, vector<Double> Uv, vector<Double> Vv, vector<Double> RS, vector<Double> VA );
        //!< Convert list of Qx,Qy,Qz in Qvec to real space using sample information
        /*!< 
         *   @param Qvec  list of Qx,Qy,Qz : Qx0,Qy0,Qz0,Qx1,Qy1,Qz1,Qx2,...,Qxn,Qyn,Qzn
         *   @param LC    Latice constants
         *   @param Uv    U vector
         *   @param Vv    V vector
         *   @param RS    Rotation info
         *   @param VA    View axes info
         *   @return list of x,y,z in real space, its size is same as Qvec
         */
    bool SetSlicedElementContainerArray( ElementContainerArray* _eca, string _bin_key="X", string _xval_key="XVAL", string _bin_unit="", string _xval_unit="", string _xrange_key=UTSUSEMI_KEY_HEAD_XRANGE );

    bool LoadXtalParam( string file );
    bool SaveXtalParam( string file );
    bool SetLatticeConstants( PyObject* LC );
    bool SetUVvector( PyObject* UV, PyObject* VV );
    bool SetUBMatrix( PyObject* UBM );
    bool SetRotationSteps( string steps );
    bool Projection( PyObject* arg_ax1, PyObject* arg_ax2, PyObject* arg_ax3, PyObject* arg_ax4 );
    bool Projection();
    bool Slice( PyObject* Ax1, PyObject* Ax2, PyObject* Ax3, PyObject* Ax4, PyObject* DiagFolding, bool isAverageMode=true );
    bool Slice(bool isAverageMode=true);
    static const Int4 ROTATE_AXIS_X;
    static const Int4 ROTATE_AXIS_Y;
    static const Int4 ROTATE_AXIS_Z;

    vector<Double> ConvToUBMatrix( PyObject* LC, PyObject* UV, PyObject* VV, bool flagRound=true );
    vector<Double> ConvToUVvector( PyObject* LC, PyObject* UB, bool flagRound=true );
    vector<Double> InverseMatrix( vector<Double> a );
    void TEST_InverseMatrix( vector<Double> a );
    void SetDebugMode( bool _isDebugMode=true ){ isDEBUG=_isDebugMode; }
    vector<Double> PutInnerMatrix( string M);
        //!< Puts the inner matrix for LatticeConst , U and V, Rotation, View axes and all
        /*!<    argument string :
         *      "L"  Matrix of a* b* c*
         *      "U"  Matrix of U and V vectors
         *      "R"  Matrix of rotation
         *      "V"  Matrix of view axes
         *      "A"  A = [V] [L] [U] [R]
         *   @param M (string) "L","U","R","V" and "A"
         *   @return vector of matrix.
         */
};
#endif
