#ifndef WIRINGINFOCONVERSIONDICTIONARY
#define WIRINGINFOCONVERSIONDICTIONARY

#include <algorithm>
#include "Map.hh"
#include "UtsusemiHeader.hh"
#include "UtsusemiUnitConverter.hh"
#include "UtsusemiTofOriginCorrection.hh"
#include "WiringInfoDataBase.hh"

////////////////////////////////////////
// WiringInfoConversionTemplate
////////////////////////////////////////

//! Template for methods to convert from TOF 
/*! 
 *  
 *
 */
class WiringInfoConversionTemplate
{
private:
    UtsusemiUnitConverter* _UUC;
    
public:
    WiringInfoConversionTemplate();
        //!< Constructor
        /*!<
         */
    WiringInfoConversionTemplate( const WiringInfoConversionTemplate& obj);
        //!< Copy Constructor
        /*!<
         */
    ~WiringInfoConversionTemplate();
        //!< Destructor
        /*!<
         */
    Int4 ConvType;
    string KeyCode;
    vector<UInt4> NumOfParams;
    string Memo;
    bool isReversed;
    bool isRequiredPixelPosi;
    bool (*CheckParamsFunc)(vector<Double>, UtsusemiUnitConverter* );
    vector<Double> (*PutTofFunc)(vector<Double>*, vector<Double>*, vector<Double>*, vector<Double>*, UtsusemiUnitConverter* );
    vector<Double> (*PutXaxisFunc)(vector<Double>,  Double );
    vector<Double> (*PutLambdaFunc)(vector<Double>*, vector<Double>*, vector<Double>*, vector<Double>*, UtsusemiUnitConverter* );
    bool CheckParams( vector<Double> params );
    vector<Double> PutTofBin(vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams );
    vector<Double> PutXaxisVect(vector<Double> params, Double tof_offset=0.0);
    vector<Double> PutLambdaVec(vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams );
    vector< pair< string,string > > LabelsOfEC;

};

////////////////////////////////////////
// WiringInfoConversionDictionary
////////////////////////////////////////

//! Methods for unit conversion on histogram creation
/*! 
 *  - puts X-axis bin for direct conversion from event to histogram
 *  - puts TOF bin for histogram according to given conversion
 *  - puts lambda values for each X axis
 *  tt = WiringInfoConversionDictionary()
 *  tt.SetParams( conv_type, params )
 *  x=tt.PutXaxis()
 *  tof_bin = tt.PutTofBin( pixel_id )
 */

class WiringInfoConversionDictionary
{

private:
    UtsusemiTofOriginCorrection* _UTO;
    
protected:
    string _KeyCode;
    vector<Double> _Xaxis;
    vector<Double> _ConvParams;
    string _MessageTag;
    Double _L1;
    vector<Double> _SamplePosition;
    vector< vector<Double>* > _pixelPositionVect;
    vector< vector<Double> > _timeFocParam;
    vector< WiringInfoConversionTemplate* > Dict;
    void Initialize();
    void SetDict();
    void _Clear();
    WiringInfoConversionTemplate* _Search( string keycode );
    WiringInfoConversionTemplate* _Search( Int4 convtype );
    vector<Double> _PutTofBinOrLambda( UInt4 _pixel_id, bool isTof );

    bool isSetTimeDependBackGroundRegion; //[inamura 160709]

public:
    WiringInfoConversionDictionary();
        //!< Constructor
        /*!<
         */
    ~WiringInfoConversionDictionary();
        //!< Destructor
        /*!<
         */
    bool isValidConvType( Int4 convtype );
    Int4 PutConvType( string keycode );
        //!< Puts ConvType Number from key code
        /*!< 
         *   @param keycode (string) key code for a type of direct conversion
         *   @return ConvType (Int4)
         */
    string PutKeyCode( Int4 convtype );
        //!< Puts Key code from ConvType Number
        /*!< 
         *   @param convtype (Int4) conversion type number
         *   @return KeyCode (string)
         */
    bool CheckParams( Int4 convtype, vector<Double> params );
    vector<UInt4> PutNumOfParams( string keycode );
    vector<UInt4> PutNumOfParams( Int4 convtype );
        //!< Puts the number of parameters for given conv type.
        /*!<     PutNumOfParams( string keycode );
         *       PutNumOfParams( Int4 convtype );
         *   @param keycode  (string) 
         *   @param convtyep (Int4)
         *   @return the number of parameters (vector<UInt4>)
         */
    string PutMemo( string keycode );
        //!< Puts Memo for given ConvType
        /*!< 
         *   @param keycode (string) key code for a type of direct conversion
         *   @return Memo (string)
         */
    bool isReverseXaxis( string keycode );
    bool isReverseXaxis( Int4 convtype );
        //!< Returns whether converted tof is required to reverse
        /*!<     isReverseXaxis( string keycode );
         *       isReverseXaxis( Int4 convtype );
         *   @param keycode  (string) 
         *   @param convtyep (Int4)
         *   @return  (bool)
         */
    bool SetParams( string keycode, vector<Double> *params );
    bool SetParams( Int4 convtype, vector<Double> *params );
        //!< Sets conv type and its parameters
        /*!<     SetParams( string keycode );
         *       SetParams( Int4 convtype );
         *   @param keycode  (string) 
         *   @param convtyep (Int4)
         *   @param params   (vector<Double>*)
         *   @retval true  : succeed setting
         *   @retval false : fail setting
         */
    vector<Double> PutParams( Int4 convtype );
        //!< Puts parametes for given conv type
        /*!<     PutParams( Int4 convtype );
         *   @param convtyep (Int4)
         *   @return  params (vector<Double>*)
         */
    vector<Double> PutXaxis( Double tof_offset=0.0 );
        //!< Puts X axis for given conv type
        /*!<     PutXaxis();
         *   @param tof_offset (Double)
         *   @return  X axis (vector<Double>)
         */
    vector<Double> PutTofBin( UInt4 pixelId );
    vector<Double> PutTofBin( UInt4 pixelId, Int4 convtype, vector<Double> *params );
        //!< Puts Tof Binning for given conv type
        /*!<  (1)   PutTofBin( UInt4 pixelId )
         *    (2)   PutTofBin( UInt4 pixelId, Int4 convtype, vector<Double> *params )
         *   Both return TOF binning of given pixel ID.
         *   (1) requires SetParams(...) before.
         *   (2) includes SetParams(...), so you must give convtype and params.
         *   @param pixelId   (UInt4)
         *   @param convtype  (Int4)
         *   @param params    (vector<Double>*)
         *   @return  TOF binning (vector<Double>)
         */
    vector<Double> PutLambda( UInt4 pixelId );
        //!< Puts Lambda values for given pixel ID and its X axis values
        /*!<     PutLambda( UInt4 pixelId );
         *   @param pixelId (UInt4)
         *   @return  Lambda values (vector<Double>)
         */
    void SetPixelPosition( vector< vector<Double>* > pixelPositionVect );
    void SetTimeFocusingParam( vector< vector<Double> >* timeFocusParam ); 
    void SetL1( Double L1 );
    void SetSamplePosition( Double px, Double py, Double pz );
    pair<string,string> PutXLabel( Int4 convtype=-1 );
    pair<string,string> PutYLabel( Int4 convtype=-1 );
    pair<string,string> PutELabel( Int4 convtype=-1 );
    
    bool SetTofShift( UInt4 cType );
    bool SetTofShift( UInt4 cType, vector<Double> params );
    
    void SetTimeDependBackGroundRegion( Double tof0=0., Double tof1=0. ); //[inamura 160809]
    vector< vector<Double>* >* TimeDependBackGroundRegion;           //[inamura 160809]
    bool SetFrameBoundaryInfoList( UInt4 frameNo, Double boundary, string unit );
    vector< pair<UInt4,Double>* > FrameBoundaryInfoList; /** first: calculated frameNo, second: calculated boundary tof [inamura 170410]  */
    
    static bool CheckParamsType001( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType001( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType001( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType001( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType002( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType002( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType002( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType003( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType003( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType003( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static vector<Double> PutTofBinConvType012Type013( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType012Type013( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType020( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType020( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType020( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType020( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType021( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType021( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType021( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType021( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType022( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType022( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType022( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType022( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType023( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType023( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType023( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType023( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType024( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType024( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType024( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType024( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType025( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType025( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType025( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType025( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType027( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType027( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType027( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType027( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );

    static bool CheckParamsType028( vector<Double> params, UtsusemiUnitConverter* uuc );
    static vector<Double> PutXaxisConvType028( vector<Double> params, Double tof_offset);
    static vector<Double> PutTofBinConvType028( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
    static vector<Double> PutLambdaConvType028( vector<Double>* xaxis, vector<Double>* params, vector<Double>* pv, vector<Double>* oparams, UtsusemiUnitConverter* uuc );
};
#endif
