#ifndef BOOSTXMLPARSER
#define BOOSTXMLPARSER

/* Header.hh should be included on the top to avoid environments
 * specific issues (see Header.hh for the detail).
 */
#include "Header.hh"

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <boost/foreach.hpp>
#include <boost/version.hpp>

#include "Map.hh"
#include "StringTools.hh"

//////////////////////////////////
// BoostXmlParser
/////////////////////////////////

//! Function for read XML file using boost::property_tree
/*! This gives a simple way to access to elements 
 *  by using path-to-element text.
 *  Example for path format
 *  @code
 *  <node1>
 *      <node2 i="1">
 *           <node3>Target</node3>
 *      </node2>
 *      <node2 i="2">...</node2>
 *  </node1>
 *  @endcode
 *  When you want to touch "Target",
 *  the path is "node1/node2,i=1/node3"
 *  or use XPath format ( poor functions )
 *  like "node1/node2[@i='1']/node3"
 */

class BoostXmlParser
{
private:
    string _MessageTag;
    string _CurrentKey;
    string _MakeNewKey( string path="" );
    StringTools* _st;

protected:
    Map< boost::property_tree::ptree* >* _TreeMap;
    void Initialize();
    boost::property_tree::ptree& _getNode( boost::property_tree::ptree& pIn, string path, bool& status, bool makeNew );
        //!< Picks up node indicated by path.
        /*!< 
         *   _getNode( boost::property_tree::ptree& pIn, string path, bool& status, bool makeNew )
         *   @param pIn      (ptree)  ptree of node
         *   @param path     (string) path to element to be removed
         *   @param status   (bool)   status will be set in this method
         *   @param makeNew  (bool)   whether make new node or not when no node indicated by given path
         *   @return ptree 
         */
    bool _eraseNode( boost::property_tree::ptree& pIn, string path, string attr="" );
        //!< erases node or attribute directed by given path
        /*!< 
         *   _eraseNode( boost::property_tree::ptree& pIn, string path, string attr="" )
         *   @param pIn    (ptree)  ptree of node
         *   @param path   (string) path to element to be removed
         *   @param attr   (string) attribute name if you want to delete an attribute
         *   @retval true  :succeeded
         *   @retval false :failed
         */
    string _trimPath( string path );
        //!< Trims given path
        /*!< 
         *   1) removes spaces, 2) converts XPath format to our one, 
         *   3) trimes slash at head and tail of path.
         *   _trimPath( string path )
         *   @param path  (string) path
         *   @reurn trimed string
         */
    bool _isQuietMode; // no message when xml path is not found.
    bool _isLoadXmlString;
    bool _isDebugMode;

public:
    BoostXmlParser();
        //!< Constructor
        /*!<
         */
    BoostXmlParser( string xmlfile );
        //!< Constructor
        /*!<
         *    BoostXmlParser( sting xmlfile )
         *
         *    @param xmlfile (string) path to xml file
         */
    BoostXmlParser( string key, string xmlfile );
        //!< Constructor
        /*!<
         *    BoostXmlParser( string key, string xmlfile )
         *
         *    @param key     (string) Key for xml info
         *    @param xmlfile (string) path to xml file
         */
    ~BoostXmlParser();
        //!< Destructor
        /*!<
         */
    bool CreateNewTree( string key="" );
        //!< Creates new property tree indexed by key.
        /*!< 
         *   CreateNewTree( [string key] )
         *
         *   @param key (string) key for xml info
         *   @retval true   succeeded
         *   @retval false  failed
         */
    bool AddElement( string path, string cont );
    bool AddElement( string key, string path, string cont );
    bool AddElement( string path, vector<string> attr_v, vector<string> val_v);
    bool AddElement( string key, string path, vector<string> attr_v, vector<string> val_v);
        //!< Adds elements and attributes to a tree
        /*!< 
         *   AddToTree( [string key,] string path, string cont );
         *   AddToTree( [string key,] string path, vector<string> attr_v, vector<string> val_v );
         *
         *   @param key   (string) key for xml info
         *   @param path  (string) path to elements and attributes to be added
         *   @param cont  (string) contents to be added
         *   @param att_v (vector<string>) attribute names list
         *   @param val_v (vector<string>) attribute values list
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool RemoveElement( string path, string attr );
    bool RemoveElement( string key, string path, string attr );
        //!< Removes elements or attribute
        /*!< 
         *   RemoveTree( string key, string path, string attr )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to element including attribute to be removed
         *   @param attr (string) attribute name. if empty, delete tree of given path
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool Save( string path );
    bool Save( string key, string path );
        //!< Saves tree to xml file.
        /*!< 
         *   Save( string key, string path )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to file
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool Load( string arg );
        //!< Load tree from xml file.
        /*!< 
         *   Load( string key )
         *
         *   @param key  (string) key for xml info
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool Load( string key, string arg );
        //!< Load tree from xml file.
        /*!< 
         *   Load( string key, string path )
         *
         *   @param key  (string) key for xml info
         *   @param arg (string) path/to/file or xml string
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool LoadFromString( string xmlstring );
        //!< Load tree from string including xml.
        /*!< 
         *   LoadFromString( string xmlstring )
         *
         *   @param key  (string) key for xml info
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool LoadFromString( string key, string xmlstring );
        //!< Load tree from string including xml.
        /*!< 
         *   LoadFromString( string key, string xmlstring )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) string of xml description 
         *   @retval true  succeeded
         *   @retval false failed
         */
    bool isLoadXmlString(){ return _isLoadXmlString; }
        //!< Checks whether XML information is given as string or not (file)
        /*!< 
         *   isLoadXmlString()
         *
         *   @retval true  XML information is given as string
         *   @retval false XML information is given by XML file reading
         */
    string OutToString( string key, string path, bool withIndent );
        //!< Output element tree included by given element path as string of xml.
        /*!< 
         *   OutToString( string key, string path, bool withIndent )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to xml element
         *   @param withIndent (bool) whether output string is formatted by indent
         *   @return xml strings
         */
    string OutToString( string path, bool withIndent );
        //!< Output element tree included by given element path as string of xml.
        /*!< 
         *   OutToString( string path, bool withIndent )
         *
         *   @param path (string) path to xml element
         *   @param withIndent (bool) whether output string is formatted by indent
         *   @return xml strings
         */
    string OutToString( bool withIndent );
        //!< Output element tree included by given element path as string of xml.
        /*!< 
         *   OutToString( bool withIndent )
         *
         *   @param withIndent (bool) whether output string is formatted by indent
         *   @return xml strings
         */
    string OutAllToString( string key, bool withIndent=false ){ return OutToString( key, "", withIndent ); }
        //!< Output all element tree as string of xml.
        /*!< 
         *   OutAllToString( string key, bool withIndent=false )
         *
         *   @param key  (string) key for xml info
         *   @param withIndent (bool) whether output string is formatted by indent
         *   @return xml strings
         */
    string OutAllToString( bool withIndent=false ){ return OutToString( "", withIndent ); }
        //!< Output all element tree as string of xml.
        /*!< 
         *   OutAllToString( bool withIndent=false )
         *
         *   @param withIndent (bool) whether output string is formatted by indent
         *   @return xml strings
         */

    string PutContent( string path );
    string PutContent( string path, string attr);
    string PutContent( string key, string path, string attr);
    string putTextContent( string key, string path, string attr="");
        //!< Puts contents or attribute directed by a given path
        /*!< putTextContent is used for old code to use MiniXmlReader
         *   PutContent( [string key,] string path, string attr )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return string value
         */
    string PutContent( boost::property_tree::ptree* curNode, string path, string attr );
        //!< Puts contents or attribute directed by a given path under current node
        /*!< 
         *   PutContent( boost::property_tree::ptree* curNode, string path, string attr )
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return string value
         */
    vector<string> PutAttValList( string path, string attr );
    vector<string> PutAttValList( string key, string path, string attr );
        //!< Puts all attribute-values included in a given path
        /*!< 
         *   PutAttValList( [string key,] string path, string attr )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return (vector<string>) list of found attributes
         */
    vector<string> PutAttValList( boost::property_tree::ptree* curNode, string path, string attr );
        //!< Puts all attribute-values included in a given path under current node
        /*!< 
         *   PutAttValList( boost::property_tree::ptree* curNode, string path, string attr )
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return (vector<string>) list of found attributes
         */
    vector<string> PutElemContList( string path, string elename );
    vector<string> PutElemContList( string key, string path, string elename );
        //!< Puts all contents of the element included in a given path
        /*!< 
         *   PutAttValList( [string key,] string path, string attr )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to element
         *   @param elename (string) included element name you want to get
         *   @return (vector<string>) list of found contents
         */
    vector<string> PutElemContList( boost::property_tree::ptree* curNode, string path, string elename );
        //!< Puts all contents of the element included in a given path
        /*!< 
         *   PutElemContList( boost::property_tree::ptree* curNode, string path, string attr )
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path (string) path to element
         *   @param elename (string) included element name you want to get
         *   @return (vector<string>) list of found contents
         */
    UInt4 PutNumOfElements( string path );
    UInt4 PutNumOfElements( string key, string path );
        //!< Puts the number of elements included in a given path
        /*!< 
         *   PutNumOfElements( [string key,] string path )
         *
         *   @param key  (string) key for xml info
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return (UInt4) number of elements
         */
    UInt4 PutNumOfElements( boost::property_tree::ptree* curNode, string path );
        //!< Puts the number of elements included in a given path from current node
        /*!< 
         *   PutNumOfElements( boost::property_tree::ptree* curNode, string path )
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path (string) path to element
         *   @param attr (string) attribute name you want to get
         *   @return (UInt4) number of elements
         */
    bool hasPath( string path );
    bool hasPath( string key, string path );
        //!< checks the existence of given path-to-element 
        /*!< 
         *   bool hasPath( [string key,] string path )
         *
         *   @param key  (string) key name for loaded XML info
         *   @param path (string) path-to-element
         *   @retval true   given path exists
         *   @retval false  given path does not exist
         */
    bool hasPath( boost::property_tree::ptree* curNode, string path );
        //!< checks the existence of given path-to-element
        /*!< 
         *   bool hasPath( boost::property_tree::ptree* curNode, string path )
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path (string) path-to-element
         *   @retval true   given path exists
         *   @retval false  given path does not exist
         */
    vector<string> PutKeys();
        //!< Puts key list
        /*!< 
         *   @param None
         *   @return string vector of key list
         */
    bool hasKey( string key );
        //!< Checks whether xml with given key is loaded or not
        /*!< 
         *   @param key (string) key name
         *   @retval true  loaded xml with given key 
         *   @retval false not loaded xml with given key 
         */
    UInt4 PutSize();
        //!< Puts the number of added xml info
        /*!< 
         *   @param None
         *   @return UInt4 value
         */
    void Clear( string key="", bool isForced=false );
        //!< Deletes property tree by a given key
        /*!< 
         *   Clear( string key="", bool isForced=false )
         *
         *   @param key  (string) key for xml info
         *   @param isForced (bool) no warning message if key is not found
         *   @return None
         */
    bool ChangeCurrent( string key );
        //!< Changes current tree by a given key
        /*!< 
         *   ChangeCurrent( string key )
         *
         *   @param key  (string) key for xml info
         *   @retval true  : succeeded
         *   @retval false : failed
         */
    string PutCurrentKey(){ return _CurrentKey; }
        //!< Puts the key of current tree
        /*!< 
         *   PutCurrentKey()
         *
         *   @return key (string)
         */
    bool CopyTree( string org_key, string new_key );
    void SetQuiet( bool isquiet=true ){ _isQuietMode=isquiet; }
    boost::property_tree::ptree* PutNode( string path, string attr );
        //!< Takes out the node given path and attributes
        /*!< 
         *   PutNode( string path, string attr );
         *
         *   @param path  (string) path to elements and attribute you want to get
         *   @param attr  (string) attribute name you want to get
         *   @return node (boost::property_tree::ptree)
         */
    boost::property_tree::ptree* PutNode( string key, string path, string attr );
        //!< Takes out the node given path and attributes
        /*!< 
         *   PutNode( string key, string path, string attr );
         *
         *   @param key   (string) key name for loaded XML info
         *   @param path  (string) path to elements and attribute you want to get
         *   @param attr  (string) attribute name you want to get
         *   @return node (boost::property_tree::ptree)
         */
    boost::property_tree::ptree* PutNode( boost::property_tree::ptree* curNode, string path, string attr );
        //!< Takes out the node given path and attributes
        /*!< 
         *   PutNode( boost::property_tree::ptree* curNode, string path, string attr );
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path    (string) path to elements and attribute you want to get under current node
         *   @param attr    (string) attribute name you want to get
         *   @return node   (boost::property_tree::ptree)
         */
    vector<boost::property_tree::ptree*> PutFoundChildNodes( boost::property_tree::ptree* curNode, string path);
        //!< Takes out all nodes found by current node and given path
        /*!< 
         *   PutFoundChildNodes( boost::property_tree::ptree* curNode, string path);
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path    (string) path to elements and attribute you want to get under current node
         *   @return node   (boost::property_tree::ptree)
         */
    vector<string> PutChildrenElemNames( string path );
        //!< Put list of elements names included in given path
        /*!< 
         *   PutChildrenElemNames( string path );
         *
         *   @param path    (string) path to elements and attribute you want to get under current node
         *   @return vector<string>  List of names
         */
    vector<string> PutChildrenElemNames( string key, string path );
        //!< Put list of elements names included in given path
        /*!< 
         *   PutChildrenElemNames( string key, string path );
         *
         *   @param key   (string) key name for loaded XML info
         *   @param path    (string) path to elements and attribute you want to get under current node
         *   @return vector<string>  List of names
         */
    vector<string> PutChildrenElemNames( boost::property_tree::ptree* curNode, string path );
        //!< Put list of elements names included in given path
        /*!< 
         *   PutChildrenElemNames( boost::property_tree::ptree* curNode, string path );
         *
         *   @param curNode (boost::property_tree::ptree) current node
         *   @param path    (string) path to elements and attribute you want to get under current node
         *   @return vector<string>  List of names
         */
    void SetDebugMode( bool isDebugMode=true );
        //!< Turn on or set debug mode to show behind messages about data treatments.
        /*!< Argument isDebugMode affects on QuietMode.
         *
         *   PutChildrenElemNames( string path );
         *
         *   @param isDebugMode (bool) 
         *   @retval None
         */
};
#endif
