/*
$Id: ElementContainer.cc 2374 2012-04-19 01:57:28Z jisuzuki $
*/

#include <functional>
#include <numeric>
#include "ElementContainer.hh"

#include "SplitString.hh"
#include "DoubleBinArrange.hh"
#include "StringTools.hh"

/*
 * index of methods
 *
 *
 * 1. constructor, operator=, destructor
 * 2. private function used in operators
 * 3. operators, +=, -=, *=, /=
 * 4. removed.
 * 5. ReBin and Binning; rebinning
 * 6. SetKeys and Dumpkey
 * 7. series of Add() method
 * 8. InputHeader, InputUnitHeader
 * 9. Copy, AddBlankVecvot, Remove, Replace, AddValue, SetValue
 * 10. series of Dump, DumpValue
 * 11. StringToInt, IntToString, DoubleToString (removed)
 * 12. series of Put()
 * 13. others
 * 14. sum, integral, average, min, max
 */


/**********************************************
 *** 0. global functions (not class methods)***
 **********************************************/

///////////////////////////////////////////
bool 
Compare( const vector<Double> &v1, const vector<Double> &v2 )
{
  UInt4 size = (UInt4)(v1.size());

  if( size ==0 || v2.size() == 0 ){
    cout << "Compare(vector,vector) (in ElementContainer.cc)" << endl;
    cout << "The size of input vector is zero."      << endl;
    cout << "I cannot compare the vectors."          << endl;
    return false;
  }

  if( size != v2.size() ){
    return false;
  }

  Double Y = 0.0;
  Double X = 0.0;
  
  for( UInt4 i=0; i<size; i++ ){
    Y = Y + v1[i];
    X = X + v2[i];
  }
  Y = Y / (Double)size;
  X = X / (Double)size;

  Double sx2 = 0.0;
  Double sy2 = 0.0;
  Double sxy = 0.0;

  for( UInt4 i=0; i<size; i++ ){
    sx2 = sx2 + (v2[i]-X)*(v2[i]-X);
    sy2 = sy2 + (v1[i]-Y)*(v1[i]-Y);
    sxy = sxy + ( v2[i] - X) * ( v1[i] - Y );
  }

  Double slope = sxy / sx2;
  Double intercept = Y - slope * X;
  Double r2 = sxy * sxy / ( sx2 * sy2 );

  if( slope <= 0.99 )       return false;
  if( slope >= 1.01 )       return false;
  if( intercept <= -0.001 ) return false;
  if( intercept >=  0.001 ) return false;
  if( r2    <= 0.999 )      return false;

  return true;
}
///////////////////////////////////////////

/** makes new string for the unit string
 **
 * @param  L unit of left object
 * @param  ope operator
 * @param  R unit of right object
 * @return new string according to the following rule.
 *       same  not same
 *   +    L      L+R
 *   -    L      L-R
 *   *   L*L     L*R
 *   /   None    L/R
 *  When L or R contains + or -, it is enclosed in parentheses.
 *  When L or R is None, "None" is returned.
 *  When the operator is illegal, "IllegalOperator" is returned.
 */
string ReArrangeUnit( const string &L, const string &ope, const string &R ) {
  //return "(" + L + ope + R + ")";  // [TI 2015.09.30] Change the rule
  if (L == "None" or R == "None"){
    return "None";
  }else{
    ostringstream oss;
    if (L == R){
      if (ope == "+" or ope == "-"){
        return L;
      } else if (ope == "*"){
        if (L.find("+") == string::npos and L.find("-") == string::npos){
          oss << L << ope << L;
        } else {
          oss << "(" << L << ")*(" << L << ")";
        }
      } else if (ope == "/"){
        oss << "None";
      } else {
        cout << "Illegal operator" << endl;
        return "IllegalOperator";
      }
    } else {
      if (ope == "+" or ope == "-"){
        oss << L << ope << R;
      } else if (ope == "*" or ope == "/"){
        if (L.find("+") == string::npos and L.find("-") == string::npos){
          oss << L << ope;
        } else {
          oss << "(" << L << ")" << ope;
        }
        if (R.find("+") == string::npos and R.find("-") == string::npos){
          oss << R;
        } else {
          oss << "(" << R << ")";
        }
      } else {
        cout << "Illegal operator" << endl;
        return "IllegalOperator";
      }
    }
    return oss.str();
  }    
}

/**********************************************
 *** 1. constructor, operator=, destructor  ***
 **********************************************/

////////////////////////////////////////////////////////
ElementContainer::
ElementContainer() :
  Xkey("None"), Ykey("None"), Ekey("None"), safesum(false)
{
  header     = new HeaderBase();
  UnitHeader = new HeaderBase();
  HistFlag = true;
}
////////////////////////////////////////////////////////
ElementContainer::
ElementContainer( const HeaderBase &Header ) :
  Xkey("None"), Ykey("None"), Ekey("None"), safesum(false)
{
  header = new HeaderBase( Header );
  UnitHeader = new HeaderBase();
  HistFlag = true;
}
////////////////////////////////////////////////////////
ElementContainer::
ElementContainer( const ElementContainer &ob ) :
  Xkey(ob.Xkey), Ykey(ob.Ykey), Ekey(ob.Ekey), safesum(ob.safesum),
  M(ob.M)
{
  header = new HeaderBase( *ob.header );
  UnitHeader = new HeaderBase( *ob.UnitHeader );
  HistFlag = ob.HistFlag;
}
/////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator=( const ElementContainer &ob )
{
  *header     = *ob.header;
  *UnitHeader = *ob.UnitHeader;

  M = ob.M;

  Xkey = ob.Xkey;
  Ykey = ob.Ykey;
  Ekey = ob.Ekey;

  safesum = ob.safesum;
  HistFlag = ob.HistFlag;

  return *this;
}
////////////////////////////////////////////////////////
ElementContainer::
 ~ElementContainer()
{
  delete header;
  delete UnitHeader;
  M.clear();
}


/**********************************************
 *** 2. private function used in operators  ***
 **********************************************/

////////////////////////////////////////////////////////
void ElementContainer ::
Formatter2(
  const std::vector<Double> *rx, const std::vector<Double> *&ry, 
  const std::vector<Double> *&re) {

  // references to vectors in myself
  std::vector<Double> &lx = M.ref(PutXKey());
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());
  DoubleBinArrange *a = NULL;

  if (lx.size() > (*rx).size() ) {
    // rebin myself
    a = new DoubleBinArrange( (UInt4)(ly.size()), (UInt4)((*ry).size()) );  
    // rivised by T.Ito & Y.Inamura (2009.12.21)

    a -> SetHist(lx, ly, le, *rx);
    //a -> Averaging();
    a -> Binning();
    lx = a -> PutResultBinVector();
    ly = a -> PutResultValue();
    le = a -> PutResultError();

  } else if (Compare(lx, *rx) == false ) {
	// lx.size() > rx.size() or size is equal but bins are not identical
	// rebin'ed vectors are generated 
        a = new DoubleBinArrange( (UInt4)((*ry).size()), (UInt4)(ly.size()) );
	a -> SetHist( *rx, *ry, *re, lx );
	//a -> Averaging();
	a -> Binning();
	ry = new std::vector<Double> (a -> PutResultValue());
	re = new std::vector<Double> (a -> PutResultError());
  }
  delete a;
}

/**********************************************
 *** 3. operators, +, -, *, /               ***
 **********************************************/

////////////////////////////////////////////////////////
bool ElementContainer::
IsMaskData( vector<Double> &v, const vector<Double> *vp ){

  for( UInt4 i=0; i<v.size(); i++ ){
    if( v[i] < 0.0 ){ return true; }
  }
  for( UInt4 i=0; i<vp->size(); i++ ){
    if( (*vp)[i] < 0.0 ){ return true; }
  }
  return false;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
MaskedPlus(const ElementContainer &r )
{
  vector<Double> *lex = M( PutXKey() );
  vector<Double> *ley = M( PutYKey() );
  vector<Double> *lee = M( PutEKey() );

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rex = &r.M.ref(r.PutXKey());
  const std::vector<Double> *rey = &r.M.ref(r.PutYKey());
  const std::vector<Double> *ree = &r.M.ref(r.PutEKey());

  vector<Double> v0y;
  vector<Double> v0e;
  vector<Double> v1y;
  vector<Double> v1e;

  
  vector<Double> *x = new vector<Double>();
  DoubleBinArrange *dba0 = NULL;
  DoubleBinArrange *dba1 = NULL;
  
  if( lex->size() <= rex->size() ){
	
    //x = lex;
    for( UInt4 i=0; i<lex->size(); i++ ){ x->push_back( (*lex)[i]); }

    v0y = *ley;
    v0e = *lee;

    dba1 = new DoubleBinArrange( (UInt4)(rex->size())-1, (UInt4)(x->size())-1 );
    dba1 -> SetHist( *rex, *rey, *ree, *x );
    dba1 -> Binning( -1 );
    v1y = dba1->PutResultValue(); 
    v1e = dba1->PutResultError(); 
    
  } 
  else{
    //x = rex;
    for( UInt4 i=0; i<rex->size(); i++ ){ x->push_back( (*rex)[i]); }
    v1y = *rey;
    v1e = *ree;

    dba0 = new DoubleBinArrange( (UInt4)(lex->size())-1, (UInt4)(x->size())-1 );
    dba0 -> SetHist( *lex, *ley, *lee, *x );
    dba0 -> Binning( -1 );
    v0y = dba0->PutResultValue(); 
    v0e = dba0->PutResultError();
  } 

  vector<Double> *y = new vector<Double>( x->size()-1 );
  vector<Double> *e = new vector<Double>( x->size()-1 );

  for( UInt4 i=0; i<y->size(); i++ ){
    if( v0e[i] < 0.0 && v1e[i] >= 0.0 ){
      (*y)[i] = v1y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] >= 0.0 ){
      (*y)[i] = v0y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] < 0.0){
      (*y)[i] =  v0y[i] + v1y[i];
      (*e)[i] = -1.0;
    }
    else{
      (*y)[i] = v0y[i] + v1y[i];
      (*e)[i] = sqrt( v0e[i]*v0e[i] + v1e[i]*v1e[i] );
    }
  }
  
  string xkey = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();
  Replace( xkey, *x );
  Replace( ykey, *y );
  Replace( ekey, *e );
  SetKeys( xkey, ykey, ekey );
  
  delete dba0;
  delete dba1;
  delete x;
  delete y;
  delete e;
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
MaskedSub(const ElementContainer &r )
{
  vector<Double> *lex = M( PutXKey() );
  vector<Double> *ley = M( PutYKey() );
  vector<Double> *lee = M( PutEKey() );

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rex = &r.M.ref(r.PutXKey());
  const std::vector<Double> *rey = &r.M.ref(r.PutYKey());
  const std::vector<Double> *ree = &r.M.ref(r.PutEKey());

  vector<Double> v0y;
  vector<Double> v0e;
  vector<Double> v1y;
  vector<Double> v1e;

  
  vector<Double> *x = new vector<Double>();
  DoubleBinArrange *dba0 = NULL;
  DoubleBinArrange *dba1 = NULL;
  
  if( lex->size() <= rex->size() ){

    //x = lex;
    for( UInt4 i=0; i<lex->size(); i++ ){ x->push_back( (*lex)[i]); }

    v0y = *ley;
    v0e = *lee;

    dba1 = new DoubleBinArrange( (UInt4)(rex->size())-1, (UInt4)(x->size())-1 );
    dba1 -> SetHist( *rex, *rey, *ree, *x );
    dba1 -> Binning( -1 );
    v1y = dba1->PutResultValue(); 
    v1e = dba1->PutResultError(); 
    
  } 
  else{
    //x = rex;
    for( UInt4 i=0; i<rex->size(); i++ ){ x->push_back( (*rex)[i]); }
    v1y = *rey;
    v1e = *ree;

    dba0 = new DoubleBinArrange( (UInt4)(lex->size())-1, (UInt4)(x->size())-1 );
    dba0 -> SetHist( *lex, *ley, *lee, *x );
    dba0 -> Binning( -1 );
    v0y = dba0->PutResultValue(); 
    v0e = dba0->PutResultError();
  } 

  vector<Double> *y = new vector<Double>( x->size()-1 );
  vector<Double> *e = new vector<Double>( x->size()-1 );

  for( UInt4 i=0; i<y->size(); i++ ){
    if( v0e[i] < 0.0 && v1e[i] >= 0.0 ){
      (*y)[i] = -v1y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] >= 0.0 ){
      (*y)[i] = v0y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] < 0.0){
      (*y)[i] =  v0y[i] - v1y[i];
      (*e)[i] = -1.0;
    }
    else{
      (*y)[i] = v0y[i] - v1y[i];
      (*e)[i] = sqrt( v0e[i]*v0e[i] + v1e[i]*v1e[i] );
    }
  }
  
  string xkey = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();
  Replace( xkey, *x );
  Replace( ykey, *y );
  Replace( ekey, *e );
  SetKeys( xkey, ykey, ekey );
  
  delete dba0;
  delete dba1;
  delete x;
  delete y;
  delete e;
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
MaskedMul(const ElementContainer &r )
{
  vector<Double> *lex = M( PutXKey() );
  vector<Double> *ley = M( PutYKey() );
  vector<Double> *lee = M( PutEKey() );

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rex = &r.M.ref(r.PutXKey());
  const std::vector<Double> *rey = &r.M.ref(r.PutYKey());
  const std::vector<Double> *ree = &r.M.ref(r.PutEKey());

  vector<Double> v0y;
  vector<Double> v0e;
  vector<Double> v1y;
  vector<Double> v1e;

  
  vector<Double> *x = new vector<Double>();
  DoubleBinArrange *dba0 = NULL;
  DoubleBinArrange *dba1 = NULL;
  
  if( lex->size() <= rex->size() ){

    //x = lex;
    for( UInt4 i=0; i<lex->size(); i++ ){ x->push_back( (*lex)[i]); }

    v0y = *ley;
    v0e = *lee;

    dba1 = new DoubleBinArrange( (UInt4)(rex->size())-1, (UInt4)(x->size())-1 );
    dba1 -> SetHist( *rex, *rey, *ree, *x );
    dba1 -> Binning( -1 );
    v1y = dba1->PutResultValue(); 
    v1e = dba1->PutResultError(); 
    
  } 
  else{
    //x = rex;
    for( UInt4 i=0; i<rex->size(); i++ ){ x->push_back( (*rex)[i]); }
    v1y = *rey;
    v1e = *ree;

    dba0 = new DoubleBinArrange( (UInt4)(lex->size())-1, (UInt4)(x->size())-1 );
    dba0 -> SetHist( *lex, *ley, *lee, *x );
    dba0 -> Binning( -1 );
    v0y = dba0->PutResultValue(); 
    v0e = dba0->PutResultError();
  } 

  vector<Double> *y = new vector<Double>( x->size()-1 );
  vector<Double> *e = new vector<Double>( x->size()-1 );

  for( UInt4 i=0; i<y->size(); i++ ){
    if( v0e[i] < 0.0 && v1e[i] >= 0.0 ){
      (*y)[i] = v1y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] >= 0.0 ){
      (*y)[i] = v0y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] < 0.0){
      (*y)[i] =  v0y[i] * v1y[i];
      (*e)[i] = -1.0;
    }
    else{
      (*y)[i] = v0y[i] * v1y[i];
      (*e)[i] = sqrt( v1y[i]*v0e[i]*v1y[i]*v0e[i] + v0y[i]*v1e[i]*v0y[i]*v1e[i] );
    }
  }
  
  string xkey = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();
  Replace( xkey, *x );
  Replace( ykey, *y );
  Replace( ekey, *e );
  SetKeys( xkey, ykey, ekey );
  
  delete dba0;
  delete dba1;
  delete x;
  delete y;
  delete e;
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
MaskedDiv(const ElementContainer &r )
{
  vector<Double> *lex = M( PutXKey() );
  vector<Double> *ley = M( PutYKey() );
  vector<Double> *lee = M( PutEKey() );

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rex = &r.M.ref(r.PutXKey());
  const std::vector<Double> *rey = &r.M.ref(r.PutYKey());
  const std::vector<Double> *ree = &r.M.ref(r.PutEKey());

  vector<Double> v0y;
  vector<Double> v0e;
  vector<Double> v1y;
  vector<Double> v1e;

  
  vector<Double> *x = new vector<Double>();
  DoubleBinArrange *dba0 = NULL;
  DoubleBinArrange *dba1 = NULL;
  
  if( lex->size() <= rex->size() ){

    //x = lex;
    for( UInt4 i=0; i<lex->size(); i++ ){ x->push_back( (*lex)[i]); }

    v0y = *ley;
    v0e = *lee;

    dba1 = new DoubleBinArrange( (UInt4)(rex->size())-1, (UInt4)(x->size())-1 );
    dba1 -> SetHist( *rex, *rey, *ree, *x );
    dba1 -> Binning( -1 );
    v1y = dba1->PutResultValue(); 
    v1e = dba1->PutResultError(); 
    
  } 
  else{
    //x = rex;
    for( UInt4 i=0; i<rex->size(); i++ ){ x->push_back( (*rex)[i]); }
    v1y = *rey;
    v1e = *ree;

    dba0 = new DoubleBinArrange( (UInt4)(lex->size())-1, (UInt4)(x->size())-1 );
    dba0 -> SetHist( *lex, *ley, *lee, *x );
    dba0 -> Binning( -1 );
    v0y = dba0->PutResultValue(); 
    v0e = dba0->PutResultError();
  } 

  vector<Double> *y = new vector<Double>( x->size()-1 );
  vector<Double> *e = new vector<Double>( x->size()-1 );

  for( UInt4 i=0; i<y->size(); i++ ){

    if( v1y[i] == 0.0 ){
      (*y)[i] =  0.0;
      (*e)[i] = -1.0;
    }
    else if( v0e[i] < 0.0 && v1e[i] >= 0.0 ){
      (*y)[i] = v0y[i] / v1y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] >= 0.0 ){
      (*y)[i] = v0y[i] / v1y[i];
      (*e)[i] = -1.0;
    }
    else if( v1e[i] < 0.0 && v0e[i] < 0.0){
      (*y)[i] =  v0y[i] / v1y[i];
      (*e)[i] = -1.0;
    }
    else{
      (*y)[i] = v0y[i] / v1y[i];
      (*e)[i] = sqrt( v1y[i]*v0e[i]*v1y[i]*v0e[i] + v0y[i]*v1e[i]*v0y[i]*v1e[i] ) / (v1y[i]*v1y[i]);
    }
  }
  
  string xkey = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();
  Replace( xkey, *x );
  Replace( ykey, *y );
  Replace( ekey, *e );
  SetKeys( xkey, ykey, ekey );

  delete dba0;
  delete dba1;
  delete x;
  delete y;
  delete e;
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator+=(const ElementContainer &r )
{
  // references to vectors in myself
  std::vector<Double> &lx = M.ref(PutXKey());
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rx = &r.M.ref(r.PutXKey());
  const std::vector<Double> *ry = &r.M.ref(r.PutYKey());
  const std::vector<Double> *re = &r.M.ref(r.PutEKey());

  /*
  if( IsMaskData( le, re ) == true ){
    return MaskedPlus( r );
  }
  */

  if( // if dispersed type data
     ( lx. size() == ly. size() ) &&
     ( lx. size() == le. size() ) &&
     ( rx->size() == ry->size() ) &&
     ( rx->size() == re->size() ) &&
     ( lx. size() == rx->size() )
      ){ 
    UInt4 size = (UInt4)(lx.size());
    for( UInt4 i=0; i<size; i++ ){
      ly[i] = ly[i] + (*ry)[i];
      le[i] = sqrt( (*re)[i]*(*re)[i] + le[i]*le[i] );
    }
    string NewYUnit = ReArrangeUnit( PutUnit( Ykey ), "+", r.PutUnit( r.PutYKey() ) );
    string NewEUnit = ReArrangeUnit( PutUnit( Ekey ), "+", r.PutUnit( r.PutEKey() ) );
    ReSetUnit(PutYKey(), NewYUnit);
    ReSetUnit(PutEKey(), NewEUnit);
  }
  else if( 
	  ( lx. size() == ly. size()+1 ) &&
	  ( lx. size() == le. size()+1 ) &&
	  ( rx->size() == ry->size()+1 ) &&
	  ( rx->size() == re->size()+1 ) )
    { // if histogram type data

      vector<Double> *NewBin = NULL; // for calculation result
      vector<Double> NewLy;
      vector<Double> NewLe;
      vector<Double> NewRy;
      vector<Double> NewRe; 

      BinConverter *bc = NULL;
      
      if( Compare( lx, *rx ) == true ){
	 // when both bins are same, do not convert bin
	NewBin = new vector<Double>( rx->size() );
	for( int i=0; i<rx->size(); i++ ){ (*NewBin)[i] = (*rx)[i]; }
	NewLy = ly;
	NewLe = le;
	NewRy = *ry;
	NewRe = *re;

      }

      else{
    // when bins are different, do covert bins
	bc = new BinConverter( lx, ly, le, rx, ry, re );
	// delete bc; cause the error when NewLy etc are used.

	if( bc -> prepare() == -1 ){
	  cout << "Bin Conversion error!!" << endl;
	  return *this;
	}
	NewBin = bc -> NewBin; // for results
	NewLy = bc -> al->PutResultValue();
	NewLe = bc -> al->PutResultError();
	NewRy = bc -> ar->PutResultValue();
	NewRe = bc -> ar->PutResultError();
      }
      
      vector<Double> y; // for y results
      vector<Double> e; // for e results

      for( UInt4 i=0; i<NewBin->size()-1; i++ ){

	y.push_back( NewLy[i] + NewRy[i] );
	double err = sqrt( NewRe[i]*NewRe[i] + NewLe[i]*NewLe[i] );

	if( NewRe[i] < 0.0 || NewLe[i] < 0.0 ){
	  e.push_back( -1.0 * err );
	}
	else{
	  e.push_back( err );
	}
      }

      // remake ElementContainer
      string _xkey = PutXKey();
      string _ykey = PutYKey();
      string _ekey = PutEKey();

      string _x_unit(PutUnit( Xkey ));
      string NewYUnit( ReArrangeUnit(PutUnit( Ykey ), "+", r.PutUnit( r.PutYKey() )) );
      string NewEUnit( ReArrangeUnit(PutUnit( Ekey ), "+", r.PutUnit( r.PutEKey() )) );

      Remove( _xkey );
      Remove( _ykey );
      Remove( _ekey );

      Add( _xkey, *NewBin, _x_unit );
      Add( _ykey, y, NewYUnit );
      Add( _ekey, e, NewEUnit );
      SetKeys( _xkey, _ykey, _ekey );
      //
      
      if( bc != NULL ){
	delete bc;
      }
      //delete NewBin;
    }
  else{
    cout << "operation failed" << endl;
  }

  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator+=(const std::pair<Double, Double> &r )
{
  // references to vectors in myself
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  UInt4 size = (UInt4)(le . size());
  for( UInt4 i=0; i<size; i++ ) ly[i] += r.first;

  double r2 = r.second*r.second;
  for( UInt4 i=0; i<size; i++ ) le[i] *= le[i];
  for( UInt4 i=0; i<size; i++ ) le[i] += r2;
  for( UInt4 i=0; i<size; i++ ) le[i] = sqrt(le[i]);

  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator-=(const ElementContainer &r )
{
  // references to vectors in myself
  std::vector<Double> &lx = M.ref(PutXKey());
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rx = &r.M.ref(r.PutXKey());
  const std::vector<Double> *ry = &r.M.ref(r.PutYKey());
  const std::vector<Double> *re = &r.M.ref(r.PutEKey());

  /*
  if( IsMaskData( le, re ) == true ){
    return MaskedSub( r );
  }
  */
  if( // if dispersed type data
     ( lx. size() == ly. size() ) &&
     ( lx. size() == le. size() ) &&
     ( rx->size() == ry->size() ) &&
     ( rx->size() == re->size() ) &&
     ( lx. size() == rx->size() )
      ){ 
    UInt4 size = (UInt4)(lx.size());
    for( UInt4 i=0; i<size; i++ ){
      ly[i] = ly[i] - (*ry)[i];
      le[i] = sqrt( (*re)[i]*(*re)[i] + le[i]*le[i] );
    }
    string NewYUnit = ReArrangeUnit( PutUnit( Ykey ), "-", r.PutUnit( r.PutYKey() ) );
    string NewEUnit = ReArrangeUnit( PutUnit( Ekey ), "-", r.PutUnit( r.PutEKey() ) );
    ReSetUnit(PutYKey(), NewYUnit);
    ReSetUnit(PutEKey(), NewEUnit);
  }
  else if( 
	  ( lx. size() == ly. size()+1 ) &&
	  ( lx. size() == le. size()+1 ) &&
	  ( rx->size() == ry->size()+1 ) &&
	  ( rx->size() == re->size()+1 ) )
    { // if histogram type data


      vector<Double> *NewBin = NULL; // for calculation result
      vector<Double> NewLy;
      vector<Double> NewLe;
      vector<Double> NewRy;
      vector<Double> NewRe; 

      BinConverter *bc = NULL;
      
      if( Compare( lx, *rx ) == true ){
	 // when both bins are same, do not convert bin
	NewBin = new vector<Double>( rx->size() );
	for( int i=0; i<rx->size(); i++ ){ (*NewBin)[i] = (*rx)[i]; }
	NewLy = ly;
	NewLe = le;
	NewRy = *ry;
	NewRe = *re;
      }

      else{
	// when bins are different, do convert bins
	bc = new BinConverter( lx, ly, le, rx, ry, re );
	// delete bc; cause the error when NewLy etc are used.

	if( bc -> prepare() == -1 ){
	  cout << "Bin Conversion error!!" << endl;
	  return *this;
	}
	NewBin = bc -> NewBin; // for results
	NewLy = bc -> al->PutResultValue();
	NewLe = bc -> al->PutResultError();
	NewRy = bc -> ar->PutResultValue();
	NewRe = bc -> ar->PutResultError();
      }
      
      vector<Double> y; // for y results
      vector<Double> e; // for e results

      for( UInt4 i=0; i<NewBin->size()-1; i++ ){
	y.push_back( NewLy[i] - NewRy[i] );
	double err = sqrt( NewRe[i]*NewRe[i] + NewLe[i]*NewLe[i] );

	if( NewRe[i] < 0.0 || NewLe[i] < 0.0 ){
	  e.push_back( -1.0 * err );
	}
	else{
	  e.push_back( err );
	}

      }

      // remake ElementContainer
      string _xkey = PutXKey();
      string _ykey = PutYKey();
      string _ekey = PutEKey();

      string _x_unit(PutUnit( Xkey ));
      string NewYUnit( ReArrangeUnit(PutUnit( Ykey ), "+", r.PutUnit( r.PutYKey() )) );
      string NewEUnit( ReArrangeUnit(PutUnit( Ekey ), "+", r.PutUnit( r.PutEKey() )) );

      Remove( _xkey );
      Remove( _ykey );
      Remove( _ekey );
      Add( _xkey, *NewBin, _x_unit );
      Add( _ykey, y, NewYUnit );
      Add( _ekey, e, NewEUnit );
      SetKeys( _xkey, _ykey, _ekey );
      //
      
      if( bc != NULL ){
	delete bc;
      }


    }
  else{
    cout << "operation failed" << endl;
  }
  
  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator-=(const std::pair<Double, Double> &r )
{
  // references to vectors in myself
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  UInt4 size = (UInt4)(le . size());
  for( UInt4 i=0; i<size; i++ ) ly[i] -= r.first;

  double r2 = r.second*r.second;
  for( UInt4 i=0; i<size; i++ ) le[i] *= le[i];
  for( UInt4 i=0; i<size; i++ ) le[i] += r2;
  for( UInt4 i=0; i<size; i++ ) le[i] = sqrt(le[i]);

  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator*=(const ElementContainer &r )
{
  // references to vectors in myself
  std::vector<Double> &lx = M.ref(PutXKey());
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rx = &r.M.ref(r.PutXKey());
  const std::vector<Double> *ry = &r.M.ref(r.PutYKey());
  const std::vector<Double> *re = &r.M.ref(r.PutEKey());

  /*
  if( IsMaskData( le, re ) == true ){
    return MaskedMul( r );
  }
  */

  if( // if dispersed type data
     ( lx. size() == ly. size() ) &&
     ( lx. size() == le. size() ) &&
     ( rx->size() == ry->size() ) &&
     ( rx->size() == re->size() ) &&
     ( lx. size() == rx->size() )
      ){ 
    UInt4 size = (UInt4)(lx.size());
    for( UInt4 i=0; i<size; i++ ){
      double r0 = (*re)[i]*ly[i];
      double r1=le[i]*(*ry)[i];
      le[i] = sqrt( r0*r0 + r1*r1 );
      ly[i] = ly[i] * (*ry)[i];
    }
    string NewYUnit = ReArrangeUnit( PutUnit( Ykey ), "*", r.PutUnit( r.PutYKey() ) );
    string NewEUnit = ReArrangeUnit( PutUnit( Ekey ), "*", r.PutUnit( r.PutEKey() ) );
    ReSetUnit(PutYKey(), NewYUnit);
    ReSetUnit(PutEKey(), NewEUnit);
  }
  else if( 
	  ( lx. size() == ly. size()+1 ) &&
	  ( lx. size() == le. size()+1 ) &&
	  ( rx->size() == ry->size()+1 ) &&
	  ( rx->size() == re->size()+1 ) )
    { // if histogram type data

      vector<Double> *NewBin = NULL; // for calculation result
      vector<Double> NewLy;
      vector<Double> NewLe;
      vector<Double> NewRy;
      vector<Double> NewRe; 

      BinConverter *bc = NULL;
      
      if( Compare( lx, *rx ) == true ){
	 // when both bins are same, do not convert bin
	NewBin = new vector<Double>( rx->size() );
	for( int i=0; i<rx->size(); i++ ){ (*NewBin)[i] = (*rx)[i]; }
	NewLy = ly;
	NewLe = le;
	NewRy = *ry;
	NewRe = *re;
      }

      else{
	// when bins are different, do convert bins
	bc = new BinConverter( lx, ly, le, rx, ry, re );
	// delete bc; cause the error when NewLy etc are used.

	if( bc -> prepare() == -1 ){
	  cout << "Bin Conversion error!!" << endl;
	  return *this;
	}
	NewBin = bc -> NewBin; // for results
	NewLy = bc -> al->PutResultValue();
	NewLe = bc -> al->PutResultError();
	NewRy = bc -> ar->PutResultValue();
	NewRe = bc -> ar->PutResultError();
      }
      
      vector<Double> y; // for y results
      vector<Double> e; // for e results

      for( UInt4 i=0; i<NewBin->size()-1; i++ ){
	y.push_back( NewLy[i] * NewRy[i] );
	double err = sqrt( ( NewLy[i]*NewRe[i] )*( NewLy[i]*NewRe[i] ) + 
			   ( NewRy[i]*NewLe[i] )*( NewRy[i]*NewLe[i] ) );

	if( NewRe[i] < 0.0 || NewLe[i] < 0.0 ){
	  e.push_back( -1.0 * err );
	}
	else{
	  e.push_back( err );
	}

      }

      // remake ElementContainer
      string _xkey = PutXKey();
      string _ykey = PutYKey();
      string _ekey = PutEKey();

      string _x_unit(PutUnit( Xkey ));
      string NewYUnit( ReArrangeUnit(PutUnit( Ykey ), "+", r.PutUnit( r.PutYKey() )) );
      string NewEUnit( ReArrangeUnit(PutUnit( Ekey ), "+", r.PutUnit( r.PutEKey() )) );

      Remove( _xkey );
      Remove( _ykey );
      Remove( _ekey );
      Add( _xkey, *NewBin, _x_unit );
      Add( _ykey, y, NewYUnit );
      Add( _ekey, e, NewEUnit );
      SetKeys( _xkey, _ykey, _ekey );
      //
      
      if( bc != NULL ){
	delete bc;
      }


    }
  else{
    cout << "operation failed" << endl;
  }

  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator*=(const std::pair<Double, Double> &r )
{
  // references to vectors in myself
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  UInt4 size = (UInt4)(le . size());

  for( UInt4 i=0; i<size; i++ ){
    double r0 = r.second*ly[i];
    double r1=le[i]*r.first;
    le[i] = sqrt( r0*r0 + r1*r1 );
  }
  for( UInt4 i=0; i<size; i++ ){
    ly[i] *= r.first;
  }
  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator/=(const ElementContainer &r )
{
  // references to vectors in myself
  std::vector<Double> &lx = M.ref(PutXKey());
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  // pointers to vectors in r or new rebin'ed vectors
  const std::vector<Double> *rx = &r.M.ref(r.PutXKey());
  const std::vector<Double> *ry = &r.M.ref(r.PutYKey());
  const std::vector<Double> *re = &r.M.ref(r.PutEKey());

  /*
  if( IsMaskData( le, re ) == true ){
    return MaskedDiv( r );
  }
  */

  if( // if dispersed type data
     ( lx. size() == ly. size() ) &&
     ( lx. size() == le. size() ) &&
     ( rx->size() == ry->size() ) &&
     ( rx->size() == re->size() ) &&
     ( lx. size() == rx->size() )
      ){ 
    UInt4 size = (UInt4)(lx.size());
    for( UInt4 i=0; i<size; i++ ){
      if( (*ry)[i] == 0.0 ){
      ly[i] = 0.0;
      le[i]  = 1.0;
    }
    else {  
      double r0 = (*re)[i]*ly[i];
      double r1 = le[i]*(*ry)[i];
      le[i] = sqrt( r0*r0 + r1*r1 ) / ((*ry)[i]*(*ry)[i]);
      ly[i] = ly[i] / (*ry)[i];
    }
    }
    string NewYUnit = ReArrangeUnit( PutUnit( Ykey ), "/", r.PutUnit( r.PutYKey() ) );
    string NewEUnit = ReArrangeUnit( PutUnit( Ekey ), "/", r.PutUnit( r.PutEKey() ) );
    ReSetUnit(PutYKey(), NewYUnit);
    ReSetUnit(PutEKey(), NewEUnit);
  }
  else if( 
	  ( lx. size() == ly. size()+1 ) &&
	  ( lx. size() == le. size()+1 ) &&
	  ( rx->size() == ry->size()+1 ) &&
	  ( rx->size() == re->size()+1 ) )
    { // if histogram type data
      vector<Double> *NewBin = NULL; // for calculation result
      vector<Double> NewLy;
      vector<Double> NewLe;
      vector<Double> NewRy;
      vector<Double> NewRe; 

      BinConverter *bc = NULL;
      
      if( Compare( lx, *rx ) == true ){
	 // when both bins are same, do not convert bin
	NewBin = new vector<Double>( rx->size() );
	for( int i=0; i<rx->size(); i++ ){ (*NewBin)[i] = (*rx)[i]; }
	NewLy = ly;
	NewLe = le;
	NewRy = *ry;
	NewRe = *re;
      }

      else{
	// when bins are different, do covert bins
	bc = new BinConverter( lx, ly, le, rx, ry, re );
	// delete bc; cause the error when NewLy etc are used.

	if( bc -> prepare() == -1 ){
	  cout << "Bin Conversion error!!" << endl;
	  return *this;
	}
	NewBin = bc -> NewBin; // for results
	NewLy = bc -> al->PutResultValue();
	NewLe = bc -> al->PutResultError();
	NewRy = bc -> ar->PutResultValue();
	NewRe = bc -> ar->PutResultError();
      }
      
      vector<Double> y; // for y results
      vector<Double> e; // for e results

      for( UInt4 i=0; i<NewBin->size()-1; i++ ){

	if( NewRy[i] == 0.0 ){
	  y.push_back( 0.0 );
	  e.push_back( 1.0 );
	}
	else{
	  y.push_back( NewLy[i] / NewRy[i] );
	  
	  double err = ( 1.0 / NewRy[i] / NewRy[i] ) * 
	    sqrt( ( NewLy[i]*NewRe[i] )*( NewLy[i]*NewRe[i] ) + 
		  ( NewRy[i]*NewLe[i] )*( NewRy[i]*NewLe[i] ) );
	  
	  if( NewRe[i] < 0.0 || NewLe[i] < 0.0 ){
	    e.push_back( -1.0 * err );
	  }
	  else{
	    e.push_back( err );
	  }  
	}
      }

      // remake ElementContainer
      string _xkey = PutXKey();
      string _ykey = PutYKey();
      string _ekey = PutEKey();

      string _x_unit(PutUnit( Xkey ));
      string NewYUnit( ReArrangeUnit(PutUnit( Ykey ), "+", r.PutUnit( r.PutYKey() )) );
      string NewEUnit( ReArrangeUnit(PutUnit( Ekey ), "+", r.PutUnit( r.PutEKey() )) );

      Remove( _xkey );
      Remove( _ykey );
      Remove( _ekey );
      Add( _xkey, *NewBin, _x_unit );
      Add( _ykey, y, NewYUnit );
      Add( _ekey, e, NewEUnit );
      SetKeys( _xkey, _ykey, _ekey );
      //
      
      if( bc != NULL ){
	delete bc;
      }

      
    }
  else{
    cout << "operation failed" << endl;
  }
  
  return *this;
}

////////////////////////////////////////////////////////
ElementContainer &ElementContainer ::
operator/=(const std::pair<Double, Double> &r )
{
  // references to vectors in myself
  std::vector<Double> &ly = M.ref(PutYKey());
  std::vector<Double> &le = M.ref(PutEKey());

  UInt4 size = (UInt4)(le . size());

  
  double r2 = r.first*r.first;
  for( UInt4 i=0; i<size; i++ ){
    double r0 = r.second*ly[i];
    double r1=le[i]*r.first;
    le[i] = sqrt( r0*r0 + r1*r1 ) / r2;
  }
  for( UInt4 i=0; i<size; i++ ){
    ly[i] /= r.first;
  }

  /*
  for( UInt4 i=0; i<size; i++ ){
    ly[i] /= r.first;
  }
  double r2 = r.first*r.first;
  for( UInt4 i=0; i<size; i++ ){
    double r0 = r.second*ly[i];
    double r1=le[i]*r.first;
    le[i] = sqrt( r0*r0 + r1*r1 ) / r2;
  }
  */

  return *this;
}

/**********************************************
 *** 4. removed                             ***
 **********************************************/

/**********************************************
 *** 5. ReBin and Binning; rebinning        ***
 **********************************************/

////////////////////////////////////////////////////////
ElementContainer ElementContainer::
ReBin( const vector<Double> &v )
{
  if( Xkey == "None" ||
      Ykey == "None" ||
      Ekey == "None" ){
    ElementContainer tmp;
    return tmp;
  }

  vector<Double> ResultBin   = v;
  vector<Double> SourceBin   = PutX();
  vector<Double> SourceValue = PutY();
  vector<Double> SourceError = PutE();

  DoubleBinArrange *a = 
    new DoubleBinArrange( (UInt4)(SourceBin.size())-1, (UInt4)(ResultBin.size())-1 );
  a -> SetHist( SourceBin, SourceValue, SourceError, ResultBin );
  a -> Binning();

  ElementContainer result;
  result . InputHeader( PutHeader() );
  result . Add( Xkey, a->PutResultBinVector(), 
		UnitHeader->PutString( Xkey ) );
  result . Add( Ykey, a->PutResultValue(),
		UnitHeader->PutString( Ykey ) );
  result . Add( Ekey, a->PutResultError(),
		UnitHeader->PutString( Ekey ) );
  result . SetKeys( Xkey, Ykey, Ekey );

  delete a;
  return result;
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
ReBin( UInt4 n ){
  return ReBin( PrepareXbin( n ) );
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Binning( UInt4 n ){
  return Binning( PrepareXbin( n ) );
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Averaging( UInt4 n ){
  return Averaging( PrepareXbin( n ) );
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
MergeWithWeight( ElementContainer &e ){

  ElementContainer r = ElementContainer( PutHeader() );
  
  vector<Double> y0 = PutY();
  vector<Double> e0 = PutE();
  vector<Double> y1 = e.PutY();
  vector<Double> e1 = e.PutE();

  vector<Double> y  = vector<Double>( y0.size() );
  vector<Double> er = vector<Double>( e0.size() );

  for( UInt4 i=0; i<y0.size(); i++ ){

    double b = 1.0;
    double er0 =  e0[i];
    double er1 =  e1[i];
    
    if(er0 > 0.0 && er1 > 0.0){
      b = 1.0 / ( er0 * er0 ) + 1.0 / ( er1 * er1 );
      y[i]  = ( y0[i] / ( er0 * er0 ) + y1[i] / ( er1 * er1 ) ) / b;
      er[i] = 1 / sqrt( b );
      //cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n0" << endl;
    }
    else if( er0 <= 0.0 && er1 > 0){
      y[i]  = y1[i];
      er[i] = er1;
      //cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n1" << endl;
    }
    else if( er1 <= 0.0 && er0 > 0){
      y[i]  = y0[i];
      er[i] = er0;
      //cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n2" << endl;
    }
    else if( er0 <= 0.0 && er1 <= 0.0){
      y[i] = 0.0;
      er[i] = 0.0;
      //cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n3" << endl;
        if( er0 < 0.0 && er1 < 0.0){
          y[i]  = 0.0;
          er[i] = -1.0;
          //cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n4" << endl;
        }
    }
    /*else if( er0 == 0.0 && er1 == 0.0){
      y[i]  = 0.0;
      er[i] = 0.0;
      cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n0" << endl;
    }*/
    else{
      cout << "y0="<<y0[i] <<", e0=" <<er0 <<", y1="<< y1[i] <<", e1="<<e1[i] <<", ry="<<y[i] <<", re="<< er[i] <<", n5" << endl;
    }
  }

  r.Add( PutXKey(), PutX() );
  r.Add( PutYKey(), y  );
  r.Add( PutEKey(), er );
  r.SetKeys( PutXKey(), PutYKey(), PutEKey() );

  return r;
}
////////////////////////////////////////////////////////
vector<Double> ElementContainer::
PrepareXbin( UInt4 n ){
  vector<Double> v;
  vector<Double> x = PutX();

  UInt4 t = ( (UInt4)(x.size())-1 ) / n;
  for( UInt4 i=0; i<=t; i++ ){
    v.push_back( x[ i*n ] );
  }
  
  if( (x.size()-1) % n != 0 ){
    v.push_back( x[ x.size() -1 ] );
  }
  return v;
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Binning( const vector<Double> &v )
{
  return ReBin( v );
}
/*
ElementContainer ElementContainer::
Binning( const vector<Double> &v )
{
  if( Xkey == "None" ||
      Ykey == "None" ||
      Ekey == "None" ){
    ElementContainer tmp;
    return tmp;
  }
  vector<Double> ResultBin   = v;
  vector<Double> SourceBin   = PutX();
  vector<Double> SourceValue = PutY();
  vector<Double> SourceError = PutE();

  DoubleBinArrange *a = 
    new DoubleBinArrange( SourceBin.size()-1, ResultBin.size()-1 );
  a -> SetHist( SourceBin, SourceValue, SourceError, ResultBin );
  a -> Binning(); 
  
  ElementContainer result;
  result . InputHeader( PutHeader() );
  result . Add( Xkey, a->PutResultBinVector(),
		UnitHeader->PutString( Xkey ) );
  result . Add( Ykey, a->PutResultValue(),
		UnitHeader->PutString( Ykey ) );
  result . Add( Ekey, a->PutResultError(),
		UnitHeader->PutString( Ekey ) );
  result . SetKeys( Xkey, Ykey, Ekey );

  delete a;
  return result;
}
*/
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Averaging( const vector<Double> &v )
{
  if( Xkey == "None" ||
      Ykey == "None" ||
      Ekey == "None" ){
    ElementContainer tmp;
    return tmp;
  }
  vector<Double> ResultBin   = v;
  vector<Double> SourceBin   = PutX();
  vector<Double> SourceValue = PutY();
  vector<Double> SourceError = PutE();

  DoubleBinArrange *a = 
    new DoubleBinArrange( (UInt4)(SourceBin.size())-1, (UInt4)(ResultBin.size())-1 );
  a -> SetHist( SourceBin, SourceValue, SourceError, ResultBin );
  a -> Averaging(); 
  
  ElementContainer result;
  result . InputHeader( PutHeader() );
  result . Add( Xkey, a->PutResultBinVector(),
		UnitHeader->PutString( Xkey ) );
  result . Add( Ykey, a->PutResultValue(),
		UnitHeader->PutString( Ykey ) );
  result . Add( Ekey, a->PutResultError(),
		UnitHeader->PutString( Ekey ) );
  result . SetKeys( Xkey, Ykey, Ekey );

  delete a;
  return result;
}


/**********************************************
 *** 6. SetKeys and Dumpkey                 ***
 **********************************************/

////////////////////////////////////////////////////////
bool ElementContainer::
SetKeys( const string &X, const string &Y, const string &E )
{
  if( CheckKey( X ) != 1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "The key of " << X << " cannot be found in this container." << endl;
    return false;
  }
  else if( CheckKey( Y ) != 1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "The key of " << Y << " cannot be found in this container." << endl;
    return false;
  }
  else if( CheckKey( E ) != 1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "The key of " << E << " cannot be found in this container." << endl;
    return false;
  }
  else if( PutSize( X ) <= 1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "Because the size of the vector named as " << X
	 << " is zero or one. " 
	 << "A histogram cannot be defined." << endl;
      return false;
  }
  else if( ( PutSize( X ) == PutSize( Y ) ) && ( PutSize( X ) == PutSize( E ) ) ){
    // Appended by Jiro SUZUKI, on 2011.12.14
    // Modified by Jiro SUZUKI, on 2013.01.30
    
    Xkey = X;
    Ykey = Y;
    Ekey = E;

    // Appended by Jiro SUZUKI, on 2016.03.14
    HistFlag = false;
    return true;

    // comment out on 2031.01.30 [start]
    /*
    vector<Double> * OrigX0 = (*this)( X );
    vector<Double> * OrigX  = new vector<Double>( OrigX0->size() );
    for( UInt4 i=0; i<OrigX->size(); i++ ){
      (*OrigX)[i] = (*OrigX0)[i];
    }

    vector<Double> * NewX  = new vector<Double>( OrigX->size() + 1 );
    Remove( X );

    ( *NewX )[0] = (*OrigX)[0] - ( (*OrigX)[1] - (*OrigX)[0] ) / 2.0;

    for( UInt4 i=1; i<NewX->size()-1; i++ ){
      (*NewX)[i] = ( (*OrigX)[i-1] + (*OrigX)[i] ) / 2.0;
    }

    ( *NewX )[NewX->size()-1] = (*OrigX)[NewX->size()-2] + 
      ( (*OrigX)[NewX->size()-2] - (*OrigX)[NewX->size()-3] ) / 2.0;


    Add( X + "_orig", *OrigX );
    Add( X, *NewX );
    SetKeys( X, Y, E );
    return;
    */
    // comment out on 2031.01.30 [end]
  }
  else if( PutSize( X )  != PutSize( Y )+1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "The size of Y-vector is wrong." << endl;
    return false;
  }
  else if( PutSize( X ) != PutSize( E )+1 ){
    cout << "ElementContainer::SetKeys(string,string,string)" << endl;
    cout << "The size of E-vector is wrong." << endl;
    return false;
  }
  else{
    Xkey = X;
    Ykey = Y;
    Ekey = E;
    
    // Appended by Jiro SUZUKI, on 2016.03.14
    HistFlag = true;
    return true;
  }
}
bool ElementContainer::
isHist(){
  return HistFlag;
}
////////////////////////////////////////////////////////
void ElementContainer::
DumpKey()
{
  //M.DumpKeyList();

  cout << " " << "\t" << "Key" << "\t" << "Unit" << endl;
  vector<string> KeyList = M.PutKeyList();
  for( UInt4 i=0; i<KeyList.size(); i++ ){
    cout << i << "\t" 
	 << KeyList[ i ] << "\t"
	 << PutUnit( KeyList[ i ] ) << endl;
  }
}


/**********************************************
 *** 7. series of Add() method              ***
 **********************************************/

////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, vector< Double > value, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  _Add(Key, value, Unit);
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, Double *&Array, UInt4 ArraySize, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  vector<Double> TmpVec( ArraySize );
  for( UInt4 i=0; i<ArraySize; i++ ){
    TmpVec[i] = Array[ i ];
  }

  _Add(Key, TmpVec, Unit);
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, vector< Float > value, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  UInt4 Size = (UInt4)(value.size());
  vector<Double> TmpVec( Size );
  for( UInt4 i=0; i<Size; i++ ){
    TmpVec[i] = (Double)value[ i ];
  }
  
  _Add(Key, TmpVec, Unit);
}
///////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, vector< UInt4 > value, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  UInt4 Size = (UInt4)(value.size());
  vector<Double> TmpVec( Size );
  for( UInt4 i=0; i<Size; i++ ){
    TmpVec[i] = (Double)value[ i ];
  }
  
  _Add(Key, TmpVec, Unit);
}
///////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, vector< UInt2 > value, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  UInt4 Size = (UInt4)(value.size());
  vector<Double> TmpVec( Size );
  for( UInt4 i=0; i<Size; i++ ){
    TmpVec[i] = (Double)value[ i ];
  }

  _Add(Key, TmpVec, Unit);
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, PyObject *value, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  vector< Double > TmpVec = __gCppToPython.ListToDoubleVector( value );
 
  if( TmpVec.size() == 0 ){
    cout << "ElementContainer::Add(string,PyObject*)" << endl;
    cout << Key << " cannnot convert into vector<Double>." << endl;
    cout << "Is the type of each element number ?" << endl;
  }
  else{
    _Add(Key, TmpVec, Unit);
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, Float *&Array, UInt4 ArraySize, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  vector<Double> TmpVec( ArraySize );
  for( UInt4 i=0; i<ArraySize; i++ ){
    TmpVec[i] = (Double)Array[ i ];
  }
  _Add(Key, TmpVec, Unit);
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, UInt4 *&Array, UInt4 ArraySize, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }
  vector<Double> TmpVec( ArraySize );
  for( UInt4 i=0; i<ArraySize; i++ ){
    TmpVec[i] = (Double)Array[ i ];
  }
  _Add(Key, TmpVec, Unit);
}
////////////////////////////////////////////////////////
void ElementContainer::
Add( const string &Key, UInt2 *&Array, UInt4 ArraySize, const string &Unit )
{
  if( M.CheckKey( Key ) != 0 ){
    cout << "\"" << Key << "\" has been installed in this container." << endl;
    cout << "Please choose another Key." << endl;
    return;
  }

  vector<Double> TmpVec( ArraySize );
  for( UInt4 i=0; i<ArraySize; i++ ){
    TmpVec[i] = (Double)Array[ i ];
  }
  _Add(Key, TmpVec, Unit);
}


/**********************************************
 *** 8. InputHeader, InputUnitHeader        ***
 **********************************************/

////////////////////////////////////////////////////////
void ElementContainer::
InputHeader( const HeaderBase &Header )
{
  delete header;

  header = new HeaderBase( Header );

  return;
}
////////////////////////////////////////////////////////
void ElementContainer::
InputUnitHeader( const HeaderBase &uHeader )
{
  delete UnitHeader;

  UnitHeader = new HeaderBase( uHeader );

  vector<string> KeyList = M.PutKeyList();

  for( UInt4 i=0; i<KeyList.size(); i++ ){
    if( UnitHeader->CheckKey( KeyList[ i ] ) == 0 ){
      UnitHeader->Add( KeyList[ i ], "None" );
    }
  }
  return;
}


/**********************************************
 *** 9. Copy, AddBlankVecvot, Remove,       ***
 ***   Replace, AddValue, SetValue          ***
 **********************************************/

////////////////////////////////////////////////////////
void  ElementContainer::
Copy( const string &s, const string &x )
{
  M.Copy( s, x );
  string u = UnitHeader -> PutString( s );
  UnitHeader -> Add( x, u );
}
////////////////////////////////////////////////////////
void ElementContainer::
AddBlankVector( const string &Key, UInt4 Size, const string &Unit )
{
  vector<Double> TmpVec( Size );
  for( UInt4 i=0; i<Size; i++ ){
    TmpVec[i] = (Double)i;
  }
  M.Add( Key, TmpVec );
  UnitHeader -> Add( Key, Unit );
}
////////////////////////////////////////////////////////
void ElementContainer::
Remove( const string &Key )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::Remove(string)" << endl;
    cout << Key << " is not found in this container." << endl;
    return;
  }
  M.Erase( Key );
  UnitHeader -> Erase( Key );

  if( Xkey == Key || 
      Ykey == Key ||
      Ekey == Key ){
    Xkey = "None";
    Ykey = "None";
    Ekey = "None";
    HistFlag = true;
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
Replace( const string &Key, const vector<Double> &value )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::Replace(string,vector<Double>)" 
	 << endl;
    cout << Key << " is not found in this container." << endl;
    return;
  }
  Remove( Key );
  Add( Key, value );
}
////////////////////////////////////////////////////////
void ElementContainer::
Replace( const string &Key, PyObject *value )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::Replace(string,vector<Double>)" 
	 << endl;
    cout << Key << " is not found in this container." << endl;
    return;
  }
  Remove( Key );
  Add( Key, value );
}
////////////////////////////////////////////////////////
void ElementContainer::
AppendValue( const string &Key, const vector<Double> &value )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::AppendValue(string,vector<Double>)" 
	 << endl;
    cout << Key << " is not found in this container." << endl;
    return;
  }
  vector<Double> *TmpVec = M( Key );
  UInt4 Size = (UInt4)(value.size());
  for( UInt4 i=0; i<Size; i++ ){
    TmpVec->push_back( value[ i ] );
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
SetValue( const string &Key, UInt4 Number, Double value )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::SetValue(string,UInt4, Double)" 
	 << endl;
    cout << Key << " is not found in this container." << endl;
    return;
  }
  vector< Double > *vec = M( Key );

  if( Number >= vec->size() ){
    cout << "ElementContainer::SetValue(string,UInt4,Double)" << endl;
    cout << "The value of \"Number\" is too large to set the value." << endl;
    return;
  }
  
  ( *vec )[Number] = value;
}


/**********************************************
 *** 10. series of Dump, DumpValue          ***
 **********************************************/

////////////////////////////////////////////////////////
void ElementContainer::
Dump()
{
  cout << "*** header object start" << endl;
  header -> Dump();
  cout << "*** header object end"   << endl;
  cout << endl;

  UInt4 TableSize = PutTableSize();
  cout << "The number of vectors is ";
  cout << TableSize << endl;
  cout << endl;

  cout << "x key = "
       << Xkey << endl;
  cout << "y key = "
       << Ykey << endl;
  cout << "e key = "
       << Ekey << endl;
  cout << endl;


  cout << "Index"  << "\t"
       << "Key"    << "\t"
       << "Size"   << "\t"
       << "Unit"   << "\t"
       << "Values" << endl;
  DumpValue();
}
////////////////////////////////////////////////////////
PyObject* ElementContainer::
PyDump(){
  vector<string> vs;
  vs.clear();
  string s;
  

  // start procedure for HeaderBase
  s = "*** header object start";
  vs.push_back( s );
  PyObject *hpo = header -> PyDump();
  CppToPython *hcp = new CppToPython();
  vector<string> hvs = hcp -> ListToStringVector( hpo );
  delete hcp;
  for( UInt4 i=0; i<hvs.size(); i++ ){
    vs.push_back( hvs[ i ] );
  }
  s = "*** header object end";
  vs.push_back( s );
  // end procedure for HeaderBase


  StringTools *st = new StringTools();

  UInt4 TableSize = PutTableSize();
  s = "The number of vectors is ";
  s = s + st->UInt4ToString( TableSize );
  vs.push_back( s );
  
  s = "x key = " + Xkey + ", y key = " + Ykey + ", e key = " + Ekey;
  vs.push_back( s );
  
  s = "Index" + string( "\t" ) +
      "Key"   + string( "\t" ) +
      "Size"  + string( "\t" ) +
      "Unit"  + string( "\t" ) +
      "Values";
  vs.push_back( s );

  for( UInt4 t=0; t<TableSize; t++ ){
    string key   = PutName( t );
    string depth = st->UInt4ToString( PutSize( t ) );
    string unit  = UnitHeader->PutString( key );
    
    s = st->UInt4ToString( t ) + string( "\t" ) +
        key   + string( "\t" ) +
        depth + string( "\t" ) +
        unit  + string( "\t" );
    
    if( PutSize(t) >= 1){
      vector<Double> V = Put( t );
      s = s + string( "[" );
      for( UInt4 d=0; d<PutSize(t)-1; d++ ){
	s = s + st->DoubleToString( V[d] ) + string( "," );
      }
      s = s + st->DoubleToString( V[PutSize(t)-1] ) + string( "]" );
      vs.push_back( s );
    }
    else{
      vs.push_back( string( "The size of vector is zero." ) );
    }
  }

  CppToPython *cp = new CppToPython();
  PyObject *p = cp -> VectorStringToList( vs );
  delete cp;
  delete st;
  return p;
}
////////////////////////////////////////////////////////
void ElementContainer::
DumpValue()
{ 
  UInt4 TableSize = PutTableSize();
  for( UInt4 t=0; t<TableSize; t++ ){
    string key   = PutName( t );
    UInt4  depth = PutSize( t );
    string unit  = UnitHeader->PutString( key );
    
    cout << t     << "\t"
	 << key   << "\t"
	 << depth << "\t"
	 << unit  << "\t";
    
    if( depth >= 1){
      vector<Double> V = Put( t );
      cout << "[";
      for( UInt4 d=0; d<depth-1; d++ ){
	cout << V[d] << ",";
      }
      cout << V[depth-1] << "]" << endl;
    }
    else{
      cout << "The size of vector is zero." << endl;
    }
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
DumpFromVectorContainer()
{
  UInt4 TableSize = PutTableSize();
  for( UInt4 t=0; t<TableSize; t++ ){
    string key   = PutName( t );
    UInt4  depth = PutSize( t );
    string unit  = UnitHeader->PutString( key );
    
    cout << key   << "\t"
	 << depth << "\t";
    
    if( depth >= 1){
      vector<Double> V = Put( t );
      cout << "[";
      for( UInt4 d=0; d<depth-1; d++ ){
	cout << V[d] << ",";
      }
      cout << V[depth-1] << "]" << endl;
    }
    else{
      cout << "The size of vector is zero." << endl;
    }
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
Dump( UInt4 size )
{
  cout << "*** header object start" << endl;
  header -> dump();
  cout << "*** header object end"   << endl;
  cout << endl;

  UInt4 TableSize = PutTableSize();
  cout << "The number of vectors is ";
  cout << TableSize << endl;
  cout << endl;

  cout << "x key = "
       << Xkey << endl;
  cout << "y key = "
       << Ykey << endl;
  cout << "e key = "
       << Ekey << endl;
  cout << endl;

  cout << "Index" << "\t"
       << "Key"  << "\t"
       << "Size"   << "\t"
       << "Unit" << "\t"
       << "Values" << endl;
  DumpValue( size );
}
////////////////////////////////////////////////////////
void ElementContainer::
DumpValue( UInt4 size )
{

  UInt4 TableSize = PutTableSize();
  if( size == 0 ){
    for( UInt4 t=0; t<TableSize; t++ ){
      string key   = PutName( t );
      UInt4  depth = PutSize( t );
      string unit = UnitHeader -> PutString( key );
      
      cout << t     << "\t"
	   << key   << "\t"
	   << depth << "\t"
	   << unit  << "\t"
	   << "[...]" << endl;
    }
  }

  else{
    for( UInt4 t=0; t<TableSize; t++ ){
      string key   = PutName( t );
      UInt4  depth = PutSize( t );
      string unit = UnitHeader -> PutString( key );
      
      cout << t     << "\t"
	   << key   << "\t"
	   << depth << "\t"
	   << unit  << "\t[ ";
      
      if( depth >= 1 && depth <= size ){
	vector<Double> V = Put( t ); 
	for( UInt4 d=0; d<depth-1; d++ ){
	  cout << V[d] << ",";
	}
	cout << V[depth-1] << " ]" << endl;
      }
      
      else if( depth >= 1 && depth > size ){
	vector<Double> V = Put( t ); 
	for( UInt4 d=0; d<size-1; d++ ){
	  cout << V[d] << ",";
	}
	cout << V[size-1] << " ,..... ]" << endl;
      }
      
      else{
	cout << "The size of vector is zero.]" << endl;
      }
    }
  }
}
////////////////////////////////////////////////////////
void ElementContainer::
DumpFromVectorContainer( UInt4 size )
{
  UInt4 TableSize = PutTableSize();
  if( size == 0 ){
    for( UInt4 t=0; t<TableSize; t++ ){
      string key   = PutName( t );
      UInt4  depth = PutSize( t );
      string unit = UnitHeader -> PutString( key );
      
      cout << key   << "\t"
	   << depth << "\t"
	   << "[...]" << endl;
    }
  }

  else{
    for( UInt4 t=0; t<TableSize; t++ ){
      string key   = PutName( t );
      UInt4  depth = PutSize( t );
      
      cout << key   << "\t"
	   << depth << "\t[ ";
      
      if( depth >= 1 && depth <= size ){
	vector<Double> V = Put( t );
	for( UInt4 d=0; d<depth-1; d++ ){
	  cout << V[d] << ",";
	}
	cout << V[depth-1] << " ]" << endl;
      }
      
      else if( depth >= 1 && depth > size ){
	vector<Double> V = Put( t ); 
	for( UInt4 d=0; d<size-1; d++ ){
	  cout << V[d] << ",";
	}
	cout << V[size-1] << " ,..... ]" << endl;
      }
      
      else{
	cout << "The size of vector is zero.]" << endl;
      }
    }
  }
}


/**********************************************
 *** 11. StringToInt, IntToString, DoubleToString
 **********************************************/

// StringToInt, IntToString, DoubleToString were removed

/**********************************************
 *** 12. series of Put()                    ***
 **********************************************/

////////////////////////////////////////////////////////
vector<Double> ElementContainer::
PutX()
{
  if( Xkey != "None" ){
    return M.Put( Xkey );
  }

  cout << "ElementContainer::PutX()"   << endl;
  cout << "The X-key is not assigned." << endl;
  cout << "Returns a blanc vector."    << endl;

  vector<Double> blanc;
  blanc.clear();
  return blanc;
}
/////////////////////////////////////////////////////////
vector<Double> ElementContainer::
PutY()
{
  if( Ykey != "None" ){
    return M.Put( Ykey );
  }

  cout << "ElementContainer::PutY()"   << endl;
  cout << "The Y-key is not assigned." << endl;
  cout << "Returns a blanc vector."    << endl;

  vector<Double> blanc;
  blanc.clear();
  return blanc;
}
/////////////////////////////////////////////////////////
vector<Double> ElementContainer::
PutE()
{
  if( Ekey != "None" ){
    return M.Put( Ekey );
  }

  cout << "ElementContainer::PutE()"   << endl;
  cout << "The E-key is not assigned." << endl;
  cout << "Returns a blanc vector."    << endl;

  vector<Double> blanc;
  blanc.clear();
  return blanc;
}
/////////////////////////////////////////////////////////
string ElementContainer::
PutXYEKeys()
{
  string st;
  st = Xkey + string( "," ) 
     + Ykey + string( "," )
     + Ekey;
  return st;
}
/////////////////////////////////////////////////////////
string ElementContainer::
PutMergedKey()
{
  string st;
  st.clear();

  vector<string> vs = M.PutKeyList();

  UInt4 size = (UInt4)(vs.size());
  if( size == 0 ){
    st = "NULL";
  }
  else{
    for( UInt4 i=0; i<size-1; i++){
      st = st + vs[i] + string( "," );
    }
    st = st + vs[ size-1 ];
  }
  return st;
}
/////////////////////////////////////////////////////////
vector<UInt4> ElementContainer::
PutSizeVector()
{
  vector<string> vs = M.PutKeyList();
  UInt4 Size = (UInt4)(vs.size());
  vector<UInt4> SizeVector( Size );
  for( UInt4 i=0; i<Size; i++ ){
    SizeVector[ i ] = (UInt4)(M( vs[ i ] )->size());
  }
  return SizeVector;
}
/////////////////////////////////////////////////////////
vector<Double> ElementContainer::
PutMergedDataVector()
{
  UInt4 Total = 0;
  
  vector<UInt4> SizeVector = PutSizeVector();
  UInt4 VecSize = (UInt4)(SizeVector.size());
  for( UInt4 i=0; i<VecSize; i++ ){
    Total = Total + SizeVector[ i ];
  }

  UInt4 Counter = 0;
  vector<Double> MergedDataVector( Total );
  vector<string> vs = M.PutKeyList();
  for( UInt4 i=0; i<vs.size(); i++ ){
    vector<Double> *target = M( vs[ i ] );
    UInt4 TargetSize = (UInt4)(target -> size());
    for( UInt4 j=0; j<TargetSize; j++ ){
      MergedDataVector[Counter++] = ( *target )[ j ];
    }
  }
  return MergedDataVector;
}
///////////////////////////////////////////////////////// 
string ElementContainer::
PutUnit( const string &Key ) const
{
  if( UnitHeader->CheckKey( Key ) != 1 ){
    return "NoKey";
  }
  return UnitHeader->PutString( Key );
}


/**********************************************
 *** 13. others                             ***
 **********************************************/
/////////////////////////////////////////////////////////
void ElementContainer::
SaveToBinFile( string key, string FileName ){
  NeutronWriteBinaryData<Double> *nw 
    = new NeutronWriteBinaryData<Double>();
  nw -> Write( Put( key ), FileName );
  delete nw;
}
/////////////////////////////////////////////////////////
void ElementContainer::
ReadBinFile( string FileName, string key ){
  NeutronReadBinaryData<Double> *nr = 
    new NeutronReadBinaryData<Double>();
  Add( key, nr->ReadData( FileName ) );
  delete nr;
}
/////////////////////////////////////////////////////////
vector<Double> ElementContainer::
ReduceColumn( const string &Key )
{
  if( CheckKey( Key ) != 1 ){
    cout << "ElementContainer::ReduceColumn(string)" 
	 << endl;
    cout << Key << " is not found in this container." << endl;
    vector<Double> blanc;
    blanc.clear();
    return blanc;
  }
  vector<Double> vec = Put( Key );
  UInt4 size = (UInt4)(vec.size());

  vector<Double> NewVec( size-1 );
  for( UInt4 i=0; i<size-1; i++ ){
    NewVec[i] = ( vec[i] + vec[i+1] ) / 2.0;
  }
  
  return NewVec;
}
/////////////////////////////////////////////////////////
void ElementContainer::
BuildElementContainer( string XYEKeys, 
		       string MergedKey, 
		       vector<UInt4> SizeVector, 
		       vector<Double> MergedDataVector,
		       HeaderBase h,
		       HeaderBase uh )
{
  SplitString *ss = new SplitString();

  if( MergedKey == "NULL" || SizeVector.size() == 0 
      || MergedDataVector.size() == 0 ){
    //cout << "ElementContainer::BuildElementContainer(.....)" << endl;
    //cout << "Data is not contained in the arguments." << endl;
    return;
  }
  
  vector<string> MergedKeyVector = ss -> Split( MergedKey, "," );
  if( MergedKeyVector.size() != SizeVector.size() ){
    cout << "ElementContainer::BuildElementContainer(.....)" << endl;
    cout << "The data size of the second argument ";
    cout << "is inconsistent with the third argumet. ";
    cout << "Please check them !!" << endl;
    return;
  }

  UInt4 Total = 0;
  UInt4 SizeVectorSize = (UInt4)(SizeVector.size());
  for( UInt4 i=0; i<SizeVectorSize; i++ ){
    Total = Total + SizeVector[ i ];
  }
  if( Total != MergedDataVector.size() ){
    cout << "ElementContainer::BuildElementContainer(.....)" << endl;
    cout << "The data size of the third argument is inconsistent ";
    cout << "with the fourth argument. Please check them !!" << endl;
    return;
  }
  

  UInt4 Counter = 0;
  for( UInt4 i=0; i<SizeVectorSize; i++ ){
    UInt4 TargetSize = SizeVector[ i ];
    vector<Double> Target( TargetSize );
    for( UInt4 j=0; j<TargetSize; j++ ){
      Target[ j ] = MergedDataVector[ Counter++ ];
    }
    M.Add( MergedKeyVector[ i ], Target );
  }
  vector<string> xye = ss -> Split( XYEKeys, "," );
  if( xye[0] != "None" || 
      xye[1] != "None" || 
      xye[2] != "None" ){
    SetKeys( xye[0], xye[1], xye[2] );
  }
  InputHeader( h ); 
  delete UnitHeader;
  UnitHeader = new HeaderBase();
  *UnitHeader = uh;

  delete ss;
}
///////////////////////////////////////////////////////// 
void ElementContainer::
SetUnit( const string &Key, const string &NewUnit )
{
  if( UnitHeader->CheckKey( Key ) != 1 ){
    cout << Key << " cannot be found in this container." << endl;
    return;
  }
  UnitHeader->OverWrite( Key, NewUnit );
}
///////////////////////////////////////////////////////// 

/**********************************************
 *** 14. sum, integral, average, min, max   ***
 **********************************************/


/** binary function object used in ElementContainer::Ave() and ElementContainer::Integrate()
 *
 * S1 is used to calculate
 * (E[i] * (X[i+1] - X[i]))^2
 * instead of normal inner_product which calculates E[i]*(X[i+1]-X[i])
 */
/* typename are needed to compile with gcc */
template <class T> class S1 : public binary_function<T, T, T>
{
public:
  typedef typename binary_function<T, T, T>::result_type result_type;
  typedef typename binary_function<T, T, T>::first_argument_type first_argument_type;
  typedef typename binary_function<T, T, T>::second_argument_type second_argument_type;
  result_type operator()(first_argument_type arg1, second_argument_type arg2) {
    return arg1*arg1*arg2*arg2;
  }
};
/** binary function object used in ElementContainer::Ave() and ElementContainer::Integrate()
 *
 * S2 is used to calculate 1 / (Error[i] * Error[i])
 */
template <class T> class S2 : public binary_function<T, T, T>
{
public:
  typedef typename binary_function<T, T, T>::result_type result_type;
  typedef typename binary_function<T, T, T>::first_argument_type first_argument_type;
  typedef typename binary_function<T, T, T>::second_argument_type second_argument_type;
  result_type operator()(first_argument_type arg1, second_argument_type arg2) {
    return 1 / (arg1 * arg2);
  }
};
/** binary function object used in ElementContainer::Ave() and ElementContainer::Integrate()
 *
 * S3 is used to calculate T[i] / (Error[i] * Error[i])
 */
template <class T> class S3 : public binary_function<T, T, T>
{
public:
  typedef typename binary_function<T, T, T>::result_type result_type;
  typedef typename binary_function<T, T, T>::first_argument_type first_argument_type;
  typedef typename binary_function<T, T, T>::second_argument_type second_argument_type;
  result_type operator()(first_argument_type arg1, second_argument_type arg2) {
    return arg1 / (arg2 * arg2);
  }
};

///////////////////////////////////////////////////////// 
/** function class to calculate summation with Kahan's algorithm
 * http://en.wikipedia.org/wiki/Kahan_summation_algorithm
 *
 */
class KahanSum {
private:
  double c;
public:
  KahanSum() : c(0.0) {};

  /** performs Kahan's Compensated Summation; it is designed to used from accumulate() as a function object
   *
   * @param  s initial value 
   * @param  d value to add to s
   * @return summation result
   */
  double operator()(double &s, const double &d) {
	double y = d - c;
	double t = s + y;
	c = (t-s) - y;
	return t;
  }
};



/////////////////////////////////////////////////////////
Double ElementContainer::
Sum( const string &key, const UInt4 i0, const UInt4 i1 ) const {
  if (!(i0<i1)) {
    cout << "ElementContainer::Sum(UInt4, UInt4)" << endl;
	cout << "i0 should be smaller than i1" << endl;
  }
  //if ((i0<0) || (i1>(M.Size(key)-1)))
  if ((i0<0) || (i1>(M.Size(key)))) { //[inamura 140228]
    cout << "ElementContainer::Sum(UInt4, UInt4)" << endl;
	cout << "i0 and i1 should be in the range of array index" << endl;
  }
  /*
  const vector<Double> *Target = M( key ); 
  
  if (safesum) {
	class KahanSum K;
	return accumulate(&((*Target)[i0+1]), &((*Target)[i1]),
					  (*Target)[i0], K);
  }
  else
  return accumulate(&((*Target)[i0]), &((*Target)[i1]), 0.0);
  */
  vector<Double>::const_iterator it = M( key )->begin();
  if (safesum) {
      class KahanSum K;
      return accumulate( it+i0+1, it+i1, *(it+i0), K );
  }
  else{
      return accumulate( it+i0, it+i1, 0.0 );
  }
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Sum( const string &key ) const {

  //return Sum(key, 0, M.Size(key)-1); 
  return Sum(key, 0, M.Size(key));   //[inamura 140228]
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Sum( void ) const {
  if ( Ykey == "None" ) {
    cout << "ElementContainer::Sum( void )" << endl;
    cout << "Ykey have not been set." << endl;
	return 0;
  }
  return Sum( Ykey );
}
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Sum( const string &key, const string &err_key, UInt4 i0, UInt4 i1 ) const {

  std::pair<Double, Double> r;
 
  const vector<Double> *Error  = M( err_key ); 

  if( M( key )->size() != Error->size() ){
    cout << "ElementContainer::Sum(string,string)" << endl;
    cout << "The size of Target and Error vector should be equal." << endl;
    return r;
  }
  
  r.first = Sum( key, i0, i1 );

  if (safesum) {

	std::vector<double> e(&((*Error)[i0]), &((*Error)[i1]));
	for (unsigned int i=0;i<e.size();++i) e[i] *= e[i];

	class KahanSum K;
	Double t = accumulate(&(e[1]), &(e[e.size()]), e[0], K);
	r.second = sqrt( t );

  } else {
  // Error is added as a inner product sum{E[i]*E[i]}
  Double t = inner_product(&((*Error)[i0]), &((*Error)[i1]), &((*Error)[i0]), 0.0);

  r.second = sqrt( t );
  }

  return r;
}
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Sum( const string &key, const string &err_key ) const {
  return Sum(key, err_key, 0, M.Size(key)-1 );
}
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Sum2( void ) const {
  if ( ( Ykey == "None" ) || ( Ekey == "None" ) ) {
    cout << "ElementContainer::Sum( void )" << endl;
    cout << "Ykey or Ekey have not been set." << endl;
	std::pair<Double, Double> v(2, 0.0);
	return v;
  }
  return Sum( Ykey, Ekey );
}
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Sum( Double ini, Double fin ) const {
  std::pair<Double, Double> r;

  if( ini >= fin ){
    cout << "ElementContainer::Sum(Double,Double)" << endl;
    cout << "The values of first and second arguments are wrong." << endl;
    return r;
  }

  if( Xkey == "None" || Ykey == "None" || Ekey == "None" ){
    cout << "ElementContainer::Sum(Double,Double)" << endl;
    cout << "Xkey, Ykey and Ekey have not been set." << endl;
    return r;
  }

  const vector<Double> *X = M(Xkey);
  UInt4 i0=0, i1=M.Size(Xkey)-1;
  while ((*X)[i0] < ini ) ++i0;
  while ((*X)[i1] > fin ) --i1;

  return Sum( Ykey, Ekey, i0, i1);
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Ave( const string &key ){
  return Sum( key ) / ( (Double)( M( key )->size() ) );
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Ave( const string &key, const string &err ){

  vector<Double> *Target = M( key );
  vector<Double> *Error  = M( err );

  if( Target->size() != Error->size() ){
    cout << "ElementContainer::Ave(string,string)" << endl;
    cout << "The size of Target and Error vector should be equal." << endl;
    return 0.0;
  }

  vector<Double>::iterator zero = find(Error->begin(), Error->end(), 0.0);
  if (zero != Error->end()) {
      cout << "ElementContainer::Ave(string,string)" << endl;
      cout << "Each error value shuld not be zero."  << endl;
      return 0.0;
  }

  Double A = 0.0;
  Double B = 0.0;

  // A = sum{T[i]/(E[i]*E[i])}
  A = inner_product(Target->begin(), Target->end(), Error->begin(), 0.0,
		      plus<Double>(), S3<Double>() );
  // B = sum{1/(E[i]*E[i])}
  B = inner_product(Error->begin(), Error->end(), Error->begin(), 0.0,
		      plus<Double>(), S2<Double>() );

  return A / B;
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Min( const string &key ){
  vector<Double> *target = M( key );

  vector<Double>::iterator Min;
  Min = min_element(target->begin(), target->end());

  return *Min;
}
/////////////////////////////////////////////////////////
Double ElementContainer::
Max( const string &key ){
  vector<Double> *target = M( key );

  vector<Double>::iterator Max;
  Max = max_element(target->begin(), target->end());

  return *Max;
}
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Integrate(){
  return Integrate( 0, (Double)(M(Ykey)->size()-1) );
}
/////////////////////////////////////////////////////////
/*
vector<Double> ElementContainer::
Integrate( UInt4 ini, UInt4 fin ){
  vector<Double> r;
  r.clear();

  if( ini >= fin ){
    cout << "ElementContainer::Integrate(UInt4,UInt4)" << endl;
    cout << "The values of first and second arguments are wrong." << endl;
    return r;
  }

  if( Xkey == "None" || Ykey == "None" || Ekey == "None" ){
    cout << "ElementContainer::Integrate(UInt4,UInt4)" << endl;
    cout << "Xkey, Ykey and Ekey have not been set." << endl;
    return r;
  }

  vector<Double> *X = M(Xkey);
  vector<Double> *Y = M(Ykey);
  vector<Double> *E = M(Ekey);

  if( Y->size() <= fin ){
    cout << "ElementContainer::Integrate(UInt4,UInt4)" << endl;
    cout << "The second argument is too large." << endl;
    return r;
  }
  
  vector<Double> bin;
  bin.resize(X->size());
  // bin[i] = X[i]-X[i-1]; bin[0]=0
  adjacent_difference(X->begin(), X->end(), bin.begin());

  vector<Double>::iterator pbin = bin.begin();
  ++pbin;

  Double sum = 0.0;
  Double err = 0.0;

  // sum = sum{Y[i]*(X[i+1]-X[i])}
  sum = inner_product(Y->begin(), Y->end(), pbin, 0.0);
  // err = sum{(E[i]*(X[i+1]-X[i]))^2}
  err = inner_product(E->begin(), E->end(), pbin, 0.0,
		      plus<Double>(), S1<Double>() );

  r.push_back( sum );
  r.push_back( sqrt( err ) );

  return r;
}
*/
/////////////////////////////////////////////////////////
std::pair<Double, Double> ElementContainer::
Integrate( Double ini, Double fin ){
  std::pair<Double, Double> r;

  if( ini >= fin ){
    cout << "ElementContainer::Integrate(Double,Double)" << endl;
    cout << "The values of first and second arguments are wrong." << endl;
    return r;
  }

  if( Xkey == "None" || Ykey == "None" || Ekey == "None" ){
    cout << "ElementContainer::Integrate(Double,Double)" << endl;
    cout << "Xkey, Ykey and Ekey have not been set." << endl;
    return r;
  }

  vector<Double> *X = M(Xkey);
  vector<Double> *Y = M(Ykey);
  vector<Double> *E = M(Ekey);

  double sum = 0.0;
  double err = 0.0;
  double area = 0.0;
  
  for (UInt4 i=0;i<((*X).size()-1); ++i){
      if ( ( (*X)[i]>=ini )&&((*X)[i+1]<=fin) ){
	  sum += (*Y)[i]*fabs((double)((*X)[i+1]-(*X)[i]));
	  area = (*E)[i]*fabs((double)((*X)[i+1]-(*X)[i]));
	  err += area*area;
      }
  }

  r.first = sum;
  r.second = sqrt( err );

  return r;
}

//////////////////////////////////////////////////////////
string ElementContainer::
PutName( UInt4 IndexNumber ){
  if( IndexNumber >= PutSize() ){
    cout << "ElementContainer::PutName(UInt4)" << endl;
    cout << "The argument is too large." << endl;
    return "None";
  }
  return (M.PutKeyList())[IndexNumber];
}
//////////////////////////////////////////////////////////
vector<string> ElementContainer::
PutKeys(){
  UInt4 s = PutSize();
  vector<string> vs;
  vs.clear();
  for( UInt4 i=0; i<s; i++ ){
    vs.push_back( PutName( i ) );
  }
  return vs;
}
//////////////////////////////////////////////////////////
PyObject * ElementContainer::
PutKeysList(){
  CppToPython *cp = new CppToPython();
  PyObject *r = cp -> VectorStringToList( PutKeys() );
  delete cp;
  return r;
}
//////////////////////////////////////////////////////////
PyObject * ElementContainer::
PutHistKeyList(){
  CppToPython *cp = new CppToPython();
  vector<string> vs;
  vs.clear();
  vs.push_back( Xkey );
  vs.push_back( Ykey );
  vs.push_back( Ekey );
  PyObject *r = cp -> VectorStringToList( vs );
  delete cp;
  return r;
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveTextFile( string FileName ){
    SaveTextFile( FileName, -1, ',' );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveTextFile( string FileName, Int4 prec ){
    SaveTextFile( FileName, prec, ',' );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveTextFile( string FileName, Char deli ){
    SaveTextFile( FileName,-1, deli );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveTextFile( string FileName, Int4 prec, Char deli ){
  SimpleIOVectorToTextFile *ss = new SimpleIOVectorToTextFile();
  
  vector<Double> xv = PutX();
  vector<Double> yv = PutY();
  vector<Double> ev = PutE();
  ss->SetTarget(xv,yv,ev);
  ss->SaveFile( FileName, prec, deli );
  delete ss;
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveHistTextFile( string FileName ){
    SaveHistTextFile( FileName, -1, ',' );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveHistTextFile( string FileName, Int4 prec ){
    SaveHistTextFile( FileName, prec, ',' );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveHistTextFile( string FileName, Char deli ){
    SaveHistTextFile( FileName,-1, deli );
}
//////////////////////////////////////////////////////////
void ElementContainer::
SaveHistTextFile( string FileName, Int4 prec, Char deli ){
  SimpleIOVectorToTextFile *ss = new SimpleIOVectorToTextFile();
  
  vector<Double> xv = PutX();
  vector<Double> yv = PutY();
  vector<Double> ev = PutE();
  ss->SetTarget(xv,yv,ev, true);
  ss->SaveFile( FileName, prec, deli );
  delete ss;
}
//////////////////////////////////////////////////////////
void ElementContainer::
LoadTextFile( string FileName ){
  SimpleIOVectorToTextFile *ss = new SimpleIOVectorToTextFile();
  
  ss->LoadFile( FileName );
  
  Add("x",ss->PutX());
  Add("y",ss->PutY());
  Add("e",ss->PutE());
  SetKeys("x","y","e");
  
  delete ss;
}
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
ElementContainer &ElementContainer::
MulMySelf( Double d )
{ // Multiply itself and returns (*this)
  vector<Double> &YVec = M.ref( PutYKey() );
  vector<Double> &EVec = M.ref( PutEKey() );
  UInt4 Size = (UInt4)(YVec.size());
  for( UInt4 i=0; i<Size; i++ ){
    (YVec)[i] *= d;
    (EVec)[i] *= d;
  }
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer::
PlusMySelf( Double d )
{ // Plus itself and returns (*this)
  vector<Double> &YVec = M.ref( PutYKey() );
  vector<Double> &EVec = M.ref( PutEKey() );
  UInt4 Size = (UInt4)(YVec.size());
  for( UInt4 i=0; i<Size; i++ ){
    (YVec)[i] += d;
  }
  return *this;
}
////////////////////////////////////////////////////////////
ElementContainer &ElementContainer::
PowMySelf( Double d )
{ // Power itself and returns (*this)
  vector<Double> &YVec = M.ref( PutYKey() );
  vector<Double> &EVec = M.ref( PutEKey() );
  UInt4 Size = (UInt4)(YVec.size());
  for( UInt4 i=0; i<Size; i++ ){
    double y = (YVec)[i];
    double e = (EVec)[i];
    (YVec)[i] = pow(y, d);
    double ed = d*pow(y, (d-1));
    (EVec)[i] = fabs( ed*e );
  }
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Mul( Double d )
{
  ElementContainer result (*this);
  return result.MulMySelf(d);
}
///////////////////////////////////////////////////////
ElementContainer ElementContainer::
Plus( Double d )
{
  ElementContainer result (*this);
  return result.PlusMySelf(d);
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Pow( Double d )
{
  ElementContainer result (*this);
  return result.PowMySelf(d);
}
///////////////////////////////////////////////////////
ElementContainer &ElementContainer::
MulMySelf( Double d, Double e )
{
  vector<Double> &ly = M.ref( PutYKey() );
  vector<Double> &le = M.ref( PutEKey() );
  UInt4 size = (UInt4)(le.size());

  for( UInt4 i=0; i<size; i++ ){
    double r0 = e * ly[i];
    double r1 = le[i] * d;
    le[i] = sqrt( r0*r0 + r1*r1 );
  }
  for( UInt4 i=0; i<size; i++ ){
    ly[i] *= d;
  }
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer &ElementContainer::
PlusMySelf( Double d, Double e )
{
  vector<Double> &ly = M.ref( PutYKey() );
  vector<Double> &le = M.ref( PutEKey() );
  UInt4 size = (UInt4)(le.size());

  for( UInt4 i=0; i<size; i++ ){ ly[i] += d; }

  Double r2 = e * e;
  for( UInt4 i=0; i<size; i++ ){ le[i] *= le[i]; }
  for( UInt4 i=0; i<size; i++ ){ le[i] += r2; }
  for( UInt4 i=0; i<size; i++ ){ le[i] = sqrt( le[i] ); }
  
  return *this;
}
////////////////////////////////////////////////////////
ElementContainer ElementContainer::
Mul( Double d, Double e )
{
  ElementContainer result (*this);
  return result.MulMySelf(d,e);
}
///////////////////////////////////////////////////////
ElementContainer ElementContainer::
Plus( Double d, Double e )
{
  ElementContainer result (*this);
  return result.PlusMySelf(d,e);
}
///////////////////////////////////////////////////////
void ElementContainer::
HistToScat(){

  if( isHist() != true ){
    cout << "This is NOT histogram type data." << endl;
    return;
  }

  vector<Double> *X = PutP( PutXKey() );
  vector<Double> *Y = PutP( PutYKey() );
  vector<Double> *E = PutP( PutEKey() );

  vector<Double> *newX = new vector<Double>();
  for( UInt4 i=0; i<X->size()-1; i++ ){
    Double ini = (*X)[ i   ];
    Double fin = (*X)[ i+1 ];
    newX->push_back( ( ini + fin ) / 2.0 );
    
    (*Y)[ i ] = (*Y)[ i ] / fabs( fin - ini );
    (*E)[ i ] = (*E)[ i ] / fabs( fin - ini );
  }

  string xkey = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();

  Copy( PutXKey(), PutXKey()+"_hist" );
  Remove( PutXKey() );
  Add( xkey, *newX );
  SetKeys( xkey, ykey, ekey );
  delete newX;
}
//////////////////////////////////////////////////////
void ElementContainer::
ScatToHist(){

  if( isHist() == true ){
    cout << "This is histogram type data." << endl;
    return;
  }

  if( CheckKey( PutXKey() + "_hist" ) != 1 ){
    cout << "This coordinate type data cannot return to a histogram type data." << endl;
    cout << "Is a vector object named " 
	 << PutXKey() << "_hist in this container ?" << endl;
    return;
  }

  vector<Double> *X = PutP( PutXKey() + "_hist" );
  vector<Double> *Y = PutP( PutYKey() );
  vector<Double> *E = PutP( PutEKey() );
  
  for( UInt4 i=0; i<X->size()-1; i++ ){
    Double ini = (*X)[ i   ];
    Double fin = (*X)[ i+1 ];
    
    (*Y)[ i ] = (*Y)[ i ] * fabs( fin - ini );
    (*E)[ i ] = (*E)[ i ] * fabs( fin - ini );
  }

  string CoordinateX = PutXKey();
  string ykey = PutYKey();
  string ekey = PutEKey();

  Remove( CoordinateX );
  Copy( CoordinateX + "_hist", CoordinateX );
  Remove( CoordinateX + "_hist" );
  SetKeys( CoordinateX, ykey, ekey );
}

bool ElementContainer::
RenameKey( const string &OldKey, const string & NewKey ){

  if( !( CheckKey( OldKey ) == 1 && CheckKey( NewKey ) == 0 ) ){
    return false;
  } // キーのチェックをする
  
  UnitHeader -> Add( NewKey, UnitHeader->PutString( OldKey ) ); // unitのコピー
  M.Copy( OldKey, NewKey ); // コピーをして
  M.Erase( OldKey );        // 不要なものを消す

  if( !( Xkey == "None" ) && Xkey == OldKey ){ // キーのコピーを必要に応じて行う
    SetKeys( NewKey, Ykey, Ekey );
  }
  if( !( Ykey == "None" ) && Ykey == OldKey ){
    SetKeys( Xkey, NewKey, Ekey );
  }
  if( !( Ekey == "None" ) && Ekey == OldKey ){
    SetKeys( Xkey, Ykey, NewKey );
  }

  return true;
}

/////////////////////////////////////////////////////////////
BinConverter::
BinConverter( vector<Double> &LX, vector<Double> &LY, vector<Double> &LE, 
	      const vector<Double> *RX, const vector<Double> *RY, const vector<Double> *RE ){
  lx = LX;
  ly = LY;
  le = LE;

  rx = RX;
  ry = RY;
  re = RE;
  
}
/////////////////////////////////////////////////////////////
BinConverter::
~BinConverter(){

  delete LBin;
  delete RBin;
  delete al;
  delete ar;

}
////////////////////////////////////////////////////////////
Int4 BinConverter::
prepare(){
  UInt4 lsize = (UInt4)(lx.size());
  UInt4 rsize = (UInt4)(rx->size());

  double lmin = lx[0];
  double lmax = lx[lsize-1];
  double rmin = (*rx)[0];
  double rmax = (*rx)[rsize-1];
      
  if( rmax < lmin || lmax < rmin ){
    // Two histogram are not overraped 
    cout << "operation failed" << endl;
    return -1;
  }

  double min = lmin;
  if( rmin > lmin ){ min = rmin; }
  double max = lmax;
  if( rmax < lmax ){ max = rmax; }
  

  LBin = new vector<Double>();
  RBin = new vector<Double>();
  NewBin = NULL;

  for( UInt4 i=0; i<lsize; i++ ){
	Double val = lx[i];
	if( val >= min && val <= max ){ LBin->push_back( val ); }
      } 
      for( UInt4 i=0; i<rsize; i++ ){
	Double val = (*rx)[i];
	if( val >= min && val <= max ){ RBin->push_back( val ); }
      }

      if( RBin->size() < LBin->size() ){ NewBin = RBin; }
      else{ NewBin = LBin; }

      if( NewBin->size() < 2 ){
	cout << "operation failed, new histogram cannot be created" << endl;
	return -1;
      }

      al = new DoubleBinArrange( lsize-1, (UInt4)(NewBin->size())-1 );
      ar = new DoubleBinArrange( rsize-1, (UInt4)(NewBin->size())-1 );
      al->SetHist( lx,   ly,  le, *NewBin );
      ar->SetHist( *rx, *ry, *re, *NewBin );
      
      al -> Binning();
      ar -> Binning();

      return 1;
}
/////////////////////////////////////////////////////////////

