#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Emaki package
Emaki data container
Revision 0.20
Author: ITO, Takayoshi (CROSS)
Last update: 26th Sep., 2012
"""

import os.path
import numpy as np
import numpy.random
from Emaki.EmakiDecorator import measure_time
import Manyo as mm
########################################
# EmakiDataContainerBase
########################################
class EmakiDataContainerBase(object):
    """
    EmakiDataMonitorFrameで扱うデータコンテナのメソッドを規定するためのベースクラス。
    これを継承して、PutOrdinateをオーバーライドする。
    """
    def __init__(self, **params):
        """
        コンストラクタ。
        パラメータ辞書の作成。
        @param params (dict) パラメータの辞書。
        """
        self.params = dict(params)
        self.is1d = True
        
    def PutParam(self, key):
        """
        与えたkeyに対する値を返す。
        @param key (str) パラメータのキー。
        """
        return self.params.get(key)

    def PutAllParams(self):
        """
        パラメータの辞書を返す。
        """
        return dict(self.params)
        
    def SetParams(self, paramDict):
        """
        パラメータの設定。
        @param paramDict (dict) パラメータの辞書。
        """
        self.params.update(paramDict)

    def Update(self, paramDict):
        """
        パラメータの再設定。
        状況に応じてこれをオーバーライドする。
        SetParamsに続いて、Initializeを行うのが通常の使い方になる。
        @param paramDict (dict) パラメータの辞書。
        """
        self.SetParams(paramDict)

    def Initialize(self):
        """
        パラメータを用いて初期化する。
        Update時にも利用することを想定して、コンストラクタから分離した。
        """
        pass

    def Calculate(self):
        """
        パラメータを用いてデータの計算を行う。
        """
        pass
    
    def PutText(self):
        """
        コメント文字列を返す。
        """
        return ''

    def PutAbscissa(self):
        """
        横軸の値のリスト(or numpy.array, vector<double>, ...)を返す。
        """
        return []
        
    def PutOrdinate(self):
        """
        縦軸の値のリスト(or numpy.array, vector<double>, ...)を返す。
        """
        return []
        
    def PutIntensity(self):
        """
        強度の値のリスト(or numpy.array, vector<double>, ...)を返す。
        デフォルトでは1次元データを念頭にPutOrdinateを呼び出す。
        """
        return self.PutOrdinate()
        
########################################
#　EmakiDataContainer
########################################
class EmakiDataContainer(EmakiDataContainerBase):
    """
    EmakiDataContainerBaseの継承例。
    """
    def PutText(self):
        return "n = %s"%(self.params['n'])

    def PutAbscissa(self):
        return range(int(self.params['n']))
        
    def PutOrdinate(self):
        return numpy.random.standard_normal(int(self.params['n']))

########################################
#　EmakiDataContainer2d
########################################
class EmakiDataContainer2d(EmakiDataContainerBase):
    """
    EmakiDataContainerBaseの継承例。
    """
    def __init__(self):
        EmakiDataContainerBase.__init__(self)
        self.is1d = False

    def PutText(self):
        return "nx = %s, ny = %s"%(self.params['nx'], self.params['ny'])

    def PutAbscissa(self):
        return range(int(self.params['nx']))
        
    def PutOrdinate(self):
        return range(int(self.params['ny']))
        
    def PutIntensity(self):
        nx = int(self.params['nx'])
        ny = int(self.params['ny'])
        a  = numpy.random.standard_normal(nx-1)
        b = a.copy()
        for i in range(1, ny-1):
            b = numpy.vstack( (b, a*(i+1)) )
        return b

class EmakiDC4ECinECM(EmakiDataContainerBase):
    @measure_time
    def Calculate(self):
        fn = str(self.params['filepath'])
        if not os.path.exists(fn):
            raise UserWarning
        r = mm.ReadSerializationFileBinary(fn)
        ecm = mm.ElementContainerMatrix()
        r.Load(ecm)
        del r
        psd_id = int(self.params['psd_id'])
        pixel_id = int(self.params['pixel_id'])
        self._x = ecm(psd_id, pixel_id).ReduceColumnList('TOF')
        self._y = ecm(psd_id, pixel_id).PutYList()

    def PutAbscissa(self):
        return self._x[:]

    def PutOrdinate(self):
        return self._y[:]

    def PutText(self):
        fn = os.path.basename(self.params['filepath'])
        start = str(self.params['psd_id'])
        end = str(self.params['pixel_id'])
        return '%s, PSD no.: %s, Pixel no.: %s'%(fn, start, end)

class EmakiDC4ECM(EmakiDataContainerBase):
    """
    ECMをTOFスライスするDataContainer。
    Parameter list:
        filepath: File path
        start_tof: Start TOF
        end_tof: End TOF
    """
    def __init__(self):
        EmakiDataContainerBase.__init__(self)
        self.is1d = False
        self._x = None
        self._y = None
        self._c = None

    def Update(self, paramDict):
        self.SetParams(paramDict)
        self.Initialize()

    @measure_time
    def Calculate(self):
        fn = str(self.params['filepath'])
        if not os.path.exists(fn):
            raise UserWarning
        r = mm.ReadSerializationFileBinary(fn)
        ecm = mm.ElementContainerMatrix()
        r.Load(ecm)
        del r
        self._x = range(ecm.PutSize()+1)
        self._y = range(ecm(0).PutSize()+1)
        start, end = float(self.params['start_tof']), float(self.params['end_tof'])
        self._c = []
        for i in range(ecm.PutSize()):
            data = []
            for j in range(ecm(i).PutSize()):
                sum_ye = ecm(i, j).Sum(start, end)
                data.append(sum_ye[0])
            self._c.append(data)
            del data
        self._c = np.array(self._c)
        del ecm

    def PutText(self):
        fn = os.path.basename(self.params['filepath'])
        start = str(self.params['start_tof'])
        end = str(self.params['end_tof'])
        return '%s, %s, %s'%(fn, start, end)

    def PutAbscissa(self):
        return self._x[:]

    def PutOrdinate(self):
        return self._y[:]

    def PutIntensity(self):
        return self._c.T

