#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Emaki package
Emaki data monitor frame
Revision 0.16
Author: ITO, Takayoshi (CROSS)
Last update: 30th Sep., 2012
"""

import time
import sys
import numpy as np
import wx
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg
if sys.hexversion >= 0x020500F0:
    import xml.etree.cElementTree as ElementTree
else:
    import elementtree.ElementTree as ElementTree
#from xml.etree import cElementTree

from Emaki import EmakiDataContainer
from Emaki import EmakiElementTreeTools
from Emaki.dialog.EmakiMultiTextEntryDialog import EmakiMultiTextEntryDialog

########################################
# EmakiDataMonitorFrame
########################################
class EmakiDataMonitorFrame(wx.Frame):
    """
    データモニタ用のwx.Frame。
    このクラスをPySimpleAppかで呼び出して利用する。
    """
    ########################################
    def __init__(self, parent=None, title='Emaki data monitor',
                    settings=[["DataContainer", "Emaki.EmakiDataContainer", "EmakiDataContainer"]]):
        """
        @param parent (wx.Window)
        @param title (str) フレームのタイトル。
        @param settings (list) 設定リスト。詳細は次に記す。
        設定自体もリストになっている。
        設定には二種類ある。リストの第一項目で区別する。
        1. データコンテナの設定
        1.0. 'DataContainer'
        1.1. (str) モジュール名
        1.2. (str) クラス名
        1.3. (list) データコンテナ設定ダイアログ用の辞書のリスト
        2. 開始時に表示されるグラフの設定
        2.0. 'Plot'
        2.1. (int) データコンテナを表す整数
        2.2. (dict) データコンテナ設定の辞書
        2.3. (dict) プロット設定の辞書
        """
        wx.Frame.__init__(self, parent, wx.ID_ANY, title=title, size=(400,100))
        self.dc_classes = [] # 選択可能なデータコンテナクラスのリスト。
        self.dc_param_infos = [] # 選択可能なデータコンテナクラスのパラメータのリスト。
        self.data_containers = [] # 表示中のデータコンテナのインスタンスのリスト。
        self.plot_config_menu_ids = [] # 表示中のプロット用configメニューのIDのリスト。 
        self.dc_param_menu_ids = [] # 表示中のプロットのデータコンテナ用parameterメニューのIDのリスト。 
        self.ImportDataContainerClasses(settings) # settingsのクラスをimportする。
        self._InitializeMainFrame() # フレームの初期設定。
        self._MkToolBar() # ツールバーの作成と設定。
        self._MkMenuBar() # メニューバーの作成と設定。
        self._MkStatusBar() # ステータスバーの作成と設定。
        self._MkInitialPlots(settings) # 初期プロットの作成と設定。
        self._MkTimer(settings) # タイマーの作成と設定。
        
    ########################################
    def _InitializeMainFrame(self):
        """
        フレームの初期設定。
        """
        ## プロットを追加していくsizerをあらかじめ作成しておく。
        box = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(box)

    ########################################
    def _MkToolBar(self):
        """
        ツールバーの作成と設定。
        """
        tb = self.CreateToolBar()
        tsize= (24, 24)
        ## 設定ファイルの読み書き。
        open_bmp = wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_TOOLBAR, tsize)
        tb.AddTool(wx.ID_OPEN, bitmap=open_bmp, shortHelpString='Open')
        save_bmp = wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_TOOLBAR, tsize)
        tb.AddLabelTool(wx.ID_SAVE, "Save", bitmap=save_bmp, shortHelp='Save')
        tb.AddSeparator()

        ## プロットの追加と削除。
        # 追加アイコンの画像。
        xpm_add = [
"24 24 4 1",
" 	c None",
".	c #CCCCCC",
"+	c #006400",
"@	c #FFFFFF",
"........................",
"........................",
"........................",
"........................",
"........++++++++........",
"........++++++++@.......",
"........++++++++@.......",
"........++++++++@.......",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....@@@@++++++++@@@@@...",
"........++++++++@.......",
"........++++++++@.......",
"........++++++++@.......",
"........@@@@@@@@@.......",
"........................",
"........................",
"........................"]
        bmp_add = wx.BitmapFromXPMData(xpm_add) # プロット追加アイコンのビットマップ化。
        id_add =wx.NewId()
        tb.AddControl(wx.BitmapButton(tb, id_add, bmp_add, name="Add"))
        tb.SetToolShortHelp(id_add, 'Add a new plot')
        # 削除アイコンの画像。
        xpm_remove = [
"24 24 4 1",
" 	c None",
".	c #CCCCCC",
"+	c #006400",
"@	c #FFFFFF",
"........................",
"........................",
"........................",
"........................",
"........................",
"........................",
"........................",
"........................",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....++++++++++++++++@...",
"....@@@@@@@@@@@@@@@@@...",
"........................",
"........................",
"........................",
"........................",
"........................",
"........................",
"........................"]
        bmp_remove = wx.BitmapFromXPMData(xpm_remove) # プロット削除アイコンのビットマップ化。
        id_remove =wx.NewId()
        tb.AddControl(wx.BitmapButton(tb, id_remove, bmp_remove, name="Remove"))
        tb.SetToolShortHelp(id_remove, 'Remove a plot')
        tb.AddSeparator()

        ## 自動更新の開始と一時停止。
        # 開始アイコンの画像。
        xpm_play = [
"24 24 4 1",
" 	c None",
".	c #CCCCCC",
"+	c #191919",
"@	c #FFFFFF",
"........................",
"........................",
"........................",
"....+...................",
"....+++.................",
"....+++++...............",
"....+++++++.............",
"....+++++++++...........",
"....+++++++++++.........",
"....+++++++++++++.......",
"....+++++++++++++++.....",
"....++++++++++++++++@...",
"....+++++++++++++++@@...",
"....+++++++++++++@@@....",
"....+++++++++++@@@......",
"....+++++++++@@@........",
"....+++++++@@@..........",
"....+++++@@@............",
"....+++@@...............",
"....+@@@................",
"....@@..................",
"........................",
"........................",
"........................"]
        # 一時停止アイコンの画像。
        xpm_pause = [
"24 24 4 1",
" 	c None",
".	c #CCCCCC",
"+	c #191919",
"@	c #FFFFFF",
"........................",
"........................",
"........................",
"....+++++......+++++....",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....+++++@.....+++++@...",
"....@@@@@@.....@@@@@@...",
"........................",
"........................",
"........................"]

        bmp_play = wx.BitmapFromXPMData(xpm_play) # 開始アイコンのビットマップ化。
        self.id_play = wx.NewId()
        tb.AddControl(wx.BitmapButton(tb, self.id_play, bmp_play, name='Play'))
        bmp_pause = wx.BitmapFromXPMData(xpm_pause) # 一時停止アイコンのビットマップ化。
        self.id_pause = wx.NewId()
        tb.AddControl(wx.BitmapButton(tb, self.id_pause, bmp_pause, name='Pause'))
        tb.EnableTool(self.id_play, False) # アプリの開始時は自動更新状態なので開始アイコンをdisableに。

        self.Bind(wx.EVT_BUTTON, self.OnAddPlotDialog, id=id_add)
        self.Bind(wx.EVT_BUTTON, self.OnRemovePlot, id=id_remove)
        self.Bind(wx.EVT_BUTTON, self.OnPlay, id=self.id_play)
        self.Bind(wx.EVT_BUTTON, self.OnPause, id=self.id_pause)
        self.Bind(wx.EVT_TOOL, self.SaveConfig, id=wx.ID_SAVE)
        tb.Realize() # ツール追加後のお約束。

    ########################################
    def _MkMenuBar(self):
        """
        メニューバーの作成と設定。
        """
        ## Menu bar
        menu_bar = wx.MenuBar()

        ## File menu
        file_menu = wx.Menu()
        self.idTest = wx.NewId()
        file_menu.Append(self.idTest, 'Test')
        
        ## Edit menu
        edit_menu = wx.Menu()
        self.id_update_time = wx.NewId()
        edit_menu.Append(self.id_update_time, 'Update rate')
        self.dc_param_menu = wx.Menu()
        edit_menu.AppendSubMenu(self.dc_param_menu, 'Data container parameter')
        self.plot_config_menu = wx.Menu()
        edit_menu.AppendSubMenu(self.plot_config_menu, 'Plot config')

        ## Menu setting
        menu_bar.Append(file_menu, '&File')
        menu_bar.Append(edit_menu, '&Edit')
        self.SetMenuBar(menu_bar)
        
        ## Bind
        self.Bind(wx.EVT_MENU, self.OnTest, id=self.idTest)
        self.Bind(wx.EVT_MENU, self.OnSetUpdateRate, id=self.id_update_time)
     
    ########################################
    def _MkStatusBar(self):
        """
        ステータスバーの作成と設定。
        """
        self.CreateStatusBar()

    ########################################
    def _MkInitialPlots(self, settings):
        """
        初期プロットの作成と設定。
        @settings (list) 設定リスト。詳細はコンストラクタに記す。
        """
        for setting in settings:
            if setting[0] == "Plot":
                self.AddPlot(setting[1],
                                setting[2],
                                **setting[3]
                            )

    ########################################
    def _MkTimer(self, settings=[['Timer', 1]]):
        """
        タイマーの作成と設定。
        """
        for setting in settings:
            if setting[0] == 'Timer':
                self.timer_time = setting[1] # 繰り返し時間 (sec.)
                break
        self.timer = wx.Timer(self)
        self.timer.Start(self.timer_time*1000)
        self.Bind(wx.EVT_TIMER, self.OnTimer)

    ########################################
    def ImportDataContainerClasses(self, settings):
        """
        XML読み込み結果を用いてクラスをimportし、またインスタンス化時パラメータ項目をリストにする。
        """
        for setting in settings:
            if setting[0] == "DataContainer":
                _module_name, _class_name = setting[1:3]
                self.dc_param_infos.append(setting[3:])
                _module = __import__(_module_name,
                    globals(),
                    locals(),
                    [_class_name])
                self.dc_classes.append(getattr(_module, _class_name))
        return

    ########################################
    def OnRemovePlot(self, event):
        """
        プロットの削除。
        """
        dlg = wx.TextEntryDialog(self,
                message = 'Remove a plot',
                caption = 'Enter a remove plot number',
                defaultValue = '1')
        if dlg.ShowModal() == wx.ID_OK:
            try:
                value = int(dlg.GetValue())
            except:
                return
        else:
            return
        i = value - 1
        sz = self.GetSizer()
        children = sz.GetChildren()
        n = len(children)
        if i < n:
            sz_item = sz.GetItem(i)
            pan = sz_item.GetWindow()
            sz.Remove(i) # サイザーから当該パネルを削除して、
            pan.Destroy() # パネルを廃止して、
            del pan # パネルを削除
            del self.data_containers[i] # データコンテナインスタンスを削除
            #sz.Layout()
            fitting_size = sz.ComputeFittingWindowSize(self)
            w, h = self.GetSize()
            self.SetSize((w, fitting_size[1]))
            self.plot_config_menu.Delete(self.plot_config_menu_ids[i]) # plot configメニューから削除
            del self.plot_config_menu_ids[i] # plot config用IDを削除
            # plot configメニューの番号を振りなおす
            for index, menu_item in enumerate(self.plot_config_menu.GetMenuItems()):
                menu_item.SetText('#%d'%(index+1))
        
    ########################################
    def OnAddPlotDialog(self, event):
        """
        プロット追加ダイアログ。
        """
        if len(self.dc_classes) == 0:
            print "[!] No data container setting."
            return
        elif len(self.dc_classes) == 1:
            dc_index = 0
        else:
            dc_index = self.OnDataContainerChoiceDialog()
        if dc_index == None:
            return
        dc_param = self.dc_param_infos[dc_index]
        params = self.OnInitialParameterDialog(dc_param)
        if params == None:
            return
        self.AddPlot(dc_index, params,
                        title='Plot #%d'%(len(self.data_containers)+1),
                        xlabel='x',
                        ylabel='y',
                    )

    ########################################
    def AddPlot(self, data_container_index, data_container_params, **setting):
        """
        プロットパネルを追加する。
        @param data_container_index (int)
        @param data_container_params (list) データコンテナに渡すパラメータ
        @param **setting プロッタに渡す未定義キーワード引数
        """
        dc_index = data_container_index
        dc_class = self.dc_classes[dc_index]
        self.AddPlotConfigMenu()
        self.AddDataContainerParameterMenu()
        pan = wx.Panel(self, wx.ID_ANY, size=(400, 100), style=wx.BORDER_RAISED)
        fig = Figure(figsize=(8,4))
        canvas = FigureCanvasWxAgg(pan, -1, fig)
        ax = fig.add_axes([0.1, 0.1, 0.8, 0.75]) # rect = [left, bottom, width, height]

        dc = dc_class() # データコンテナクラスのインスタンス作成
        dc.SetParams( data_container_params ) # データコンテナインスタンスへのパラメータ設定
        #print "DEBUG:", data_container_params # デバッグ用パラメータの標準出力  
        dc.Initialize()

        dc.Calculate()

        if dc.is1d:
            # 1d plot
            ax.plot(dc.PutAbscissa(), dc.PutOrdinate()) # プロット。
        else:
            # 2d plot
            abscissa = dc.PutAbscissa()
            ordinate = dc.PutOrdinate()
            X, Y = np.meshgrid(abscissa, ordinate)
            quadmesh = ax.pcolormesh(X, Y, dc.PutIntensity()) # プロット。
            cb = fig.colorbar(quadmesh, fraction=0.05, pad=0.01)
        ax.text(0, 1.1, dc.PutText(), transform=ax.transAxes)
        self.data_containers.append(dc)
        ax.relim()
        ax.autoscale_view(scalex=True, scaley=True)
        #ax.autoscale(enable=True, axis='both', tight=False)


        navi_tb = NavigationToolbar2WxAgg(canvas)
        navi_tb.SetWindowStyle(wx.TB_VERTICAL)
        sz_canvas = wx.BoxSizer(wx.HORIZONTAL)
        sz_canvas.Add(canvas)
        sz_canvas.Add(navi_tb)
        pan.SetSizer(sz_canvas)
        sz = self.GetSizer()
        sz.Add(pan)
        #sz.Layout()
        self.SetSize(sz.ComputeFittingWindowSize(self))
        #time.sleep(0.5) # これがないとpanIniの追加がうまく行えない時がある。原因不明。
        self.ConfigurePlot(len(self.data_containers)-1, **setting)

    ########################################
    def ConfigurePlot(self, index, **setting):
        sz = self.GetSizer()
        children = sz.GetChildren()
        sz_item = children[index]
        pan = sz_item.GetWindow()
        sz_canvas = pan.GetSizer()
        for sz_item_canvas in sz_canvas.GetChildren():
            child = sz_item_canvas.GetWindow()
            if isinstance(child, FigureCanvasWxAgg):
                ax = child.figure.axes[0]
                title = setting.get('title')
                xlabel = setting.get('xlabel')
                ylabel = setting.get('ylabel')
                if title is not None:
                    ax.set_title(title)
                if xlabel is not None:
                    ax.set_xlabel(xlabel)
                if ylabel is not None:
                    ax.set_ylabel(ylabel)
                try:
                    xleft, xright = ( float(setting['xmin']), float(setting['xmax']) )
                    ax.set_autoscalex_on(False)
                    ax.set_xlim(xleft, xright, auto=False)
                except:
                    ax.set_autoscalex_on(True)
                    ax.set_xlim(auto=True)
                try:
                    yleft, yright = ( float(setting['ymin']), float(setting['ymax']) )
                    ax.set_autoscaley_on(False)
                    ax.set_ylim(yleft, yright, auto=False)
                except:
                    ax.set_autoscaley_on(True)
                    ax.set_ylim(auto=True)
                try:
                    if setting['yscale'] in ['1', 'yes', 'True', 'Y']:
                        ax.set_yscale('log')
                    else:
                        ax.set_yscale('linear')
                except:
                    pass

    ########################################
    def OnConfigurePlotDialog(self, event):
        event_id = event.GetId()
        index = self.plot_config_menu_ids.index(event_id)
        queries=[
            dict(label='Title', default='Plot #%d'%(index+1), tip='Enter a plot title.', alias='title'),
            dict(label='X-axis label', default='x', tip='Enter a x-axis label.', alias='xlabel'),
            dict(label='Y-axis label', default='y', tip='Enter a y-axis label.', alias='ylabel'),
            dict(label='X min', default='', tip='Enter a x-axis minimum.', alias='xmin'), 
            dict(label='X max', default='', tip='Enter a x-axis maximum.', alias='xmax'), 
            dict(label='Y min', default='', tip='Enter a y-axis minimum.', alias='ymin'), 
            dict(label='Y max', default='', tip='Enter a y-axis maximum.', alias='ymax'), 
            dict(label='Y log scale', default='no', tip='Enter a y-axis scale.', alias='yscale'), 
            ]
        dlg = EmakiMultiTextEntryDialog(self, title='Configure', queries=queries)
        if dlg.ShowModal() == wx.ID_OK:
            rslt = dlg.GetValue()
        else:
            rslt = None
        dlg.Destroy()
        if rslt is not None:
            self.ConfigurePlot(index, **rslt)
        return

    ########################################
    def OnInitialParameterDialog(self, param):
        """
        初期パラメータ設定ダイアログ。
        """
        dlg = EmakiMultiTextEntryDialog(self, title='Parameters', queries=param)
        if dlg.ShowModal() == wx.ID_OK:
            rslt = dlg.GetValue()
        else:
            rslt = None
        dlg.Destroy()
        return rslt

    ########################################
    def OnTimer(self, event):
        """
        タイマーで実行される処理
        """
        sz = self.GetSizer()
        children = sz.GetChildren()
        # プロットパネル全てに対して更新を実行。
        for i, sz_item in enumerate(children):
            pan = sz_item.GetWindow()
            sz_canvas = pan.GetSizer()
            for sz_item_canvas in sz_canvas.GetChildren():
                child = sz_item_canvas.GetWindow()
                if isinstance(child, FigureCanvasWxAgg):
                    ax = child.figure.axes[0]
                    self.data_containers[i].Calculate()
                    if self.data_containers[i].is1d:
                        line = ax.lines[0]
                        line.set_xdata(self.data_containers[i].PutAbscissa())
                        line.set_ydata(self.data_containers[i].PutOrdinate())
                        ax.texts[-1].set_text(self.data_containers[i].PutText())
                    else:
                        ax1 = child.figure.axes[0] # プロットのaxis
                        setting = dict() # axesをクリアする前に、タイトル等を取得
                        setting['title'] = str(ax.get_title())
                        setting['xlabel'] = str(ax.get_xlabel())
                        setting['ylabel'] = str(ax.get_ylabel())
                        if ax.get_autoscalex_on():
                            setting['xmin'], setting['xmax'] = '', ''
                        else:
                            setting['xmin'], setting['xmax'] = ax.get_xlim()
                        if ax.get_autoscaley_on():
                            setting['ymin'], setting['ymax'] = '', ''
                        else:
                            setting['ymin'], setting['ymax'] = ax.get_ylim()
                        ax1.clear() # クリアして、
                        dc = self.data_containers[i]
                        abscissa = dc.PutAbscissa()
                        ordinate = dc.PutOrdinate()
                        X, Y = np.meshgrid(abscissa, ordinate)
                        quadmesh = ax.pcolormesh(X, Y, dc.PutIntensity()) # 再プロット。
                        ax.text(0, 1.1, dc.PutText(), transform=ax.transAxes)
                        ax2 = child.figure.axes[1] # colorbarのaxis
                        ax2.clear()
                        cb = child.figure.colorbar(quadmesh, cax=ax2, ax=child.figure.axes[0]) # 再描画
                        self.ConfigurePlot(i, **setting) # 取得したタイトル等を再設定
                        """
                        collection = ax.collections[0]
                        x = self.data_containers[i].PutAbscissa()
                        y = self.data_containers[i].PutOrdinate()
                        C = self.data_containers[i].PutIntensity()
                        X, Y = np.meshgrid(x, y)
                        Ny, Nx = X.shape
                        collection._meshWidth = Nx-1
                        collection._meshHeight = Ny-1
                        coords = np.zeros(((Nx * Ny), 2), dtype=float)
                        coords[:, 0] = X.ravel()
                        coords[:, 1] = Y.ravel()
                        collection._bbox.update_from_data_xy(coords.reshape( (Nx*Ny, 2) ))
                        coords = coords.reshape( (Ny, Nx, 2) )
                        collection._cordinates = np.array( coords, np.float_)
                        C = C.ravel()
                        collection.set_array(C)
                        """
                    xauto = ax.get_autoscalex_on()
                    yauto = ax.get_autoscaley_on()
                    if xauto and yauto:
                        ax.relim() # 両軸autoならrelimで、limit設定。
                    else: # 片軸のみautoならrelimの前に現在のlimitを保存してからrelim。
                        if xauto:
                            ylim = ax.get_ylim() 
                            ax.relim()
                            ax.set_ylim(ylim)
                        elif yauto:
                            xlim = ax.get_xlim() 
                            ax.relim()
                            ax.set_xlim(xlim)
                    ax.autoscale_view(scalex=xauto, scaley=yauto)
                    #ax.autoscale(enable=True, axis='both', tight=False)
                    child.draw()
                    break
        self.SetStatusText(time.strftime('%H:%M:%S'))

    ########################################
    def OnPlay(self, event):
        tb = self.GetToolBar()
        tb.EnableTool(self.id_pause, True)
        tb.EnableTool(self.id_play, False)
        self.timer.Start()

    ########################################
    def OnPause(self, event):
        tb = self.GetToolBar()
        tb.EnableTool(self.id_pause, False)
        tb.EnableTool(self.id_play, True)
        self.SetStatusText("Pause")
        self.timer.Stop()

    ########################################
    def OnTest(self, event):
        print "Test"

    ########################################
    def OnSetUpdateRate(self, event):
        dlg = wx.TextEntryDialog(self,
            message = "Enter update rate (s)",
            caption = "Update rate",
            defaultValue = "%d"%(self.timer_time), )
        if dlg.ShowModal() == wx.ID_OK:
            try:
                self.timer_time = int(dlg.GetValue())
                tb = self.GetToolBar()
                tb.EnableTool(self.id_pause, True)
                tb.EnableTool(self.id_play, False)
                self.timer.Start(self.timer_time*1000)
            except:
                return
        else:
            return

    ########################################
    def AddPlotConfigMenu(self):
        self.plot_config_menu_ids.append(wx.NewId())
        self.plot_config_menu.Append(self.plot_config_menu_ids[-1], '#%d'%(len(self.plot_config_menu_ids)))
        self.Bind(wx.EVT_MENU, self.OnConfigurePlotDialog, id=self.plot_config_menu_ids[-1])

    ########################################
    def AddDataContainerParameterMenu(self):
        self.dc_param_menu_ids.append(wx.NewId())
        self.dc_param_menu.Append(self.dc_param_menu_ids[-1], '#%d'%(len(self.dc_param_menu_ids)))
        self.Bind(wx.EVT_MENU, self.OnUpdateDataContainerParameterDialog, id=self.dc_param_menu_ids[-1])

    ########################################
    def OnDataContainerChoiceDialog(self):
        """
        データコンテナクラス選択ダイアログ。
        """
        containers = []
        for dc in self.dc_classes:
            containers.append( '%s.%s'%(dc.__module__, dc.__name__) )
        dlg = wx.SingleChoiceDialog(self,
                message='Choice a data container',
                caption='Data container',
                choices = containers,
                )
        if dlg.ShowModal() == wx.ID_OK:
            dc_index = dlg.GetSelection()
        else:
            return None
        return dc_index

    ########################################
    def OnUpdateDataContainerParameterDialog(self, event):
        event_id = event.GetId()
        index = self.dc_param_menu_ids.index(event_id)
        dc = self.data_containers[index]
        for dc_class, dc_param_info in zip(self.dc_classes, self.dc_param_infos):
            if isinstance(dc, dc_class):
                break
        else:
            raise UserWarning, 'Unexpected error at "OnUpdateDataContainerParameterDialog".'
        cur_param = dc.PutAllParams()
        param_dicts = dc_param_info[:]
        for d in param_dicts:
            if d.get('label') in cur_param.keys():
                d['default'] = cur_param[d['label']]
            if d.get('alias') in cur_param.keys():
                d['default'] = cur_param[d['alias']]
        dlg = EmakiMultiTextEntryDialog(self, title='Parameter', queries=param_dicts)
        if dlg.ShowModal() == wx.ID_OK:
            rslt = dlg.GetValue()
        else:
            rslt = None
        dlg.Destroy()
        if rslt is not None:
            dc.Update(rslt)
        return

    ########################################
    def SaveConfig(self, event):
        root = ElementTree.Element("EmakiDataMonitorSetting")
        for index, [dc_class, info] in enumerate(zip(self.dc_classes, self.dc_param_infos)):
            _data_container = ElementTree.Element("dataContainer", {"i":"%d"%index})
            _package = ElementTree.Element("package")
            _module = ElementTree.Element("module")
            module_strings = dc_class.__module__.rsplit(".", 1)
            if len(module_strings) == 1:
                _module.text = module_strings[0]
            else:
                _package.text = module_strings[0]
                _module.text = module_strings[1]
            _data_container.append(_package)
            _data_container.append(_module)
            root.append(_data_container)
        EmakiElementTreeTools.ReconstructForEye(root)
        print ElementTree.tostring(root)
        return
