#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import sys
import time
import glob
import shutil
import math
import numpy as np
import multiprocessing

from Emaki.EmakiDecorator import measure_time
import Emaki.EmakiDetectorViewInterface as DetView
import Emaki.dialog.EmakiSimpleDialog as EmakiSimpleDialog
import Emaki.dialog.EmakiMultiTextEntryDialog as EmakiMultiTextEntryDialog
from Emaki.EmakiCalculator import PutTwoTheta, RotateWXYZ, PutResultOfUserDefinedFormula, PutGauss

if sys.hexversion >= 0x020500F0:
    import xml.etree.cElementTree as ElementTree
else:
    import elementtree.ElementTree as ElementTree

try:
    import vis.MPlot as MP
    import vis.M2Plot as M2P
    HAS_MPLOT = True
except ImportError:
    HAS_MPLOT = False

try:
    import SAS.ana.Reduction.SASMakeParamFiles as mkPrm
    HAS_MAKEPARAM = True
except ImportError:
    HAS_MAKEPARAM = False
import wx
from wx import xrc
import Manyo as mm

########################################
# MyMainApp
########################################
class MyMainApp(wx.PySimpleApp):
    ########################################
    def __init__(self, redirect=False):
        wx.PySimpleApp.__init__(self, redirect)

    ########################################
    def OnInit(self):
        xrc_path = os.path.join(os.path.dirname(os.path.abspath( __file__ )), 'EmakiDetectorViewer.xrc')
	self.res = xrc.XmlResource(xrc_path)
        self.init_frame()
        return 1

    ########################################
    def init_frame(self):
        self._loaded_ecm = None
        self._loaded_filepath = None
        self.det_view = None
        self.frm = self.res.LoadFrame(None, 'main_frame')
        icon = wx.Icon(os.path.join(os.path.dirname(os.path.abspath(__file__)), "emaki_icon.png"), wx.BITMAP_TYPE_PNG)
        self.frm.SetIcon(icon)
        self.init_menu()
        self.init_btn()
        self.init_xrange()
        self.init_cbrange()
        self.init_pixel_frm()
        self.frm.Show()
        main_frame_pos = self.frm.GetPosition()
        main_frame_size = self.frm.GetSize()
        self.pxl_info_frm.MoveXY(main_frame_pos[0], main_frame_pos[1]+main_frame_size[1]+40)
        self.pxl_info_frm.Show()
        self.frm.CreateStatusBar()
        self.frm.SetStatusText("")

    ########################################
    def init_pixel_frm(self):
        xrc_path = os.path.join(os.path.dirname(os.path.abspath( __file__ )), 'EmakiPixelInfoFrame.xrc')
        self.res_pxl_frm = xrc.XmlResource(xrc_path)
        self.pxl_info_frm = self.res_pxl_frm.LoadFrame(self.frm, 'pixel_info_frame')
        icon = wx.Icon(os.path.join(os.path.dirname(os.path.abspath(__file__)), "emaki_icon.png"), wx.BITMAP_TYPE_PNG)
        self.pxl_info_frm.SetIcon(icon)
        self.st_intensity = xrc.XRCCTRL(self.pxl_info_frm, 'st_intensity')
        self.st_x_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_x_pos')
        self.st_y_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_y_pos')
        self.st_z_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_z_pos')
        self.st_tth = xrc.XRCCTRL(self.pxl_info_frm, 'st_tth')
        self.st_azm = xrc.XRCCTRL(self.pxl_info_frm, 'st_azm')
        self.st_rx_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_rx_pos')
        self.st_ry_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_ry_pos')
        self.st_rz_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_rz_pos')
        self.st_bank_name = xrc.XRCCTRL(self.pxl_info_frm, 'st_bank_name')
        self.st_pixel_id = xrc.XRCCTRL(self.pxl_info_frm, 'st_pixel_id')
        self.st_det_id = xrc.XRCCTRL(self.pxl_info_frm, 'st_det_id')
        self.st_pixel_pos = xrc.XRCCTRL(self.pxl_info_frm, 'st_pixel_pos')

        self.btn_pixel_plot_new = xrc.XRCCTRL(self.pxl_info_frm, 'btn_pixel_plot_new')
        self.btn_pixel_plot_append = xrc.XRCCTRL(self.pxl_info_frm, 'btn_pixel_plot_append')
        self.btn_2d_plot = xrc.XRCCTRL(self.pxl_info_frm, 'btn_2d_plot')

        self.btn_pixel_plot_new.Bind(wx.EVT_BUTTON, self.OnPixelPlotNew)
        self.btn_pixel_plot_append.Bind(wx.EVT_BUTTON, self.OnPixelPlotAppend)
        self.btn_2d_plot.Bind(wx.EVT_BUTTON, self.On2dPlot)
        self.mplot = None
        self.picked_pixel = [None, None, None] # [bankId, psdId(detId), pixel_position]

    ########################################
    def init_btn(self):
        self.btn_open = xrc.XRCCTRL(self.frm, 'btn_open')
        if HAS_MAKEPARAM:
            self.btn_open.Bind(wx.EVT_BUTTON, self.OnViewFrameByRunNumber)
        else:
            self.btn_open.Bind(wx.EVT_BUTTON, self.OnViewFrame)

    ########################################
    def init_xrange(self):
        self.tc_xrange_start = xrc.XRCCTRL(self.frm, 'tc_xrange_start')
        self.tc_xrange_end = xrc.XRCCTRL(self.frm, 'tc_xrange_end')
        self.btn_xrange_apply = xrc.XRCCTRL(self.frm, 'btn_xrange_apply')
        if self._loaded_ecm == None:
            self.btn_xrange_apply.Disable()
        self.btn_xrange_apply.Bind(wx.EVT_BUTTON, self.OnApplyXRange)

    ########################################
    def init_cbrange(self):
        self.tc_cb_min = xrc.XRCCTRL(self.frm, 'tc_cb_min')
        self.tc_cb_max = xrc.XRCCTRL(self.frm, 'tc_cb_max')
        self.btn_cb_apply = xrc.XRCCTRL(self.frm, 'btn_cb_apply')
        if self.det_view == None:
            self.btn_cb_apply.Disable()
        self.btn_cb_apply.Bind(wx.EVT_BUTTON, self.OnColorbarRange)

    ########################################
    def init_menu(self):
        self.menu_bar = self.frm.GetMenuBar()
        self.menu_open = self.menu_bar.FindItemById(xrc.XRCID('menu_open'))
        self.menu_exit = self.menu_bar.FindItemById(xrc.XRCID('menu_exit'))
        self.menu_tth = self.menu_bar.FindItemById(xrc.XRCID('menu_tth'))
        self.menu_load_data = self.menu_bar.FindItemById(xrc.XRCID('menu_load_data'))
        #self.menu_colorbar = self.menu_bar.FindItemById(xrc.XRCID('menu_colorbar'))
        self.menu_formula = self.menu_bar.FindItemById(xrc.XRCID('menu_formula'))
        self.menu_tolerance = self.menu_bar.FindItemById(xrc.XRCID('menu_tolerance'))
        self.menu_mask_view = self.menu_bar.FindItemById(xrc.XRCID('menu_mask_view'))
        self.menu_dummy = self.menu_bar.FindItemById(xrc.XRCID('menu_dummy'))
        self.menu_pixel_info = self.menu_bar.FindItemById(xrc.XRCID('menu_pixel_info'))

        self.Bind(wx.EVT_MENU, self.OnViewFrame, id=self.menu_open.GetId())
        self.Bind(wx.EVT_MENU, self.OnExit, id=self.menu_exit.GetId())
        self.Bind(wx.EVT_MENU, self.OnSetTwoTheta, id=self.menu_tth.GetId())
        self.Bind(wx.EVT_MENU, self.OnLoadData, id=self.menu_load_data.GetId())
        #self.Bind(wx.EVT_MENU, self.OnColorbarRange, id=self.menu_colorbar.GetId())
        self.Bind(wx.EVT_MENU, self.OnSetUserDefinedFormula, id=self.menu_formula.GetId())
        self.Bind(wx.EVT_MENU, self.OnSetPickerTolerance, id=self.menu_tolerance.GetId())
        self.Bind(wx.EVT_MENU, self.OnCreateDummyData, id=self.menu_dummy.GetId())
        self.Bind(wx.EVT_MENU, self.OnPixelInfo, id=self.menu_pixel_info.GetId())

        self.menu_mask_view.Check()

    ########################################
    def OnViewFrameByRunNumber(self, event=None):
        queries = [['Run number', '0']]
        dlg = EmakiSimpleDialog.EmakiSimpleDialog(self.frm, title='Run number', queries=queries)
        run_number = None
        if dlg.ShowModal() == wx.ID_OK:
            run_number = dlg.PutItems()[0]
        dlg.Destroy()
        if run_number == None:
            return
        prm = mkPrm.SASMakeParamFiles(int(run_number))
        detInfoPath = prm.DetectorInfo
        #wiringInfoPath = prm.OrgWiringInfo
        wiringInfoPath = prm.WiringInfo  ##[inamrua 161111]
        print detInfoPath, wiringInfoPath
        self.det_view = DetView.EmakiDetectorViewInterface(str(detInfoPath), str(wiringInfoPath))
        self.view_frm = wx.Frame(self.frm, wx.ID_ANY, title=self.det_view.PutInstCode(), size=(800,600),style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        icon = wx.Icon(os.path.join(os.path.dirname(os.path.abspath(__file__)), "emaki_icon.png"), wx.BITMAP_TYPE_PNG)
        self.view_frm.SetIcon(icon)
        self.view_frm.CreateStatusBar()
        self.view_frm.Show()
        self.control = self.det_view.edit_traits(parent=self.view_frm, kind='panel').control
        self.det_view.SetPickerCallback(self.PickerCallback)
        if HAS_MPLOT:
            self.btn_pixel_plot_new.Enable()
            self.btn_pixel_plot_append.Enable()
            self.btn_2d_plot.Enable()
        self.btn_cb_apply.Enable()

    ########################################
    def OnViewFrame(self, event=None):
        detInfoPath = None
        dlg = wx.FileDialog(self.frm,
                           message = "Open a DetectorInfo file",
                           defaultDir = os.getcwd(),
                           wildcard = "DetectorInfo files|*.xml",
                           style = wx.OPEN | wx.CHANGE_DIR)
        if dlg.ShowModal() == wx.ID_OK:
            detInfoPath = dlg.GetPath()
        dlg.Destroy()
        if detInfoPath == None:
            return
        wiringInfoPath = None
        dlg = wx.FileDialog(self.frm,
                           message = "Open a WiringInfo file",
                           defaultDir = os.getcwd(),
                           wildcard = "WiringInfo files|*.xml",
                           style = wx.OPEN | wx.CHANGE_DIR)
        if dlg.ShowModal() == wx.ID_OK:
            wiringInfoPath = dlg.GetPath()
        dlg.Destroy()
        if wiringInfoPath == None:
            dlg = wx.MessageDialog(self.frm,
                message = "Do not you use any WiringInfo file?")
            if dlg.ShowModal() == wx.ID_OK:
                pass
            else:
                return
        self.det_view = DetView.EmakiDetectorViewInterface(str(detInfoPath), str(wiringInfoPath))
        self.view_frm = wx.Frame(self.frm, wx.ID_ANY, title=self.det_view.PutInstCode(), size=(800,600),style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        icon = wx.Icon(os.path.join(os.path.dirname(os.path.abspath(__file__)), "emaki_icon.png"), wx.BITMAP_TYPE_PNG)
        self.view_frm.SetIcon(icon)
        self.view_frm.CreateStatusBar()
        self.view_frm.Show()
        self.control = self.det_view.edit_traits(parent=self.view_frm, kind='panel').control
        self.det_view.SetPickerCallback(self.PickerCallback)
        if HAS_MPLOT:
            self.btn_pixel_plot_new.Enable()
            self.btn_pixel_plot_append.Enable()
            self.btn_2d_plot.Enable()
        self.btn_cb_apply.Enable()

    ########################################
    def OnPixelInfo(self, event=None):
        self.pixel_info_frm.Show()

    ########################################
    def OnExit(self, event=None):
        self.Exit()

    ########################################
    def OnLoadData(self, event=None):
        dlg = wx.FileDialog(self.frm,
                            message = "Open a serialized ECM file",
                            wildcard = "Serialized ECM files|*.srlz",
                            style = wx.OPEN | wx.CHANGE_DIR)
        if dlg.ShowModal() == wx.ID_OK:
            self._loaded_filepath = str(dlg.GetPath())
        dlg.Destroy()
        """ One ECM for each bank
        for info in self.det_view.bank_info:
            fn = '%s_%02d.srlz'%(str(rslt["File prefix"]), info[1])
            ecm = mm.ElementContainerMatrix()
            boostRead = mm.ReadSerializationFileBinary(fn)
            boostRead.Load(ecm)
            
            del boostRead
            ini, fin = float(rslt["ini"]), float(rslt["fin"])
            data = []
            for i in range(ecm.PutSize()):
                for j in range(ecm(i).PutSize()):
                    sum_ye = ecm(i, j).Sum(ini, fin)
                    data.append(sum_ye[0])
            self.det_view.SetScalarsToBankByBankId( info[1], data)
        """
        if not os.path.exists(self._loaded_filepath):
            print "File not found."
            return
        read = mm.ReadSerializationFileBinary(self._loaded_filepath)
        del self._loaded_ecm
        self._loaded_ecm = mm.ElementContainerMatrix()
        read.Load(self._loaded_ecm)
        del read
        self.btn_xrange_apply.Enable()
        self.frm.SetStatusText("Data: %s"%(self._loaded_filepath))

    ########################################
    def OnApplyXRange(self, event=None):
        hh = self._loaded_ecm.PutHeader()
        bank_id_list = list(hh.PutInt4Vector("BANKIDLIST"))
        bank_size_list = list(hh.PutInt4Vector("BANKSIZELIST"))
        start = float(self.tc_xrange_start.GetValue())
        end = float(self.tc_xrange_end.GetValue())
        view_mask = self.menu_mask_view.IsChecked()
        for info, config in zip(self.det_view.bank_info, self.det_view.bank_configurations):
            data = []
            SS = mm.SearchInHeader()
            SS.SetTarget(self._loaded_ecm)
            for det_id in config["det_id_list"]:
                SS.SearchArray("DETID", det_id)
                rslt_vector = SS.PutResultIndex(0)
                if rslt_vector.size() != 0:
                    eca_index = rslt_vector[0]
                    for pixel_num in range(self._loaded_ecm(eca_index).PutSize()):
                        if not view_mask:
                            sum_ye = self._loaded_ecm(eca_index, pixel_num).Sum(start, end)
                            data.append(sum_ye[0])
                        else:
                            if self._loaded_ecm(eca_index).PutHeader().PutInt4("MASKED"):
                                data.append(0.0)
                            elif self._loaded_ecm(eca_index, pixel_num).PutHeader().PutInt4("MASKED"):
                                data.append(0.0)
                            elif self.HasMask(self._loaded_ecm(eca_index, pixel_num), start, end):
                                data.append(0.0)
                            else:
                                sum_ye = self._loaded_ecm(eca_index, pixel_num).Sum(start, end)
                                data.append(sum_ye[0])
                else:
                    print "Not found in a data container: Det. ID:", det_id, ". Intensity is filled by 0."
                    #data.append( [0.0 for i in range(config["n_pxl_per_det"])] )
                    for i in range(config["n_pxl_per_det"]):
                        data.append( 0.0 ) ##[inamura 161111]
            del SS
            self.det_view.SetScalarsToBankByBankId( info[1], data)
        
    ########################################
    def HasMask(self, ec, start, end):
        """
        Is this EC has any error flag in this range?
        """
        x_list = ec.PutXList()
        e_list = ec.PutEList()
        if x_list[0] >= end or x_list[-1] <= start:
            return False
        for xx, e in zip( zip(x_list[:-1], x_list[1:]), e_list):
            if (start <= xx[0] and xx[1] <= end) and e < 0.:
                return True
            if xx[0] >= end:
                return False
        else:
            return False

    ########################################
    def OnSetTwoTheta(self, event=None):
        pool = multiprocessing.Pool()
        for bankIndex, bank_config in enumerate(self.det_view.bank_configurations):
            coordinates = bank_config["coordinates"].T # これでcoordinatesは(n, 3)
            args = list(coordinates)
            data = np.array( pool.map(PutTwoTheta, args) )
            self.det_view.SetScalarsToBank( bankIndex, data)

    ########################################
    def PickerCallback(self, picker_obj):
        #print picker_obj.selection_point # 画面上の位置 
        #print picker_obj.pick_position # 空間上の位置
        #print picker_obj.mapper_position # mapper内での位置
        picked = picker_obj.actors
        if len(picked) == 0:
            self.view_frm.SetStatusText( "No object is picked.")
            return
        if len(picked) > 1:
            self.view_frm.SetStatusText( "Pass due to multi picking.")
            return
        for i, bank_module in enumerate(self.det_view.bank_modules):
            if bank_module.actor.actor._vtk_obj in [ o._vtk_obj for o in picked]:
                bank_config = self.det_view.bank_configurations[i]
                glyph_points = bank_module.glyph.glyph_source.glyph_source.output.points.to_array()
                point_id = picker_obj.point_id/glyph_points.shape[0]
                intensity = bank_module.mlab_source.scalars[point_id]
                x_rel = bank_module.mlab_source.x[point_id]
                y_rel = bank_module.mlab_source.y[point_id]
                z_rel = bank_module.mlab_source.z[point_id]
                x_abs, y_abs, z_abs = picker_obj.picked_positions[0]
                tth = PutTwoTheta( np.array([x_abs, y_abs, z_abs]) )
                azm = np.degrees(np.arctan2(y_abs, x_abs))
                """
                template = 'Intensity: %.1f. Abs. X, Y, Z: %.1f, %.1f, %.1f. Rel. X, Y, Z: %.1f, %.1f, %.1f. Bank: %s. # %d'
                self.view_frm.SetStatusText(template%(intensity,
                                                     x_abs, y_abs, z_abs,
                                                     x_rel, y_rel, z_rel,
                                                     self.det_view.bank_info[i][0], point_id) )
                """
                self.view_frm.SetStatusText('Picked.')
                self.st_intensity.SetLabel('%.4e'%intensity)
                self.st_x_pos.SetLabel('%.1f'%x_abs)
                self.st_y_pos.SetLabel('%.1f'%y_abs)
                self.st_z_pos.SetLabel('%.1f'%z_abs)
                self.st_tth.SetLabel('%.2f'%tth)
                self.st_azm.SetLabel('%.2f'%azm)
                self.st_rx_pos.SetLabel('%.1f'%x_rel)
                self.st_ry_pos.SetLabel('%.1f'%y_rel)
                self.st_rz_pos.SetLabel('%.1f'%z_rel)
                self.st_bank_name.SetLabel('%s, %d'%(self.det_view.bank_info[i][0], self.det_view.bank_info[i][1]))
                self.picked_pixel[0] = self.det_view.bank_info[i][1]
                n_pxl_per_det = bank_config["n_pxl_per_det"]
                det_id = bank_config["det_id_list"][point_id/n_pxl_per_det]
                self.picked_pixel[1] = det_id
                pixel_position = point_id%n_pxl_per_det
                self.picked_pixel[2] = pixel_position
                self.st_det_id.SetLabel('%d'%det_id)
                self.st_pixel_pos.SetLabel('%d'%pixel_position)
                self.st_pixel_id.SetLabel('%d'%point_id)
                break

    ########################################
    def PutElementContainerToPixelPlot(self):
        SS = mm.SearchInHeader()
        SS.SetTarget(self._loaded_ecm)
        SS.SearchArray("DETID", self.picked_pixel[1])
        rslt_vector = SS.PutResultIndex(0)
        if rslt_vector.size() != 0:
            eca_index = rslt_vector[0]
        else:
            print('[!] No mathcing DETID %d'%self.picked_pixel[1])
            return None
        del SS
        return self._loaded_ecm(eca_index).Put(self.picked_pixel[2])

    ########################################
    def PutElementContainerArrayTo2dPlot(self):
        SS = mm.SearchInHeader()
        SS.SetTarget(self._loaded_ecm)
        SS.SearchArray("DETID", self.picked_pixel[1])
        rslt_vector = SS.PutResultIndex(0)
        if rslt_vector.size() != 0:
            eca_index = rslt_vector[0]
        else:
            print('[!] No mathcing DETID %d'%self.picked_pixel[1])
            return None
        del SS
        return self._loaded_ecm.Put(eca_index)

    ########################################
    def OnPixelPlotNew(self, event):
        ec = self.PutElementContainerToPixelPlot()
        if ec is None:
            raise UserWarning
            return
        if self.mplot is None:
            self.mplot = MP.MPlot(ec)
        else:
            self.mplot.NewPlot(ec)

    ########################################
    def OnPixelPlotAppend(self, event):
        if self.mplot is None:
            raise UserWarning, '[!] No plotter exists.'
            return
        dlg = wx.TextEntryDialog(self.frm,
             message = "Plotter number",
             caption = "Enter plotter number",
             defaultValue = "1", )
        if dlg.ShowModal() == wx.ID_OK:
            target = int(dlg.GetValue())
        else:
            return
        self.mplot.ChangePlot(target)
        ec = self.PutElementContainerToPixelPlot()
        if ec is None:
            raise UserWarning
            return
        self.mplot.AddData(ec)

    ########################################
    def On2dPlot(self, event):
        eca = self.PutElementContainerArrayTo2dPlot()
        if eca is None:
            raise UserWarning
            return
        M2P.M2Plot(eca)

    ########################################
    def OnColorbarRange(self, event):
        """
        cb_range = self.det_view.bank_modules[0].actor.mapper.scalar_range[:]
        queries = [
                  ['Min', '%f'%cb_range[0]],
                  ['Max', '%f'%cb_range[1]]
                ]
        dlg = EmakiSimpleDialog.EmakiSimpleDialog(self.frm, title='Colorbar range', queries=queries)
        if dlg.ShowModal() == wx.ID_OK:
            self.SetColorbarRange( *dlg.PutItems())
        dlg.Destroy()
        """
        min = float(self.tc_cb_min.GetValue())
        max = float(self.tc_cb_max.GetValue())
        self.SetColorbarRange(min, max)

    ########################################
    def SetColorbarRange(self, min_str, max_str):
        for mod in self.det_view.bank_modules:
            mod.actor.mapper.scalar_range = [float(min_str), float(max_str) ]
        return

    ########################################
    def OnSetTwoTheta(self, event=None):
        pool = multiprocessing.Pool()
        for bankIndex, bank_config in enumerate(self.det_view.bank_configurations):
            coordinates = bank_config["coordinates"].T # これでcoordinatesは(n, 3)
            args = list(coordinates)
            data = np.array( pool.map(PutTwoTheta, args) )
            self.det_view.SetScalarsToBank( bankIndex, data)

    ########################################
    def OnCreateDummyData(self, event=None):
        dlg = wx.TextEntryDialog(self.frm,
             message = "File prefix",
             caption = "Enter a prefix.",
             defaultValue = "SAS999999", )
        if dlg.ShowModal() == wx.ID_OK:
            prefix = str(dlg.GetValue())
        else:
            return
        def gauss(A, c, w, x_data):
            template = "np.exp(-(x_data - c*%d)**2/(2.0*w**2))+"
            s = 'A*('
            for i in range(1,37):
                s += template%(i)
            else:
                s = s.rstrip('+')
            s += ')'
            return eval(s)

        def calc_tof(d, L, tth):
            return d*(252.777*L*2*math.sin(math.radians(tth)/2))

        inst_code = self.det_view.PutInstCode()
        for bankIndex, bank_config in enumerate(self.det_view.bank_configurations):
            print("Dummy data creation: %d / %d"%( bankIndex, len(self.det_view.bank_configurations)))
            coordinates = bank_config["coordinates"].T
            X = np.array([i*40.0 for i in range(1,1001)])
            X_mid = np.array([i*40.0+20.0 for i in range(1,1000)])
            ecm = mm.ElementContainerMatrix()
            ecm.AddToHeader("INSTRUMENT", inst_code)
            det_id_list = bank_config["det_id_list"]
            n_det = bank_config["n_det"]
            n_pxl_per_det = bank_config["n_pxl_per_det"]
            for i, det_id in enumerate(det_id_list):
                print i, bankIndex
                eca = mm.ElementContainerArray()
                eca.AddToHeader("PSDID", det_id)
                for j in range(n_pxl_per_det):
                    x, y, z = coordinates[i*n_pxl_per_det+j].tolist()
                    tth = PutTwoTheta(coordinates[i*n_pxl_per_det+j])
                    L2 = math.sqrt(x**2+y**2+z**2)*0.001
                    L = L2 + 14.5
                    c = calc_tof(1.0, L, tth)
                    Y = gauss(1000.0, c, 40.0, X_mid)
                    E = np.sqrt(Y)
                    lmbd_bins = [tof/252.777/L for tof in X.tolist()]
                    qx_bins = [2*math.pi*x*0.001/L2/lmbd for lmbd in lmbd_bins]
                    qy_bins = [2*math.pi*y*0.001/L2/lmbd for lmbd in lmbd_bins]
                    qz_bins = [2*math.pi*(1.0-z*0.001/L2)/lmbd for lmbd in lmbd_bins]
                    qx_center = []
                    qy_center = []
                    qz_center = []
                    for k in range(len(lmbd_bins) -1):
                        qx_center.append( (qx_bins[k]+qx_bins[k+1])*0.5 )
                        qy_center.append( (qy_bins[k]+qy_bins[k+1])*0.5 )
                        qz_center.append( (qz_bins[k]+qz_bins[k+1])*0.5 )
                    ec = mm.ElementContainer()
                    ec.AddToHeader("PSDID", det_id)
                    ec.AddToHeader("PixelPosition", mm.ListToDoubleVector([x, y, z]))
                    ec.Add('TOF', X.tolist())
                    ec.Add('Count', Y.tolist())
                    ec.Add('Error', E.tolist())
                    ec.Add('qx', qx_center)
                    ec.Add('qy', qy_center)
                    ec.Add('qz', qz_center)
                    ec.SetKeys('TOF', 'Count', 'Error')
                    eca.Add(ec)
                ecm.Add(eca)
            boostWrite = mm.WriteSerializationFileBinary("%s_%02d.srlz"%(prefix, self.det_view.bank_info[bankIndex][1]))
            boostWrite.Save(ecm)
            del boostWrite
            del ecm
        print("Dummy data creation: End")

    ########################################
    def OnSetUserDefinedFormula(self, event=None):
        dlg = wx.TextEntryDialog(self.frm,
             message = "Formula",
             caption = "Enter your formula.",
             defaultValue = "sqrt(x**2 + y**2 + z**2)", )
        if dlg.ShowModal() == wx.ID_OK:
            formula = str(dlg.GetValue())
        else:
            return
        pool = multiprocessing.Pool()

        for bankIndex, bank_config in enumerate(self.det_view.bank_configurations):
            coordinates = bank_config["coordinates"].T
            args = list(coordinates)
            args = [[l, formula] for l in args]
            data = np.array( pool.map(PutResultOfUserDefinedFormula, args) )
            self.det_view.SetScalarsToBank( bankIndex, data)

    ########################################
    def OnSetPickerTolerance(self, event):
        try:
            tolerance = self.det_view.PutPickerTolerance()
        except:
            return
        dlg = wx.TextEntryDialog(self.frm,
            message = "Tolerance",
            caption = "Enter new tolerance.",
            defaultValue = "%f"%tolerance, )
        if dlg.ShowModal() == wx.ID_OK:
            try:
                tolerance_new = float(dlg.GetValue())
            except:
                return
        else:
            return
        self.det_view.SetPickerTolerance(tolerance_new)


if __name__ == '__main__':
    import sys
    _redirect = False
    app = MyMainApp(redirect=_redirect)
    app.MainLoop()

